/********************************************************************\
*****   Basisklasse fr Audio-Codec                              *****
\********************************************************************/

#ifndef _AUDIO_CODEC_H_
#define _AUDIO_CODEC_H_

#include <sys/types.h>

#include "Codec.h"
#include "A_Format.h"

//class VideoFile;
class A_Track;
class VideoFile;
// class Codec;

/**Basisklasse fr Audio Codecs.<br>
  *@author Christian Tusche
  */

class A_Codec: public Codec, public A_Format {
public:
    A_Codec( A_Track *Track );

    virtual ~A_Codec();

    /**int16 Audiodaten schreiben<br>
      *daten ist ein Feld der Form [channels][samples] und enthlt
      *fr jedes Sample einen Wert zwischen -2^15 und +2^15
      */
    virtual int encodeAudio( int16_t **daten, int samples );

    /**float Audiodaten schreiben<br>
      *daten ist ein Feld der Form [channels][samples] und enthlt
      *fr jedes Sample einen Wert zwischen -1.0 und +1.0
      */
    virtual int encodeAudio( float **daten, int samples );

    /**int16 Audiodaten lesen<br>
      *daten wie bei encodeAudio
      */
    virtual int decodeAudio( int16_t **&daten, int samples );

    /**float Audiodaten lesen<br>
      *daten wie bei encodeAudio
      */
    virtual int decodeAudio( float **&daten, int samples );

    /**gehe zu Sample
      */
    virtual int SeekSample( int64_t sample );

    /**gehe zu Zeitpunkt
      */
    virtual int SeekTime( double seconds );

    /**leifert Zeiger auf Track
      */
    A_Track *Track( void ) { return track; }

protected:
    /**Codecspezifische Implementierung zum encodieren der Samples
      */
    virtual int encode( float **daten, int sampels );

    /**Codecspezifische Implementierung zum decodieren der Samples
      */
    virtual int decode( float **daten, int sampels );

    A_Track *track;

private:
    // interne Puffer-Verwaltung

    /**Puffer anlegen<br>
      *len: Anzahl der Samples pro Puffer/Channel
      */
    void alloc_buffers( long len );

    /**Puffer fr int16 Samples
      */
    int16_t **sample_puffer_int16;

    /**Puffer fr float Samples
      */
    float **sample_puffer_float;

    /**Gre der Puffer
      */
    long sample_puffer_len;

    /**Anzahle der Channels des Puffers
      */
    int sample_puffer_channels;


};


#endif
