/***************************************************************************
                          A_Format.h  -  description
                             -------------------
    begin                : Tue Jan 29 2002
    copyright            : (C) 2002 by Christian Tusche
    email                : ctusche@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef A_FORMAT_H
#define A_FORMAT_H

#include <inttypes.h>

/**Format-Info zum Audio Track
  *@author Christian Tusche
  */

class A_Format {
public: 
    A_Format();
    A_Format( const A_Format &format );
    A_Format( int Sample_rate, int Channels );
    virtual ~A_Format();

    /**Samples pro Sekunde
     */
    virtual int SampleRate( void ) { return sample_rate; }

    /**Anzahl der Kanle
     */
    virtual int Channels( void ) { return channels; }

    /**Bits pro Sample
     */
    virtual int Bits( void ) { return bits; }
    
    /**liefert Bitrate des Codecs<br>
     *oder 0 fr unbekannte Bitrate oder variable Bitrate
     */
    virtual double BitsPerSecond( void ) { return bits_per_second; }
    
    /**Samples in einem Datenpaket<br>
     *Audioasmples knnen immer als vielfaches von diesem Wert
     *geschrieben und gelesen werden.
     */
    virtual int FrameSamples( void ) { return samples_per_pack; }
    
    /**Gre eines Datenpaketes in Bytes<br>
     *oder 0 fr unbekannte Gre oder variable Gre.
     */
    virtual int FrameBytes( void ) { return (int)(bits_per_second * samples_per_pack / (sample_rate * 8) + 0.5 ); }
    
    
    //A_Format &operator =(A_Format format);
    

        
protected:
    /**Werte aus einem anderen Formateintrag bernehmen
      */
    void SetAFormat( A_Format &format );

    /**Samples pro Sekunde einstellen
     */
    virtual void SetSampleRate( int rate ) { sample_rate = rate; }
    
    /**Samples pro Sekunde einstellen
     */
    virtual void SetChannels( int ch ) { channels = ch; }
    
    /**Bits pro Sample setzen
     */
    virtual void SetBits( int b ) { bits = b; }
    
    /**Bitrate des Codecs setzen<br>
     *0 fr unbekannte Bitrate oder variable Bitrate
     */
    virtual void SetBitsPerSecond( double bitrate ) { bits_per_second = bitrate; }
    
    /**Samples in einem Datenpaket einstellen
     */
    virtual void SetFrameSamples( int samples ) { samples_per_pack = samples; }

        
private:
    int sample_rate;            // Sampels/sec
    int channels;               // Audio-Kanle
    int bits;                   // Bits pro Sample und Kanal (8, 16)
    double bits_per_second;
    int samples_per_pack;
};

#endif
