/*****************************************************************************\
**  Author  : Christian Tusche                                               **
**  E-Mail  : ctusche@gmx.net                        **
**                                                                           **
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/

#include <stdio.h>
#include <assert.h>

#include "VideoFile.h"
#include "debug.h"
#include "translate.h"




/***** Konstruktor / Destruktor *****/
A_Track::A_Track( VideoFile *File, int track )
:Track( File, track )
{
    // Variablen initialisieren
    codec = NULL;
    read_puffer = NULL;
    read_puffer_rest = 0;
    read_puffer_gr = 0;
    read_puffer_start = 0;
    total_samples   = 0;
    read_last_pos = 0;

    File->addTrack( this );
}


A_Track::A_Track( VideoFile *File, int Sample_rate, int Channels, const char *Codec_name )
:Track( File, 0, Codec_name ), A_Format( Sample_rate, Channels )
{
    codec = NULL;
    read_puffer = NULL;
    read_puffer_rest = 0;
    read_puffer_gr = 0;
    read_puffer_start = 0;
    total_samples   = 0;
    read_last_pos = 0;
    
    File->addTrack( this );
}





A_Track::~A_Track()
{
    // DEBUG_TXT( "delete A_Track %s", codec_name );

    if( codec != NULL ) {
        delete codec;
    }

    file->removeTrack( this );
}






/***** Rohdaten lesen und schreiben *****************************************************/
// Rckgabewert: 1   -    OK
//               <1  -    Fehler
int A_Track::readData( unsigned char *&data, long &bytes )
{
    int stat;

    stat = readRaw( data, bytes );
    pos++;                    // aktueller Datenblock

    return stat;
}



int A_Track::writeData( unsigned char *data, long bytes, long samp )
{
    total_samples += samp;
    Track::bytes += bytes;
    pos++;                    // aktueller Datenblock
    total_blocks++;           // Gesamtlnge

    // Blockrate aktualisieren
    block_rate = ((double)pos * (double)SampleRate()) / total_samples;

    return writeRaw( data, bytes, samp );
}




// beliebige anzahl an Bytes in den Puffer data schreiben
// es knnen weniger Bytes gelesen werden, die tatschliche Anzahl
// in &bytes gespeichert
int A_Track::readBytes( unsigned char *data, long &bytes )
{
    int stat = 1;
    unsigned char *puffer;
    long n=0, rest = bytes, bytes_read=0;

    // Puffer ist nach Seek ungltig
    if( read_last_pos != pos )
        read_puffer_rest = 0;
    
    // noch Daten von letztem Aufruf im Puffer?
    if( read_puffer_rest > 0 ) {
        n = (bytes > read_puffer_rest)? read_puffer_rest : bytes;
        memcpy( data, read_puffer + read_puffer_start, n );
        read_puffer_rest -= n;
        read_puffer_start += n;
        rest -= n;
    }

    // mssen noch neue Daten aus der Datei gelesen werden?
    while( rest > 0 ) {
        assert( read_puffer_rest == 0 );
        stat = readData( puffer, bytes_read );
        // letzte Position merken
        read_last_pos = pos;

        if( !stat ) {
            // keine Daten mehr
            if( rest > 0 )
                bytes -= rest;

            // wenn noch was gelesen wurde -> ok
            return (bytes>0)? 1 : stat;
        }

        // gelesen Daten in Zielpuffer schreiben
        n = (rest > bytes_read)? bytes_read : rest;
        memcpy( data + bytes - rest, puffer, n );
        bytes_read -= n;
        rest -= n;
    }

    // noch Daten ber?
    if( bytes_read > 0 ) {
        if( bytes_read > read_puffer_gr ) {
            delete [] read_puffer;
            read_puffer_gr = bytes_read;
            read_puffer = new unsigned char[ read_puffer_gr ];
        }

        // fr nchsten Aufruf zwischenspeichen
        memcpy( read_puffer, puffer + n, bytes_read );
        read_puffer_rest = bytes_read;
        read_puffer_start = 0;
    }

    return stat;
}






/***** Rohdaten lesen und schreiben *****************************************************/
// eigentliche Implementierung  - mu in abgeleiteter Klasse berladen werden
int A_Track::readRaw( unsigned char *&data, long &bytes )
{
    ERROR_TXT( _("read data not implemented!") );
    return 0;
}



int A_Track::writeRaw( unsigned char *data, long bytes, int samp )
{
    ERROR_TXT( _("write data not implemented!") );
    return 0;
}

