/********************************************************************\
*****   Basisblasse fr Audio-Track                             *****
\********************************************************************/

#ifndef _AUDIO_TRACK_H_
#define _AUDIO_TRACK_H_

#include <sys/types.h>

#include "Track.h"
#include "A_Format.h"

class VideoFile;
class A_Codec;

/**Basisklasse fr Audio Tracks.<br>
  *beinhaltet alle Operationen, die mit einem Audiotrack ausgefhrt
  *werden knnen
  *@author Christian Tusche
  */

class A_Track: public virtual Track, public A_Format {
    friend class A_Codec;
public:

    A_Track( VideoFile *File, int track=0 );
    A_Track ( VideoFile *File,
              int Sample_rate,
              int Channels,
              const char *Codec_name = NULL );

    virtual ~A_Track();
    

    /**Anzahl der Samples<br>
      * Rcgabe  0: Track leer / Anzahl nicht bekannt<br>
      *         >0: Anzahl der Samples
      */
    virtual int64_t Samples( void ) { return total_samples; }

    /**Endzeit des Tracks in Sekunden
      */
    virtual double TotalTime( void ) { return (double)total_samples / SampleRate(); }

    /**Liefert Zeiger auf Audiocodec
      */
    inline A_Codec *Codec( void ) { return codec; }

    /**Einen Datenblock lesen<br>
      *speichert Zeiger auf Datenblock in *&data
      *und die Gre des Blocks in &bytes
      */
    virtual int readData( unsigned char *&data, long &bytes );

    /**Beliebige Anzahl an Bytes in den Puffer *data schreiben<br>
      *es knnen weniger Bytes gelesen werden, die tatschliche Anzahl
      *in &bytes gespeichert
      */
    virtual int readBytes( unsigned char *data, long &bytes );

    /**Rohdaten-Block schreiben
      */
    virtual int writeData( unsigned char *data, long bytes, long samp = 0 );



protected:
    /**Daten lesen - interne Implementierung
      */
    virtual int readRaw( unsigned char *&data, long &bytes );

    /**Daten schreiben - interne Implementierung
      */
    virtual int writeRaw( unsigned char *data, long bytes, int samp=0 );

    A_Codec *codec;             // Videocodec fr diesen Track

    long long total_samples;    // Anzahl der Samples im Track
    
    unsigned char *read_puffer; // Puffer fr gelesene Daten
    long read_puffer_rest;      // Bytes im Puffer
    long read_puffer_start;     // Start der Daten im Puffer
    long read_puffer_gr;        // Gre des Puffers
    long read_last_pos;         // letzte Position, von der gelesen wurde
};

#endif
