/***************************************************************************
                          Codec.h  -  description
                             -------------------
    begin                : Sat Jan 19 2002
    copyright            : (C) 2002 by Christian Tusche
    email                : ctusche@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CODEC_H
#define CODEC_H

#include <inttypes.h>

#include "optionlist.h"

class VideoFile;
class Track;

/**Gemeinsame Funktionen fr A_Codec und V_Codec
  *@author Christian Tusche
  */

class Codec: public OptionList {
public: 
    Codec( Track *tr );
    virtual ~Codec();

    /**Liefert den richtigen Namen des Codecs fr diesen
      *Dateityp.
      */
    virtual const String CodecName( void );

    /**Zustzlich Informationen zu diesem Codec auslesen<br>
      *der eigentliche Wert von "info" hngt vom Typ des IO-Moduls
      *ab! "size" gibt die Datenlnge an.<br>
      *Der Rckgabewert: 1 - info gesetzt<br>
      *                  0 - sonst
      */
    virtual int GetExtraInfo( uint8_t *&info, unsigned int &size ) { return 0; }

    /**Positioniere Track<br>
      *Rckgabe: 1 : erfolg<br>
      *          0 : Fehler<br>
      *gegebenenfalls werden Keyframes decodiert
      */
    virtual int SeekTime( double seconds ) { return track->SeekTime( seconds ); }

protected:
    /**IO-Modus
      */
    inline VideoFile::IO_Mode IOMode( void ) { return iomode; }

    Track *track;
    VideoFile *file;       // bergeordnete Videodatei
    VideoFile::IO_Mode iomode;
};

#endif
