/***************************************************************************
                          codecinfo_entry.h  -  description
                             -------------------
    begin                : Fri Jan 18 2002
    copyright            : (C) 2002 by Christian Tusche
    email                : ctusche@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CODECINFO_ENTRY_H
#define CODECINFO_ENTRY_H

#include "String.h"
#include "PluginInfo.h"


class V_Codec;
class A_Codec;
class V_Track;
class A_Track;

class CodecPlugins;


/**Formatinfo eines Codecs
  *@author Christian Tusche
  */

class Codec_Info: public PluginInfo {
    friend class CodecPlugins;
public:
    /**erstellt einen Infoeintrag fr einen Videocodec<br>
      *Dateityp: Name des Dateityps/IO-Moduls, fr das dieser Codec verwendet werden kann<br>
      *Codecname: Name des Codecs mit dem dieser aufgerufen wird<br>
      *konstruktor: C Funktion, die eine Codec-Objekt erzugt<br>
      *Codecalias: Name des Codecs, der in der Videodatei verwndet wird<br>
      *InfoTxt: Beschreibung des Codecs<br>
      *read, write: Fhigkeiten des Codecs
      */
    Codec_Info( const String Dateityp, const String Codecname, V_Codec *(*konstruktor)(V_Track *), const String Codecalias = "", const String InfoTxt = "", bool read=true, bool write=true );

    /**erstellt einen Infoeintrag fr einen Audiocodec<br>
      *Dateityp: Name des Dateityps/IO-Moduls, fr das dieser Codec verwendet werden kann<br>
      *Codecname: Name des Codecs mit dem dieser aufgerufen wird<br>
      *konstruktor: C Funktion, die eine Codec-Objekt erzugt<br>
      *Codecalias: Name des Codecs, der in der Videodatei verwndet wird<br>
      *InfoTxt: Beschreibung des Codecs<br>
      *read, write: Fhigkeiten des Codecs
      */

    Codec_Info( const String Dateityp, const String Codecname, A_Codec *(*konstruktor)(A_Track *), const String Codecalias = "", const String InfoTxt = "", bool read=true, bool write=true );

    Codec_Info( const Codec_Info &entry );

    Codec_Info();

    ~Codec_Info();

    /** Echter Name des Codecs
      */
    String CodecName( void );

    /** Codec Alias-Name
      */
    String AliasName( void );

    /**Video Codec erstellen
      */
    V_Codec *CreateVCodec( V_Track *track );

    /**Audio Codec erstellen
      */
    A_Codec *CreateACodec( A_Track *track );

    /**Beschreibung des Moduls
      */
    String InfoText( void ) { return info_txt; }

    /**Video-Codec
      */
    bool Video( void ) { return v_konstr != NULL; }

    /**Audio-Codec
      */
    bool Audio( void ) { return a_konstr != NULL; }

    /**Datei-Typ
      */
    String FileTyp( void ) { return datei_typ; }

private:
    String datei_typ;
    String codec_name;
    String alias_name;
    V_Codec *(*v_konstr)( V_Track *track );
    A_Codec *(*a_konstr)( A_Track *track );

    // Beschreibung
    String info_txt;
    
    // untersttzte Operationen
    bool can_read, can_write;
};

#endif
