/*
   bttvgrab 0.15.10 [2000-03-21]
   (c) 1998, 1999 by Joerg Walter <trouble@moes.pmnet.uni-oldenburg.de>
   Maintained by: Joerg Walter
   Current version at http://moes.pmnet.uni-oldenburg.de/bttvgrab/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <limits.h>
#include <errno.h>
#include "Convert.h"
#include "debug.h"


// YCC<->RGB code taken from IJG's jpeglib-v6b
// Copyright (C) 1991-1996, Thomas G. Lane.

#define NO_C            -128
#define CBCR_OFFSET     ((int) 128 << 16)
#define ONE_HALF        ((int) 1 << 15)
#define FIX(x)          ((int) ((x) * (double)(1L<<16) + 0.5))
#define R_Y_OFF         0                       /* offset to R => Y section */
#define G_Y_OFF         (1*256)      /* offset to G => Y section */
#define B_Y_OFF         (2*256)      /* etc. */
#define R_CB_OFF        (3*256)
#define G_CB_OFF        (4*256)
#define B_CB_OFF        (5*256)
#define R_CR_OFF        B_CB_OFF                /* B=>Cb, R=>Cr are the same */
#define G_CR_OFF        (6*256)
#define B_CR_OFF        (7*256)





/*********** storage macros ***********/
#define GET_BGR24(r,g,b,ptr) b = *(ptr++); g = *(ptr++); r = *(ptr++);
#define PUT_BGR24(r,g,b,ptr) *(ptr++) = b; *(ptr++) = g; *(ptr++) = r;
#define GET_RGB24(r,g,b,ptr) r = *(ptr++); g = *(ptr++); b = *(ptr++);
#define PUT_RGB24(r,g,b,ptr) *(ptr++) = r; *(ptr++) = g; *(ptr++) = b;
#define GET_8BIT(x,ptr) x = *(ptr++);
#define PUT_8BIT(x,ptr) *(ptr++) = x;
#define GET_RGB16(r,g,b,ptr) r = (*((unsigned short *)ptr) >> 11) & 0x1f; g = (*((unsigned short *)ptr) >> 5) & 0x3f; b = *(((unsigned short *)ptr)++) & 0x1f;
#define PUT_RGB16(r,g,b,ptr) *(((unsigned short *)ptr)++) = ((r & 0x1f) << 11) | ((g & 0x3f) << 5) | (b & 0x1f);
#define GET_YUV420(y0,y1,y2,y3,Cb,Cr,ptr1,ptr2) y0=*(ptr1++); Cr=*(ptr1++); y1=*(ptr1++); Cb=*(ptr1++); y2=*(ptr2++); y3=*(ptr2++);
#define PUT_YUV420(y0,y1,y2,y3,Cb,Cr,ptr1,ptr2) *(ptr1++)=y0; *(ptr1++)=Cr; *(ptr1++)=y1; *(ptr1++)=Cb; *(ptr2++)=y2; *(ptr2++)=y3;
#define GET_YUV420PAL(y0,y1,y2,y3,Cb,Cr,pY0,pY1,pCb,pCr) y0=*(pY0++); Cr=*(pCr++); y1=*(pY0++); Cb=*(pCb++); y2=*(pY1++); y3=*(pY1++);
#define PUT_YUV420PAL(y0,y1,y2,y3,Cb,Cr,pY0,pY1,pCb,pCr) *(pY0++)=y0; *(pCr++)=Cr; *(pY0++)=y1; *(pCb++)=Cb; *(pY1++)=y2; *(pY1++)=y3;
#define GET_YUV422S(y0,y1,Cb,Cr,ptr) y0 = *(ptr++); Cb = *(ptr++); y1 = *(ptr++); Cr = *(ptr++);
#define PUT_YUV422S(y0,y1,Cb,Cr,ptr) *(ptr++) = y0; *(ptr++) = Cb; *(ptr++) = y1; *(ptr++) = Cr;
#define GET_YUV422SP(y0,y1,Cb,Cr,ptrY,ptrCb,ptrCr) y0 = *(ptrY++); Cb = *(ptrCr++); y1 = *(ptrY++); Cr = *(ptrCb++);
#define PUT_YUV422SP(y0,y1,Cb,Cr,ptrY,ptrCb,ptrCr) *(ptrY++) = y0; *(ptrCr++) = Cb; *(ptrY++) = y1; *(ptrCb++) = Cr;
/**************************************/

/************ conversion macros ***************/
#define RGB242PAL8(r,g,b,c) c = convert_backpal[convert_256to5tab[r]][convert_256to9tab[g]][convert_256to5tab[b]];
#define RGB242RGB16(ri,gi,bi,ro,go,bo) ro = convert_8to5tab[ri]; go = convert_8to6tab[gi]; bo = convert_8to5tab[bi];
#define RGB242YUV(r,g,b,y,cb,cr) \
	y = (char)((convert_rgb_ycc_tab[r+R_Y_OFF] + convert_rgb_ycc_tab[g+G_Y_OFF] + convert_rgb_ycc_tab[b+B_Y_OFF]) >> 16); \
	cb= (char)((convert_rgb_ycc_tab[r+R_CB_OFF] + convert_rgb_ycc_tab[g+G_CB_OFF] + convert_rgb_ycc_tab[b+B_CB_OFF]) >> 16); \
	cr= (char)((convert_rgb_ycc_tab[r+R_CR_OFF] + convert_rgb_ycc_tab[g+G_CR_OFF] + convert_rgb_ycc_tab[b+B_CR_OFF]) >> 16);
#define RGB242GREY8(r,g,b,l) \
	l = (char)((convert_rgb_ycc_tab[r+R_Y_OFF] + convert_rgb_ycc_tab[g+G_Y_OFF] + convert_rgb_ycc_tab[b+B_Y_OFF]) >> 16);

#define PAL82RGB24(c,r,g,b) r = convert_palette[(int)c][0]; g = convert_palette[(int)c][1]; b = convert_palette[(int)c][2];
#define PAL82RGB16(c,r,g,b) r = convert_palette16[(int)c][0]; g = convert_palette16[(int)c][1]; b = convert_palette16[(int)c][2];
#define PAL82YUV(c,y,cb,cr) y = convert_paletteYCC[(int)c][0]; cb = convert_paletteYCC[(int)c][1]; cr = convert_paletteYCC[(int)c][2];
#define PAL82GREY8(c,l) l = convert_paletteG[(int)c];

#define RGB162RGB24(ri,gi,bi,ro,go,bo) ro = convert_5to8tab[ri]; go = convert_6to8tab[gi]; bo = convert_5to8tab[bi];
#define RGB162PAL8(r,g,b,c) c = convert_backpal[convert_32to5tab[r]][convert_64to9tab[g]][convert_32to5tab[b]];
#define RGB162YUV(r,g,b,y,cb,cr) RGB162RGB24(r,g,b,r,g,b) RGB242YUV(r,g,b,y,cb,cr)
#define RGB162GREY8(r,g,b,l) RGB162RGB24(r,g,b,r,g,b) RGB242GREY8(r,g,b,l)

#define YUV2RGB24(y,cb,cr,r,g,b) \
	r = convert_rangetable[y + convert_Crrtab[cr]]; \
	g = convert_rangetable[y + ((int) (convert_Cbgtab[cb] + convert_Crgtab[cr]) >> 16)]; \
	b = convert_rangetable[y + convert_Cbbtab[cb]];
#define YUV2RGB16(y,cb,cr,r,g,b) YUV2RGB24(y,cb,cr,r,g,b) RGB242RGB16(r,g,b,r,g,b)
#define YUV2PAL8(y,cb,cr,c) c = convert_backpal[convert_256to5tab[convert_rangetable[y + convert_Crrtab[cr]]]] \
						[convert_256to9tab[convert_rangetable[y + ((int) (convert_Cbgtab[cb] + convert_Crgtab[cr]) >> 16)]]] \
						[convert_256to5tab[convert_rangetable[y + convert_Cbbtab[cb]]]]
#define YUV2GREY8(y,cb,cr,l) l = y;

#define GREY82RGB24(l,r,g,b) r = g = b = l;
#define GREY82PAL8(l,c) RGB242PAL8(l,l,l,c)
#define GREY82RGB16(l,r,g,b) RGB242RGB16(l,l,l,r,g,b)
#define GREY82YUV(l,y,cb,cr) RGB242YUV(l,l,l,y,cb,cr)
/**********************************************/

/******************* scaling macros *******************/
#define INTERPOL2(x1,x2) x1
#define INTERPOL4(x1,x2,x3,x4) x1
#define addINTERPOLn(x,x2,n) do {x += x2; n++} while (0)
#define INTERPOLn(x,n) (x/n)
/******************************************************/









const int Convert::convert_color_bpps[CONVERT_COLORS] = {24,8,16,16,12,8,16};

const Convert::convert_init_func Convert::convert_inittable[CONVERT_COLORS] = {
        ADD(init24),
        ADD(init8),
        ADD(init16),
        ADD(init422),
        ADD(init420),
        ADD(initG),
        NULL
    };

const Convert::convert_func Convert::convert_table[CONVERT_COLORS][CONVERT_COLORS] = {
    /*BGR24 -> x */	{ ADD(nil24),   ADD(24to8),  ADD(24to16),  ADD(24to422),  ADD(24to420),  ADD(24toG)  },
    /*RGB8  -> x */ { ADD(8to24),   ADD(nil8),   ADD(8to16),   ADD(8to422),   ADD(8to420),   ADD(8toG)   },
    /*RGB16 -> x */ { ADD(16to24),  ADD(16to8),  ADD(nil16),   ADD(16to422),  ADD(16to420),  ADD(16toG)  },
    /*422   -> x */ { ADD(422to24), ADD(422to8), ADD(422to16), ADD(nil422),   ADD(422to420), ADD(422toG) },
    /*420   -> x */ { ADD(420to24), ADD(420to8), ADD(420to16), ADD(420to422), ADD(nil420),   ADD(420toG) },
    /*GREY  -> x */ { ADD(Gto24),   ADD(Gto8),   ADD(Gto16),   ADD(Gto422),   ADD(Gto420),   ADD(nilG)   }
    };







////////////////////////// 24bit
void Convert::convert_init24(int in)
{
	ASSERT( (in && !in_pal) || (!in && !out_pal), "palettized in-/output not supported for RGB24" );
	if (informat == outformat && in_altcol == out_altcol) convert_doit = &Convert::convert_nil;
}
void Convert::convert_nil24(void)
{
	int r,g,b;
	unsigned char *ptr = convert_inbuf, *ptro = convert_outbuf;
	int cnt = inwidth*inheight;

	while (cnt--) {
		if (in_altcol) {
			GET_BGR24(r,g,b,ptr);
		} else {
			GET_RGB24(r,g,b,ptr);
		}
		if (out_altcol) {
			PUT_BGR24(r,g,b,ptro);
		} else {
			PUT_RGB24(r,g,b,ptro);
		}
	}
}

void Convert::convert_24to8(void)
{
	int r,g,b;
	unsigned char *ptr = convert_inbuf, *ptro = convert_outbuf;
	int cnt = inwidth*inheight;

	while (cnt--) {
		if (in_altcol) {
			GET_BGR24(r,g,b,ptr);
		} else {
			GET_RGB24(r,g,b,ptr);
		}
		RGB242PAL8(r,g,b,r);
		PUT_8BIT(r,ptro);
	}
}

void Convert::convert_24to16(void)
{
	unsigned int r,g,b;
	unsigned char *ptr = convert_inbuf, *ptro = convert_outbuf;
	int cnt = inwidth*inheight;

	while (cnt--) {
		if (in_altcol) {
			GET_BGR24(r,g,b,ptr);
		} else {
			GET_RGB24(r,g,b,ptr);
		}
		RGB242RGB16(r,g,b,r,g,b);
		PUT_RGB16(r,g,b,ptro);
	}
}

void Convert::convert_24to422(void)
{
	int r,g,b, y1,cb1,cr1, y2,cb2,cr2;
	unsigned char *ptr = convert_inbuf;
	unsigned char *ptro0 = convert_outbuf, *ptro1 = ptro0+inwidth*inheight, *ptro2 = ptro1+inwidth/2*inheight;
	int cnt = inwidth*inheight/2;

	while (cnt--) {
		if (in_altcol) {
			GET_BGR24(r,g,b,ptr);
			RGB242YUV(r,g,b,y1,cb1,cr1);
			GET_BGR24(r,g,b,ptr);
			RGB242YUV(r,g,b,y2,cb2,cr2);
		} else {
			GET_RGB24(r,g,b,ptr);
			RGB242YUV(r,g,b,y1,cb1,cr1);
			GET_RGB24(r,g,b,ptr);
			RGB242YUV(r,g,b,y2,cb2,cr2);
		}
		if (out_pal) {
			PUT_YUV422SP(y1,y2,INTERPOL2(cb1,cb2),INTERPOL2(cr1,cr2),
				      ptro0,ptro1,ptro2);
		} else {
			PUT_YUV422S(y1,y2,INTERPOL2(cb1,cb2),INTERPOL2(cr1,cr2),ptro0);
		}
	}
}

void Convert::convert_24to420(void)
{
	int r,g,b;
	int y1,cb1,cr1, y2,cb2,cr2, y3,cb3,cr3, y4,cb4,cr4;
	unsigned char *ptr0, *ptr1 = convert_inbuf;
	unsigned char *ptro0 = NULL, *ptro01 = convert_outbuf, *ptro1 = ptro01+inwidth*inheight, *ptro2 = ptro1+inwidth/2*((inheight+1)/2);
	int w, h = (inheight+1)/2;

	while (h--) {
		ptro0 = ptro01;
		if (!out_pal) ptro01 += outwidth*2;
		else ptro01 += outwidth;

		ptr0 = ptr1;
		ptr1 += (inwidth*3);
		w = inwidth/2;

		while (w--) {
			if (in_altcol) {
				GET_BGR24(r,g,b,ptr0);
				RGB242YUV(r,g,b,y1,cb1,cr1);
				GET_BGR24(r,g,b,ptr0);
				RGB242YUV(r,g,b,y2,cb2,cr2);
				GET_BGR24(r,g,b,ptr1);
				RGB242YUV(r,g,b,y3,cb3,cr3);
				GET_BGR24(r,g,b,ptr1);
				RGB242YUV(r,g,b,y4,cb4,cr4);
			} else {
				GET_RGB24(r,g,b,ptr0);
				RGB242YUV(r,g,b,y1,cb1,cr1);
				GET_RGB24(r,g,b,ptr0);
				RGB242YUV(r,g,b,y2,cb2,cr2);
				GET_RGB24(r,g,b,ptr1);
				RGB242YUV(r,g,b,y3,cb3,cr3);
				GET_RGB24(r,g,b,ptr1);
				RGB242YUV(r,g,b,y4,cb4,cr4);
			}

			if (out_pal) {
				PUT_YUV420PAL(y1,y2,y3,y4,INTERPOL4(cb1,cb2,cb3,cb4),INTERPOL4(cr1,cr2,cr3,cr4),
					      ptro0,ptro01,ptro1,ptro2);
			} else {
				PUT_YUV420(y1,y2,y3,y4,INTERPOL4(cb1,cb2,cb3,cb4),INTERPOL4(cr1,cr2,cr3,cr4),
					   ptro0,ptro01);
			}
		}
	}
}

void Convert::convert_24toG(void)
{
	int r,g,b;
	unsigned char *ptr = convert_inbuf, *ptro = convert_outbuf;
	int cnt = inwidth*inheight;

	while (cnt--) {
		if (in_altcol) {
			GET_BGR24(r,g,b,ptr);
		} else {
			GET_RGB24(r,g,b,ptr);
		}
		RGB242GREY8(r,g,b,r);
		PUT_8BIT(r,ptro);
	}
}

/////////////////////////// 8 bit
void Convert::convert_init8(int in)
{
	ASSERT( (in && !in_pal) || (!in && !out_pal), "palettized in-/output not supported for RGB8" );
	ASSERT((in && !in_altcol) || (!in && !out_altcol), "color swapped in-/output not supported for RGB8" );
	if (informat == outformat) convert_doit = &Convert::convert_nil;
}

void Convert::convert_nil8(void)
{
	// ERROR(INTMSG("convert_nil8 called"));
}

void Convert::convert_8to24(void)
{
	int c,r,g,b;
	unsigned char *ptr = convert_inbuf, *ptro = convert_outbuf;
	int cnt = inwidth*inheight;

	while (cnt--) {
		GET_8BIT(c,ptr);
		PAL82RGB24(c,r,g,b);
		if (out_altcol) {
			PUT_BGR24(r,g,b,ptro);
		} else {
			PUT_RGB24(r,g,b,ptro);
		}
	}
}

void Convert::convert_8to16(void)
{
	int c,r,g,b;
	unsigned char *ptr = convert_inbuf, *ptro = convert_outbuf;
	int cnt = inwidth*inheight;

	while (cnt--) {
		GET_8BIT(c,ptr);
		PAL82RGB16(c,r,g,b);
		PUT_RGB16(r,g,b,ptro);
	}
}

void Convert::convert_8to422(void)
{
	unsigned int c, y1,cb1,cr1, y2,cb2,cr2;
	unsigned char *ptr = convert_inbuf;
	unsigned char *ptro0 = convert_outbuf, *ptro1 = ptro0+inwidth*inheight, *ptro2 = ptro1+inwidth/2*inheight;
	int cnt = inwidth*inheight/2;

	while (cnt--) {
		GET_8BIT(c,ptr);
		PAL82YUV(c,y1,cb1,cr1);
		GET_8BIT(c,ptr);
		PAL82YUV(c,y2,cb2,cr2);
		if (out_pal) {
			PUT_YUV422SP(y1,y2,INTERPOL2(cb1,cb2),INTERPOL2(cr1,cr2),
				      ptro0,ptro1,ptro2);
		} else {
			PUT_YUV422S(y1,y2,INTERPOL2(cb1,cb2),INTERPOL2(cr1,cr2),ptro0);
		}
	}
}

void Convert::convert_8to420(void)
{
	int c;
	int y1,cb1,cr1, y2,cb2,cr2, y3,cb3,cr3, y4,cb4,cr4;
	unsigned char *ptr0, *ptr1 = convert_inbuf;
	unsigned char *ptro0 = NULL, *ptro01 = convert_outbuf, *ptro1 = ptro0+inwidth*inheight, *ptro2 = ptro1+inwidth/2*((inheight+1)/2);
	int w, h = (inheight+1)/2;

	while (h--) {
		ptro0 = ptro01;
		if (!out_pal) ptro01 += outwidth*2;
		else ptro01 += outwidth;

		ptr0 = ptr1;
		ptr1 += inwidth;
		w = inwidth/2;

		while (w--) {
			GET_8BIT(c,ptr0);
			PAL82YUV(c,y1,cb1,cr1);
			GET_8BIT(c,ptr0);
			PAL82YUV(c,y2,cb2,cr2);
			GET_8BIT(c,ptr1);
			PAL82YUV(c,y3,cb3,cr3);
			GET_8BIT(c,ptr1);
			PAL82YUV(c,y4,cb4,cr4);
			if (out_pal) {
				PUT_YUV420PAL(y1,y2,y3,y4,INTERPOL4(cb1,cb2,cb3,cb4),INTERPOL4(cr1,cr2,cr3,cr4),
					      ptro0,ptro01,ptro1,ptro2);
			} else {
				PUT_YUV420(y1,y2,y3,y4,INTERPOL4(cb1,cb2,cb3,cb4),INTERPOL4(cr1,cr2,cr3,cr4),
					   ptro0,ptro01);
			}
		}
	}
}

void Convert::convert_8toG(void)
{
	int c;
	unsigned char *ptr = convert_inbuf, *ptro = convert_outbuf;
	int cnt = inwidth*inheight;

	while (cnt--) {
		GET_8BIT(c,ptr);
		PAL82GREY8(c,c);
		PUT_8BIT(c,ptro);
	}
}

/////////////////////////// 16 bit
void Convert::convert_init16(int in)
{
	ASSERT( (in && !in_pal) || (!in && !out_pal), "palettized in-/output not supported for RGB16" );
	ASSERT( (in && !in_altcol) || (!in && !out_altcol), "color swapped in-/output not supported for RGB16" );
	if (informat == outformat) convert_doit = &Convert::convert_nil;
}

void Convert::convert_nil16(void)
{
	// ERROR(INTMSG("convert_nil16 called"));
}

void Convert::convert_16to24(void)
{
	int r,g,b;
	unsigned char *ptr = convert_inbuf, *ptro = convert_outbuf;
	int cnt = inwidth*inheight;

	while (cnt--) {
		GET_RGB16(r,g,b,ptr);
		RGB162RGB24(r,g,b,r,g,b);
		if (out_altcol) {
			PUT_BGR24(r,g,b,ptro);
		} else {
			PUT_RGB24(r,g,b,ptro);
		}
	}
}

void Convert::convert_16to8(void)
{
	int r,g,b;
	unsigned char *ptr = convert_inbuf, *ptro = convert_outbuf;
	int cnt = inwidth*inheight;

	while (cnt--) {
		GET_RGB16(r,g,b,ptr);
		RGB162PAL8(r,g,b,b);
		PUT_8BIT(b,ptro);
	}
}

void Convert::convert_16to422(void)
{
	int r,g,b, y1,cb1,cr1, y2,cb2,cr2;
	unsigned char *ptr = convert_inbuf;
	unsigned char *ptro0 = convert_outbuf, *ptro1 = ptro0+inwidth*inheight, *ptro2 = ptro1+inwidth/2*inheight;
	int cnt = inwidth*inheight/2;

	while (cnt--) {
		GET_RGB16(r,g,b,ptr);
		RGB162YUV(r,g,b,y1,cb1,cr1);
		GET_RGB16(r,g,b,ptr);
		RGB162YUV(r,g,b,y2,cb2,cr2);
		if (out_pal) {
			PUT_YUV422SP(y1,y2,INTERPOL2(cb1,cb2),INTERPOL2(cr1,cr2),
				      ptro0,ptro1,ptro2);
		} else {
			PUT_YUV422S(y1,y2,INTERPOL2(cb1,cb2),INTERPOL2(cr1,cr2),ptro0);
		}
	}
}

void Convert::convert_16to420(void)
{
	int r,g,b;
	int y1,cb1,cr1, y2,cb2,cr2, y3,cb3,cr3, y4,cb4,cr4;
	unsigned char *ptr0, *ptr1 = convert_inbuf;
	unsigned char *ptro0 = NULL, *ptro01 = convert_outbuf, *ptro1 = ptro0+inwidth*inheight, *ptro2 = ptro1+inwidth/2*((inheight+1)/2);
	int w, h = (inheight+1)/2;

	while (h--) {
		ptro0 = ptro01;
		if (!out_pal) ptro01 += outwidth*2;
		else ptro01 += outwidth;

		ptr0 = ptr1;
		ptr1 += (inwidth*2);
		w = inwidth/2;

		while (w--) {
			GET_RGB16(r,g,b,ptr0);
			RGB162YUV(r,g,b,y1,cb1,cr1);
			GET_RGB16(r,g,b,ptr0);
			RGB162YUV(r,g,b,y2,cb2,cr2);
			GET_RGB16(r,g,b,ptr1);
			RGB162YUV(r,g,b,y3,cb3,cr3);
			GET_RGB16(r,g,b,ptr1);
			RGB162YUV(r,g,b,y4,cb4,cr4);

			if (out_pal) {
				PUT_YUV420PAL(y1,y2,y3,y4,INTERPOL4(cb1,cb2,cb3,cb4),INTERPOL4(cr1,cr2,cr3,cr4),
					      ptro0,ptro01,ptro1,ptro2);
			} else {
				PUT_YUV420(y1,y2,y3,y4,INTERPOL4(cb1,cb2,cb3,cb4),INTERPOL4(cr1,cr2,cr3,cr4),
					   ptro0,ptro01);
			}
		}
	}
}

void Convert::convert_16toG(void)
{
	int r,g,b;
	unsigned char *ptr = convert_inbuf, *ptro = convert_outbuf;
	int cnt = inwidth*inheight;

	while (cnt--) {
		GET_RGB16(r,g,b,ptr);
		RGB162GREY8(r,g,b,b);
		PUT_8BIT(b,ptro);
	}
}

/////////////////////////// YUV422
void Convert::convert_init422(int in)
{
	ASSERT( (in && in_altcol) || (!in  && out_altcol), "only color swapped in-/output supported for YUV422" );
	if (informat == outformat && in_pal == out_pal) convert_doit = &Convert::convert_nil;
}

void Convert::convert_nil422(void)
{
	int y0, y1, cb, cr;
	unsigned char *ptr0 = convert_inbuf, *ptr1 = ptr0+inwidth*inheight, *ptr2 = ptr1+inwidth/2*inheight;
	unsigned char *ptro0 = convert_outbuf, *ptro1 = ptro0+inwidth*inheight, *ptro2 = ptro1+inwidth/2*inheight;
	int cnt = inwidth*inheight/2;

	while (cnt--) {
		if (in_pal) {
			GET_YUV422SP(y0,y1,cb,cr,ptr0,ptr1,ptr2);
		} else {
			GET_YUV422S(y0,y1,cb,cr,ptr0);
		}
		if (out_pal) {
			PUT_YUV422SP(y0,y1,cb,cr,ptro0,ptro1,ptro2);
		} else {
			PUT_YUV422S(y0,y1,cb,cr,ptro0);
		}
	}
}

void Convert::convert_422to24(void)
{
	int r,g,b, y0, y1, cb, cr;
	unsigned char *ptr0 = convert_inbuf, *ptr1 = ptr0+inwidth*inheight, *ptr2 = ptr1+inwidth/2*inheight;
	unsigned char *ptro = convert_outbuf;
	int cnt = inwidth*inheight/2;

	while (cnt--) {
		if (in_pal) {
			GET_YUV422SP(y0,y1,cb,cr,ptr0,ptr1,ptr2);
		} else {
			GET_YUV422S(y0,y1,cb,cr,ptr0);
		}
		if (out_altcol) {
			YUV2RGB24(y0,cb,cr,r,g,b);
			PUT_BGR24(r,g,b,ptro);
			YUV2RGB24(y1,cb,cr,r,g,b);
			PUT_BGR24(r,g,b,ptro);
		} else {
			YUV2RGB24(y0,cb,cr,r,g,b);
			PUT_BGR24(r,g,b,ptro);
			YUV2RGB24(y1,cb,cr,r,g,b);
			PUT_RGB24(r,g,b,ptro);
		}
	}
}

void Convert::convert_422to8(void)
{
	int c, y0, y1, cb, cr;
	unsigned char *ptr0 = convert_inbuf, *ptr1 = ptr0+inwidth*inheight, *ptr2 = ptr1+inwidth/2*inheight;
	unsigned char *ptro = convert_outbuf;
	int cnt = inwidth*inheight/2;

	while (cnt--) {
		if (in_pal) {
			GET_YUV422SP(y0,y1,cb,cr,ptr0,ptr1,ptr2);
		} else {
			GET_YUV422S(y0,y1,cb,cr,ptr0);
		}
		YUV2PAL8(y0,cb,cr,c);
		PUT_8BIT(c,ptro);
		YUV2PAL8(y1,cb,cr,c);
		PUT_8BIT(c,ptro);
	}
}

void Convert::convert_422to16(void)
{
	int r,g,b, y0, y1, cb, cr;
	unsigned char *ptr0 = convert_inbuf, *ptr1 = ptr0+inwidth*inheight, *ptr2 = ptr1+inwidth/2*inheight;
	unsigned char *ptro = convert_outbuf;
	int cnt = inwidth*inheight/2;

	while (cnt--) {
		if (in_pal) {
			GET_YUV422SP(y0,y1,cb,cr,ptr0,ptr1,ptr2);
		} else {
			GET_YUV422S(y0,y1,cb,cr,ptr0);
		}
		YUV2RGB16(y0,cb,cr,r,g,b);
		PUT_RGB16(r,g,b,ptro);
		YUV2RGB16(y1,cb,cr,r,g,b);
		PUT_RGB16(r,g,b,ptro);
	}
}

void Convert::convert_422to420(void)
{
	int y0,y1,cb1,cr1;
	int y2,y3,cb2,cr2;
	unsigned char *ptr0 = convert_inbuf, *ptr1 = ptr0+inwidth*inheight, *ptr2 = ptr1+inwidth/2*inheight;
	unsigned char *ptro0 = NULL, *ptro01 = convert_outbuf, *ptro1 = ptro01+inwidth*inheight, *ptro2 = ptro1+inwidth/2*((inheight+1)/2);
	int w, h = (inheight+1)/2;

	while (h--) {
		ptro0 = ptro01;
		if (!out_pal) ptro1 += outwidth*2;
		else ptro01 += outwidth;

		if (!in_pal) {
			ptr1 = ptr0;
			ptr0 += inwidth*2;
		}
		w = inwidth/2;

		while (w--) {
			if (in_pal) {
				GET_YUV422SP(y0,y1,cb1,cr1,ptr0,ptr1,ptr2);
				GET_YUV422SP(y2,y3,cb2,cr2,ptr0,ptr1,ptr2);
			} else {
				GET_YUV422S(y0,y1,cb1,cr1,ptr1);
				GET_YUV422S(y2,y3,cb2,cr2,ptr0);
			}
			if (out_pal) {
				PUT_YUV420PAL(y0,y1,y2,y3,INTERPOL2(cb1,cb2),INTERPOL2(cr1,cr2),
					      ptro0,ptro01,ptro1,ptro2);
			} else {
				PUT_YUV420(y0,y1,y2,y3,INTERPOL2(cb1,cb2),INTERPOL2(cr1,cr2),
					   ptro0,ptro01);
			}
		}
	}
}

void Convert::convert_422toG(void)
{
	int y0,y1,cb,cr;
	unsigned char *ptr0 = convert_inbuf, *ptr1 = ptr0+inwidth*inheight, *ptr2 = ptr1+inwidth/2*inheight;
	unsigned char *ptro = convert_outbuf;
	int cnt = inwidth*inheight/2;

	while (cnt--) {
		if (in_pal) {
			GET_YUV422SP(y0,y1,cb,cr,ptr0,ptr1,ptr2);
		} else {
			GET_YUV422S(y0,y1,cb,cr,ptr0);
		}
		PUT_8BIT(y0,ptro);
		PUT_8BIT(y1,ptro);
	}
}

/////////////////////////// YUV420
void Convert::convert_init420(int in)
{
	ASSERT( (in && !in_altcol) || (!in  && !out_altcol), "color swapped in-/output not supported for YUV420" );
	if (informat == outformat && in_pal == out_pal) convert_doit = &Convert::convert_nil;
}

void Convert::convert_nil420(void)
{
	int y0, y1, y2, y3, cb, cr;
	unsigned char *ptr0, *ptr01 = convert_inbuf, *ptr1 = ptr01+inwidth*inheight, *ptr2 = ptr1+inwidth/2*((inheight+1)/2);
	unsigned char *ptro0 = NULL, *ptro01 = convert_outbuf, *ptro1 = ptro01+inwidth*inheight, *ptro2 = ptro1+inwidth/2*((inheight+1)/2);
	int w, h = (inheight+1)/2;

	while (h--) {
		ptr0 = ptr01;
		if (!in_pal) ptr01 += inwidth*2;
		else ptr01 += inwidth;

		ptro0 = ptro1;
		ptro1 += inwidth*3;

		w = inwidth/2;
		while (w--) {
			if (in_pal) {
				GET_YUV420PAL(y0,y1,y2,y3,cb,cr,ptr0,ptr01,ptr1,ptr2);
			} else {
				GET_YUV420(y0,y1,y2,y3,cb,cr,ptr0,ptr01);
			}
			if (out_pal) {
				PUT_YUV420PAL(y0,y1,y2,y3,cb,cr,ptro0,ptro01,ptro1,ptro2);
			} else {
				PUT_YUV420(y0,y1,y2,y3,cb,cr,ptro0,ptro01);
			}
		}
	}
}

void Convert::convert_420to24(void)
{
	int r,g,b, y0, y1, y2, y3, cb, cr;
	unsigned char *ptr0, *ptr01 = convert_inbuf, *ptr1 = ptr01+inwidth*inheight, *ptr2 = ptr1+inwidth/2*((inheight+1)/2);
	unsigned char *ptro1 = convert_outbuf, *ptro0;
	int w, h = (inheight+1)/2;

	while (h--) {
		ptr0 = ptr01;
		if (!in_pal) ptr01 += inwidth*2;
		else ptr01 += inwidth;

		ptro0 = ptro1;
		ptro1 += inwidth*3;

		w = inwidth/2;
		while (w--) {
			if (in_pal) {
				GET_YUV420PAL(y0,y1,y2,y3,cb,cr,ptr0,ptr01,ptr1,ptr2);
			} else {
				GET_YUV420(y0,y1,y2,y3,cb,cr,ptr0,ptr01);
			}

			if (out_altcol) {
				YUV2RGB24(y0,cb,cr,r,g,b);
				PUT_BGR24(r,g,b,ptro0);
				YUV2RGB24(y1,cb,cr,r,g,b);
				PUT_BGR24(r,g,b,ptro0);
				YUV2RGB24(y2,cb,cr,r,g,b);
				PUT_BGR24(r,g,b,ptro1);
				YUV2RGB24(y3,cb,cr,r,g,b);
				PUT_BGR24(r,g,b,ptro1);
			} else {
				YUV2RGB24(y0,cb,cr,r,g,b);
				PUT_RGB24(r,g,b,ptro0);
				YUV2RGB24(y1,cb,cr,r,g,b);
				PUT_RGB24(r,g,b,ptro0);
				YUV2RGB24(y2,cb,cr,r,g,b);
				PUT_RGB24(r,g,b,ptro1);
				YUV2RGB24(y3,cb,cr,r,g,b);
				PUT_RGB24(r,g,b,ptro1);
			}
		}
	}
}

void Convert::convert_420to8(void)
{
	int c, y0, y1, y2, y3, cb, cr;
	unsigned char *ptr0, *ptr01 = convert_inbuf, *ptr1 = ptr01+inwidth*inheight, *ptr2 = ptr1+inwidth/2*((inheight+1)/2);
	unsigned char *ptro1 = convert_outbuf, *ptro0;
	int w, h = (inheight+1)/2;

	while (h--) {
		ptr0 = ptr01;
		if (!in_pal) ptr01 += inwidth*2;
		else ptr01 += inwidth;

		ptro0 = ptro1;
		ptro1 += inwidth;

		w = inwidth/2;
		while (w--) {
			if (in_pal) {
				GET_YUV420PAL(y0,y1,y2,y3,cb,cr,ptr0,ptr01,ptr1,ptr2);
			} else {
				GET_YUV420(y0,y1,y2,y3,cb,cr,ptr0,ptr01);
			}
			YUV2PAL8(y0,cb,cr,c);
			PUT_8BIT(c,ptro0);
			YUV2PAL8(y1,cb,cr,c);
			PUT_8BIT(c,ptro0);
			YUV2PAL8(y2,cb,cr,c);
			PUT_8BIT(c,ptro1);
			YUV2PAL8(y3,cb,cr,c);
			PUT_8BIT(c,ptro1);
		}
	}
}

void Convert::convert_420to16(void)
{
	int r,g,b, y0, y1, y2, y3, cb, cr;
	unsigned char *ptr0, *ptr01 = convert_inbuf, *ptr1 = ptr01+inwidth*inheight, *ptr2 = ptr1+(inwidth/2)*(inheight/2);
	unsigned char *ptro1 = convert_outbuf, *ptro0;
	int w, h = (inheight+1)/2;

	while (h--) {
		ptr0 = ptr01;
		if (!in_pal) ptr01 += inwidth*2;
		else ptr01 += inwidth;

		ptro0 = ptro1;
		ptro1 += inwidth*2;

		w = inwidth/2;
		while (w--) {
			if (in_pal) {
				GET_YUV420PAL(y0,y1,y2,y3,cb,cr,ptr0,ptr01,ptr1,ptr2);
			} else {
				GET_YUV420(y0,y1,y2,y3,cb,cr,ptr0,ptr01);
			}
			YUV2RGB16(y0,cb,cr,r,g,b);
			PUT_RGB16(r,g,b,ptro0);
			YUV2RGB16(y1,cb,cr,r,g,b);
			PUT_RGB16(r,g,b,ptro0);
			YUV2RGB16(y2,cb,cr,r,g,b);
			PUT_RGB16(r,g,b,ptro1);
			YUV2RGB16(y3,cb,cr,r,g,b);
			PUT_RGB16(r,g,b,ptro1);
		}
	}
}

void Convert::convert_420to422(void)
{
	int y0, y1, y2, y3, cb, cr;
	unsigned char *ptr0, *ptr01 = convert_inbuf, *ptr1 = ptr01+inwidth*inheight, *ptr2 = ptr1+inwidth/2*((inheight+1)/2);
	unsigned char *ptro0 = convert_outbuf, *ptro1 = ptro0+inwidth*inheight, *ptro2 = ptro1+inwidth/2*inheight;
	int w, h = (inheight+1)/2;

	while (h--) {
		ptr0 = ptr01;
		if (!in_pal) ptr01 += inwidth*2;
		else ptr01 += inwidth;

		if (!out_pal) {
			ptro1 = ptro0;
			ptro0 += inwidth*2;
		}

		w = inwidth/2;
		while (w--) {
			if (in_pal) {
				GET_YUV420PAL(y0,y1,y2,y3,cb,cr,ptr0,ptr01,ptr1,ptr2);
			} else {
				GET_YUV420(y0,y1,y2,y3,cb,cr,ptr0,ptr01);
			}
			if (out_pal) {
				PUT_YUV422SP(y0,y1,cb,cr,ptro0,ptro1,ptro2);
				PUT_YUV422SP(y2,y3,cb,cr,ptro0,ptro1,ptro2);
			} else {
				PUT_YUV422S(y0,y1,cb,cr,ptro1);
				PUT_YUV422S(y2,y3,cb,cr,ptro0);
			}
		}
	}
}

void Convert::convert_420toG(void)
{
	int y0, y1, y2, y3, cb, cr;
	unsigned char *ptr0, *ptr01 = convert_inbuf, *ptr1 = ptr01+inwidth*inheight, *ptr2 = ptr1+inwidth/2*((inheight+1)/2);
	unsigned char *ptro1 = convert_outbuf, *ptro0;
	int w, h = (inheight+1)/2;

	while (h--) {
		ptr0 = ptr01;
		if (!in_pal) ptr01 += inwidth*2;
		else ptr01 += inwidth;

		ptro0 = ptro1;
		ptro1 += inwidth;

		w = inwidth/2;
		while (w--) {
			if (in_pal) {
				GET_YUV420PAL(y0,y1,y2,y3,cb,cr,ptr0,ptr01,ptr1,ptr2);
			} else {
				GET_YUV420(y0,y1,y2,y3,cb,cr,ptr0,ptr01);
			}
			PUT_8BIT(y0,ptro0);
			PUT_8BIT(y1,ptro0);
			PUT_8BIT(y2,ptro1);
			PUT_8BIT(y3,ptro1);
		}
	}
}

/////////////////////////// grey
void Convert::convert_initG(int in)
{
	ASSERT( (in && !in_pal) || (!in && !out_pal), "palettized in-/output not supported for GREY8" );
	ASSERT( (in && !in_altcol) || (!in && !out_altcol), "color swapped in-/output not supported for GREY8" );
	if (informat == outformat) convert_doit = &Convert::convert_nil;
}

void Convert::convert_nilG(void)
{
	// ERROR(INTMSG("convert_nilG called"));
}

void Convert::convert_Gto24(void)
{
	int b;
	unsigned char *ptr = convert_inbuf, *ptro = convert_outbuf;
	int cnt = inwidth*inheight;

	while (cnt--) {
		GET_8BIT(b,ptr);
		PUT_RGB24(b,b,b,ptro);
	}
}

void Convert::convert_Gto16(void)
{
	int r,g,b;
	unsigned char *ptr = convert_inbuf, *ptro = convert_outbuf;
	int cnt = inwidth*inheight;

	while (cnt--) {
		GET_8BIT(b,ptr);
		GREY82RGB16(b,r,g,b);
		PUT_RGB16(r,g,b,ptro);
	}
}

void Convert::convert_Gto422(void)
{
	int y1,y2;
	unsigned char *ptr = convert_inbuf;
	unsigned char *ptro0 = convert_outbuf, *ptro1 = ptro0+inwidth*inheight, *ptro2 = ptro1+inwidth/2*inheight;
	int cnt = inwidth*inheight/2;

	while (cnt--) {
		GET_8BIT(y1,ptr);
		GET_8BIT(y2,ptr);
		if (out_pal) {
			PUT_YUV422SP(y1,y2,NO_C,NO_C,ptro0,ptro1,ptro2);
		} else {
			PUT_YUV422S(y1,y2,NO_C,NO_C,ptro0);
		}
	}
}

void Convert::convert_Gto420(void)
{
	int y1,y2,y3,y4;
	unsigned char *ptr0, *ptr1 = convert_inbuf;
	unsigned char *ptro0 = NULL, *ptro01 = convert_outbuf, *ptro1 = ptro0+inwidth*inheight, *ptro2 = ptro1+inwidth/2*((inheight+1)/2);
	int w, h = (inheight+1)/2;

	while (h--) {
		ptro0 = ptro01;
		if (!out_pal) ptro01 += outwidth*2;
		else ptro01 += outwidth;

		ptr0 = ptr1;
		ptr1 += inwidth;

		w = inwidth/2;
		while (w--) {
			GET_8BIT(y1,ptr0);
			GET_8BIT(y2,ptr0);
			GET_8BIT(y3,ptr1);
			GET_8BIT(y4,ptr1);
			if (out_pal) {
				PUT_YUV420PAL(y1,y2,y3,y4,NO_C,NO_C,
					      ptro0,ptro01,ptro1,ptro2);
			} else {
				PUT_YUV420(y1,y2,y3,y4,NO_C,NO_C,ptro0,ptro01);
			}
		}
	}
}

void Convert::convert_Gto8(void)
{
	int c;
	unsigned char *ptr = convert_inbuf, *ptro = convert_outbuf;
	int cnt = inwidth*inheight;

	while (cnt--) {
		GET_8BIT(c,ptr);
		GREY82PAL8(c,c);
		PUT_8BIT(c,ptro);
	}
}





void Convert::convert_nil(void)
{
    memcpy(convert_outbuf, convert_inbuf, convert_bytes);
}









/****************************************************************************************/
/***** Konstruktor / Destruktor *********************************************************/
/****************************************************************************************/


/***** Farbkonvertierung initialisieren *********************/
Convert::Convert( int infmt, int outfmt, int width, int height )
{
    int tg[9] = {0,32,64,96,128,160,192,224,255};
    int trb[5] = {0,64,128,192,255};
    int i, last;
    int r,g,b, ro, go, bo, l, y, cb, cr;
    double x;

    /***** Bildgre *****/
    ASSERT(width, "unknown image width" );
    ASSERT(height, "unknown image height" );
    inwidth=width;
    inheight=height;
    outwidth=width;
    outheight=height;

    // Variablen initialisieren
    in_pal = 0;
    out_pal = 0;
    in_altcol = 0;
    out_altcol = 0;
    convert_bytes = 0;

    convert_outbuf = NULL;
    convert_inbuf = NULL;

    convert_rangetable = NULL;
    convert_doit = NULL;


    /****** Konvertierungstabellen initialisieren *****/

    // 16 <-> 24 bit conversion
    x = 255.0/31.0;
    for (i = 0; i < 32; i++) convert_5to8tab[i] = (int)((x*((double)i))+0.5);
    x = 255.0/63.0;
    for (i = 0; i < 64; i++) convert_6to8tab[i] = (int)((x*((double)i))+0.5);
    x = 31.0/255.0;
    for (i=0; i < 256; i++) convert_8to5tab[i] = (int)((x*((double)i))+0.5);
    x = 63.0/255.0;
    for (i=0; i < 256; i++) convert_8to6tab[i] = (int)((x*((double)i))+0.5);

    // YCbCr <-> 24bit conversion
    for (i = 0; i < 256; i++) {
        convert_rgb_ycc_tab[i+R_Y_OFF] = FIX(0.29900) * i;
        convert_rgb_ycc_tab[i+G_Y_OFF] = FIX(0.58700) * i;
        convert_rgb_ycc_tab[i+B_Y_OFF] = FIX(0.11400) * i + ONE_HALF;
        convert_rgb_ycc_tab[i+R_CB_OFF] = (-FIX(0.16874)) * i;
        convert_rgb_ycc_tab[i+G_CB_OFF] = (-FIX(0.33126)) * i;
        convert_rgb_ycc_tab[i+B_CB_OFF] = FIX(0.50000) * i + CBCR_OFFSET + ONE_HALF-1;
        convert_rgb_ycc_tab[i+G_CR_OFF] = (-FIX(0.41869)) * i;
        convert_rgb_ycc_tab[i+B_CR_OFF] = (-FIX(0.08131)) * i;
    }
    for (i = 0, x = -128; i < 256; i++, x++) {
        convert_Crrtab[i] = ((int)(FIX(1.40200) * x + ONE_HALF) >> 16);
        convert_Cbbtab[i] = ((int)(FIX(1.77200) * x + ONE_HALF) >> 16);
        convert_Crgtab[i] = ((int)((- FIX(0.71414)) * x));
        convert_Cbgtab[i] = ((int)((- FIX(0.34414)) * x + ONE_HALF));
    }
    convert_rangetable = convert_range+256;
    memset(convert_range,0,256);
    for (i = 0; i < 256; i++) convert_rangetable[i] = i;
    memset(convert_rangetable+256,255,384);
    memset(convert_rangetable+256+384,0,384);
    memcpy(convert_rangetable+256+384+384,convert_rangetable,128);


    // 8bit pal <-> 24bit conversion
    last = 0;
    for (i = 0; i < 256; i++) {
        if (last < 4 && (i-trb[last]) > (trb[last+1]-i)) last++;
        convert_256to5tab[i] = last;
    }
    for (i = 0; i < 32; i++) convert_32to5tab[i] = convert_256to5tab[convert_5to8tab[i]];
    last = 0;
    for (i = 0; i < 256; i++) {
        if (last < 8 && (i-tg[last]) > (tg[last+1]-i)) last++;
        convert_256to9tab[i] = last;
    }
    for (i = 0; i < 64; i++) convert_64to9tab[i] = convert_256to9tab[convert_6to8tab[i]];

    for (i = 0; i < 16; i++) convert_palette[i][0] = convert_palette[i][1] = convert_palette[i][2] = 0;
    for (g=0; g<9; g++) for (r=0; r<5; r++) for (b=0;b<5;b++) {
        convert_palette[i][0] = trb[r];
        convert_palette[i][1] = tg[g];
        convert_palette[i][2] = trb[b];
        convert_backpal[r][g][b] = i;

        RGB242RGB16(trb[r],tg[g],trb[b],ro,go,bo);
        convert_palette16[i][0] = ro;
        convert_palette16[i][1] = go;
        convert_palette16[i][2] = bo;

        RGB242GREY8(trb[r],tg[g],trb[b],l);
        convert_paletteG[i] = l;

        RGB242YUV(trb[r],tg[g],trb[b],y,cb,cr);
        convert_paletteYCC[i][0] = y;
        convert_paletteYCC[i][1] = cb;
        convert_paletteYCC[i][2] = cr;

        i++;
    }
    for (; i < 256; i++) convert_palette[i][0] = convert_palette[i][1] = convert_palette[i][2] = 0;




    /***** Farbformat fr Konvertierung einstellen *****/
    in_altcol = 0;
    out_altcol = 0;
    if( infmt & BGR ) {
        infmt -= BGR;
        in_altcol = 1;
    }

    if( outfmt & BGR ) {
        outfmt -= BGR;
        out_altcol = 1;
    }

    in_pal = 0;
    out_pal = 0;
    if( infmt & PAL ) {
        infmt -= PAL;
        in_pal = 1;
    }

    if( outfmt & PAL ) {
        outfmt -= PAL;
        out_pal = 1;
    }

    informat = infmt;
    outformat= outfmt;





    /***** Methode fr Konvertierung aus Tabelle lesen *****/
    convert_doit = convert_table[infmt][outfmt];

    // Konvertierungesmethoden initialisieren
    if( convert_doit ) {
        (this->*(convert_inittable[infmt]))(1);
        (this->*(convert_inittable[outfmt]))(0);

        convert_bytes = ((outheight+1)*outwidth*convert_color_bpps[outformat]+7)/8;

        // Ausgabepuffer erstellen
        //convert_outbuf = new unsigned char[convert_bytes];
    }

}






/***** Farbkonvertierung beenden *****/
Convert::~Convert(void)
{
    delete[] convert_outbuf;
}





/***** Konvertierung ausfhren *****/
void Convert::do_convert( unsigned char *inbuf, unsigned char *outbuf )
{

    if( !convert_doit /*|| convert_doit == &Convert::convert_nil */) {
        // outbuf = inbuf;
        return;
    }
    
    convert_inbuf = inbuf;
    convert_outbuf = outbuf;
    (this->*convert_doit)();

    return;
}
