/***************************************************************************
                          FrameData.h  -  description
                             -------------------
    begin                : Son Dez 15 2002
    copyright            : (C) 2002 by Christian Tusche
    email                : ctusche@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FRAMEDATA_H
#define FRAMEDATA_H

#include <inttypes.h>
#include <unistd.h>


/**Speichert die Daten eines Bildes
  *@author Christian Tusche
  */

class VideoFrame;

class FrameData {
    friend class VideoFrame;
public:
    /**Neuen Videoframe mit der Gre<br>
      *  x * y<br>
      *erstellen<br>
      *Es wird ein Framebuffer angelegt, oder der Speicherbereich *memseg mit
      *der Gre size als Framebuffer verwendet.
      */    
    FrameData( int x, int y, uint8_t *memseg = NULL, size_t size = 0 );
    
    
    ~FrameData();
    
    /**Daten aus *bild werden in den internen Framebuffer kopiert!
      */
    void SetFrame( FrameData *bild );
    
    /**Daten aus *bild werden in den internen Framebuffer kopiert!
      */
    void SetFrame( uint8_t *bild );

    /**Daten aus *y, *u, *v werden in den internen Framebuffer kopiert!
      */
    void SetFrame( uint8_t *y, uint8_t *u, uint8_t *v );

    /**Zeiger auf Datenfeld - linearer Framebuffer<br>
      *Rckgabewert: Gre der Bilddaten
      */
    int FramePointer( uint8_t *&bild );

    /**Zeiger auf Datenfeld - planar<br>
      *Rckgabewert: Gre der Bilddaten von Y+U+V
      */
    int FramePointer( uint8_t *&y, uint8_t *&u, uint8_t *&v );


    /**Farbraum der Bilddaten abfragen
      */
    int ColorModel( void ) { return c_model; }

    /**Farbraum der Bilddaten festlegen
      */
    void ColorModel( int cmodel );
    
private:
    /**Neue Puffer anlegen
      */
    void alloc_data(void);

    /**Puffer freigeben
      */
    void free_data( void );

    /**yuv-Planes erzeugen
      */
    void yuv_planes( void );


    uint8_t *data;                               // linearer Framebuffer
    size_t data_size;                            // Gre der zur verfgungstehenden Daten
    uint8_t *y_plane, *u_plane, *v_plane;        // yuv-Planes
    size_t frame_size;                           // Gre der Bildedaten
    size_t y_size, u_size, v_size;               // Gre der Komponentendaten
    
    int c_model;                                 // Farbraum

    int breite;
    int hoehe;

    bool alloc_buf;            // data selbst angelegt
    //bool data_ok;              // data zeigt auf gltigen Framebuffer
};

#endif
