/***************************************************************************
                          Scale.h  -  Skalierung von Bildern
                             -------------------
    begin                : Son Dez 15 2002
    copyright            : (C) 2002 by Christian Tusche
    email                : ctusche@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _FRAME_RESCALER_H_
#define _FRAME_RESCALER_H_
 
#include <inttypes.h>
#include "cmodels.h"
#include "Convert.h"
 
#define __align8
//#define __align8 __attribute__ ((aligned (8)))

#define NB_COMPONENTS 3

#define PHASE_BITS 4
#define NB_PHASES  (1 << PHASE_BITS)
#define NB_TAPS    4
#define FCENTER    1  /* index of the center of the filter */


class FrameRescaler {
public:
    FrameRescaler( int cmodel, int iwidth, int iheight, int owidth, int oheight );
    ~FrameRescaler();
    
    void Scale( uint8_t *output, uint8_t *input);
    int ColorModel( void ) { return c_model; }

private:
    void build_filter(int16_t *filter, float factor);
    int16_t get_phase( int pos );

    void h_resample_fast(int16_t *dst, int dst_width, uint8_t *src, int src_width,
                         int src_start, int src_incr, int16_t *filters, int inc );
#if NB_TAPS == 4
    void h_resample_fast4_inc1(int16_t *dst, int dst_width, uint8_t *src, int src_width,
                         int src_start, int src_incr, int16_t *filters );
#endif
    void v_resample(uint8_t *dst, int dst_width, int16_t *src, int wrap, int16_t *filter, int inc );

    void h_resample_slow(int16_t *dst, int dst_width, uint8_t *src, int src_width,
                            int src_start, int src_incr, int16_t *filters, int inc );

    void h_resample(int16_t *dst, int dst_width, uint8_t *src, int src_width,
                       int src_start, int src_incr, int16_t *filters, int inc );

    void component_resample( uint8_t *output, int owrap, int owidth, int oheight,
                             uint8_t *input, int iwrap, int iwidth, int iheight, int inc);



    void scale_yuv420( uint8_t *input, uint8_t *output );
    void scale_yuv422( uint8_t *input, uint8_t *output );
    void scale_rgb24( uint8_t *input, uint8_t *output );
    void scale_grey8( uint8_t *input, uint8_t *output );

    //int mm_flags;
    int i_width, i_height, o_width, o_height;
    int h_incr, v_incr;
    int16_t h_filters[NB_PHASES * NB_TAPS] __align8; /* horizontal filters */
    int16_t v_filters[NB_PHASES * NB_TAPS] __align8; /* vertical filters */
    uint8_t *temp_buf;
    int16_t *line_buf;
    int c_model;
    Convert *cmodel_converter;
};

#endif
