/***************************************************************************
                       PluginList.h  -  description
                             -------------------
    begin                : Sat Jan 5 2002
    copyright            : (C) 2002 by Christian Tusche
    email                : ctusche@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _PLUGIN_LIST_H
#define _PLUGIN_LIST_H

#include "String.h"
#include <vector>
using std::vector;




/**Liste der verfgbaren Codecs
  *@author Christian Tusche
  */

template <class Info_Typ>
class PluginList: public vector<Info_Typ*> {
public: 
    PluginList();
    ~PluginList();

    /**Name des Codec-Moduls
      */
    void Libname( const char *name );

    /**fgt Formateintrge zur liste hinzu
      */
    PluginList<Info_Typ> &operator += ( PluginList &liste );

    /**fgt einen Formateintrag zur liste hinzu
      */
    PluginList<Info_Typ> &operator += ( const Info_Typ &element );

    /**fgt einen Formateintrag zur liste hinzu
      */
    PluginList<Info_Typ> &operator += ( const Info_Typ *element );

    /**ein Eintrag aus der Liste
      */
    Info_Typ &Element( unsigned int num );

    /**Anzahl der Eintrge
      */
    unsigned int NumElements( void ){ return size(); }

protected:
    String lib_name;
    Info_Typ std_format;
};














/****************************************************************************************/
/***** Implementierung ******************************************************************/
/****************************************************************************************/
template <class Info_Typ>
PluginList<Info_Typ>::PluginList()
{
}




template <class Info_Typ>
PluginList<Info_Typ>::~PluginList()
{
    while( size() > 0 ) {
        delete back();
        pop_back();
    }
}






/**Name des Codec-Moduls
  */
template <class Info_Typ>
void PluginList<Info_Typ>::Libname( const char *name )
{
    lib_name = name;
}






/**fgt Formateintrge zur liste hinzu
  */
template <class Info_Typ>
PluginList<Info_Typ> &PluginList<Info_Typ>::operator += ( PluginList<Info_Typ> &liste )
{
    for( unsigned int i=0; i < liste.size(); i++ ) {
        (*this) += liste.Element( i );
    }

    return *this;
}





template <class Info_Typ>
PluginList<Info_Typ> &PluginList<Info_Typ>::operator += ( const Info_Typ &element )
{
    Info_Typ *format = new Info_Typ( element );
    format->SetLibname( lib_name );
    push_back( format );
    return *this;
}



/**fgt einen Formateintrag zur liste hinzu
  */
template <class Info_Typ>
PluginList<Info_Typ> &PluginList<Info_Typ>::operator += ( const Info_Typ *element )
{
    return *this += *element;
}





/**ein Eintrag aus der Liste
  */
template <class Info_Typ>
Info_Typ &PluginList<Info_Typ>::Element( unsigned int num )
{
    if( num < size() )
        return **(begin() + num );

    return std_format;
}

#endif
