/* 
Copyright (C) 1988 Free Software Foundation
    written by Doug Lea (dl@rocky.oswego.edu)

This file is part of the GNU C++ Library.  This library is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This library is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free Software
Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* 
  Regex class implementation
 */

#include "Regex.h"




Regex::~Regex()
{
    regfree( &preg );
    delete [] rmatch;
}




Regex::Regex( const char* t )
{
    rmatch = new regmatch_t[ maxMatch ];
    int regflags = 0;
 
    regcomp(&preg, t, regflags);
}




int Regex::match_info(int& start, int& length, int nth) const
{
    if( nth >= maxMatch )
        return 0;
    else {
        start = rmatch[nth].rm_so;
        length = rmatch[nth].rm_eo - rmatch[nth].rm_so;
        return start >= 0 && length >= 0;
    }
}





int Regex::search(const char *s, int len, int& matchlen, int startpos) const
{
    int matchpos, pos;
    char *str;

    if (startpos >= 0) {
        pos = startpos;
    } else {
        pos = len + startpos;
    }

    str = new char[len+1];
    strncpy( str, s, len );
    str[ len ] = 0;
 
    if( regexec(&preg, str + pos, maxMatch, rmatch, 0) ) {
        matchlen = 0;
        delete [] str;
        return -1;
    }
    delete [] str;

    matchpos = rmatch[0].rm_so + pos;
    matchlen = rmatch[0].rm_eo - rmatch[0].rm_so;

    //DEBUG_TXT( "matchpos=%i, matchlen=%i", matchpos, matchlen );
    return matchpos;
}





int Regex::match(const char*s, int len, int p) const
{
    int matchlen=0;
    
    return search( s, len, matchlen, p );
}




/*
 some built-in Regular expressions
*/

const Regex RXwhite("[ \n\t\r\v\f]+" );
const Regex RXint("-?[0-9]+" );
const Regex RXdouble("-?\\(\\([0-9]+\\.[0-9]*\\)\\|\\([0-9]+\\)\\|\\(\\.[0-9]+\\)\\)\\([eE][---+]?[0-9]+\\)?" );
const Regex RXalpha("[A-Za-z]+" );
const Regex RXlowercase("[a-z]+" );
const Regex RXuppercase("[A-Z]+" );
const Regex RXalphanum("[0-9A-Za-z]+" );
const Regex RXidentifier("[A-Za-z_][A-Za-z0-9_]*" );

