/********************************************************************\
*****   Basisblasse fr Audio- und Video-Tracks                  *****
\********************************************************************/

#ifndef _TRACK_H_
#define _TRACK_H_

#include <sys/types.h>
#include <inttypes.h>
#include "String.h"


class VideoFile;
class Codec;
class CodecPlugins;
class V_Track;
class A_Track;


/**Gemeinsame Methoden von Audio- und Video-Tracks.
  *@author Christian Tusche
  */

class Track {
    friend class Codec;
    friend class CodecPlugins;
    friend class V_Track;
    friend class A_Track;
public:
    Track( VideoFile *File, int track = 0, const String Codec_name = "" );
    virtual ~Track();

    /**Name des Codecs
      */
    inline const String CodecName( void ) { return codec_name; }

    /**Nummer dieses Tracks
      */
    virtual int Number(void) { return nummer; }

    /** Aktuelle Position ( Bild / Block )
      */
    virtual long long Position(void) { return pos; }

    /** Aktuelle Position in Sekunden
      */
    virtual double CurrentTime( void ) { return (double)pos / block_rate; }

    /** Bisher geschriebene Bytes
      */
    virtual long long BytesWritten(void) { return bytes; }

    /**Daten werden in Blcken gespeichert<br>
      *1 Block = 1 Bild bei Videotracks<br>
      *liefert Blockrare (Blcke pro Sekunde)
      */
    virtual double Rate( void ) { return block_rate; }

    /**leifert Gesamtanzahl der Blcke im Track
      */
    virtual long Frames( void ) { return total_blocks; }

    /**liefert die Endzeit des Tracks in Sekunden
      */
    virtual double TotalTime( void ) { return (double)total_blocks / block_rate; }
    
    /**Positioniere Track<br>
      *Rckgabe: 1 : erfolg<br>
      *          0 : Fehler
      */
    virtual int SeekTime( double seconds ) { return Seek( (long)(seconds * block_rate) ); }

    /**Positioniere Track auf gegebenen Datensatz<br>
      *Rckgabe: 1 : erfolg<br>
      *          0 : Fehler<br>
      *Die tatschliche Position kann spter mit Position() bestimmt werden
      */
    virtual int Seek( long block );


    /**Zustzliche Informationen mitteilen<br>
      *der eigentliche Wert von "info" hngt vom Typ des IO-Moduls<br>
      *Diese Funktion sollte nur vom Codec aufgerufen werden<br>
      *ab! "size" gibt die Datenlnge an.<br>
      *Der Rckgabewert: 1 - info verstanden<br>
      *                  0 - sonst
      */
    virtual int CodecExtraInfo( uint8_t *&info, unsigned int &size ) { return 0; }

    /**IO-Modus
     */
    inline VideoFile::IO_Mode IOMode( void ) { return iomode; }


protected:
    /**Interne Bezeichnung des Codecs
      */
    inline const String &CodecID( void ) { return codec_id; }

    /**Interne Bezeichnung des Codecs
      */
    inline void SetCodecID( const String &name ) { codec_id = name; }

    /**Intern darf der Codec-Name verndert werden
      */
    inline void SetCodecName( const String &name ) { codec_name = name; }

    /**Blockrate setzen
      */
    virtual void SetRate( double rate ) { block_rate = rate; }

    /**Anzahl der Frames setzen
      */
    virtual void SetFrames( long frames ) { total_blocks = frames; }

    /**Position setzen
      */
    virtual void SetPosition( long position ) { pos = position; }


    CodecPlugins *AV_Codecs;    // Plugin-Loader
    VideoFile *file;            // bergeordnete Klasse


private:
    long pos;                   // aktuelle Position ( Bild / Sample )
    double block_rate;          // Daten werden in Blcken gespeichert
    long total_blocks;          // Gesamtanzahl der Blcke im Track                                // 1 Block = 1 Bild bei Videotracks
    VideoFile::IO_Mode iomode;
    int nummer;
    String codec_id;            // Kennung des Codecs zur internen Benutzung in IO-Modul
    String codec_name;          // Name des Codecs
    long long bytes;            // Anzahl der Bytes, die bisher geschrieben wurden
};

#endif
