/*****************************************************************************\
**  Author  : Christian Tusche                                               **
**  E-Mail  : ctusche@gmx.net                        **
**                                                                           **
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/

#include <stdio.h>

#include "VideoFile.h"
#include "V_Codec.h"
#include "debug.h"
#include "translate.h"


/***** Konstruktor / Destruktor *****/
V_Codec::V_Codec( V_Track *Track)
:Codec( Track ), V_Format( *Track )
{
    // Variablen initialisieren
    color_model=RGB24;
    color_model_intern=RGB24;
    //enc_converter = NULL;
    //dec_converter = NULL;
    track = Track;

    // gengend groen Puffer anfordern (RGB32)
    //frame_puffer_gr = breite * hoehe * 4;
    //frame_puffer = new unsigned char[frame_puffer_gr];
    
    //enc_frame_puffer = new VideoFrame( breite, hoehe );
    enc_frame_puffer = NULL;
}





V_Codec::~V_Codec()
{
    //delete enc_converter;
    //delete dec_converter;
    //delete [] frame_puffer;
    //delete enc_frame_puffer;
}





// Farbraum festlegen
bool V_Codec::ColorModel( int cmodel )
{
    color_model = cmodel;
    color_model_intern = cmodel;

    // Farbraum-Konverter lschen
    //delete enc_converter;
    //delete dec_converter;
    //enc_converter = NULL;
    //dec_converter = NULL;

    return true;
}





bool V_Codec::ColorModel_intern( int cmodel )
{
    color_model_intern = cmodel;

    /*
    // Farbraum-Konverter lschen
    delete enc_converter;
    delete dec_converter;
    enc_converter = NULL;
    dec_converter = NULL;

    // Farbraumkonverter erstellen
    if( color_model != color_model_intern ) {
        enc_converter = new Convert( color_model, color_model_intern, breite, hoehe );
        dec_converter = new Convert( color_model_intern, color_model, breite, hoehe );
    }
    */
    
    return true;
}










// Video-Bild schreiben
int V_Codec::encodeVideo( VideoFrame *bild )
{
    int  x = 0, y = 0;
    if( iomode != VideoFile::write )
        return 0;

    // Gre des Bildes berprfen
    bild->OutSize( x,  y );
    if( x != breite || y != hoehe ) {
        // Bild skalieren
        bild->SetOutSize( breite, hoehe );
    }
    
    // Farbraum fr Ausgabedaten festlegen
    ColorModel( bild->ColorModel() );
    bild->OutColorModel( color_model );
    
    enc_frame_puffer = bild;

    // eigentlichen Codec aufrufen
    return encode( enc_frame_puffer );
}




// Video-Bild lesen
int V_Codec::decodeVideo( VideoFrame *bild )
{
    int stat;
    int x=0, y=0;

    if( iomode != VideoFile::read )
        return 0;

    // Gre des Bildes berprfen
    bild->Size( x,  y );
    if( x != breite || y != hoehe )
        return 0;

    // Farbraum festlegen
    bild->ColorModel( color_model );
    
    // Codec aufrufen
    stat = decode( bild );

    /*
    // Farbraumkonvertierung ausfhren
    if( dec_converter ) {
        dec_converter->do_convert( frame_puffer, bild );
    } else {
        bild = frame_puffer;
    }
    */
    
    return stat;
}





int V_Codec::SeekFrame( long bild )
{
    return track->Seek( bild );
}



int V_Codec::SeekTime( double seconds )
{
    if( iomode != VideoFile::read )
        return 0;

    long frame = (long long)( seconds * track->Rate() );
    return SeekFrame( frame );
}



int V_Codec::CopyFrame( void )
{
    if( iomode != VideoFile::write )
        return 0;

    if( enc_frame_puffer == NULL )
        return 0;

    // Farbraum festlegen
    enc_frame_puffer->OutColorModel( color_model );
    
    return encode( enc_frame_puffer );
}




// Video-Bild schreiben
int V_Codec::encode( VideoFrame *bild )
{
    ERROR_TXT( _("encode Video not implemented!") );
    return 0;
}




// Video-Bild lesen
int V_Codec::decode( VideoFrame *bild )
{
    ERROR_TXT( _("decode video not implemented!") );
    return 0;
}
