/*****************************************************************************\
**  Author  : Christian Tusche                                               **
**  E-Mail  : ctusche@gmx.net                        **
**                                                                           **
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/

#include <stdio.h>

#include "VideoFile.h"
#include "debug.h"
#include "translate.h"


/***** Konstruktor / Destruktor *****/
V_Track::V_Track( VideoFile *File, int track)
:Track( File, track )
{
    // Variablen initialisieren
    codec = NULL;

    frame_daten = NULL;
    frame_daten_bytes = 0;

    // Track in Liste einreihen
    File->addTrack( this );
}



V_Track::V_Track( VideoFile *File, double Rate, int x, int y, const char *Codec_name )
:Track( File, 0, Codec_name ), V_Format( x, y )
{
    SetRate( Rate );
    
    codec = NULL;

    frame_daten = NULL;
    frame_daten_bytes = 0;

    File->addTrack( this );
}




V_Track::~V_Track()
{
    // DEBUG_TXT( "delete V_Track %s", codec_name );

    if( codec != NULL ) {
        delete codec;
    }

    file->removeTrack( this );
}





/**testet, ob das aktuelle Bild ein Keyframe ist
  */
bool V_Track::IsKeyFrame( void )
{
    long bild = Position();
    return GetKeyFrame( bild ) == bild;
}












/***** Rohdaten lesen und schreiben *****************************************************/
// Rckgabewert: 1   -    OK
//               <1  -    Fehler
int V_Track::readData( unsigned char *&data, long &bytes )
{
    int stat;

    stat = readRaw( data, bytes );
    pos++;

    frame_daten = data;
    frame_daten_bytes = bytes;

    return stat;
}



int V_Track::writeData( unsigned char *data, long bytes, int keyframe )
{

    frame_daten = data;
    frame_daten_bytes = bytes;
    pos++;
    total_blocks++;
    Track::bytes += bytes;
    return writeRaw( data, bytes, keyframe );
}





/***** Rohdaten lesen und schreiben *****************************************************/
// eigentliche Implementierung  - mu in abgeleiteter Klasse berladen werden
int V_Track::readRaw( unsigned char *&data, long &bytes )
{
    ERROR_TXT( _("read data not implemented!") );
    return 0;
}



int V_Track::writeRaw( unsigned char *data, long bytes, int keyframe )
{
    ERROR_TXT( _("write data not implemented!") );
    return 0;
}
