/********************************************************************\
*****   Basisblasse fr Video-Track                             *****
\********************************************************************/

#ifndef _VIDEO_TRACK_H_
#define _VIDEO_TRACK_H_

#include <sys/types.h>

#include "Track.h"
#include "V_Format.h"

class VideoFile;

/**Basisklasse fr Video Tracks.<br>
  *beinhaltet alle Operationen, die mit einem Videotrack ausgefhrt
  *werden knnen
  *@author Christian Tusche
  */


class V_Track: public virtual Track, public V_Format {
    friend class V_Codec;
public:
    V_Track( VideoFile *File, int track=0 );

    V_Track( VideoFile *File,
             double Rate,
             int x, int y,
             const char *Codec_name = NULL );

    virtual ~V_Track();


    /**liefert den letzten Keyframe zur angegebenen Position
      */
    virtual long GetKeyFrame( long bild ) { return bild; }

    /**testet, ob das aktuelle Bild ein Keyframe ist
      */
    virtual bool IsKeyFrame( void );

    /**Rohdaten lesen
      */
    virtual int readData( unsigned char *&data, long &bytes );

    /**Rohdaten schreiben
      */
    virtual int writeData( unsigned char *data, long bytes, int keyframe=0 );

    static const int IS_KEYFRAME=1;

    /**Liefert Zeiger auf Videocodec
      */
    inline V_Codec *Codec( void ) { return codec; }


protected:
    /**Rohdaten lesen
      */
    virtual int readRaw( unsigned char *&data, long &bytes );

    /**Rohdaten schreiben
      */
    virtual int writeRaw( unsigned char *data, long bytes, int keyframe=0 );

    V_Codec *codec;             // Videocodec fr diesen Track

    unsigned char *frame_daten; // Rohdatenfeld von letztem Bild
    long frame_daten_bytes;     // Feldgre
};

#endif
