/********************************************************************\
*****   Basisblasse fr Videodateiformate                        *****
\********************************************************************/

#ifndef _VIDEO_FILE_H_
#define _VIDEO_FILE_H_

#include <sys/types.h>
#include <list>
using std::list;

#include "String.h"

#include "Convert.h"
#include "PluginList.h"
#include "Errors.h"

class V_Codec;
class V_Track;
class A_Codec;
class A_Track;
class Track;
class Codec;
class CodecPlugins;
class IOModulPlugins;
class Codec_Info;
class IOModul_Info;

/**Basisklasse fr IO-Module.<br>
  *@author Christian Tusche
  */

class VideoFile {
    friend class V_Codec;
    friend class V_Track;
    friend class A_Codec;
    friend class A_Track;
    friend class Track;
    friend class Codec;
public:
    typedef enum IOMODE { read, write } IO_Mode;

    /**aktiviere gettext
      */
    static void Init( void );
    
    /**erstelle ein IO-Modul
      */
    static VideoFile *Create_VideoFile( const String file, const String typ, IO_Mode mode = write );

    /**Liste mit IO-Modulen
      */
    static PluginList<IOModul_Info> *IOModulList( void );

    /**Liste mit Codecs
      */
    static PluginList<Codec_Info> *AVCodecList( void );



    
    VideoFile( const String file, const String typ, IO_Mode mode = write );
    virtual ~VideoFile();

    /** Video codec initialisieren<br>
      *    Codec: Name des Videocodecs<br>
      *    Es wird ein neuer Track mit dem angegebenen Codec erstellt.<br>
      *    Zurckgegeben wird ein Zeiger auf den Codec des Tracks.
      */
    virtual V_Codec *initVideoCodec( double Rate,
                                     int x, int y,
                                     const char *Codec = NULL );


    /** Audio codec initialisieren
      */
    virtual A_Codec *initAudioCodec( int Sample_rate,
                                     int Channels,
                                     const char *Codec = NULL );

    /**Zeiger auf Video Codec<br>
      *wenn der Track nochnicht geffnet ist, wird er zum lesen
      *geffnet.
      */
    virtual V_Codec *VideoCodec( int track );

    /**Zeiger auf Audio Track<br>
      *wenn der Track nochnicht geffnet ist, wird er zum lesen
      *geffnet.
      */
    virtual A_Codec *AudioCodec( int track );


    /**Anzahl der Video-Tracks
      */
    virtual int VideoTracks(void) { return v_tracks; }

    /**Anzahl der Audio-Tracks
      */
    virtual int AudioTracks(void) { return a_tracks; }
    
        
    /**Anzahl der in die Videodatei geschriebenen Bytes
      */
    virtual long long BytesWritten( void );

    /**Name des IO-Moduls fr diese Instanz
      */
    virtual const String ModulName( void ) { return iomodul; }

    // Fehlernachricht
    //virtual void Fehler( const char* fmt, ... );

    /**Fehler-Status
      */
    virtual Errors &ErrorStatus( void ) { return err_stat; }
      
protected:
    // Video-Track ffnen
    virtual V_Track *openVideoTrack( int track );
    virtual V_Track *openVideoTrack( double Rate, int x, int y, const char *Codec );

    // Audio-Track ffnen
    virtual A_Track *openAudioTrack( int track );
    virtual A_Track *openAudioTrack( int Sample_rate, int Channels, const char *Codec );

    list<A_Track*> ATrackListe;
    list<V_Track*> VTrackListe;

    V_Track *getVideoTrack( int track );
    A_Track *getAudioTrack( int track );
    void addTrack( V_Track *track );
    void addTrack( A_Track *track );
    void removeTrack( V_Track *track );
    void removeTrack( A_Track *track );

    void deleteTracks( void );

    int v_tracks, a_tracks;

    /**Anzahl der Video-Tracks setzen
      */
    void SetVideoTracks( int n ) { v_tracks = n; }

    /**Anzahl der Audio-Tracks setzen
      */
    void SetAudioTracks( int n ) { a_tracks = n; }

    IO_Mode iomode;         // lesen/schreiben
    String iofile;          // Datei fr Video ein/ausgabe
    String iomodul;         // Name des Backends

    static CodecPlugins *AV_Codecs;
    static IOModulPlugins *IO_Moduls;

    // Status
    Errors err_stat;

    // VideoFile initialisiert
    static bool init;
};

#include "V_Track.h"
#include "V_Codec.h"
#include "A_Track.h"
#include "A_Codec.h"
#include "Codec_Info.h"
#include "IOModul_Info.h"

#endif
