/*****************************************************************************\
                          VideoFrame.C  -  description
                             -------------------
    begin                : Son Dez 15 2002
    copyright            : (C) 2002 by Christian Tusche
    email                : ctusche@gmx.net
\*****************************************************************************/

/*****************************************************************************\
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/

#include <unistd.h>
#include <string.h>

// #define TIMERS 1

#include "VideoFrame.h"
#include "cmodels.h"
#include "debug.h"
#include "translate.h"



VideoFrame::VideoFrame( int x, int y, uint8_t *memseg, size_t size )
{
    // Daten des Bildes
    bilddaten = new FrameData( x, y, memseg, size );
    ausgabebild = NULL;
    
    // Zwischenspeicher
    crop_out = NULL;
    scale_out = NULL;
    cmodel_out = NULL;
    
    cmodel_converter = NULL;
    re_scaler = NULL;
    
    ausgabe_ok = false;

    box_x1 = 1;
    box_y1 = 1;
    box_x2 = bilddaten->breite;
    box_y2 = bilddaten->hoehe;

    out_breite = x;
    out_hoehe = y;

    out_cmodel = bilddaten->ColorModel();
}



VideoFrame::~VideoFrame()
{
    delete bilddaten;
    delete crop_out;
    delete scale_out;
    delete cmodel_out;
}



/**Bilddaten in Frame speichern - VideoFrame
  */
void VideoFrame::SetFrame( VideoFrame *bild )
{
    if( bild == NULL ) {
        ausgabe_ok = false;
        return;
    }
    
    // Gre prfen
    int x=0, y=0;
    bild->OutSize( x, y );
    if( x != bilddaten->breite || y != bilddaten->hoehe ) {
        return;
        // Gre anpassen ???
        // SetSize( x, y );
    }

    // Farbraum setzen
    bild->OutColorModel( ColorModel() );
    
    if( !bild->ausgabe_ok ) {
        bild->ausgabe_erzeugen();
    }

    if( bild->ausgabebild == bild->bilddaten || !bilddaten->alloc_buf ) {
        // *bild hat keine Kopie der Ausgabe - Ausgabe sind Originaldaten!
        // kopiere Ausgabe
        bilddaten->SetFrame( bild->ausgabebild );
        ausgabe_ok = false;
        
    } else {
        // tausche eigene Bilddaten mit Ausgabedaten in *bild
        FrameData *temp = bilddaten;
        
        bilddaten = bild->ausgabebild;

        // Ausgabe i *bild ersetzen - 3 Mglichkeiten
        if( bild->crop_out == bilddaten )
            bild->crop_out = temp;
        else if( bild->scale_out == bilddaten )
            bild->scale_out = temp;
        else
            bild->cmodel_out = temp;
                
        bild->ausgabe_ok = false;
        ausgabe_ok = false;
    }
}



/**Bilddaten in Frame speichern - lineare Framebuffer<br>
  *Daten aus *bild werden in den interne Framebuffer kopiert.
  */
void VideoFrame::SetFrame( uint8_t *bild )
{
    if( bild != NULL ) {
        bilddaten->SetFrame( bild );
    }
    ausgabe_ok = false;
}






/**Bilddaten in Frame speichern - planar<br>
  *Daten aus *bild werden in den interne Framebuffer kopiert.
  */
void VideoFrame::SetFrame( uint8_t *y, uint8_t *u, uint8_t *v )
{
    bilddaten->SetFrame( y, u, v );
    ausgabe_ok = false;
}






/**Zeiger auf Datenfeld - linearer Framebuffer<br>
  *Rckgabewert: Gre der Bilddaten
  */
int VideoFrame::FramePointer( uint8_t *&bild )
{
    ausgabe_ok = false;
    return bilddaten->FramePointer( bild );;
}




/**Zeiger auf Datenfeld - planar<br>
  *Rckgabewert: Gre der Bilddaten der Y-Komponente
  */
int VideoFrame::FramePointer( uint8_t *&y, uint8_t *&u, uint8_t *&v )
{
    ausgabe_ok = false;
    return bilddaten->FramePointer( y, u, v );
}






/**Zeiger auf Ausgabebild - linearer Framebuffer<br>
  *Rckgabewert: Gre der Bilddaten
  */
int VideoFrame::GetFrame( uint8_t *&bild )
{
    uint8_t *b = NULL;
    
    if( !ausgabe_ok ) {
        ausgabe_erzeugen();
    }

    ausgabebild->FramePointer( b );

    bild = b;

    return ausgabebild->frame_size;
}






/**Zeiger auf Ausgabebild - planar<br>
  *Rckgabewert: Gre der Bilddaten Y+U+V
  */
int VideoFrame::GetFrame( uint8_t *&y, uint8_t *&u, uint8_t *&v )

{
    uint8_t *_y = NULL, *_u = NULL, *_v = NULL;

    if( !ausgabe_ok ) {
        ausgabe_erzeugen();
    }

    ausgabebild->FramePointer( _y, _u, _v );

    y = _y;
    u = _u;
    v = _v;

    return ausgabebild->frame_size;
}







/**Legt den Bildausschnitt fest, der ausgegeben werden soll<br>
  *Erste Ecke des Rechtecks:  x1, y1<br>
  *Zweite Ecke des Rechtecks: x2, y2<br>
  *Die Koordinaten werden von der oberen linken Ecke aus gezhlt.
  */
void VideoFrame::CropBox( int x1, int y1, int x2, int y2 )
{
    int breite = bilddaten->breite;
    int hoehe  = bilddaten->hoehe;
    
    if( (x2-x1) * (y2-y1) == 0 ) {
        box_x1 = 1;
        box_y1 = 1;
        box_x2 = breite;
        box_y2 = hoehe;
        ausgabe_ok = false;
        
    } else {

        // Bereich prfen und Box einstellen
        x1 = (x1 >= 1 ? x1 : 1);
        x1 = (x1 >= breite ? breite : x1);
        x2 = (x2 >= 1 ? x2 : 1);
        x2 = (x2 >= breite ? breite : x2);
        y1 = (y1 >= 1 ? y1 : 1);
        y1 = (y1 >= hoehe ? hoehe : y1);
        y2 = (y2 >= 1 ? y2 : 1);
        y2 = (y2 >= hoehe ? hoehe : y2);
        
        box_x1 = (x2 > x1 ? x1 : x2);
        box_x2 = (x2 > x1 ? x2 : x1);
        box_y1 = (y2 > y1 ? y1 : y2);
        box_y2 = (y2 > y1 ? y2 : y1);
    
        ausgabe_ok = false;
    }

    out_breite = box_x2-box_x1+1;
    out_hoehe = box_y2-box_y1+1;
    
    setup_crop();
    setup_scale();
    setup_cmodel();
}






/**Skalierte Gre des Ausgabebildes festlegen
      */
void VideoFrame::SetOutSize( int x, int y )
{
    if( x == out_breite && y == out_hoehe ) {
        return;
    }
    
    if( x == 0 || y == 0 || ( x == box_x2-box_x1+1 && y == box_y2-box_y1+1 ) ) {
        // auf Standard zurcksetzen
        out_breite = box_x2-box_x1+1;
        out_hoehe =  box_y2-box_y1+1;
    } else {    
        out_breite = x;
        out_hoehe = y;
    }
    
    setup_scale();
    setup_cmodel();
}

    



/**Liefert die Gre des auszugebenden Bildes
  */
void VideoFrame::OutSize( int &x, int &y )
{
    x = out_breite;
    y = out_hoehe;
}





/**Liefert die Gre des ursprnglichen Bildes
  */
void VideoFrame::Size( int &x, int &y )
{
    x = bilddaten->breite;
    y = bilddaten->hoehe;
}




/**Setzt gewnschten Farbraum fr die Bildausgabe
  */
void VideoFrame::OutColorModel( int cmodel )
{
    // Farbraum unbekannt
    if( cmodel & CMODEL_MASK > NUM_CMODELS )
        return;

    if( out_cmodel != cmodel ) {
        ausgabe_ok = false;
        out_cmodel = cmodel;

        setup_cmodel();
    }
}




/**Farbraum der Bilddaten abfragen
  */
int VideoFrame::ColorModel( void )
{
    return bilddaten->ColorModel();
}





/**Farbraum der Bilddaten festlegen
  */
void VideoFrame::ColorModel( int cmodel )
{
    if( bilddaten->ColorModel() == cmodel )
        return;
        
    bilddaten->ColorModel( cmodel );

    setup_crop();
    setup_scale();
    setup_cmodel();
}




/**erzeuge das Ausgabebild - Farbraumkonvertierung/ausschneiden
  */
void VideoFrame::ausgabe_erzeugen( void )
{
    FrameData *temp = bilddaten;
    uint8_t *frame_in = NULL, *frame_out = NULL;
    
    if( ausgabe_ok )
        return;
    
    // Bild ausschneiden
    if( !(box_x1 == 1 && box_y1 == 1 && box_x2 == bilddaten->breite && box_y2 == bilddaten->hoehe) ) {
        TIMER_START(1);
        crop_frame( temp, crop_out );
        temp = crop_out;
        TIMER_STOP(1, "Crop Colormodel %i", bilddaten->ColorModel() );
    }

    // skalieren
    if( re_scaler != NULL ) {
        temp->FramePointer( frame_in );
        scale_out->FramePointer( frame_out );
        re_scaler->Scale( frame_in, frame_out );
        temp = scale_out;
    }
    
    // Farbraum konvertieren
    if( cmodel_converter != NULL ) {
        temp->FramePointer( frame_in );
        cmodel_out->FramePointer( frame_out );
        cmodel_converter->do_convert( frame_in, frame_out );
        temp = cmodel_out;
    }

    ausgabebild = temp;
    ausgabe_ok = true;
}






/**Rechteck ausschneiden
  */
void VideoFrame::crop_frame( FrameData *in, FrameData *out )
{
    int cmodel = bilddaten->ColorModel();
    int pixel_gr, i;
    int breite = bilddaten->breite;
    int zeilen = box_y2 - box_y1 + 1;
    int spalten = box_x2 - box_x1 + 1;
    int x1 = box_x1 - 1;
    int y1 = box_y1 - 1;
    uint8_t *a = NULL, *b = NULL;
    uint8_t *a_y = NULL, *b_y = NULL;
    uint8_t *a_u = NULL, *b_u = NULL;
    uint8_t *a_v = NULL, *b_v = NULL;

    in->FramePointer( a );
    out->FramePointer( b );

    // Lineare Formate
    if( !(cmodel & PLANAR) && ( (cmodel & CMODEL_MASK) != YUV420 ||
                                (cmodel & CMODEL_MASK) != YUV422 ) ) {
    
        pixel_gr = CMODEL_BPP[cmodel & CMODEL_MASK] / 8;

        // Zeilen kopieren
        for( i = 0; i < zeilen; ++i ) {
            memcpy( b + i * spalten * pixel_gr,
                    a + ((i+y1) * breite + x1) * pixel_gr,
                    spalten * pixel_gr );
        }

    // YUV-Formate
    } else if( !(cmodel & PLANAR) && ( (cmodel & CMODEL_MASK) == YUV420 ||
                                       (cmodel & CMODEL_MASK) == YUV422 ) ) {
        ERROR_TXT( _("unknown color model - CropBox not possible") );

    // YUV - planar
    } else if( ( (cmodel & CMODEL_MASK) == YUV420 || (cmodel & CMODEL_MASK) == YUV422 ) ) {
        in->FramePointer( a_y, a_u, a_v );
        out->FramePointer( b_y, b_u, b_v );
        
        // Startpunk auf Vielfaches von 2 runden
        if( cmodel & YUV420 ) {
            y1 = (y1 >> 1) * 2;
        }
        x1 = (x1 >> 1) * 2;
    
        // Y-Plane
        // Zeilen kopieren
        for( i = 0; i < zeilen; ++i ) {
            memcpy( b_y + i * spalten,
                    a_y + (i + y1) * breite + x1,
                    spalten );
        }
        
        if( cmodel == YUV420P ) {
            zeilen = (zeilen + 1) >> 1;
            y1 = y1 >> 1;
        }
        
        spalten = spalten >> 1;
        x1 = x1 >> 1;
        breite = breite >> 1;

        // U/V-Planes
        // Zeilen kopieren
        for( i = 0; i < zeilen; ++i ) {
            memcpy( b_u + i * spalten, a_u + (i + y1) * breite + x1, spalten );
                    
            memcpy( b_v + i * spalten, a_v + (i + y1) * breite + x1, spalten );
        }
        
            
    // unbekanntes Format
    } else {
        ERROR_TXT( _("unknown color model - CropBox not possible") );
    }
}








/**Umwandlungen einrichten
  */
void VideoFrame::setup_crop( void )
{
    delete crop_out;
    if( box_x2-box_x1+1 != bilddaten->breite || box_y2-box_y1+1 != bilddaten->hoehe ) {
        crop_out = new FrameData( box_x2-box_x1+1, box_y2-box_y1+1 );
        crop_out->ColorModel( bilddaten->ColorModel() );
    } else {
        crop_out = NULL;
    }
}



void VideoFrame::setup_scale( void )
{
    // Skalierung erstellen
    if( out_breite != box_x2-box_x1+1 || out_hoehe != box_y2-box_y1+1 ) {
        if( scale_out == NULL || scale_out->breite != out_breite ||
                                 scale_out->hoehe != out_hoehe ) {
            delete scale_out;
            scale_out = new FrameData( out_breite, out_hoehe );
        }
        scale_out->ColorModel( bilddaten->ColorModel() );
        
        delete re_scaler;
        re_scaler = new FrameRescaler( bilddaten->ColorModel(), box_x2-box_x1+1, box_y2-box_y1+1, out_breite, out_hoehe );
    } else {
        delete scale_out;
        delete re_scaler;
        scale_out = NULL;
        re_scaler = NULL;
    }
}




void VideoFrame::setup_cmodel( void )
{
    int cmodel;
    if( re_scaler != NULL )
        cmodel = re_scaler->ColorModel();
    else
        cmodel = bilddaten->ColorModel();
        
    // Farbraumkonvertierung festlegen
    delete cmodel_converter;
    
    if( cmodel != out_cmodel ) {
        if( cmodel_out == NULL || cmodel_out->breite != out_breite ||
                                  cmodel_out->hoehe != out_hoehe ) {
            delete cmodel_out;
            cmodel_out = new FrameData( out_breite, out_hoehe );
        }
        cmodel_out->ColorModel( out_cmodel );
        cmodel_converter = new Convert( cmodel, out_cmodel, out_breite, out_hoehe );
    } else {
        delete cmodel_out;
        cmodel_out = NULL;
        cmodel_converter = NULL;
    }
}
