/*****************************************************************************\
                          codecplugins.C  -  description
                             -------------------
    begin                : Sun Jan 6 2002
    copyright            : (C) 2002 by Christian Tusche
    email                : ctusche@gmx.net
\*****************************************************************************/

/*****************************************************************************\
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/

#include "config.h"

#include "codecplugins.h"
#include "debug.h"

#include "VideoFile.h"
#include "translate.h"

CodecPlugins::CodecPlugins()
:PluginLoader()
{
    // Video-Plugins
    AddDir( PREFIX"/lib/VideoFile/video" );

    // Audio-Plugins
    AddDir( PREFIX"/lib/VideoFile/audio" );
}





CodecPlugins::~CodecPlugins()
{
}



/**Video Codec erstellen
  */
V_Codec *CodecPlugins::CreateVCodec( V_Track *track, const String Dateityp, const String &Codecname )
{
    INFO_TXT( _("Create codec-plugin for filetype \"%s\" with codec \"%s\""), Dateityp.chars(), Codecname.chars() );

    Codec_Info *gefunden = FindCodec( track, Dateityp, Codecname );

    if( gefunden != NULL ) {
        // den echten Namen des Codecs im Track aktualisieren
        track->SetCodecName( gefunden->CodecName() );
        track->SetCodecID( gefunden->AliasName() );

        // Codec erstellen
        return gefunden->CreateVCodec( track );
    }
    
    track->SetCodecID( track->CodecName() );
    return NULL;
}




/**Audio Codec erstellen
  */
A_Codec *CodecPlugins::CreateACodec( A_Track *track, const String Dateityp, const String &Codecname )
{
    INFO_TXT( _("Create codec-plugin for filetype \"%s\" with codec \"%s\""), Dateityp.chars(), Codecname.chars() );

    Codec_Info *gefunden = FindCodec( track, Dateityp, Codecname );

    if( gefunden != NULL ) {
        // den echten Namen des Codecs speichern
         // den echten Namen des Codecs im Track aktualisieren
        track->SetCodecName( gefunden->CodecName() );
        track->SetCodecID( gefunden->AliasName() );

        // Codec erstellen
        return gefunden->CreateACodec( track );
    }

    track->SetCodecID( track->CodecName() );
    return NULL;
}






int CodecPlugins::RegisterModul( modul_data *data )
{
    if( data->Typ != "V_Codec" && data->Typ != "A_Codec" ) {
        return 0;
    }

    // Liste der Codecformate des Moduls zu Liste der
    // unterstzten Formate hinzufgen
    PluginList<Codec_Info> *liste;
    liste = (PluginList<Codec_Info> *)data->ModulInfo();

    // Dateinamen des Moduls speichern
    reg_codecs.Libname( data->Name.chars() );
    reg_codecs += *liste;

    return 1;
}









Codec_Info *CodecPlugins::FindCodec( Track *track, const String &Dateityp, const String &Codecname )
{
    Codec_Info *format = NULL;

    for( unsigned int i=0; i < reg_codecs.NumElements(); i++ ) {
        format = &reg_codecs.Element( i );
        if( track->iomode == VideoFile::read ) {
            // bei lesendem Zugriff wurde der Codecname vom IOModul vorgegeben
            // es wird also der entsprechende Aliasname gesucht
            if( ( Dateityp == NULL || format->datei_typ == Dateityp ) &&
                format->alias_name == Codecname && format->can_read ) {

                return format;
            }
        } else {
            // bei lesendem Zugriff wurde der Codecname vom Benutzer vorgegeben
            // es wird noch dem richtigen Codecnamen gesucht
            if( ( Dateityp == NULL || format->datei_typ == Dateityp ) &&
                format->codec_name == Codecname && format->can_write ) {

                return format;
            }
        }
    }

    return NULL;
}
