/*****************************************************************************\
                          pluginloader.C  -  description
                             -------------------
    begin                : Fri Jan 4 2002
    copyright            : (C) 2002 by Christian Tusche
    email                : ctusche@gmx.net
\*****************************************************************************/

/*****************************************************************************\
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/

#include <dirent.h>
#include <dlfcn.h>
#include <Regex.h>

#include "pluginloader.h"
#include "debug.h"
#include "translate.h"

PluginLoader::PluginLoader()
{

}





PluginLoader::~PluginLoader()
{
    // geladene Module schlieen
    CloseModuls();
}







int PluginLoader::AddDir( const char *dir )
{
    // ffne Verzeichnis
    INFO_TXT( ("Load modules in directory \"%s\""), dir);
    DIR *dir_handle = opendir( dir );
    if( dir_handle == NULL ) {
        DEBUG_TXT( _("Unable to open directory \"%s\"!"), dir );
        return 0;
    }

    // Verzeichniseintrge verarbeiten
    struct dirent *eintrag;
    String name;
    int n=0;
    Regex expr( "^lib.*\\.plugin\\.so." VIDEO_FILE_VER "$" );

    while( ( eintrag = readdir( dir_handle ) ) != NULL ) {
        //nur echte Dateien verarbeiten
        //if( eintrag->d_type != DT_REG )
        //    continue;

        name = eintrag->d_name;
        if( name.contains( expr ) ) {
            name.prepend( "/" );
            name.prepend( dir );
            n += AddModul( name.chars() );
        }
    }

    closedir( dir_handle );

    INFO_TXT( _("loaded %i modules"), n );
    return n;
}






int PluginLoader::AddModul( const char *libname )
{
    modul_data *data;

    // DEBUG_TXT_3( "Versuche Plugin aus lib \"%s\" zu laden.", libname );

    // ffne Modul
    void *dl_handle = dlopen( libname, RTLD_LAZY | RTLD_LOCAL );
    if( dl_handle == NULL ) {
        DEBUG_TXT( _("dlopen of \"%s\" failed!"), libname );
        return 0;
    }

    data = new modul_data;
    data->dl_handle = dl_handle;
    data->Name = libname;

    // lade Symbole aus lib
    data->InitModul   = (const char *(*)(void)) dlsym( dl_handle, "plugin_init" );
    data->ModulInfo   =       (void *(*)(void)) dlsym( dl_handle, "plugin_info" );
    data->DeleteModul =        (void (*)(void)) dlsym( dl_handle, "plugin_delete" );

    // InitModul und ModulInfo mssen auf jeden fall vorhanden sein
    if( data->InitModul == NULL ||
        data->ModulInfo == NULL ) {

        // nicht alle Symbole gefunden
        DEBUG_TXT( _("Not all symbols in \"%s\" found!"), libname );
        dlclose( dl_handle );
        delete data;
        return 0;
    }

    // initialisiere Modul
    const char *Typ = data->InitModul();
    if( Typ == NULL ) {
        // keine gltige Typinformation - Initialisierung ist fehlgeschlagen
        DEBUG_TXT( _("plugin_init in \"%s\" failed!"), libname );
        dlclose( dl_handle );
        delete data;
        return 0;
    }
    data->Typ = Typ;

    // registriere Modul
    if( ! RegisterModul( data ) ) {
        // fehlgeschlagen
        DEBUG_TXT( _("Modul \"%s\" could not be registered!"), libname );
        if( data->DeleteModul != NULL )
            data->DeleteModul();

        dlclose( dl_handle );
        delete data;
        return 0;
    }

    // reihe Modul in Liste ein
    module.push_back( data );

    DEBUG_TXT( _("Loaded module from \"%s\""), libname );
    return 1;
}





void PluginLoader::CloseModuls( void )
{
    modul_data *data;
    // alle Module schlieen
    while( module.size() > 0 ) {
        data = module.back();

        // moduldestruktor ausfhren, wenn einer vorhanden ist
        if( data->DeleteModul != NULL )
            data->DeleteModul();

        module.pop_back();
        delete data;
    }
}
