/***************************************************************************
                          pluginloader.h  -  description
                             -------------------
    begin                : Fri Jan 4 2002
    copyright            : (C) 2002 by Christian Tusche
    email                : ctusche@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PLUGINLOADER_H
#define PLUGINLOADER_H

#include "String.h"
#include <vector>
using std::vector;


/**ffnen und verwalten von dynamisch ladbaren Modulen
  *@author Christian Tusche
  */

class PluginLoader {
public: 
    PluginLoader();
    virtual ~PluginLoader();

    /**eine Verzeichnis einlesen und zum Modulspeicher hinzufgen.<br>
      *Rckgabe: anzahl der geladenen Module
      */
    virtual int AddDir( const char *dir );

    /**eine einzelene lib zum Modulspeicher hinzufgen.<br>
      *Rckgabe: 1 - Erfolg
      *          0 - Fehler
      */
    virtual int AddModul( const char *libname );

    /**alle offenen Module schlieen
      */
    virtual void CloseModuls( void );

protected:
    /**Datenstruktur fr ein Modul
      */
    struct modul_data {
        String Name;                        // Dateiname
        void *dl_handle;                    // Handle fr libdl

        // dynamisch geladene Funktionen
        const char *(*InitModul)(void);     // Modul initialisieren - liefert String mit Modultyp
        void *(*ModulInfo)(void);           // Info-Struktur des Moduls abfragen
        void (*DeleteModul)(void);          // Modul beenden

        String Typ;                         // Modultyp
    };

    /**Liste mit geladenen Modulen
      */
    vector<modul_data *> module;

    /**Modul intern registrieren<br>
      *Rckgabewert: 1 - Modul registriert<br>
      *              0 - Modul nicht regitriert
      */
    virtual int RegisterModul( modul_data *data ) { return 0; }
};

#endif
