/*****************************************************************************\
                          ausgabedatei.C  -  description
                             -------------------
    begin                : Mon Dec 24 2001
    copyright            : (C) 2001 by Christian Tusche
    email                : ctusche@gmx.net
\*****************************************************************************/

/*****************************************************************************\
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/


#include <stdio.h>
#include <assert.h>

#include "ausgabedatei.h"
#include "VideoFile/VideoFile.h"


AusgabeDatei::AusgabeDatei()
{
    AV_Ausgabe = NULL;
    A_Aus = NULL;
    V_Aus = NULL;
    video = false;
    audio = false;
    nummer = 0;
    neuer_Name = "";
}







AusgabeDatei::~AusgabeDatei()
{
    close();
}





int AusgabeDatei::open( EingabeDatei *Eingabe, int breite, int hoehe, int a_channels )
{
    /***** Datei ffnen *******************************************************/
    String name = getNextName();

    if( name == "-" ) {
        AV_Ausgabe = VideoFile::Create_VideoFile( "", v_format, VideoFile::write );
    } else {
        AV_Ausgabe = VideoFile::Create_VideoFile( name, v_format, VideoFile::write );
    }

    if( AV_Ausgabe == NULL )
        return 0;

    if( Eingabe->video ) {
        if( breite == 0 || hoehe == 0 ) {
            Eingabe->Frame()->OutSize( breite, hoehe );
        }
        
        V_Aus = AV_Ausgabe->initVideoCodec( Eingabe->fps, breite, hoehe, vcodec->CodecName().chars() );
        if( V_Aus != NULL ) {
            video = true;
            // Optionen setzen
            vcodec->SetCodecOptions( V_Aus );
        }
    }

    if ( Eingabe->audio ) {
        if( a_channels == 0 ) {
            a_channels = Eingabe->Channels();
        }
        A_Aus = AV_Ausgabe->initAudioCodec( Eingabe->a_rate, a_channels, acodec->CodecName().chars() );
        if( A_Aus != NULL ) {
            audio = true;
            acodec->SetCodecOptions( A_Aus );
         }
    }

    if( V_Aus == NULL && A_Aus == NULL )
        return 0;

    return 1;
}









int AusgabeDatei::open( double rate, int breite, int hoehe,
                  int sound_rate, int sound_channels )
{
    /***** Datei ffnen *******************************************************/
    assert( v_name == a_name );

    const String name = getNextName();

    if( name == "-" ) {
        AV_Ausgabe = VideoFile::Create_VideoFile( "", v_format, VideoFile::write );
    } else {
        AV_Ausgabe = VideoFile::Create_VideoFile( name, v_format, VideoFile::write );
    }

    if( AV_Ausgabe == NULL )
        return 0;

    if( rate > 0 ) {
        V_Aus = AV_Ausgabe->initVideoCodec( rate, breite, hoehe, vcodec->CodecName().chars() );
        if( V_Aus != NULL ) {
            video = true;
            // Optionen setzen
            vcodec->SetCodecOptions( V_Aus );
        }
    }

    if ( sound_rate > 0 ) {
        A_Aus = AV_Ausgabe->initAudioCodec( sound_rate, sound_channels, acodec->CodecName().chars() );
        if( A_Aus != NULL ) {
            audio = true;
            acodec->SetCodecOptions( A_Aus );
        }
    }

    if( V_Aus == NULL && A_Aus == NULL )
        return 0;

    return 1;
}








void AusgabeDatei::close( void )
{
    delete AV_Ausgabe;
    AV_Ausgabe = NULL;
}







const String AusgabeDatei::getNextName( void )
{
    char fmt[100];

    assert( v_name == a_name );

    sprintf( fmt, "%i", nummer );

    if( v_name.contains( "%n" ) ) {
        neuer_Name  = v_name.before( "%n" );
        neuer_Name += fmt;
        neuer_Name += v_name.after( "%n" );
    } else {
        neuer_Name  = v_name;
        if( nummer > 0 )
            neuer_Name += fmt;
    }
    nummer++;
    return neuer_Name;
}




const char *AusgabeDatei::getName( void )
{
    return neuer_Name.chars();
}



/** Video Frame schreiben
  */
int AusgabeDatei::encodeVideo( VideoFrame *bild )
{
    if( V_Aus != NULL ) {
        return V_Aus->encodeVideo( bild );
    }
    return 0;
}





/** Video Rohdaten schreiben
  */
int AusgabeDatei::writeVideoData( unsigned char *data, long bytes, int keyframe )
{
    if( V_Aus != NULL ) {
        return V_Aus->Track()->writeData( data, bytes, keyframe );
    }
    return 0;
}








/** Audio schreiben
  */
int AusgabeDatei::encodeAudio( int16_t **puffer, int samples )
{
    if( A_Aus != NULL ) {
        return A_Aus->encodeAudio( puffer, samples );
    }
    return 0;
}





/** Farbraum fr Videoeingabe einstellen
  */
bool AusgabeDatei::ColorModel( int cmodel )
{
    if( V_Aus != NULL )
        return V_Aus->ColorModel( cmodel );
    return false;
}





/**Bisher geschriebene Bytes
  */
long long AusgabeDatei::BytesWritten( void )
{
    return AV_Ausgabe->BytesWritten();
}
