/*****************************************************************************\
                          codec_optionen.C  -  description
                             -------------------
    begin                : Mon Jan 21 2002
    copyright            : (C) 2002 by Christian Tusche
    email                : ctusche@gmx.net
\*****************************************************************************/

/*****************************************************************************\
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/

#include <stdio.h>

#include "codec_optionen.h"
#include "VideoFile/VideoFile.h"

#define GETTEXT_DOMAIN "av_convert"
#include "VideoFile/translate.h"


Codec_Optionen::Codec_Optionen()
{
}



Codec_Optionen::~Codec_Optionen()
{
    while( opt_liste.size() > 0 ) {
        delete opt_liste.back();
        opt_liste.pop_back();
    }
}



void Codec_Optionen::SetArg( const char *fmt )
{
    String arg( fmt );

    if( arg.contains( "=" ) ) {
        // Option in Liste einreihen
        if( arg.length() > 0 ) {
            option *eintrag = new option;
            eintrag->name = arg.before( "=" );
            eintrag->wert = arg.after( "=" );
            opt_liste.push_back( eintrag );
        }
    } else {
         if( codec_name == "" ) {
            codec_name = arg;
        }
    }
}






String Codec_Optionen::CodecName( void )
{
    return codec_name;
}





void Codec_Optionen::SetCodecOptions( Codec *codec )
{
    for( unsigned int i=0; i<opt_liste.size(); i++ ) {
        if( codec->Option( opt_liste[i]->name ).Name() == "" ) {
            // Option nicht vorhanden
            fprintf( stderr, _("Option \"%s\" is not supported by codec \"%s\".\n"),
                     opt_liste[i]->name.chars(), codec_name.chars() );
            // eine Liste mit mglichen Optionen ausgeben
            ListOptions( codec );
            exit( 1 );
        }

        // Option setzen
        codec->Option( opt_liste[i]->name ) = opt_liste[i]->wert;
    }
}





void Codec_Optionen::ListOptions( Codec *codec )
{
    fprintf( stderr, _("Codec options:\n") );

    for( unsigned int i=0; i < codec->NumOptions(); i++ ) {
        if( codec->Option(i).Name() != "" ) {
            fprintf( stderr, " %-20s = %6s    [%s]\n", (const char*)codec->Option(i).Name(),
                                                       (const char*)codec->Option(i),
                                                       (const char*)codec->Option(i).Info() );
        } else {
                fprintf( stderr, " %-20s = %7s\n", (const char*)codec->Option(i).Name(),
                                                   (const char*)codec->Option(i) );
        }
    }
}
