/*****************************************************************************\
                          eingabedatei.C  -  description
                             -------------------
    begin                : Mon Dec 24 2001
    copyright            : (C) 2001 by Christian Tusche
    email                : ctusche@gmx.net
\*****************************************************************************/

/*****************************************************************************\
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/

#include <stdio.h>
#include <assert.h>

#include "eingabedatei.h"
#include "VideoFile/VideoFile.h"

#define GETTEXT_DOMAIN "av_convert"
#include "VideoFile/translate.h"

EingabeDatei::EingabeDatei()
{
    /***** Variablen initialisieren *******************************************/
    VideoEingabe = AudioEingabe = NULL;
    V_Ein = NULL;
    A_Ein = NULL;
    bilder = 0;
    a_samples = 0;
    breite = hoehe = a_rate = a_channels = 0;
    fps = 25.0;
    audio_puffer_gr = 22050;
    a_frames_gelesen = 0;
    samples_gelesen = 0;
    audio = video = false;
    bild_daten = NULL;
    box_x1 = box_x2 = box_y1 = box_y2 = 0;
}






EingabeDatei::~EingabeDatei()
{
    close();
}







/***** Dateien ffnen *****/
int EingabeDatei::open( void )
{
    int stat = 1;

    /***** Dateien ffnen *****************************************************/
    if( v_name != "" ) {
        if( v_name != "-" ) {
            VideoEingabe = VideoFile::Create_VideoFile( v_name, v_format, VideoFile::read );
        } else {
            VideoEingabe = VideoFile::Create_VideoFile( "", v_format, VideoFile::read );
        }
    }

    if( a_name != "" && v_name != "" && a_name == v_name ) {
        AudioEingabe = VideoEingabe;
    } else if( a_name != "" && ( v_name == "" || a_name != v_name ) ) {
        if( a_name != "-" ) {
            AudioEingabe = VideoFile::Create_VideoFile( a_name, a_format, VideoFile::read );
        } else {
            AudioEingabe = VideoFile::Create_VideoFile( "", a_format, VideoFile::read );
        }
    }


    if( VideoEingabe == NULL && AudioEingabe == NULL ) {
        fprintf( stderr, _("No input files!\n") );
        return 0;
    }

    // ffne Videotrack
    if( VideoEingabe != NULL ) {
        V_Ein = VideoEingabe->VideoCodec( 0 );
    }

    if( V_Ein != NULL ) {
        video = true;
        breite = V_Ein->Width();
        hoehe  = V_Ein->Height();
        fps    = V_Ein->Track()->Rate();
        bilder = V_Ein->Track()->Frames();

        // positionieren
        if( V_Ein->SeekTime( Start_Video() ) == 0 ) {
            fprintf( stderr, _("Error seeking video input!\n") );
            stat = 0;
        }

        // erstelle Speicher fr Bilddaten
        bild_daten = new VideoFrame( breite, hoehe );
        bild_daten->CropBox( box_x1, box_y1, box_x2, box_y2 );
    }

    // ffne Audiotrack
    if( AudioEingabe != NULL ) {
        A_Ein = AudioEingabe->AudioCodec( 0 );
    }

    if( A_Ein != NULL ) {
        audio = true;
        a_rate = A_Ein->SampleRate();
        a_channels = A_Ein->Channels();
        a_samples = A_Ein->Track()->Samples();

        // positionieren
        if( A_Ein->SeekTime( Start_Audio() ) == 0 ) {
            fprintf( stderr, _("Error seeking audio input!\n") );
            stat = 0;
        }


        // Gre des Puffers fr Audio-Daten auf 1 Bild anpassen
        audio_puffer_gr = (double)a_rate / fps;
     }

     if( !audio && ! video ) {
         fprintf( stderr, _("Error: No audio or video tracks found!\n") );
         return 0;
     }
     
     return stat;
}







long EingabeDatei::Frames( void )
{
    return (long)(Seconds() * fps);
}






void EingabeDatei::close( void )
{
    // Dateien schlieen
    delete VideoEingabe;
    if( AudioEingabe != VideoEingabe )
        delete AudioEingabe;
        
    delete bild_daten;

    VideoEingabe = NULL;
    AudioEingabe = NULL;
    bild_daten = NULL;
}







int EingabeDatei::decodeVideo( VideoFrame *&bild )
{
    if( V_Ein != NULL ) {
        bild = bild_daten;
        return V_Ein->decodeVideo( bild );
    }
    return 0;
}



int EingabeDatei::readVideoData( unsigned char *&data, long &bytes, int &keyframe )
{
    if( V_Ein != NULL ) {
        keyframe = V_Ein->Track()->IsKeyFrame();
        return V_Ein->Track()->readData( data, bytes );
    }
    keyframe = 0;
    return 0;
}




int EingabeDatei::decodeAudio( int16_t **&puffer, int &samples )
{
    int stat;
    int16_t **audio_daten=NULL;

    if( A_Ein != NULL ) {
        a_frames_gelesen++;
        samples = (long)((double)a_frames_gelesen * audio_puffer_gr - samples_gelesen);
        assert( samples > 0 );

        stat = A_Ein->decodeAudio( audio_daten, samples );

        puffer = audio_daten;
        samples_gelesen += samples;

        return stat;
    }
    return 0;
}



bool EingabeDatei::ColorModel( int cmodel )
{
    if( V_Ein != NULL ) {
        return V_Ein->ColorModel( cmodel );
    }

    return false;
}




int EingabeDatei::SeekTime( double seconds )
{
    // Video positionieren
    if( V_Ein != NULL ) {
        V_Ein->SeekTime( seconds );
    }

    // Audio positionieren
    if( A_Ein != NULL ) {
        A_Ein->SeekTime( seconds );
    }

    return 1;
}





/**auf letzten Keyframe positoinieren
  */
long EingabeDatei::SeekKeyFrame( void )
{
    long key = 0;
    long bild = 0;

    // Video positionieren
    if( V_Ein != NULL ) {
        bild = V_Ein->Track()->Position();
        key =  V_Ein->Track()->GetKeyFrame( bild );
        //V_Ein->SeekFrame( key );
        V_Ein->Track()->Seek( key );

        // Audio positionieren
        if( A_Ein != NULL ) {
            double seconds = (bild - key) / V_Ein->Track()->Rate();
            A_Ein->SeekTime( A_Ein->Track()->CurrentTime() - seconds );
        }
    }


    return bild - key;
}




/**Bildausschnitt festlegen
  */
void EingabeDatei::CropBox( int x1, int y1, int x2, int y2 )
{
    box_x1 = x1;
    box_x2 = x2;
    box_y1 = y1;
    box_y2 = y2;
}
