/***************************************************************************
                          eingabedatei.h  -  description
                             -------------------
    begin                : Mon Dec 24 2001
    copyright            : (C) 2001 by Christian Tusche
    email                : ctusche@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef EINGABEDATEI_H
#define EINGABEDATEI_H

#include "VideoFile/VideoFile.h"
#include "io_datei.h"


class AusgabeDatei;

/**Verwaltet Datenquelle fr Audio und Video
  *@author Christian Tusche
  */

class EingabeDatei: public IO_Datei {
    friend class AusgabeDatei;
public: 
    EingabeDatei();
    ~EingabeDatei();

    /**Dateien ffen
      */
    int open( void );

    /**Datein schlieen
      */
    void close( void );

    /**Anzahl der Bilder, die aus dieser Datei bearbeitet werden sollen
      */
    long Frames( void );

    /**Video Frame decodieren
      */
    int decodeVideo( VideoFrame *&bild );

    /**Video Rohdaten lesen
      */
    int readVideoData( unsigned char *&data, long &bytes, int &keyframe );

    /**Audio decodieren
      */
    int decodeAudio( int16_t **&puffer, int &samples );

    /** Farbraum fr Videoausgabe einstellen
      */
    bool ColorModel( int cmodel );

    /**Eingabedatei positionieren
      */
    int SeekTime( double seconds );

    /**auf letzten Keyframe positoinieren
      */
    long SeekKeyFrame( void );
    
    /**Breite
      */
    int Width( void ) { return breite; }

    /**Hhe
      */
    int Height( void ) { return hoehe; }

    /**Bildrate
       */
    double Rate( void ) { return fps; }

    /**Audio-Kanle
      */
    int Channels( void ) { return a_channels; }

    /**Bildausschnitt festlegen
      */
    void CropBox( int x1, int y1, int x2, int y2 );

    /**VideoFrame
      */
    VideoFrame *Frame( void ) { return bild_daten; }

    bool audio, video;
protected:
    VideoFile *VideoEingabe, *AudioEingabe;
    V_Codec *V_Ein;
    A_Codec *A_Ein;
    long bilder;
    long long a_samples;
    int breite, hoehe, a_rate, a_channels;
    double fps;
    double audio_puffer_gr;
    long a_frames_gelesen;
    long long samples_gelesen;
    VideoFrame *bild_daten;

    // Bildausschnitt
    int box_x1, box_x2, box_y1, box_y2;
};

#endif
