/*****************************************************************************\
                          io_datei.cpp  -  description
                             -------------------
    begin                : Mon Dec 24 2001
    copyright            : (C) 2001 by Christian Tusche
    email                : ctusche@gmx.net
\*****************************************************************************/

/*****************************************************************************\
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/

#include <stdlib.h>

#include "debug.h"
#include "io_datei.h"
#include "stdio.h"

#define GETTEXT_DOMAIN "av_convert"
#include "VideoFile/translate.h"

IO_Datei::IO_Datei()
{
    v_name = "";
    a_name = "";
    v_format = "";
    a_format = "";

    a_start = 0;
    v_start = 0;
    a_ende = 0;
    v_ende = 0;
}


IO_Datei::~IO_Datei()
{
}




void IO_Datei::parseArg( const char *arg, String &name, String &format, double &start, double &ende, double &offset )
{
    String Arg( arg );
    Regex Zeit( "\\[.*\\]$" );
    String start_stop;
    String tmp;

    start = 0;
    ende = 0;

    if( !Arg.contains( "::" ) ) {
        name = Arg;
        format = "";
    } else {
        format = Arg.before( "::" );
        name = Arg.after( "::" );
    }

    if( name.contains( Zeit ) ){
        start_stop = name.at( Zeit );
        start_stop = start_stop.after("[");
        start_stop = start_stop.before("]");

        if( start_stop.contains( ":" ) ) {
            tmp = start_stop.after( ":" );
            start_stop = start_stop.before( ":" );
            offset = strtod( tmp.chars(), NULL );
        }

        if( start_stop.contains("-") ) {
            tmp = start_stop.before("-");
            start = strtod( tmp.chars(), NULL );
            tmp = start_stop.after("-");
            ende = strtod( tmp.chars(), NULL );
        } else {
            start = strtod( start_stop.chars(), NULL );
        }

        if( start < 0 )
            start = 0;
        if( ende < 0 )
            ende = 0;

        DEBUG_TXT( "start     = %f, end     = %f, offset     = %f\n", start, ende, offset );

        name = name.before( Zeit );
    }
}



void IO_Datei::parseArg( const char *arg )
{
    double offset = 0;
    parseArg( arg , v_name, v_format, v_start, v_ende, offset );
    a_name = v_name;
    a_format = v_format;
    
    a_start = v_start + offset;
    a_ende = v_ende + offset;

    if( a_start < 0 ) {
        v_start -= a_start;
        a_start = 0;
    }

}




void IO_Datei::parseArg_Video( const char *arg )
{
    double offset = 0;
    parseArg( arg , v_name, v_format, v_start, v_ende, offset );
}




void IO_Datei::parseArg_Audio( const char *arg )
{
    double offset = 0;
    parseArg( arg , a_name, a_format, a_start, a_ende, offset );
}








const char *IO_Datei::getName_Video( void )
{
    return v_name.chars();
}



const char *IO_Datei::getName_Audio( void )
{
    return a_name.chars();
}




double IO_Datei::Seconds( void )
{
    double a_sec = a_ende - a_start, v_sec = v_ende - v_start;

    if( a_sec < 0 )
        a_sec = 0;
    if( v_sec < 0 )
        v_sec = 0;

    if( v_sec != 0 && a_sec != 0 && a_sec < v_sec )
        return a_sec;
    else if( v_sec != 0 )
        return v_sec;
    else
        return a_sec;
}



/**setze Codec fr Audioausgabe
  */
void IO_Datei::SetACodec( Codec_Optionen *opt )
{
    acodec = opt;
}




/**setze Codec fr Videoausgabe
  */
void IO_Datei::SetVCodec( Codec_Optionen *opt )
{
    vcodec = opt;
}
