/*****************************************************************************\
                          pluginliste.C  -  description
                             -------------------
    begin                : Die Aug 27 2002
    copyright            : (C) 2002 by Christian Tusche
    email                : ctusche@gmx.net
\*****************************************************************************/

/*****************************************************************************\
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/

#include <stdio.h>

#include "pluginliste.h"
#include "VideoFile/VideoFile.h"

#define GETTEXT_DOMAIN "av_convert"
#include "VideoFile/translate.h"

Pluginliste::Pluginliste()
{
}


Pluginliste::~Pluginliste()
{
}



void Pluginliste::Print_IO_Moduls( void )
{
    int n = VideoFile::IOModulList()->NumElements();

    IOModul_Info *plugin_info;

    fprintf( stderr, _("\nloaded IO-Modules:\n\n"));
    fprintf( stderr, _("file-type       description\n"));
    fprintf( stderr, "-----------------------------------------------------------\n");
    
    for( int i=0; i<n; i++ ) {
        plugin_info = &VideoFile::IOModulList()->Element( i );

        // Info ausgeben
        fprintf( stderr, "%-15s %s\n", plugin_info->FileTyp().chars(), plugin_info->InfoText().chars() );
    }
    fprintf( stderr, "-----------------------------------------------------------\n");
    fprintf( stderr, _("loaded %i IO-Modules\n\n"), n);
    
}



void Pluginliste::Print_Codecs( bool audio, bool video )
{
    int m = VideoFile::AVCodecList()->NumElements();
    int k = 0, l=0;

    Codec_Info *codec_info;
    String letzter_name;
    vector<String> Dateitypen;
    
    // Liste mit Dateitypen erstellen
    for( int i=0; i<m; i++ ) {
        int j = 0;
        
        letzter_name = VideoFile::AVCodecList()->Element( i ).FileTyp();
        
        // ist Dateityp schon in der Liste?
        for( j=0; j < Dateitypen.size(); j++ ) {
            if( Dateitypen[j] == letzter_name )
                break;
        }
        if( j < Dateitypen.size() && Dateitypen[j] == letzter_name )
            continue;
        
        // noch nicht in Liste
        Dateitypen.push_back( letzter_name );
    }
            
                                          
    for( int i=0; i<Dateitypen.size(); i++ ) {
        l=0;
        

        letzter_name = "";

        for( int j=0; j<m; j++ ) {
            codec_info = &VideoFile::AVCodecList()->Element( j );

            if( codec_info->FileTyp() != Dateitypen[i] )
                continue;

            if( !(audio && codec_info->Audio() || video && codec_info->Video()) )
                continue;

            // Mehrfache aufzhlung des gleichen Codecs vermeiden
            // das geht so nur, wenn alle Eintrge mit gleichem Codec-Namen direkt
            // hintereinander stehen
            if( codec_info->CodecName() == letzter_name )
                continue;

            letzter_name = codec_info->CodecName();
            
            // Einleitung ausgeben
            if( l==0 ) {
                fprintf( stderr, _("\nfiletype: %s\n"), Dateitypen[i].chars() );
                fprintf( stderr, _("codec          description\n"));
                fprintf( stderr, "-----------------------------------------------------------\n");
            }                
             
            // Info ausgeben
            fprintf( stderr, "%-15s %s\n", codec_info->CodecName().chars(), codec_info->InfoText().chars() );

            k++; l++;
        }
    }
    fprintf( stderr, "-----------------------------------------------------------\n");
    fprintf( stderr, _("loaded %i codecs\n\n"), k );
}




void Pluginliste::Print_A_Codecs( void )
{
    fprintf( stderr, _("\nLoaded audio codec modules:\n\n"));

    Print_Codecs( true, false );
}


void Pluginliste::Print_V_Codecs( void )
{
    fprintf( stderr, _("\nLoaded video codec moduls:\n\n"));

    Print_Codecs( false, true );
}
