/***************************************************************************\
*****                                                                   *****
*****         Videodateien konvertieren                                 *****
*****                                                                   *****
\***************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <limits.h>
#include <assert.h>
#include <vector>
using std::vector;

#include <libintl.h>

#include "config.h"
#include "debug.h"
#include "CmdlineInterface/eingabedatei.h"
#include "CmdlineInterface/ausgabedatei.h"
#include "CmdlineInterface/codec_optionen.h"
#include "CmdlineInterface/pluginliste.h"

#define GETTEXT_DOMAIN "av_convert"
#include "VideoFile/translate.h"



//static long Start  = 0;
//static long Laenge = 0;
static int Blank = 0;
static Codec_Optionen A_Optionen, V_Optionen;
static int deinterlace = 0;
static int filter = 0;
static int datei_gr = 0;
static bool direct_copy = false;
static int audio_out_channel = 0;
//Bild-Ausschnitt
static int box_x1=0, box_y1=0, box_x2=0, box_y2=0;
//Ausgabe-Gre
static int ausgabe_breite = 0, ausgabe_hoehe = 0;


//static EingabeDatei *Eingabe = NULL;
static vector<EingabeDatei*> Eingabe_Liste;
static AusgabeDatei *Ausgabe=NULL;


// Filter
static unsigned char *previous_Y=NULL;
static VideoFrame *filter_ausgabe = NULL, *prev_frame = NULL;






/* Konvertierung vr eine Eingabedatei ausfhren */
void Eingabe_verarbeiten( EingabeDatei *eingabe, AusgabeDatei *ausgabe );

/* Optionen von der Kommandozeile einlesen */
void options( int argc, char *argv[], char *envp[] );



/* Filter */
/* - deinterlace*/
void YUV420P_deinterlace( VideoFrame *frame,
                          int SpatialTolerance, int TemporalTolerance, int mode);

/* - Zeilen oben und unten ausblenden */
void YUV420P_blank_frame( VideoFrame *frame, int blank );

/* - mitteln */
void YUV420P_h_interpolate_frame( VideoFrame *frame );
void YUV420P_h_interpolate_time( VideoFrame *frame );
void YUV420P_h_interpolate_time_plane( uint8_t *plane, uint8_t *previous, int w, int h);










int main( int argc, char *argv[], char *envp[] )
{
    int breite, hoehe;
    EingabeDatei *Eingabe=NULL;

    // bersetzungen aktivieren
    setlocale( LC_ALL, "" );
    textdomain( GETTEXT_DOMAIN );
    bindtextdomain( GETTEXT_DOMAIN, LOCALEDIR );
    
    // Optionen verarbeiten
    options( argc, argv, envp );

    /***** ffne Eingabedatei ***********************************************************/
    Eingabe = Eingabe_Liste[0];

    // Bildausschnitt einstellen
    if( (box_x2 - box_x1) * (box_y2-box_y1) != 0 ) {
        Eingabe->CropBox( box_x1, box_y1, box_x2, box_y2 );
    }
    
    if( !Eingabe->open() ){
        fprintf( stderr, _("Error: Unable to open input file!\n") );
        exit( 1 );
    }
    
    // setze Farbraum
    Eingabe->ColorModel( YUV420P );

    if( Eingabe->video ) {
        // Puffer fr Filter
        Eingabe->Frame()->OutSize( breite, hoehe );

        if( breite * hoehe > 0 ) {
            //previous_Y = new unsigned char[ breite * hoehe ];
            filter_ausgabe = new VideoFrame( breite, hoehe );
        }
    }


    /***** ffne Ausgabedatei ***********************************************************/
    int ch = ( audio_out_channel > 0 ? 1 : 0 );
    if( !Ausgabe->open( Eingabe, ausgabe_breite, ausgabe_hoehe, ch ) ){
        fprintf( stderr, _("Error: Unable to open output file!\n") );
        exit( 1 );
    }


    /***** Daten verarbeiten ************************************************************/
    for( unsigned i=0; i < Eingabe_Liste.size(); i++ ){
        fprintf( stderr, _("Processing file %i of %i\n"), i+1, Eingabe_Liste.size() );

        if( i>0 ) {
            Eingabe = Eingabe_Liste[i];
            
            // Bildausschnitt einstellen
            if( (box_x2 - box_x1) * (box_y2-box_y1) != 0 ) {
                Eingabe->CropBox( box_x1, box_y1, box_x2, box_y2 );
            }

            if( !Eingabe->open() ){
                fprintf( stderr, _("Error: Unable to open input file!\n") );
                exit( 1 );
            }
            
            // setze Farbraum
            Eingabe->ColorModel( YUV420P );
        }

        Eingabe_verarbeiten( Eingabe, Ausgabe );
        Eingabe->close();
    }


    /***** beenden **********************************************************************/
    fprintf( stderr, _("close files...\n") );
    delete Ausgabe;

    while( Eingabe_Liste.size() > 0 ) {
        Eingabe = Eingabe_Liste.back();
        Eingabe_Liste.pop_back();
        delete Eingabe;
    }

    delete filter_ausgabe;

    return 0;
}












/*****************************************************************************/
/***** Eingabedatei verarbeiten                                          *****/
/*****************************************************************************/
void Eingabe_verarbeiten( EingabeDatei *eingabe, AusgabeDatei *ausgabe )
{
    long Laenge = eingabe->Frames();
    int i;
    static int n, breite=0, hoehe=0;
    unsigned char *buf=NULL;
    int16_t **audio_daten=NULL;
    int audio_puffer_gr=0;
    long long bytes = 0;
    int keyframe;
    long size;
    VideoFrame *frame;
    int channels=0, out_channel=0, ch=0;

    if( !(eingabe->video && ausgabe->video) && !(eingabe->audio && eingabe->audio) ) {
        fprintf( stderr, _("Error: No audio or video tracks found!\n") );
        return;
    }
    
    if( eingabe->video )
        eingabe->Frame()->OutSize( breite, hoehe );

    if( eingabe->audio ) {
        channels = eingabe->Channels();
        out_channel = ( audio_out_channel > channels ? 1 : audio_out_channel );
        ch = ( audio_out_channel > 0 ? 1 : 0 );
    }
    
    for( i=0; i < Laenge || Laenge < 1; i++ ) {
        // prfe auf Dateigre
        bytes = ausgabe->BytesWritten();
        if( datei_gr > 0 ) {
            if(  bytes >= datei_gr ) {
                // neue Datei anfangen
                fprintf( stderr, _("\nclose output file\n") );
                ausgabe->close();
                ausgabe->open( eingabe, ausgabe_breite, ausgabe_hoehe, ch );
                ausgabe->ColorModel( YUV420P );
                fprintf( stderr, _("new output file %s\n"), ausgabe->getName() );
                n=0;
            }
        }

        // Verarbeite Bild
        if( i % 10 == 0 ) {
            fprintf( stderr, "\r" );
            fprintf( stderr, _("Frame [%i]  average bitrate = %.1f kbps                "), i, bytes / (125 * (n?n:1) / eingabe->Rate()) );
        }

        // Video verarbeiten
        if( eingabe->video && ausgabe->video ) {
            if( direct_copy ) {
                // beginne mit letztem Keyframe
                if( n==0 ) {
                    i -= eingabe->SeekKeyFrame();
                }

                // Bild einlesen - Abbruch, wenn letztes Bild (eof)
                if( !eingabe->readVideoData( buf, size, keyframe ) )
                    break;

                // Bild schreiben
                if( !ausgabe->writeVideoData( buf, size, keyframe ) )
                    break;

            } else {
                // Bild einlesen - Abbruch, wenn letztes Bild (eof)
                if( !eingabe->decodeVideo( frame ) )
                    break;

                if( deinterlace ){
                    filter_ausgabe->ColorModel( YUV420P );
                    filter_ausgabe->SetFrame( frame );
                    frame = filter_ausgabe;
                    YUV420P_deinterlace( frame, 10, 10, 1 );  
                }

                if( filter != 0 ) {
                    if( frame != filter_ausgabe ) {
                        filter_ausgabe->ColorModel( YUV420P );
                        filter_ausgabe->SetFrame( frame );
                        frame = filter_ausgabe;
                    }
                    if( filter == 1 ) {
                         YUV420P_h_interpolate_time( frame );
                    } else if( filter == 2 ) {
                         YUV420P_h_interpolate_frame( frame );
                    }
                }
                 
                if( Blank > 0 ) {
                    if( frame != filter_ausgabe ) {
                        filter_ausgabe->ColorModel( YUV420P );
                        filter_ausgabe->SetFrame( frame );
                        frame = filter_ausgabe;
                    }
                    YUV420P_blank_frame( frame, Blank );
                }

                // Bild schreiben
                if( !ausgabe->encodeVideo( frame ) )
                    break;
            }
        }


        // Audio verarbeiten
        if( eingabe->audio && ausgabe->audio ) {
            if( !eingabe->decodeAudio( audio_daten, audio_puffer_gr ) )
                break;

            if( out_channel > 0 ) {
                if( !ausgabe->encodeAudio( &audio_daten[out_channel-1], audio_puffer_gr ) )
                    break;
            } else {
                if( !ausgabe->encodeAudio( audio_daten, audio_puffer_gr ) )
                    break;
            }
        }

        n++;
    }

    // deinterlacer zurcksetzen
    delete [] previous_Y;
    previous_Y = NULL;
    delete prev_frame;
    prev_frame = NULL;

    fprintf( stderr, "\r" );    
    fprintf( stderr, _("Frame [%i]  average bitrate = %.1f kbps                "), i-1, bytes / (125 * (n?n:1) / eingabe->Rate()) );
    fprintf( stderr, "\n" );
}











/****************************************************************************************/
/***** Optionen einlesen                                                            *****/
/****************************************************************************************/
void options(int argc, char *argv[], char *envp[])
{
    int optchar, letzte_eingabe='i';
    bool eingabe=false, ausgabe=false;
    EingabeDatei *Eingabe = NULL;

    while(( optchar = getopt(argc, argv, "i:v:a:o:V:A:b:dg:Dh:c:f:s:C:")) != -1 ) {
        switch (optchar) {
        case 'i':
            // neues Element in Liste aufnehmen
            Eingabe = new EingabeDatei;
            Eingabe->parseArg( optarg );
            Eingabe_Liste.push_back( Eingabe );
            eingabe = true;
            letzte_eingabe = optchar;
            break;
        case 'v':
            if( Eingabe == NULL || letzte_eingabe == optchar || letzte_eingabe == 'i' ) {
                Eingabe = new EingabeDatei;
                Eingabe_Liste.push_back( Eingabe );
                eingabe = true;
            }
            Eingabe->parseArg_Video( optarg );
            letzte_eingabe = optchar;
            break;
        case 'a':
            if( Eingabe == NULL || letzte_eingabe == optchar || letzte_eingabe == 'i' ) {
                Eingabe = new EingabeDatei;
                Eingabe_Liste.push_back( Eingabe );
                eingabe = true;
            }
            Eingabe->parseArg_Audio( optarg );
            letzte_eingabe = optchar;
            break;
        case 'V':
            V_Optionen.SetArg( optarg );
            break;
        case 'A':
            A_Optionen.SetArg( optarg );
            break;

        case 'o':
            if( Ausgabe == NULL ) {
                Ausgabe = new AusgabeDatei;
                ausgabe = true;
            }
            Ausgabe->parseArg( optarg );
            break;

        case 'b':
            Blank = atoi( optarg );
            break;
        case 'd':
            deinterlace = 1;
            break;
        case 'f':
            filter = atoi( optarg );
            break;
        case 'g':
            datei_gr = atoi( optarg ) * 1024 * 1024;
            break;
        case 'D':
            direct_copy = true;
            box_x1 = box_y1 = box_x2 = box_y2 = 0;
            break;

        case 'c':
            if( direct_copy )
                break;
            sscanf( optarg, "%i,%i,%i,%i", &box_x1, &box_y1, &box_x2, &box_y2 );
            DEBUG_TXT( _("crop box (%i,%i) - (%i,%i)"), box_x1, box_y1, box_x2, box_y2 );
            break;
        case 's':
            if( direct_copy )
                break;
            sscanf( optarg, "%ix%i", &ausgabe_breite, &ausgabe_hoehe );
            DEBUG_TXT( _("output size: %i x %i"), ausgabe_breite, ausgabe_hoehe );
            break;
        case 'C':
            audio_out_channel = atoi( optarg );
            break;
        
        case 'h':
            if( strcmp( optarg, "IO-Modules") == 0 ) {
                Pluginliste::Print_IO_Moduls();
                break;
            }
            if( strcmp( optarg, "A-Codecs") == 0 ) {
                Pluginliste::Print_A_Codecs();
                break;
            }
            if( strcmp( optarg, "V-Codecs") == 0 ) {
                Pluginliste::Print_V_Codecs();
                break;
            }
            
        default:
            printf( _("Syntax: %s [options] -i/v/a <[fmt::]file[starttime-endtime]> -o <fmt::file> \n"), argv[0] );
            printf( "\n" );
            printf( _("   -i <fmt::file[starttime-endtime]>   : input file\n") );
            printf( _("      >starttime< and >endtime< give the part of the input file in seconds\n") );
            printf( _("      Offset sets a time difference between the video- and\n") );
            printf( _("      audio-stream.\n") );
            printf( _("      this parameter can be repeated for multiple input files\n") );
            printf( _("      av_convert -i Avi::file_1.avi[100-200] -i Avi::file_1.avi[300-500] -i Avi::file_2.avi ...\n") );
            printf( "\n" );
            printf( _("   -v <fmt::file[starttime-endtime]>   : video input file\n") );
            printf( _("   -a <fmt::file[starttime-endtime]>   : audio input file\n") );
            printf( _("   -o <fmt::file>                      : output file\n") );
            printf( _("                                          fmt: IO-Modul\n") );
            printf( _("   -h IO-Modules                       : list IO-Modules\n") );
            printf( "\n" );
            printf( _("   -V <Codec>                : set video codec\n") );
            printf( _("   -V <option=value>         : set codec options\n") );
            printf( _("      -V DIVX -V Bitrate=800 -V MinQuantisation=3 -V ...\n") );
            printf( _("   -h V-Codecs               : list video codecs\n") );
            printf( "\n" );
            printf( _("   -A <Codec>                : set audio codec\n") );
            printf( _("   -A <option=value>         : set codec options\n") );
            printf( _("   -h A-Codecs               : liste audio codecs\n") );
            printf( "\n" );
            printf( _("Filteroptions\n") );
            printf( _("   -d                : deinterlace\n") );
            printf( _("   -f n              : filter: n=1 : temporal interpolation\n") );
            printf( _("                               n=2 : vertical interpolation\n") );
            printf( _("   -b n              : clear n lines in top and bottom\n") );
            printf( "\n" );
            printf( _("Outputoptions\n") );
            printf( _("   -g <size>         : maximum filesize for the output file in MB\n") );
            printf( _("                       %%n in the filenamen gets replaced with file number\n") );
            printf( _("   -c <x1,y1,x2,y2>  : crop box from inputframes\n") );
            printf( _("   -s <X>x<Y>        : output size\n") );
            printf( _("   -C n              : audio channel for output (mono)\n") );
            printf( _("   -D                : direct copy frame data\n") );
            exit(0);
        }
    }

    // Eingabedatei und Ausgabedatei gesetzt
    if( !eingabe || !ausgabe ) {
        fprintf( stderr, _("Help: %s -h\n"), argv[0] );
        exit(0);
    }

    // Codec fr Audio und Video-Ausgabe festlegen
    if( Ausgabe != NULL ) {
        Ausgabe->SetVCodec( &V_Optionen );
        Ausgabe->SetACodec( &A_Optionen );
    }
}








/****************************************************************************************/
/***** Zeilen ausblenden                                                            *****/
/****************************************************************************************/
void YUV420P_blank_frame( VideoFrame *frame, int blank )
{
    unsigned char *yuv[3];
    int w = 0, h = 0;

    if( frame->ColorModel() != YUV420P ) {
        DEBUG_TXT( _("Colormodel %i not supported"), frame->ColorModel() );
        return;
    }

    frame->Size( w, h );
    if( frame->FramePointer( yuv[0], yuv[1], yuv[2] ) <= 0 )
        return;
   
    // Zeilen oben ausblenden
    memset( yuv[0], 0, w*Blank );                               // y-Plane
    memset( yuv[1], 128  , w/2 * ((blank+1)/2) );               // u-Plane
    memset( yuv[2], 128, w/2 * ((blank+1)/2) );                 // v-Plane

    // Zeilen unten ausblenden
    memset( yuv[0] + w*h - w*blank, 0, w*blank );               // y-Plane
    memset( yuv[1] + w/2 * ((h + 1)/2) - w/2 * ((blank+1)/2),
            128, w/2 * (blank+1)/2 );                           // u-Plane
    memset( yuv[2] + w/2 * ((h + 1)/2) - w/2 * ((blank+1)/2),
            128, w/2 * (blank+1)/2 );                           //v-Plane
}









/*******************************************************************************/
/***** Filterfunktionen ********************************************************/
/*******************************************************************************/
void YUV420P_h_interpolate_frame( VideoFrame *frame )
{
    unsigned char *yuv[3];
    unsigned char *H1, *H2;
    //unsigned char *Z;
    unsigned char P;
    int w=0, h=0;

    if( frame->ColorModel() != YUV420P )
        return;

    frame->Size( w, h );
    if( frame->FramePointer( yuv[0], yuv[1], yuv[2] ) <= 0 )
        return;

    // y - Plane interpolieren
    for( int i=0; i < h-1; i++ ) {
        //Z = yuv[0] + i*w;
        H1 = yuv[0] + i*w;
        for( int j=0; j < w; j++ ) {
            //for( H1 = Z; H1 < Z + w; H1++ ) {
            //H1 = Z + j;
            H2 = H1 + w;

            P = (*H2 + *H1) >> 1;
            *H1 = P;

            H1++;
        }
    }

}






void YUV420P_h_interpolate_time( VideoFrame *frame )
{
    uint8_t *yuv[3], *previous[3];
    int w=0, h=0;

    if( frame->ColorModel() != YUV420P )
        return;

    frame->Size( w, h );
    if( frame->FramePointer( yuv[0], yuv[1], yuv[2] ) <= 0 )
        return;


    if( prev_frame == NULL ) {
        prev_frame = new VideoFrame( w, h );
        prev_frame->ColorModel( YUV420P );
        prev_frame->SetFrame( yuv[0], yuv[1], yuv[2] );
        //memcpy( previous_Y, yuv[0], w * h );
        return;
    }

    prev_frame->FramePointer( previous[0], previous[1], previous[2] );

    // y - Plane interpolieren
    YUV420P_h_interpolate_time_plane( yuv[0], previous[0], w, h);

    // u - Plane interpolieren
    YUV420P_h_interpolate_time_plane( yuv[1], previous[1], w>>1, (h+1)>>1 );

    // v - Plane interpolieren
    YUV420P_h_interpolate_time_plane( yuv[1], previous[2], w>>1, (h+1)>>1 );

    //prev_frame->SetFrame( yuv[0], yuv[1], yuv[2] );
}





// interpolate plane
inline void YUV420P_h_interpolate_time_plane( uint8_t *plane, uint8_t *previous, int w, int h)
{
    uint8_t *H1, *H2;
    //unsigned char *Z1, *Z2;
    //unsigned char *N1, *N2;
    //int P;
    uint8_t P;

    for( int i=0; i < h; i++ ) {
        //Z1 = yuv[0] + i*w;
        //Z2 = previous + i*w;
        H1 = plane + i*w;
        H2 = previous + i*w;
        for( int j=0; j < w; j++ ) {
            //H1 = Z1 + j;
            //H2 = Z2 + j;

            //P = ( *H2 - *H1 );
            //if( P < 10 && P > -10 ) {
            if( (*H1 ^ *H2) < 8 ) {
                //N1 = H1 + w;
                //N2 = H2 + w;

                //P = ( *H2 + *H1 + *N1 + *N2 ) >> 2;
                P = ( *H2 + *H1 ) >> 1;
                *H2 = *H1;
                *H1 = P;
            } else {
                *H2 = *H1;
            }
            H1++;
            H2++;
        }
    }
}







/****************************************************************************************/
/***** Deinterlacer                                                                 *****/
/****************************************************************************************/
void YUV420P_deinterlace(VideoFrame *frame,
        int SpatialTolerance, int TemporalTolerance, int mode)
{
    int i,j,weave;
    int cnt = 0, width = 0, height = 0;
    unsigned char *M0, *T0, *B0, *M1, *T1, *B1, *frame_buf;

    if( frame->ColorModel() != YUV420P )
        return;

    frame->Size( width, height );
    if( frame->FramePointer( frame_buf ) <= 0 )
        return;

    
    if( previous_Y == NULL ) {
        previous_Y = new unsigned char[ width * height ];
        memcpy( previous_Y, frame_buf, width * height );
    }
    
    switch (mode) {
    case 1:
        for (i=1; i<height-1; i+=2) {
            M1 = frame_buf + i*width;
            T1 = M1 - width;
            B1 = M1 + width;
            M0 = previous_Y + i*width;
            T0 = M0 - width;
            B0 = M0 + width;
            for (j=0; j<width; j++, M0++, T0++, B0++, M1++, T1++, B1++) {
                register int f;
                f = *T1 - *M1;
                f *= f;
                weave = (f < SpatialTolerance);
                if (!weave) {
                    f = *B1 - *M1;
                    f *= f;
                    weave = (f < SpatialTolerance);
                    if (!weave) {
                        f = *M0 - *M1;
                        f *= f;
                        weave = (f < TemporalTolerance);
                        f = *T0 - *T1;
                        f *= f;
                        weave = (f < TemporalTolerance) && weave;
                        f = *B0 - *B1;
                        f *= f;
                        weave = (f < TemporalTolerance) && weave;
                        if (!weave) {
                            f = *T1 + *B1;
                            *M1 = f>>1;
                            cnt++;
                        }
                    }
                }
            }
        }

        break;
    case 2:
        for (i=1; i<height-1; i+=2) {
            M1 = frame_buf + i*width;
            T1 = M1 - width;
            B1 = M1 + width;
            for (j=0; j<width; j++, M1++, T1++, B1++) {
                register int f = *T1 + *B1;
                *M1 = f>>1;
            }
        }
        break;
    }
    /* just copy the last line */
    M1 = frame_buf + (height - 1)*width;
    T1 = M1 - width;
    memcpy(M1,T1,width);

    memcpy( previous_Y, frame_buf, width*height );
}
