/*****************************************************************************\
**  Author  : Christian Tusche                                               **
**  E-Mail  : ctusche@gmx.net                        **
**                                                                           **
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/


#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/soundcard.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <assert.h>

#include "debug.h"
#include "OSS.h"


/***** Optionen *****/
#define Puffer_Anz 10               // Anzahl der Puffer
#define samples    4800             // Anzahl der Samples, die auf einmal gelesen werden








/****************************************************************************************/
/***** Konstruktor / Destruktor *********************************************************/
/****************************************************************************************/
OSS_th::OSS_th( void )
{
    dsp_file = 0;
    // bufsize = 0;
    fifo = NULL;
    Puffer = NULL;
    demux_Aus = NULL;

    strcpy( dsp_device, "/dev/dsp" );

    EQNULL( (void *)sem_init( &fifo_bereit,0,0), "sem_init" );
    EQNULL( (void *)pthread_mutex_init( &fifo_mutex, NULL ), "pthread_mutex_init" );

}






/***** Thread beenden *****/
OSS_th::~OSS_th( void )
{
    int i;

    // Thread beenden
    beende_Thread();

    // Aufrumen
    if( dsp_file > 0 ) {
        close( dsp_file );
        dsp_file = -1;
    }

    /* Puffer freigeben */
    if( Puffer != NULL ){
        for( i=0; i < Puffer_Anz; i++ )
            delete[] Puffer[i].Daten;
        delete[] Puffer;
    }

    /* demux Puffer freigeben */
    if( demux_Aus != NULL ){
        for( i=0; i < sound_channels; i++ )
            delete[] demux_Aus[i];
        delete[] demux_Aus;
    }
}










/***** Modul initialisieren und Thread starten *****/
void OSS_th::starte_Thread( void )
{
    int i;

    if( dsp_file ) return;
    if( monitor_audio ) {
        SAFE_OPEN( dsp_file, dsp_device, O_RDWR );
    } else {
        SAFE_OPEN( dsp_file, dsp_device, O_RDONLY );
    }

    Puffer_Gr = samples * sound_bits/8 * sound_channels;

    /* FIFO fr Aufnahmepuffer initialisieren */
    fifo = new int[ Puffer_Anz ];
    Puffer = new Audio_Puffer[ Puffer_Anz ];
    for( i=0; i < Puffer_Anz; i++) {
        Puffer[i].Daten = new char[ Puffer_Gr ];
        Puffer[i].belegt = 0;
    }
    for (i = 0; i < Puffer_Anz; i++)
        fifo[i] = -1;

    /* Puffer fr demultiplex Ausgabe initialisieren */
    demux_Aus = new (short int *)[ sound_channels ];
    for( i=0; i < sound_channels; i++ )
        demux_Aus[i] = new short int[ samples ];

    while( !sem_trywait(&fifo_bereit) );

    /* thread starten */
    grab_Thread::starte_Thread();

    /* warte bis thread gestartet */
    sem_wait( &fifo_bereit );
}






/***** eigentlicher Thread *****/
void OSS_th::Schleife(void)
{
    int tmp, i, frei;

    tmp = 0x00ff000b; // 256 byte fragments, number of them not limited
    NOTNEG1( ioctl( dsp_file, SNDCTL_DSP_SETFRAGMENT, &tmp), error_str( "fragment %x", tmp ) );

    /* Bits einstellen */
    tmp = AFMT_S16_LE;
    if( sound_bits == 8)
        tmp = AFMT_U8;
    NOTNEG1( ioctl( dsp_file, SNDCTL_DSP_SETFMT, &tmp), error_str("sound format %i" ,tmp) );

    /* Kanle */
    NOTNEG1( ioctl( dsp_file, SNDCTL_DSP_CHANNELS, &sound_channels), error_str("%i sound channels", sound_channels) );

    /* Samplerate */
    NOTNEG1( ioctl( dsp_file, SNDCTL_DSP_SPEED, &sound_rate), error_str("sampling rate %i", sound_rate) );

    /* signalisiere Thread gestartet */
    sem_post( &fifo_bereit );

    while( 1 ) {
        // Auf Abbruch testen
	testcancel();
	
        /* freien Puffer suchen */
        for( frei=0; frei < Puffer_Anz; frei++ )
            if( !Puffer[frei].belegt )
                break;
        /* FIFO voll! */
        if( frei == Puffer_Anz ) {
            usleep( 100000 );
            continue;
        }

        /* Daten von DSP einlesen */
        ASSERT( read( dsp_file, Puffer[frei].Daten, Puffer_Gr ) == Puffer_Gr, error_str("Fehler beim lesen von %s", dsp_device) );
        Puffer[frei].belegt = 1;
        // Audio wieder auf DSP-Device ausgeben
        if( monitor_audio ) {
            write( dsp_file, Puffer[frei].Daten, Puffer_Gr );
        }

        /* Puffer in FIFO einreihen */
        pthread_mutex_lock(&fifo_mutex);
        sem_post( &fifo_bereit );
        /* freien Platz suchen */
        for (i = 0; i < Puffer_Anz; i++)
            if( fifo[i] == -1 )
                break;
        ASSERT( i < Puffer_Anz, "Fehler in Audio-FIFO!" );

        /* Puffer in FIFO eintragen */
        fifo[i] = frei;

        pthread_mutex_unlock( &fifo_mutex );
    }
}








/***** den nchsten Puffer auslesen *****/
/* Argument:     Zeiger auf das Datenfeld fr Puffer wird in "*Daten" gespeichert
   Rckgabewert: Anzahl der gespeicherten Samples */
int OSS_th::lese_Daten( char *&Daten )
{

    /* warte auf Daten im FIFO */
    if( !sem_trywait( &fifo_bereit ) ) {

        pthread_mutex_lock( &fifo_mutex );

        assert( fifo[0] != -1 );

        Daten = Puffer[ fifo[0] ].Daten;

        pthread_mutex_unlock( &fifo_mutex );

        return samples;
    } else {
        return 0;
    }
}





/***** den gelesenen Puffer freigeben *****/
void OSS_th::release( void )
{
    int i;

    pthread_mutex_lock( &fifo_mutex );

    /* Puffer freigeben */
    Puffer[ fifo[0] ].belegt = 0;

    /* FIFO weiterschieben */
    for( i=0; i < Puffer_Anz - 1; i++ ) {
        fifo[i] = fifo[i+1];
    }
    fifo[i] = -1;

    pthread_mutex_unlock( &fifo_mutex );
}






/***** Daten im Puffer demultiplexen *****/
/* Argument:     Zeiger auf Datenfeld der Form "demux[channels][samples]"
   Rckgabewert: Anzahl der gespeicherten Samples */
int OSS_th::get( int16_t **&demux )
{
    int i, j, s;
    char *Daten;

    /* Daten einlesen */
    s = lese_Daten( Daten );
    if( s==0 )
        return 0;

    ASSERT( s==samples, "Fehler in Audio-FIFO!" );

    for( i=0; i < sound_channels; i++ ) {
        for( j=0; j < samples; j++ ) {
            switch( sound_bits ){
            case 8:
                ((char *)&demux_Aus[i][j])[0] = 0;
                ((char *)&demux_Aus[i][j])[1] = Daten[ j * sound_channels + i ];
                break;
            case 16:
                // demux_Aus[i][j] = (int16_t)Daten[ j * sound_channels * 2 + i * 2 ]
                //               |( (int16_t)Daten[ j * sound_channels * 2 + i * 2 + 1 ] << 8 );
                demux_Aus[i][j] = ((int16_t *)Daten)[ j * sound_channels + i  ];
                break;
            default:
                s = 0;
            }
        }
    }

    demux = demux_Aus;

    return s;
}
