/*****************************************************************************\
**  Author  : Christian Tusche                                               **
**  E-Mail  : ctusche@gmx.net                        **
**                                                                           **
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/


#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <assert.h>

#include "v4l.h"
#include "debug.h"
#include <VideoFile/cmodels.h>


/****************************************************************************************/
/***** Konstruktor / Destruktor *********************************************************/
/****************************************************************************************/
/* Initialisieren */
Video_th::Video_th( )
{

    // Variablen initialisieren
    log_fp = NULL;

    /***** Standard-Optionen *****/
    strcpy( log , "grab.log" );
    //strcpy( log , "" );
    fbs = 32;

    Norm( NORM_PAL );
    Source( 0 );
    Size( 384, 288 );
    cModel( RGB24 );
    Rate( 25.0 );

    // Puffer
    buf = NULL;
    buf_queue = NULL;
    EQNULL((void *)pthread_mutex_init(&buf_mutex,NULL), "pthread_mutex_init" );
    EQNULL((void *)sem_init(&newframe,0,0), "sem_init" );
    buf_fbs = 0;


    // Logdatei
    if (*log) {
        log_fp = fopen( log, "w" );
    }
}






// Thread beenden
Video_th::~Video_th()
{

    if( buf_queue != NULL ) {
        delete[] buf_queue;
    }

    if( log_fp ) {
        fclose( log_fp );
    }
}






// Info zu Videodevice ausgeben
void Video_th::info(char *bptr)
{
    return;
}





/****************************************************************************************/
/***** Puffer-Verwaltung ****************************************************************/
/****************************************************************************************/

/*** Bild aus Puffer lesen ***/
VideoFrame *Video_th::get()
{
    int x, stat;

    sem_wait(&newframe);
    pthread_mutex_lock(&buf_mutex);

    x = buf_queue[0];
    stat = buf[x].Status;

    pthread_mutex_unlock(&buf_mutex);

    assert( stat == ST_READY );

    //return buf[x].MemSeg;
    // Bild im VideoFrame aktualisieren
    buf[x].Frame->SetFrame((uint8_t*)NULL);
    return buf[x].Frame;
}





int Video_th::get_picnr()
{
    int n;

    pthread_mutex_lock(&buf_mutex);
    n = buf[buf_queue[0]].Nummer;
    pthread_mutex_unlock(&buf_mutex);

    return n;
}






void Video_th::release()
{
    int i;
    pthread_mutex_lock(&buf_mutex);

    buf[buf_queue[0]].Status = ST_EMPTY;

    for (i = 1; i < fbs; i++)
        buf_queue[i-1] = buf_queue[i];
    buf_queue[fbs-1]  = -1;

    pthread_mutex_unlock(&buf_mutex);
}





/**Puffer erstellen
  */
void Video_th::alloc_buf( uint8_t *memseg, size_t framesize, int n )
{
    int i;
    
    // Puffer anlegen
    buf = new Bild_Puffer[n];
    buf_queue = new int[n];

    for (i = 0; i < n; i++) {
        buf[i].Status = ST_EMPTY;
        buf[i].Nummer = -1;
        buf_queue[i] = -1;
    }

    // Speicherbereich zuordnen und VideoFrame erstellen
    buf[0].MemSeg = memseg;
    buf[0].Frame = new VideoFrame( width, height, buf[0].MemSeg, framesize );
    buf[0].Frame->ColorModel( format );

    for (i = 1; i < n; i++) {
        buf[i].MemSeg = buf[i-1].MemSeg + framesize;
        buf[i].Frame = new VideoFrame( width, height, buf[i].MemSeg, framesize );
        buf[i].Frame->ColorModel( format );
    }

    buf_fbs = n;
}






/**Puffer vernichten
  */
void Video_th::free_buf( void )
{
    if( buf == NULL || buf_fbs == 0 ) {
        return;
    }
    
    for( int i=0; i<buf_fbs; i++ ) {    
        delete buf[i].Frame;
    }
    
    delete [] buf;
    buf_fbs = 0;
}








/****************************************************************************************/
/***** Logdatei schreiben ***************************************************************/
/****************************************************************************************/
void Video_th::Log( const char *fmt, ... )
{
    va_list var;

    if( log_fp ) {
        va_start( var, fmt );
        vfprintf( log_fp, fmt, var );
    }
}








/****************************************************************************************/
/***** Optionen einstellen **************************************************************/
/****************************************************************************************/
void Video_th::Size( int x, int y )
{
    width = x;
    height = y;
}



void Video_th::cModel( int cm )
{
    format = cm;
}



void Video_th::Norm( int n )
{
    norm = n;
}



void Video_th::Source( int s )
{
    source = s;
}


void Video_th::Rate( float r )
{
    rate = r;
}
