#ifndef _VIDEO_TH_H_
#define _VIDEO_TH_H_

#include <stdarg.h>
#include <semaphore.h>
#include "grab_Thread.h"
#include "VideoFile/VideoFrame.h"

/* Videonormen */
#define NORM_PAL   0
#define NORM_NTSC  1
#define NORM_SECAM 2
#define NORM_AUTO  3


/* Bildpuffer */
typedef struct {
    unsigned char *MemSeg;
    int Nummer;
    int Status;
    VideoFrame *Frame;
} Bild_Puffer;


class Video_th: public grab_Thread {
public:
    Video_th();
    virtual ~Video_th();

    // Ein Bild aus Puffer lesen
    virtual VideoFrame *get();

    // Puffer freigeben
    virtual void release();

    virtual int get_picnr();

    // Info ber Videodevice
    virtual void info(char *bptr);

    // Optionen einstellen
    virtual void Size( int x, int y );
    virtual void cModel( int cm );
    virtual void Norm( int n );
    virtual void Source( int s );
    virtual void Rate( float r );

protected:
    /** Puffer erstellen
      */
    virtual void alloc_buf( uint8_t *memseg, size_t framesize, int n );

    /**Puffer vernichten
      */
    virtual void free_buf( void );
    
    // Logdatei schreiben
    void Log( const char *fmt, ... );

    /* Optionen */
    int width, height;
    int format;             // Farbformat
    float rate;             // Bildrate
    int norm;               // Videonorm
    int source;             // Datenquelle
    int fbs;                // Anzahl der maximal benutzten Framebuffer
    char log[PATH_MAX];     // Logdatei

    FILE *log_fp;

    // Puffer
    pthread_mutex_t buf_mutex;
    sem_t newframe;
    int *buf_queue;
    Bild_Puffer *buf;
    int buf_fbs;

    // Konstanten
    static const int ST_EMPTY=0;
    static const int ST_ARMED=1;
    static const int ST_READY=2;
    static const int ST_INUSE=3;
};

#endif
