/*
   bttvgrab 0.15.10 [2000-03-21]
   (c) 1998, 1999 by Joerg Walter <trouble@moes.pmnet.uni-oldenburg.de>
   Maintained by: Joerg Walter
   Current version at http://moes.pmnet.uni-oldenburg.de/bttvgrab/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <fcntl.h>
#include <errno.h>
#include <linux/types.h>
#include <pthread.h>
#include <assert.h>

#include "v4l.h"
#include "debug.h"
#include <VideoFile/cmodels.h>


#define iioctl(a,b,c) NOTNEG1(ioctl(a,b,c), #b )






/****************************************************************************************/
/*** Konstruktor / Destruktor ***********************************************************/
/****************************************************************************************/

/* Initialisieren */
v4l_th::v4l_th( void )
{

    // Variablen initialisieren
    strcpy( device, "/dev/video" );
    grab_fbttv = 0;
    picnr = 0;
    memset(&grab_channel,0,sizeof(grab_channel));
    memseg = NULL;

    /***** Standard-Optionen *****/
    Norm( NORM_PAL );
    Source( 0 );
    Size( 384, 288 );
    cModel( RGB24 );
    Rate( 25.0 );

}






// Thread beenden
v4l_th::~v4l_th()
{
    struct video_audio grab_audio;

    // Thread beenden
    beende_Thread();

    // Aufrumen

    /* Audio-Mixer einstellen - Mute einschalten */
    ioctl(grab_fbttv,VIDIOCGAUDIO,&grab_audio);
    // grab_audio.volume=32767;
    grab_audio.flags |= VIDEO_AUDIO_MUTE;
    ioctl(grab_fbttv,VIDIOCSAUDIO,&grab_audio);

    // Puffer vernichten
    if( memseg != NULL ) {
        munmap(buf[0].MemSeg, (grab_mbuf.size/grab_mbuf.frames) * fbs );
    }
    free_buf();

    if( grab_fbttv ) {
        if (grab_fbttv != -1) close(grab_fbttv);
    }
}








/****************************************************************************************/
/*** Thread fr Aufzeichnung starten ****************************************************/
/****************************************************************************************/
void v4l_th::starte_Thread()
{
    struct video_capability vcapability;
    struct video_audio grab_audio;

    if (grab_fbttv) return;

    SAFE_OPEN(grab_fbttv, device, O_RDWR);


    /* Audio-Mixer einstellen - Mute ausschalten */
    ioctl(grab_fbttv,VIDIOCGAUDIO,&grab_audio);
    // grab_audio.volume=32767;
    grab_audio.flags &= ~VIDEO_AUDIO_MUTE;
    ioctl(grab_fbttv,VIDIOCSAUDIO,&grab_audio);


    /* Videonorm und Signalquelle einstellen */
    ioctl(grab_fbttv,VIDIOCSCHAN,&grab_channel);

    /* Anzahl und Gre der Framebuffer */
    iioctl(grab_fbttv,VIDIOCGMBUF,&grab_mbuf);
    if (grab_mbuf.frames < fbs)
        fbs = grab_mbuf.frames;


    /***** Speicher fr Framebuffer anlegen *****/
    /* Gre des Bildes berprfen */
    iioctl(grab_fbttv,VIDIOCGCAP,&vcapability);
    // RANGE(width,vcapability.minwidth,vcapability.maxwidth,"width",MSG);
    // RANGE(height,vcapability.minheight,vcapability.maxheight,"height",MSG);


    /* Speichersegment fr Bild festlegen - Memory maped IO */
    memseg = (uint8_t *)mmap(0,(grab_mbuf.size/grab_mbuf.frames) * fbs, PROT_READ|PROT_WRITE,MAP_SHARED, grab_fbttv, 0);
    ASSERT( memseg != MAP_FAILED,
            error_str( "mmap %i bytes", (grab_mbuf.size/grab_mbuf.frames)*fbs) );

    alloc_buf( memseg, grab_mbuf.size/grab_mbuf.frames, fbs );

    while (!sem_trywait(&newframe));

    // start grab thread
    grab_Thread::starte_Thread();

    // warte auf erstes Bild
    sem_wait(&newframe);
}








/****************************************************************************************/
/*************** Der eigentliche grab-Thread ********************************************/
/****************************************************************************************/
void v4l_th::Schleife(void)
{
    struct video_mmap *gb;
    struct timeval grabtime, grabstarttime;
    int dieses_Bild, neues_Bild, tmp, i, freier_Puffer, bufnum;
    long tv_tol;
    int64_t usec, grab_usec;

    pthread_mutex_lock(&buf_mutex);
    gb = new struct video_mmap[fbs];

    for (i = 0; i < fbs; i++) {
        gb[i].width = width;
        gb[i].height = height;
        gb[i].format = format_v4l;
        gb[i].frame=i;
     }

    bufnum = fbs;
    picnr = 0;

    tv_tol = (long)( 1 / rate * 0.8E6 );

    // assert( bufnum >2 );

    /****************** Arbeitsschleife ************************/
    iioctl(grab_fbttv, VIDIOCMCAPTURE, &(gb[1]));
    iioctl(grab_fbttv, VIDIOCMCAPTURE, &(gb[0]));

    buf[0].Status = ST_ARMED;
    buf[1].Status = ST_ARMED;

    pthread_mutex_unlock(&buf_mutex);


    dieses_Bild = 0;
    neues_Bild = 1;

    /* Signal an bergeordneten Thread */
    sem_post(&newframe);

    while( 1 ) {
        // Auf Abbruch testen
        testcancel();

        /* warte auf Bild inm Puffer "dieses_Bild" */
        iioctl(grab_fbttv, VIDIOCSYNC, &dieses_Bild);

        pthread_mutex_lock(&buf_mutex);

        buf[dieses_Bild].Status = ST_READY;
        buf[dieses_Bild].Nummer = picnr++;


        // Zeit des Bildes
        gettimeofday(&grabtime,0);

        // Beim ersten Bild die Startzeit speichern
        if( buf[dieses_Bild].Nummer == 0 )
            grabstarttime = grabtime;

	// ist-Zeit dieses Bildes
	grab_usec = (int64_t)(  grabtime.tv_sec *1E6       + grabtime.tv_usec
	                      - grabstarttime.tv_sec * 1E6 - grabstarttime.tv_usec );
		  
        // relative soll-Zeit des Bildes
	usec = (int64_t)( buf[dieses_Bild].Nummer / rate * 1E6);

        // zustzliches Bild einfgen wenn soll-Zeit < ist-Zeit
	while( usec + tv_tol < grab_usec ) {
            Log( "%i filled\n",buf[dieses_Bild].Nummer);
            buf[dieses_Bild].Nummer++;
            picnr++;
	    
	    // soll-Zeit neu berechnen
	    usec = (int64_t)( buf[dieses_Bild].Nummer / rate * 1E6);

        }


        // Bild verwerfen wenn soll-Zeit > ist-Zeit
	if( usec > grab_usec + tv_tol ) {
            Log( "%i droped\n",buf[dieses_Bild].Nummer);

            // Puffer "dieses_Bild" verwerfen und nochmal aufzeichnen
            buf[dieses_Bild].Status = ST_EMPTY;

            tmp = dieses_Bild;
            dieses_Bild = neues_Bild;
            neues_Bild = tmp;
            // buf[dieses_Bild].Nummer--;
            picnr--;

        // Bild speichern
        } else {
	    // suche nach freiem Puffer fr nchstes Bild
            for( freier_Puffer = 0; freier_Puffer < bufnum; freier_Puffer++ ) {
                if( buf[freier_Puffer].Status == ST_EMPTY )
		    break;
	    }

            // alle Puffer voll -> Bild verwerfen und nochmal aufzeichnen
            if( bufnum > 2 && freier_Puffer == bufnum ) {
		Log( "%i lost\n",buf[dieses_Bild].Nummer );
		tmp = dieses_Bild;
		dieses_Bild = neues_Bild;
        	neues_Bild = tmp;

	    } else {
		/* Warte bis ein Puffer frei ist - nur wenn nur 2 Puffer da sind */
    		while( freier_Puffer == bufnum ) {
        	
        	    // alle Puffer voll -> warten
        	    pthread_mutex_unlock(&buf_mutex);
        	    if (freier_Puffer == bufnum) {
            		usleep( (long)(1 / (2 * rate) * 1E6) );
			testcancel();
        	    }
        	    pthread_mutex_lock(&buf_mutex);
		
		    // suche nach freiem Puffer fr nchstes Bild
        	    for (freier_Puffer = 0; freier_Puffer < bufnum; freier_Puffer++) {
            		if (buf[freier_Puffer].Status == ST_EMPTY)
			    break;
		    }
    		} 

		// Bild speichern
        	Log( "%i %li %li\n",buf[dieses_Bild].Nummer,grabtime.tv_sec,grabtime.tv_usec);

        	// Puffer in Wateschlange einreihen
        	for (i = 0; i < bufnum; i++)
            	    if (buf_queue[i] == -1) {
                	buf_queue[i] = dieses_Bild;
                	break;
            	    }

        	dieses_Bild = neues_Bild;
        	neues_Bild = freier_Puffer;
        	sem_post(&newframe);
	    }
        }

	
        // neues Bild in Puffer "neues_Bild" grabben
        buf[neues_Bild].Status = ST_ARMED;

        pthread_mutex_unlock(&buf_mutex);

        // printf("fb %i = Bild %i\n",neues_Bild, picnr);
        iioctl( grab_fbttv, VIDIOCMCAPTURE, &(gb[neues_Bild]) );

    }

}















/****************************************************************************************/
/***** Info anzeigen ********************************************************************/
/****************************************************************************************/
void v4l_th::info(char *bptr)
{
    struct video_capability vcapability;
    struct video_tuner vtuner;
    struct video_buffer vbuffer;
    struct video_channel vchan;
    struct video_audio vaudio;
    int i;
    char *device_cap[] = {
		"capture", "tuner", "teletext", "overlay", "chromakey", "clipping",
		"frameram", "scales", "monochrome", "subcapture", NULL };

	SAFE_OPEN(grab_fbttv, device,O_RDWR);

	iioctl(grab_fbttv,VIDIOCGCAP,&vcapability);

	iioctl(grab_fbttv,VIDIOCGFBUF,&vbuffer);

	sprintf(bptr,"card_name     : %s\n",vcapability.name);
	bptr += strlen(bptr);
	sprintf(bptr,"flags         :");
	bptr += strlen(bptr);
	for (i = 0; device_cap[i] != NULL; i++) {
		if (vcapability.type & (1 << i)) {
			sprintf(bptr," %s",device_cap[i]);
			bptr += strlen(bptr);
		}
	}

	sprintf(bptr,"\nchannels      : %d (",vcapability.channels);
	bptr += strlen(bptr);
	for (i = 0; i < vcapability.channels; i++) {
		vchan.channel = i;
		if (ioctl(grab_fbttv,VIDIOCGCHAN,&vchan) != -1) {
			sprintf(bptr," %s",vchan.name);
			bptr += strlen(bptr);
		}
	}
	sprintf(bptr," )\n");
	bptr += strlen(bptr);

	sprintf(bptr,"audio devices : %d (",vcapability.audios);
	bptr += strlen(bptr);
	for (i = 0; i < vcapability.audios; i++) {
		vaudio.audio = i;
		if (ioctl(grab_fbttv,VIDIOCGAUDIO,&vaudio) != -1) {
			sprintf(bptr," %s",vaudio.name);
			bptr += strlen(bptr);
		}
	}
	sprintf(bptr," )\n");
	bptr += strlen(bptr);

	sprintf(bptr,"size (max/min): %d*%d/%d*%d\n",vcapability.maxwidth,vcapability.maxheight,vcapability.minwidth,vcapability.minheight);
	bptr += strlen(bptr);

	vtuner.tuner = 0;
	if (ioctl(grab_fbttv,VIDIOCGTUNER,&vtuner) != -1) {
		sprintf(bptr,"tuner         : %s (Freq.: %lu-%lu)\n",vtuner.name,vtuner.rangelow,vtuner.rangehigh);
		bptr += strlen(bptr);
	}

	sprintf(bptr,"buffer   : base=%p size=%dx%d depth=%d bpl=%d\n",vbuffer.base,vbuffer.width,vbuffer.height,vbuffer.depth,vbuffer.bytesperline);
}








/****************************************************************************************/
/***** Optionen einstellen **************************************************************/
/****************************************************************************************/
void v4l_th::cModel( int cm )
{
    switch( cm ) {
    case RGB24:
        format_v4l = VIDEO_PALETTE_RGB24;
        break;
    case RGB8:
        format_v4l = VIDEO_PALETTE_HI240;
        break;
    case RGB16:
        format_v4l = VIDEO_PALETTE_RGB565;
        break;
    case YUV422:
        format_v4l = VIDEO_PALETTE_YUV422;
        break;
    case YUV420P:
        format_v4l = VIDEO_PALETTE_YUV420P;
        break;
    case GREY8:
        format_v4l = VIDEO_PALETTE_GREY;
        break;
    case RGB15:
        format_v4l = VIDEO_PALETTE_RGB555;
        break;
    case RGB32:
        format_v4l = VIDEO_PALETTE_RGB32;
        break;
    default:
        format_v4l = VIDEO_PALETTE_RGB24;
        cm = RGB24;
    }

    Video_th::cModel( cm );
}





void v4l_th::Norm( int n )
{
    Video_th::Norm( n );

    switch( n ) {
    case NORM_PAL:
        grab_channel.norm = VIDEO_MODE_PAL;
        break;
    case NORM_NTSC:
        grab_channel.norm = VIDEO_MODE_NTSC;
        break;
    case NORM_SECAM:
        grab_channel.norm = VIDEO_MODE_SECAM;
        break;
    case NORM_AUTO:
    default:
        grab_channel.norm = VIDEO_MODE_AUTO;
        break;
    }
}




void v4l_th::Source( int s )
{
    Video_th::Source( s );
    grab_channel.channel = source;
}
