/*
   bttvgrab 0.15.10 [2000-03-21]
   (c) 1998, 1999 by Joerg Walter <trouble@moes.pmnet.uni-oldenburg.de>
   Maintained by: Joerg Walter
   Current version at http://moes.pmnet.uni-oldenburg.de/bttvgrab/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <fcntl.h>
#include <errno.h>
#include <linux/types.h>
#include <pthread.h>
#include <assert.h>

#include "v4l.h"
#include "debug.h"
#include <VideoFile/cmodels.h>


#define iioctl(a,b,c) NOTNEG1(ioctl(a,b,c), #b )






/****************************************************************************************/
/*** Konstruktor / Destruktor ***********************************************************/
/****************************************************************************************/

/* Initialisieren */
v4l_th::v4l_th( void )
{

    // Variablen initialisieren
    grab_fbttv = 0;
    picnr = 0;
    memset(&grab_channel,0,sizeof(grab_channel));

    /***** Standard-Optionen *****/
    Norm( NORM_PAL );
    Source( 0 );
    Size( 384, 288 );
    cModel( RGB24 );
    Rate( 25.0 );

}






// Thread beenden
v4l_th::~v4l_th()
{
    struct video_audio grab_audio;

    // Thread beenden
    beende_Thread();

    // Aufrumen

    /* Audio-Mixer einstellen - Mute einschalten */
    ioctl(grab_fbttv,VIDIOCGAUDIO,&grab_audio);
    // grab_audio.volume=32767;
    grab_audio.flags |= VIDEO_AUDIO_MUTE;
    ioctl(grab_fbttv,VIDIOCSAUDIO,&grab_audio);


    if( buf != NULL ) {
        // if( buf[0].MemSeg != (void *)-1 )
            munmap(buf[0].MemSeg,(grab_mbuf.size/grab_mbuf.frames) * fbs );
        delete[] buf;
    }

    if (grab_fbttv) {
        if (grab_fbttv != -1) close(grab_fbttv);
    }
}








/****************************************************************************************/
/*** Thread fr Aufzeichnung starten ****************************************************/
/****************************************************************************************/
void v4l_th::starte_Thread()
{
    struct video_capability vcapability;
    struct video_audio grab_audio;
    int i;

    if (grab_fbttv) return;

    SAFE_OPEN(grab_fbttv, device, O_RDWR);


    /* Audio-Mixer einstellen - Mute ausschalten */
    ioctl(grab_fbttv,VIDIOCGAUDIO,&grab_audio);
    // grab_audio.volume=32767;
    grab_audio.flags &= ~VIDEO_AUDIO_MUTE;
    ioctl(grab_fbttv,VIDIOCSAUDIO,&grab_audio);


    /* Videonorm und Signalquelle einstellen */
    ioctl(grab_fbttv,VIDIOCSCHAN,&grab_channel);

    /* Anzahl und Gre der Framebuffer */
    iioctl(grab_fbttv,VIDIOCGMBUF,&grab_mbuf);
    if (grab_mbuf.frames < fbs)
        fbs = grab_mbuf.frames;


    /***** Speicher fr Framebuffer anlegen *****/
    buf = new Bild_Puffer[fbs];
    buf_queue = new int[fbs];

    for (i = 0; i < fbs; i++) {
        buf[i].Status = ST_EMPTY;
        buf[i].Nummer = -1;
        buf_queue[i] = -1;
    }

    /* Gre des Bildes berprfen */
    iioctl(grab_fbttv,VIDIOCGCAP,&vcapability);
    // RANGE(width,vcapability.minwidth,vcapability.maxwidth,"width",MSG);
    // RANGE(height,vcapability.minheight,vcapability.maxheight,"height",MSG);


    /* Speichersegment fr Bild festlegen - Memory maped IO */
    buf[0].MemSeg = (unsigned char *)
           mmap(0,(grab_mbuf.size/grab_mbuf.frames) * fbs, PROT_READ|PROT_WRITE,MAP_SHARED, grab_fbttv, 0);

    ASSERT( buf[0].MemSeg != MAP_FAILED,
            error_str( "mmap %i bytes", (grab_mbuf.size/grab_mbuf.frames)*fbs) );

    for (i = 1; i < fbs; i++) {
        buf[i].MemSeg = buf[i-1].MemSeg + ( grab_mbuf.size/grab_mbuf.frames );
    }

    while (!sem_trywait(&newframe));

    // start grab thread
    grab_Thread::starte_Thread();

    // warte auf erstes Bild
    sem_wait(&newframe);
}








/****************************************************************************************/
/*************** Der eigentliche grab-Thread ********************************************/
/****************************************************************************************/
void v4l_th::Schleife(void)
{
    struct video_mmap *gb;
    struct timeval grabtime, time, grabstarttime;
    int dieses_Bild, neues_Bild, tmp, i, freier_Puffer, bufnum;
    long sec, usec, tv_tol;

    pthread_mutex_lock(&buf_mutex);
    gb = new struct video_mmap[fbs];

    for (i = 0; i < fbs; i++) {
        gb[i].width = width;
        gb[i].height = height;
        gb[i].format = format;
        gb[i].frame=i;
     }

    bufnum = fbs;
    picnr = 0;

    tv_tol = (long)( 1 / rate * 1E6 );

    // assert( bufnum >2 );

    /****************** Arbeitsschleife ************************/
    iioctl(grab_fbttv, VIDIOCMCAPTURE, &(gb[1]));
    iioctl(grab_fbttv, VIDIOCMCAPTURE, &(gb[0]));

    buf[0].Status = ST_ARMED;
    buf[1].Status = ST_ARMED;

    pthread_mutex_unlock(&buf_mutex);


    dieses_Bild = 0;
    neues_Bild = 1;

    /* Signal an bergeordneten Thread */
    sem_post(&newframe);

    while( 1 ) {
        // Auf Abbruch testen
        testcancel();

        /* warte auf Bild inm Puffer "dieses_Bild" */
        iioctl(grab_fbttv, VIDIOCSYNC, &dieses_Bild);

        pthread_mutex_lock(&buf_mutex);

        buf[dieses_Bild].Status = ST_READY;
        buf[dieses_Bild].Nummer = picnr++;

        /* suche nach freiem Puffer fr nchstes Bild */
        for (freier_Puffer = 0; freier_Puffer < bufnum; freier_Puffer++)
            if (buf[freier_Puffer].Status == ST_EMPTY) break;


        /* alle Puffer voll -> letzten Puffer berschreiben */
        if (freier_Puffer == bufnum) {
            // dieses_Bild = neues_Bild;
            tmp = dieses_Bild;
            dieses_Bild = neues_Bild;
            neues_Bild = tmp;

            Log( "%i lost\n",buf[neues_Bild].Nummer );



        } else {

            // Zeit des Bildes
            gettimeofday(&grabtime,0);

            // Beim ersten Bild die Startzeit speichern
            if( buf[dieses_Bild].Nummer == 0 )
                grabstarttime = grabtime;

            // relative soll-Zeit des Bildes
            sec  = (int)(buf[dieses_Bild].Nummer / rate);
            usec = (int)( (buf[dieses_Bild].Nummer - sec * rate) / rate * 1E6);



            // absolute Soll-Zeit des Bildes + zwei Bilder toleranz
            time.tv_sec  = grabstarttime.tv_sec  + sec;
            time.tv_usec = grabstarttime.tv_usec + usec + tv_tol;
            while( time.tv_usec >= 1E6 ) {
                time.tv_sec++;
                time.tv_usec -= (long)1E6;
            }

            // zustzliches Bild einfgen wenn soll-Zeit < ist-Zeit
            if( time.tv_sec <= grabtime.tv_sec && time.tv_usec < grabtime.tv_usec ) {
                Log( "%i filled\n",buf[dieses_Bild].Nummer);
                buf[dieses_Bild].Nummer++;
                picnr++;
            }



            // Soll-Zeit des Bildes - zwei Bilder toleranz ( ist-Zeit + 4 Bilder )
            grabtime.tv_usec += 2 * tv_tol;
            while( time.tv_usec >= 1E6 ) {
                grabtime.tv_sec++;
                grabtime.tv_usec -= (long)1E6;
            }

            // Bild verwerfen wenn soll-Zeit > ist-Zeit
            if( time.tv_sec >= grabtime.tv_sec && time.tv_usec > grabtime.tv_usec ) {
                Log( "%i droped\n",buf[dieses_Bild].Nummer);

                // Puffer "dieses_Bild" nochmal aufzeichnen
                tmp = dieses_Bild;
                dieses_Bild = neues_Bild;
                neues_Bild = tmp;
                // buf[dieses_Bild].Nummer--;
                picnr--;

            // Bild speichern
            } else {
                Log( "%i %li %li\n",buf[dieses_Bild].Nummer,grabtime.tv_sec,grabtime.tv_usec);
                // printf("(%i)\n",buf[dieses_Bild].Nummer );


                for (i = 0; i < bufnum; i++)
                    if (buf_queue[i] == -1) {
                        buf_queue[i] = dieses_Bild;
                        break;
                    }


                dieses_Bild = neues_Bild;
                neues_Bild = freier_Puffer;
                // printf( "neues_Bild = %i\n", neues_Bild );
                sem_post(&newframe);
            }

        }

        // neues Bild in Puffer "neues_Bild" grabben
        buf[neues_Bild].Status = ST_ARMED;

        pthread_mutex_unlock(&buf_mutex);

        printf("fb %i = Bild %i\n",neues_Bild, picnr);
        iioctl( grab_fbttv, VIDIOCMCAPTURE, &(gb[neues_Bild]) );

    }

}















/****************************************************************************************/
/***** Info anzeigen ********************************************************************/
/****************************************************************************************/
void v4l_th::info(char *bptr)
{
    struct video_capability vcapability;
    struct video_tuner vtuner;
    struct video_buffer vbuffer;
    struct video_channel vchan;
    struct video_audio vaudio;
    int i;
    char *device_cap[] = {
		"capture", "tuner", "teletext", "overlay", "chromakey", "clipping",
		"frameram", "scales", "monochrome", "subcapture", NULL };

	SAFE_OPEN(grab_fbttv, device,O_RDWR);

	iioctl(grab_fbttv,VIDIOCGCAP,&vcapability);

	iioctl(grab_fbttv,VIDIOCGFBUF,&vbuffer);

	sprintf(bptr,"card_name     : %s\n",vcapability.name);
	bptr += strlen(bptr);
	sprintf(bptr,"flags         :");
	bptr += strlen(bptr);
	for (i = 0; device_cap[i] != NULL; i++) {
		if (vcapability.type & (1 << i)) {
			sprintf(bptr," %s",device_cap[i]);
			bptr += strlen(bptr);
		}
	}

	sprintf(bptr,"\nchannels      : %d (",vcapability.channels);
	bptr += strlen(bptr);
	for (i = 0; i < vcapability.channels; i++) {
		vchan.channel = i;
		if (ioctl(grab_fbttv,VIDIOCGCHAN,&vchan) != -1) {
			sprintf(bptr," %s",vchan.name);
			bptr += strlen(bptr);
		}
	}
	sprintf(bptr," )\n");
	bptr += strlen(bptr);

	sprintf(bptr,"audio devices : %d (",vcapability.audios);
	bptr += strlen(bptr);
	for (i = 0; i < vcapability.audios; i++) {
		vaudio.audio = i;
		if (ioctl(grab_fbttv,VIDIOCGAUDIO,&vaudio) != -1) {
			sprintf(bptr," %s",vaudio.name);
			bptr += strlen(bptr);
		}
	}
	sprintf(bptr," )\n");
	bptr += strlen(bptr);

	sprintf(bptr,"size (max/min): %d*%d/%d*%d\n",vcapability.maxwidth,vcapability.maxheight,vcapability.minwidth,vcapability.minheight);
	bptr += strlen(bptr);

	vtuner.tuner = 0;
	if (ioctl(grab_fbttv,VIDIOCGTUNER,&vtuner) != -1) {
		sprintf(bptr,"tuner         : %s (Freq.: %lu-%lu)\n",vtuner.name,vtuner.rangelow,vtuner.rangehigh);
		bptr += strlen(bptr);
	}

	sprintf(bptr,"buffer   : base=%p size=%dx%d depth=%d bpl=%d\n",vbuffer.base,vbuffer.width,vbuffer.height,vbuffer.depth,vbuffer.bytesperline);
}








/****************************************************************************************/
/***** Optionen einstellen **************************************************************/
/****************************************************************************************/
void v4l_th::cModel( int cm )
{
    Video_th::cModel( cm );

    switch( cm ) {
    case RGB24:
        format = VIDEO_PALETTE_RGB24;
        break;
    case PAL8:
        format = VIDEO_PALETTE_HI240;
        break;
    case RGB16:
        format = VIDEO_PALETTE_RGB565;
        break;
    case YUV422:
        format = VIDEO_PALETTE_YUV422;
        break;
    case YUV420P:
        format = VIDEO_PALETTE_YUV420P;
        break;
    case GREY8:
        format = VIDEO_PALETTE_GREY;
        break;
    case RGB15:
        format = VIDEO_PALETTE_RGB555;
        break;
    case RGB32:
        format = VIDEO_PALETTE_RGB32;
        break;
    default:
        format = VIDEO_PALETTE_RGB24;
    }
}





void v4l_th::Norm( int n )
{
    Video_th::Norm( n );

    switch( n ) {
    case NORM_PAL:
        grab_channel.norm = VIDEO_MODE_PAL;
        break;
    case NORM_NTSC:
        grab_channel.norm = VIDEO_MODE_NTSC;
        break;
    case NORM_SECAM:
        grab_channel.norm = VIDEO_MODE_SECAM;
        break;
    case NORM_AUTO:
    default:
        grab_channel.norm = VIDEO_MODE_AUTO;
        break;
    }
}




void v4l_th::Source( int s )
{
    Video_th::Source( s );
    grab_channel.channel = source;
}
