#include <stdio.h>
#include <errno.h>
#include <stdarg.h>
#include <stdlib.h>

// #define _DEBUG_MSG 1
// #define _TIMERS 1


#undef ASSERT
#define ASSERT(a,b)     if (!(a)) { fprintf(stderr, "%s [%i]: %s\n", __FILE__, __LINE__, b ); exit(1);}


#define EQNULL(a,b)                     ASSERT((a) == NULL,b)
#define NOTNEG1(a,b)                    ASSERT((a) != -1,b)


#define SAFE_OPEN(var,name,mode)        NOTNEG1( (var) = open( (name),(mode) ), \
                                        error_str( "Fehler beim ffnen von %s\n", name ) )



// Messung der bentigten Zeit fr Funktionsaufrufe
#ifdef _TIMERS
#include <sys/time.h>

#define TIMER_START( timerid )          struct timeval __START_TIMER_##timerid, __STOP_TIMER_##timerid;\
                                        gettimeofday( &__START_TIMER_##timerid, 0 )

#define TIMER_STOP( timerid, a... )     { gettimeofday( &__STOP_TIMER_##timerid, 0 );\
                                          long long usecs = __STOP_TIMER_##timerid.tv_sec * 1000000 + __STOP_TIMER_##timerid.tv_usec;\
                                          usecs -= __START_TIMER_##timerid.tv_sec * 1000000 + __START_TIMER_##timerid.tv_usec;\
                                          fprintf( stderr, "%s [%i]: Timer(%s) = %.2f msec, %s\n", __FILE__, __LINE__, #timerid, (float)usecs/1000.0, error_str( a ) ); }
#else
#define TIMER_START( timerid )
#define TIMER_STOP( timerid, a... )
#endif



#define PRINT_DEBUG_TXT(a...)   fprintf( stderr, "%s [%i]: %s\n", __FILE__, __LINE__, error_str( a ) )

#define ERROR_TXT(a...)         PRINT_DEBUG_TXT(a)

#ifdef _DEBUG_MSG
#if _DEBUG_MSG >= 1
#define INFO_TXT(a...)          PRINT_DEBUG_TXT(a)
#endif

#if _DEBUG_MSG >= 2
#define DEBUG_TXT(a...)         PRINT_DEBUG_TXT(a)
#define DEBUG_PROC(a)		a
#endif

#if _DEBUG_MSG >= 3
#define DEBUG_TXT_3(a...)       PRINT_DEBUG_TXT(a)
#define DEBUG_PROC_3(a)		a
#endif

#endif

#ifndef INFO_TXT
#define INFO_TXT(a...)
#endif

#ifndef DEBUG_TXT
#define DEBUG_TXT(a...)
#define DEBUG_PROC(a)
#endif

#ifndef DEBUG_TXT_3
#define DEBUG_TXT_3(a...)
#define DEBUG_PROC_3(a)
#endif



/* ISO C Standard
#ifdef DEBUG
#define DEBUG_TXT(a...)    fprintf( stderr, "%s [%i]: %s\n", __FILE__, __LINE__, error_str( __VA_ARGS__ ) )
#else
#define DEBUG_TXT(a...)
#endif
*/



static char error_Text[ 4000 ];

static inline char *error_str( const char *fmt, ... )
{
    va_list var;
    int i;

    va_start( var, fmt );
    i = vsprintf( error_Text, fmt, var );

    return error_Text;
}
