/*****************************************************************************\
**  Author  : Christian Tusche                                               **
**  E-Mail  : ctusche@gmx.net                        **
**                                                                           **
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/

#include <string.h>

#include "RIFF_Chunk.h"




RIFF_Chunk::RIFF_Chunk()
{
    buf = NULL;
}


RIFF_Chunk::~RIFF_Chunk()
{
    delete[] buf;
}






/**Einlesen der Datenstruktur aus einer Datei
  */
bool RIFF_Chunk::ReadStream( RIFF_File *file, size_t &length )
{
    size_t bytes = 0;
    size_t gelesen = 0;
    uint8_t *tmp_buf;
    
    do {
        // Beginne mit midestgre der Datenstruktur
        // Nach dem ersten einlesen kann sich der Wert von Size()
        // ndern, wenn optionale Daten in der Struktur enthalten sind.
        // Mit der neuen Gre werden in einem zweiten Durchlauf die
        // restlichen Daten geladen und der Chunk vollstndig gelesen.
        bytes = Size();

        // es knnen auch weniger als 'Size()' Bytes gelesen werden
        // das ist ok, wenn 'Read()' true liefert
        if( length < bytes )
            bytes = length;
        //    return false;

        tmp_buf = new uint8_t[bytes];

        // Datem aus vorhergehendem Durchlauf bernehmen
        if( gelesen > 0 && buf != NULL )
            memcpy( tmp_buf, buf, gelesen );

        if( buf != NULL )
            delete[] buf;
                
        buf = tmp_buf;    

        // Zustzliche Daten in Puffer laden       
        file->read( buf + gelesen, bytes - gelesen );
        gelesen = bytes;

        // Gesamten Puffer in Datenstruktur einlesen
        if( !Read( buf, bytes ) )
            return false;

    } while( gelesen < Size() && gelesen < length );

    length -= gelesen;
    return true;
}


    
/**Schreiben der Datenstruktur in einer Datei
  */
bool RIFF_Chunk::WriteStream( RIFF_File *file, size_t &length )
{
    size_t bytes = Size();
    
    if( length < bytes )
        return false;

    delete[] buf;
    buf = new uint8_t[bytes];

    Write( buf, bytes );
    if( !file->write( buf, Size() ) )
        return false;
        
    length -= Size();

    return true;
}

