/********************************************************************\
*****   lesen und schreiben in RIFF Dateien                      *****
*****   architekturunabhngige Methoden, um int/uint - Werte     *****
*****   in RIFF-Dateien zu schreiben bzw. zu lesen               *****
\********************************************************************/

#ifndef _RIFF_FILE_H_
#define _RIFF_FILE_H_


#include <unistd.h>
#include <inttypes.h>

#include "RIFF_ITypes.h"


class RIFF_File: public RIFF_ITypes {
public:
    RIFF_File( const char *datei, bool io_read );
    ~RIFF_File();

    /**uint64_t lesen
      */
    bool read( uint64_t &x );
    
    /**uint32_t lesen
      */
    bool read( uint32_t &x );

    /**uint16_t lesen
      */
    bool read( uint16_t &x );

    /**uint8_t lesen
      */
    bool read( uint8_t &x );


    /**uint64_t schreiben
      */
    bool write( uint64_t x );
    
    /**uint32_t schreiben
      */
    bool write( uint32_t x );

    /**uint16_t schreiben
      */
    bool write( uint16_t x );

    /**uint8_t schreiben
      */
    bool write( uint8_t x );


    /**Chunk-Header lesen - liest <br>
      *   uint32_t Typ<br>
      *      und<br>
      *   uint32_t Length
      */
    bool read_chunk_header( uint32_t &type, uint32_t &length );


    /**n Bytes lesen
      */
    bool read( uint8_t *buf, size_t n );

    /**n Bytes schreiben
      */
    bool write( uint8_t *buf, size_t n );
    
    
    /**Bytes bis zum Dateiende<br>
      *nur bei lesendem Zugriff mglich
      */
    off64_t bytes_left(void);


    /**Bytes berspringen
      */
    bool skip( off64_t bytes );

    /**Datei positionieren
      */
    bool seek( off64_t offset, int mode = SEEK_SET );

    /**Datei-Position
      */
    off64_t pos( void );

private:
    bool rd;

    uint64_t total_bytes;

    int file;
};

#endif
