/********************************************************************\
*****   Methoden fr IO in RIFF Dateien                          *****
*****   Typenumwandlungen                                        *****
\********************************************************************/

#ifndef _RIFF_ITYPES_H_
#define _RIFF_ITYPES_H_

#include <inttypes.h>
#include <stdlib.h>

class RIFF_ITypes {
public:
    //RIFF_ITypes() {};
    //~RIFF_ITypes() {};

    static inline bool read_int( uint64_t &x, uint8_t *&Z, size_t &length ) {
        if( length >= 8 ) {
            x = (uint64_t)Z[7] << 56 | (uint64_t)Z[6] << 48 | (uint64_t)Z[5] << 40 | (uint64_t)Z[4] << 32
              | (uint64_t)Z[3] << 24 | (uint64_t)Z[2] << 16 | (uint64_t)Z[1] << 8  | (uint64_t)Z[0];
            Z += 8;
            length -= 8;
            return true;
        }

        return false;
    }


    static inline bool read_int( uint64_t &x, uint8_t Z[8] ) {
        x = (uint64_t)Z[7] << 56 | (uint64_t)Z[6] << 48 | (uint64_t)Z[5] << 40 | (uint64_t)Z[4] << 32
          | (uint64_t)Z[3] << 24 | (uint64_t)Z[2] << 16 | (uint64_t)Z[1] << 8  | (uint64_t)Z[0];

        return true;
    }

    

    static inline bool read_int( uint32_t &x, uint8_t *&Z, size_t &length ) {
        if( length >= 4 ) {
            x = (uint32_t)Z[3] << 24 | (uint32_t)Z[2] << 16 | (uint32_t)Z[1] << 8 | (uint32_t)Z[0];
            Z += 4;
            length -= 4;
            return true;
        }

        return false;
    }


    static inline bool read_int( uint32_t &x, uint8_t Z[4] ) {
        x = (uint32_t)Z[3] << 24 | (uint32_t)Z[2] << 16 | (uint32_t)Z[1] << 8 | (uint32_t)Z[0];
        return true;
    }

    
    static inline bool read_int( uint16_t &x, uint8_t *&Z, size_t &length ) {
        if( length >= 2 ) {
            x = (uint32_t)Z[1] << 8 | (uint32_t)Z[0];
            Z += 2;
            length -= 2;
            return true;
        }

        return false;
    }


    static inline bool read_int( uint16_t &x, uint8_t Z[2] ) {
        x = (uint32_t)Z[1] << 8 | (uint32_t)Z[0];
        return true;
    }


    static inline bool read_int( uint8_t &x, uint8_t *&Z, size_t &length ) {
        if( length >= 2 ) {
            x = Z[0];
            Z += 1;
            length -= 1;
            return true;
        }

        return false;
    }


    static inline bool read_int( uint8_t &x, uint8_t Z[1] ) {
        x = Z[0];
        return true;
    }





    static inline bool write_int( uint64_t x, uint8_t *&Z, size_t &length ) {
        if( length >= 8 ) {
            *(Z++) = (uint8_t)(x & 0xff);
            *(Z++) = (uint8_t)(x >> 8  & 0xff);
            *(Z++) = (uint8_t)(x >> 16 & 0xff);
            *(Z++) = (uint8_t)(x >> 24 & 0xff);
            *(Z++) = (uint8_t)(x >> 32 & 0xff);
            *(Z++) = (uint8_t)(x >> 40 & 0xff);
            *(Z++) = (uint8_t)(x >> 48 & 0xff);
            *(Z++) = (uint8_t)(x >> 56 & 0xff);
            length -= 8;
            return true;
        }

        return false;
    }



    static inline bool write_int( uint64_t x, uint8_t Z[8] ) {
        *(Z++) = (uint8_t)(x & 0xff);
        *(Z++) = (uint8_t)(x >> 8  & 0xff);
        *(Z++) = (uint8_t)(x >> 16 & 0xff);
        *(Z++) = (uint8_t)(x >> 24 & 0xff);
        *(Z++) = (uint8_t)(x >> 32 & 0xff);
        *(Z++) = (uint8_t)(x >> 40 & 0xff);
        *(Z++) = (uint8_t)(x >> 48 & 0xff);
        *(Z)   = (uint8_t)(x >> 56 & 0xff);

        return true;
    }



    
    static inline bool write_int( uint32_t x, uint8_t *&Z, size_t &length ) {
        if( length >= 4 ) {
            *(Z++) = (uint8_t)(x & 0xff);
            *(Z++) = (uint8_t)(x >> 8  & 0xff);
            *(Z++) = (uint8_t)(x >> 16 & 0xff);
            *(Z++) = (uint8_t)(x >> 24 & 0xff);
            length -= 4;
            return true;
        }

        return false;
    }


        

    static inline bool write_int( uint32_t x, uint8_t Z[4] ) {
        *(Z++) = (uint8_t)(x & 0xff);
        *(Z++) = (uint8_t)(x >> 8  & 0xff);
        *(Z++) = (uint8_t)(x >> 16 & 0xff);
        *(Z)   = (uint8_t)(x >> 24 & 0xff);

        return true;
    }



    static inline bool write_int( uint16_t x, uint8_t *&Z, size_t &length  ) {
        if( length >= 2 ) {
            *(Z++) = (uint8_t)(x & 0xff);
            *(Z++)   = (uint8_t)(x >> 8  & 0xff);
            length -= 2;
            return true;
        }

        return false;
    }


    static inline bool write_int( uint16_t x, uint8_t Z[2] ) {
        *(Z++) = (uint8_t)(x & 0xff);
        *(Z)   = (uint8_t)(x >> 8  & 0xff);
           
        return true;
    }


    static inline bool write_int( uint8_t x, uint8_t *&Z, size_t &length  ) {
        if( length >= 1 ) {
            *(Z++) = x;
            length -= 1;
            return true;
        }

        return false;
    }


    static inline bool write_int( uint8_t x, uint8_t Z[1] ) {
        *(Z)   = x;

        return true;
    }



    
    static inline int stream_num( uint32_t x ) {
        char z[3];
        z[0]=(char)x; z[1]=(char)(x>>8); z[2]=0;
        return strtol(z, NULL, 16);
    }
};

#endif
