/*****************************************************************************\
**  Author  : Christian Tusche                                               **
**  E-Mail  : ctusche@gmx.net                        **
**                                                                           **
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/


/********************************************************************\
*****   Methoden fr IO in Avi Dateien                           *****
*****   Datentyp AVI-Header                                      *****
\********************************************************************/


#include <string.h>

#include "Wave_Format.h"
#include "debug.h"



WaveFormat::WaveFormat()
{
    // Datenstruktur
    Format = 0;
    Channels = 2;
    SamplesPerSec = 44100;
    AvgBytesPerSec = 0;
    BlockAlign = 1;
    BitsPerSample = 16;
    DataSize = 0;
    Data = NULL;

}



WaveFormat::~WaveFormat()
{
    delete [] Data;
}



    

/**Einlesen der Datenstruktur aus einem Puffer
  */
bool WaveFormat::Read( uint8_t *buf, size_t length )
{
    if( length < 16 )
        return false;

    read_int( Format, buf, length );
    read_int( Channels, buf, length );
    read_int( SamplesPerSec, buf, length );
    read_int( AvgBytesPerSec, buf, length );
    read_int( BlockAlign, buf, length );
    read_int( BitsPerSample, buf, length );

    if( length < 2 )
        return true;
        
    read_int( DataSize, buf, length );

    if( DataSize > 0 && length >= DataSize) {
        delete [] Data;
        Data = new uint8_t[DataSize];

        memcpy( Data, buf, DataSize );
    }

    return true;
}





/**Schreiben der Datenstruktur in einen Puffer
  */
bool WaveFormat::Write( uint8_t *buf, size_t length )
{
    if( length < Size() )
        return false;

    write_int( Format, buf, length );
    write_int( Channels, buf, length );
    write_int( SamplesPerSec, buf, length );
    write_int( AvgBytesPerSec, buf, length );
    write_int( BlockAlign, buf, length );
    write_int( BitsPerSample, buf, length );
    write_int( DataSize, buf, length );

    if( DataSize > 0 && Data != NULL ) {
        memcpy( buf, Data, DataSize );
    }

    return true;
}





/**Werte Ausgeben
  */
void WaveFormat::printInfo( void )
{
    printf( "    Format = 0x%04x\n", Format );
    printf( "    Channels = %i\n", Channels );
    printf( "    SamplesPerSec = %i\n", SamplesPerSec );
    printf( "    AvgBytesPerSec = %i\n", AvgBytesPerSec );
    printf( "    BlockAlign = %i\n", BlockAlign );
    printf( "    BitsPerSample = %i\n", BitsPerSample );
    printf( "    DataSize = %i\n", DataSize );
}
