/*****************************************************************************\
**  Author  : Christian Tusche                                               **
**  E-Mail  : ctusche@gmx.net                        **
**                                                                           **
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/

#include <stdio.h>


#include "PcmCodec.h"
#include "debug.h"
#include "VideoFile/translate.h"





/***** Konstruktor / Destruktor *****/
PcmCodec::PcmCodec( A_Track *Track )
:A_Codec( Track )
{
    // Variablen initialisieren
    puffer = NULL;
    puffer_gr = 0;

    SetFrameSamples( 1 );
    AddOption( "Bits", _("8 or 16 bits per sample") ) = 16;
}




PcmCodec::~PcmCodec()
{
    delete [] puffer;
}






// Audiodaten schreiben
int PcmCodec::encode( float **daten, int samples )
{
    int i,j;
    long bytes = (Channels() * Bits() * samples) / 8;
    int16_t sample;

    // Puffer anfordern
    if( bytes > puffer_gr ) {
        delete [] puffer;
        puffer = new unsigned char[bytes];
        puffer_gr = bytes;
    }

    switch( Bits() ) {
    case 16:
        // Kanle in einen Datenstrom multiplexen
        if( file->ModulName() == "Quicktime" ) {
        
            for( i=0; i<samples; i++ ) {
                for( j=0; j<Channels(); j++ ) {
                    sample = (int16_t)( daten[j][i] * (1<<15) );
                    puffer[2*(i*Channels()+j)+1] = (unsigned char)(sample & 0x00FF);
                    puffer[2*(i*Channels()+j)]   = (unsigned char)(sample >> 8);
                }
            }
        
        } else {
        
            for( i=0; i<samples; i++ ) {
                for( j=0; j<Channels(); j++ ) {
                    sample = (int16_t)( daten[j][i] * (1<<15) );
                    puffer[2*(i*Channels()+j)]   = (unsigned char)(sample & 0x00FF);
                    puffer[2*(i*Channels()+j)+1] = (unsigned char)(sample >> 8);
                }
            }
            
        }
        break;
    default:
        ERROR_TXT( _("encode audio not implemented!") );
        return 0;
    }

    // Puffer in Datei schreiben
    return track->writeData( puffer, bytes, samples );
}






// Audiodaten lesen
int PcmCodec::decode( float **daten, int samples )
{
    int i,j;
    long bytes = (Channels() * Bits() * samples) / 8;
    int16_t sample;

    // Puffer anfordern
    if( bytes > puffer_gr ) {
        delete [] puffer;
        puffer = new unsigned char[bytes];
        puffer_gr = bytes;
    }

    // Puffer aus Datei lesen
    if( !track->readBytes( puffer, bytes ) )
        return 0;

    switch( Bits() ) {
    case 16:
        // Kanle in einen Datenstrom multiplexen
        
        if( file->ModulName() == "Quicktime" ) {
        
            for( i=0; i<samples; i++ ) {
                for( j=0; j<Channels(); j++ ) {
                    sample = ((int16_t)puffer[2*(i*Channels()+j)] << 8) | (int16_t)puffer[2*(i*Channels()+j)+1];
                    daten[j][i] = (float)sample / (float)(1<<15);
                }
            }
                
        } else {
        
            for( i=0; i<samples; i++ ) {
                for( j=0; j<Channels(); j++ ) {
                    sample = ((int16_t)puffer[2*(i*Channels()+j)+1] << 8) | (int16_t)puffer[2*(i*Channels()+j)];
                    daten[j][i] = (float)sample / (float)(1<<15);
                }
            }
            
        }
        break;
    default:
        ERROR_TXT( _("decode audio not implemented!") );
        return 0;
    }

    return 1;
}









// Optionen
int PcmCodec::SetOption( OptionEntry *opt )
{
    DEBUG_TXT_3( "setze Option %s = %s", (const char*)opt->Name(), (const char*)*opt );

    if( opt->Name() == "Bits" ) {
        if( (int)*opt != 16 )
            *opt = 16;
        SetBits( (int)*opt );
        SetBitsPerSecond( Bits() * Channels() * SampleRate() );
    } else {
        return 0;
    }

    return 1;
}

