/********************************************************************\
*****   Methoden fr IO in AVI Dateien                           *****
\********************************************************************/

#ifndef _AVIFILE_H_
#define _AVIFILE_H_

#include "avifile/AviWriteFile.h"
#include "avifile/AviWriteStream.h"
#include "avifile/AviRead.h"
#include "VideoFile/VideoFile.h"

//#define GET_INDEX 1

class AviFileVtr;
class AviFileAtr;







/****************************************************************************************/
/***** Quicktime - Klasse ***************************************************************/
/****************************************************************************************/

class AviFileIO:public VideoFile {
    friend class AviFileV;
    friend class AviFileVtr;
    friend class AviFileA;
    friend class AviFileAtr;
public:
    AviFileIO( const String Dateiname, IO_Mode mode=write );
    ~AviFileIO();

protected:
    /*** Videofunktionen ***/
    // Video codec initialisieren
    //    Codec: Name des internen Videocodecs des Backends
    V_Track *openVideoTrack( double Rate,
                             int x, int y,
                             const char *Codec = NULL );


    /*** Audiofunktionen ***/
    // Audio codec initialisieren
    A_Track *openAudioTrack( int Sample_rate,
                             int Channels,
                             const char *Codec = NULL );
                             
    // Video-Track ffnen
    V_Track *openVideoTrack( int track );
    // Audio-Track ffnen
    A_Track *openAudioTrack( int track=0 );

    AviWriteFile *wDatei;
    AviReadFile *rDatei;
};









/****************************************************************************************/
/***** AVI - Videotrack *****************************************************************/
/****************************************************************************************/
class AviFileVtr:public V_Track {
public:
    AviFileVtr( AviFileIO *File, int track );

    AviFileVtr( AviFileIO *File,
            double rate,
            int x, int y,
            const char *Codec = NULL );

    ~AviFileVtr();


    // Bild-Position setzen
    int Seek( long block );

    long GetKeyFrame( long bild );
    bool IsKeyFrame( void );

private:
    AviFileIO *file;            // bergeordnete Klasse
    AviWriteFile *wDatei;       // Avi Datei (schreiben)
    AviReadFile *rDatei;        // lesen
    AviWriteStream *wStream;
    AviReadStream *rStream;
    char* FrameBuf;             // Speicher fr die Daten eines Bildes
    unsigned long BufGr;        // Gre von FrameBuf
    bool keyframe_ok;

protected:
    // Rohdaten lesen und schreiben
    virtual int readRaw( unsigned char *&data, long &bytes );
    virtual int writeRaw( unsigned char *data, long bytes, int keyframe=0 );
};











/****************************************************************************************/
/***** Avi - Audiotrack *****************************************************************/
/****************************************************************************************/
class AviFileAtr:public A_Track {
public:
    AviFileAtr( AviFileIO *File, int track );
    AviFileAtr( AviFileIO *File,
                int Sample_rate,
                int Channels,
                const char *Codec_name = NULL );

    ~AviFileAtr();

    // Audio-Position setzen
    int Seek( long block );

protected:
    int readRaw( unsigned char *&data, long &bytes );
    int writeRaw( unsigned char *data, long bytes, int samples );

private:
    AviFileIO *file;            // bergeordnete Klasse
    AviWriteFile *wDatei;       // Avi Datei (schreiben)
    AviReadFile *rDatei;        // lesen
    AviWriteStream *wStream;
    AviReadStream *rStream;
    unsigned char *puffer;
    unsigned puffer_gr;
    long samplesize;
    long bytes_per_second;
};

#endif
