//#include <crtdbg.h>

#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <assert.h>
#include <unistd.h>
#include <ctype.h>
#include <vector>
using std::vector;

#include <iostream>
using std::iostream;
using std::cerr;
using std::endl;


#include "avifmt.h"
#include "AVIReadHandler.h"
#include "File64.h"

#include "except.h"

#define __MODULE__ "AVIReadHandler"

//#define STREAMING_DEBUG

// HACK!!!!
#define _RPT4 printf
//pthread_mutex_t g_diskcs;
bool g_disklockinited=false;

#define GetLastError() 0
typedef long long __int64;



#define __max(a,b) (((a) > (b)) ? (a) : (b))
#define __min(a,b) (((a) < (b)) ? (a) : (b))
#ifndef max
#define max(a,b)   (((a) > (b)) ? (a) : (b))
#endif
#ifndef min
#define min(a,b)   (((a) < (b)) ? (a) : (b))
#endif




///////////////////////////////////////////////////////////////////////////

IvAVIReadStream::~IvAVIReadStream() {
}
IvAVIReadHandler::~IvAVIReadHandler() {
}

///////////////////////////////////////////////////////////////////////////

class AVIStreamNode;
class AVIReadHandler;
class AVIReadStream;

class AVIStreamNode {
public:
	AVIStreamHeader				hdr;
	char					*pFormat;
	long					lFormatLen;
	AVIIndex				index;
	__int64					bytes;
	bool					keyframe_only;
	int					handler_count;
	int					streaming_count;
	__int64					stream_push_pos;
	__int64					stream_bytes;
	int					stream_pushes;
	long					length;
	long					frames;

	AVIStreamNode();
	~AVIStreamNode();
	
	AVIStreamNode &operator =( AVIStreamNode &node );
};




AVIStreamNode::AVIStreamNode()
{
	pFormat = NULL;
	bytes = 0;
	handler_count = 0;
	streaming_count = 0;
	
	stream_bytes = 0;
	stream_pushes = 0;
}



AVIStreamNode::~AVIStreamNode()
{
	delete pFormat;
}


AVIStreamNode &AVIStreamNode::operator =( AVIStreamNode &node )
{
    memcpy( this, &node, sizeof( AVIStreamNode ) );

    if( node.lFormatLen > 0 ) {
        pFormat = new char[ node.lFormatLen ];
        memcpy( pFormat, node.pFormat, node.lFormatLen );
    }

    return *this;
}







///////////////////////////////////////////////////////////////////////////

class AVIFileDesc {
public:
    int    hFile;
    int    hFileUnbuffered;
    __int64		i64Size;
};




class AVIStreamNode;

class AVIReadHandler : public IvAVIReadHandler, private File64 {
public:
	bool		fDisableFastIO;

	AVIReadHandler(const char *);
	~AVIReadHandler();

	void AddRef();
	void Release();
	IvAVIReadStream *GetStream( uint32_t fccType, long lParam);
	void EnableFastIO(bool);
	bool isOptimizedForRealtime();
//	bool isStreaming();
	bool isIndexFabricated();
//	bool AppendFile(const char *pszFile);
	bool getSegmentHint(const char **ppszPath);

//	void EnableStreaming(int stream);
//	void DisableStreaming(int stream);
	void AdjustRealTime(bool fRealTime);
	__int64 getStreamPtr();
	long ReadData(int stream, void *buffer, __int64 position, long len);
	HRESULT Header(MainAVIHeader* mh)
	{
	    if(mh==0)return 0;
	    *mh=m_header;
	    return 0;
	}
	InputStream* cache;
private:
//	enum { STREAM_SIZE = 65536 };
	enum { STREAM_SIZE = 1048576 };
	enum { STREAM_RT_SIZE = 65536 };
	enum { STREAM_BLOCK_SIZE = 4096 };

	int ref_count;
	__int64 i64StreamPosition;
	int			streams;
	char		*streamBuffer;
	int			sbPosition;
	int			sbSize;
	bool		hyperindexed;
	long		fStreamsActive;
	int			nRealTime;
	int			nActiveStreamers;
	bool		fFakeIndex;
	__int64		i64Size;
	int			nFiles, nCurrentFile;
	char *		pSegmentHint;
	MainAVIHeader m_header;
	
	vector<AVIStreamNode *>		listStreams;
//	vector<AVIFileDesc *>		listFiles;

	int     _construct(const char *pszFile);
	int     _parseFile( vector<AVIStreamNode *>& streams);
	bool    _parseStreamHeader( vector<AVIStreamNode*>& streams, uint32_t dwLengthLeft );
	bool    _parseIndexBlock( vector<AVIStreamNode *>& streams, int count, __int64 );
	int     _parseExtendedIndexBlock( vector<AVIStreamNode *>& streams, AVIStreamNode *pasn, __int64 fpos, uint32_t dwLength );
	void    _destruct();

	char *  _StreamRead(long& bytes);
	void    _SelectFile(int file);

};



IvAVIReadHandler *CreateAVIReadHandler(const char *pszFile)
{
    return new AVIReadHandler(pszFile);
}



class AVIReadStream : public IvAVIReadStream {
	friend class AVIReadHandler;

public:
	AVIReadStream(AVIReadHandler *, AVIStreamNode *, int);
	~AVIReadStream();

//	HRESULT BeginStreaming(long lStart, long lEnd, long lRate);
//	HRESULT EndStreaming();
	HRESULT Info(AVISTREAMINFO *pasi, long lSize);
	bool IsKeyFrame(long lFrame) const; 
	HRESULT Read(long lStart, long lSamples, void *lpBuffer, long cbBuffer, long *plBytes, long *plSamples);
	long Start();
	long End();
	long PrevKeyFrame(long lFrame);
	long NextKeyFrame(long lFrame);
	long NearestKeyFrame(long lFrame);
	HRESULT FormatSize(long lFrame, long *plSize);
	HRESULT ReadFormat(long lFrame, void *pFormat, long *plSize);
//	bool isStreaming();
	void ClearAVICache(){parent->cache->Clear();}
	void Reinit();
	
	long Length() const {return length;}

	HRESULT Header(AVIStreamHeader* pheader)
	{
	    if(pheader!=0)*pheader=psnData->hdr;
	    return 0;
	}    
/*	HRESULT ChunkTable(AVIIndex** ppIndex)
	{
	    if(ppIndex!=0)*ppIndex=&psnData->index;
	    return 0;
	}*/

private:
	AVIReadHandler *parent;
	AVIStreamNode *psnData;
	AVIIndexEntry2 *pIndex;
	InputStream *cache;
	long& length;
	long& frames;
	long sampsize;
	int streamno;
	bool fStreamingEnabled;
	bool fStreamingActive;
	int iStreamTrackCount;
	long lStreamTrackValue;
	long lStreamTrackInterval;
	bool fRealTime;

	__int64		i64AVICachedPosition;
	AVIIndexEntry2	*pAVICachedEntry;

};











///////////////////////////////////////////////////////////////////////////

AVIReadStream::AVIReadStream(AVIReadHandler *parent, AVIStreamNode *psnData, int streamno)
:length(psnData->length)
,frames(psnData->frames)
{
	this->parent = parent;
	this->psnData = psnData;
	this->streamno = streamno;

	fStreamingEnabled = false;
	fStreamingActive = false;
	fRealTime = false;

	parent->AddRef();

	pIndex = psnData->index.index2Ptr();
	sampsize = psnData->hdr.dwSampleSize;

	if (sampsize) {
		i64AVICachedPosition = 0;
		pAVICachedEntry = pIndex;
	}

	//psnData->listHandlers.push_pack( *this );
}

AVIReadStream::~AVIReadStream() {
	//EndStreaming();
	parent->Release();
}



void AVIReadStream::Reinit() {
	pIndex = psnData->index.index2Ptr();
	i64AVICachedPosition = 0;
	pAVICachedEntry = pIndex;
}




HRESULT AVIReadStream::Info(AVISTREAMINFO *pasi, long lSize) {
	AVISTREAMINFO asi;

	memset(&asi, 0, sizeof asi);

	asi.fccType				= psnData->hdr.fccType;
	asi.fccHandler			= psnData->hdr.fccHandler;
	asi.dwFlags				= psnData->hdr.dwFlags;
	asi.wPriority			= psnData->hdr.wPriority;
	asi.wLanguage			= psnData->hdr.wLanguage;
	asi.dwScale				= psnData->hdr.dwScale;
	asi.dwRate				= psnData->hdr.dwRate;
	asi.dwStart				= psnData->hdr.dwStart;
	asi.dwLength			= psnData->hdr.dwLength;
	asi.dwInitialFrames		= psnData->hdr.dwInitialFrames;
	asi.dwSuggestedBufferSize = psnData->hdr.dwSuggestedBufferSize;
	asi.dwQuality			= psnData->hdr.dwQuality;
	asi.dwSampleSize		= psnData->hdr.dwSampleSize;
	asi.rcFrame.top			= psnData->hdr.rcFrame.top;
	asi.rcFrame.left		= psnData->hdr.rcFrame.left;
	asi.rcFrame.right		= psnData->hdr.rcFrame.right;
	asi.rcFrame.bottom		= psnData->hdr.rcFrame.bottom;

	if( lSize < (int)sizeof asi )
		memcpy(pasi, &asi, lSize);
	else {
		memcpy(pasi, &asi, sizeof asi);
		memset((char *)pasi + sizeof asi, 0, lSize - sizeof asi);
	}

	return 0;
}



bool AVIReadStream::IsKeyFrame(long lFrame) const {
	if (sampsize)
		return true;
	else {
		if (lFrame < 0 || lFrame >= length)
			return false;

		return !(pIndex[lFrame].size & 0x80000000);
	}
}



HRESULT AVIReadStream::Read(long lStart, long lSamples, void *lpBuffer, long cbBuffer, long *plBytes, long *plSamples) {
	long lActual;

	if (lStart < 0 || lStart >= length || (lSamples <= 0 && lSamples != AVISTREAMREAD_CONVENIENT)) {
		// umm... dummy!  can't read outside of stream!

		if (plBytes) *plBytes = 0;
		if (plSamples) *plSamples = 0;

		return 0;
	}

	// blocked or discrete?

	if (sampsize) {
		AVIIndexEntry2 *avie2, *avie2_limit = pIndex+frames;
		__int64 byte_off = (__int64)lStart * sampsize;
		__int64 bytecnt;
		__int64 actual_bytes=0;
		__int64 block_pos;

		// too small to hold a sample?

		if (lpBuffer && cbBuffer < sampsize)
			return AVIERR_BUFFERTOOSMALL;

		// find the frame that has the starting sample -- try and work
		// from our last position to save time
		if (byte_off >= i64AVICachedPosition) {
			block_pos = i64AVICachedPosition;
			avie2 = pAVICachedEntry;
			byte_off -= block_pos;
		} else {
			block_pos = 0;
			avie2 = pIndex;
		}

		while(byte_off >= (avie2->size & 0x7FFFFFFF)) {
			byte_off -= (avie2->size & 0x7FFFFFFF);
			block_pos += (avie2->size & 0x7FFFFFFF);
			++avie2;
		}

		pAVICachedEntry = avie2;
		i64AVICachedPosition = block_pos;

		// Client too lazy to specify a size?

		if (lSamples == AVISTREAMREAD_CONVENIENT) {
			lSamples = ((avie2->size & 0x7FFFFFFF) - byte_off) / sampsize;

			if (!lSamples && avie2+1 < avie2_limit)
				lSamples = ((avie2[0].size & 0x7FFFFFFF) + (avie2[1].size & 0x7FFFFFFF) - byte_off) / sampsize;

			if (lSamples < 0)
				lSamples = 1;
		}

		// trim down sample count

		if (lpBuffer && lSamples > cbBuffer / sampsize)
			lSamples = cbBuffer / sampsize;

		if (lStart+lSamples > length)
			lSamples = length - lStart;
	
        bytecnt = lSamples * sampsize;

		if (lpBuffer) {
			while(bytecnt > 0) {
				long tc;

				tc = (avie2->size & 0x7FFFFFFF) - byte_off;
				if (tc > bytecnt)
					tc = (long)bytecnt;

				lActual = parent->cache->Read((char*)lpBuffer, (Unsigned)streamno, ((int)avie2-(int)pIndex)/sizeof(AVIIndexEntry2),
				     (Unsigned)tc, (Unsigned)byte_off);

				if (lActual < 0)
					break;

				actual_bytes += lActual;
				++avie2;
				byte_off = 0;

				if (lActual < tc)
					break;
				bytecnt -= tc;
				lpBuffer = (char *)lpBuffer + tc;
			}

			if (actual_bytes < sampsize) {
				if (plBytes) *plBytes = 0;
				if (plSamples) *plSamples = 0;
				return AVIERR_FILEREAD;
			}

			actual_bytes -= actual_bytes % sampsize;

			if (plBytes) *plBytes = actual_bytes;
			if (plSamples) *plSamples = actual_bytes / sampsize;

			lStreamTrackValue = lStart + actual_bytes / sampsize;

		} else {
			if (plBytes) *plBytes = (long)bytecnt;
			if (plSamples) *plSamples = lSamples;
		}

	} else {
		AVIIndexEntry2 *avie2 = &pIndex[lStart];

		if (lpBuffer && (avie2->size & 0x7FFFFFFF) > cbBuffer) {
			return AVIERR_BUFFERTOOSMALL;
		}

		if (lpBuffer) {

			lActual = parent->cache->Read((char*)lpBuffer, (Unsigned)streamno, ((int)avie2-(int)pIndex)/sizeof(AVIIndexEntry2),
			     (Unsigned)(avie2->size & 0x7FFFFFFF), 0);

			if (lActual != (avie2->size & 0x7FFFFFFF)) {
				if (plBytes) *plBytes = 0;
				if (plSamples) *plSamples = 0;
				return AVIERR_FILEREAD;
			}
		}

		if (plBytes) *plBytes = avie2->size & 0x7FFFFFFF;
		if (plSamples) *plSamples = 1;
	}
	return 0;
}



long AVIReadStream::Start() {
	return 0;
}

long AVIReadStream::End() {
	return length;
}

long AVIReadStream::PrevKeyFrame(long lFrame) {
	if (sampsize)
		return lFrame>0 ? lFrame-1 : -1;

	if (lFrame < 0)
		return -1;

	if (lFrame >= length)
		lFrame = length;

	while(lFrame >= 0) {
		if (!(pIndex[lFrame].size & 0x80000000))
			return lFrame;
		lFrame--;
	}

	return -1;
}

long AVIReadStream::NextKeyFrame(long lFrame) {
	if (sampsize)
		return lFrame<length ? lFrame+1 : -1;

	if (lFrame < 0)
		return 0;

	if (lFrame >= length)
		return -1;

	while(++lFrame < length)
		if (!(pIndex[lFrame].size & 0x80000000))
			return lFrame;

	return -1;
}

long AVIReadStream::NearestKeyFrame(long lFrame) {
	long lprev;

	if (sampsize)
		return lFrame;

	if (IsKeyFrame(lFrame))
		return lFrame;

	lprev = PrevKeyFrame(lFrame);

	if (lprev < 0)
		return 0;
	else
		return lprev;
}

HRESULT AVIReadStream::FormatSize(long lFrame, long *plSize) {
	*plSize = psnData->lFormatLen;
	return 0;
}

HRESULT AVIReadStream::ReadFormat(long lFrame, void *pFormat, long *plSize) {
	if (!pFormat) {
		*plSize = psnData->lFormatLen;
		return 0;
	}

	if (*plSize < psnData->lFormatLen) {
		memcpy(pFormat, psnData->pFormat, *plSize);
	} else {
		memcpy(pFormat, psnData->pFormat, psnData->lFormatLen);
		*plSize = psnData->lFormatLen;
	}

	return 0;
}


///////////////////////////////////////////////////////////////////////////

AVIReadHandler::AVIReadHandler(const char *s) {
	this->hFile = -1;
	this->hFileUnbuffered = -1;
	ref_count = 1;
	streams=0;
	hyperindexed = false;
	fStreamsActive = 0;
	fDisableFastIO = false;
	streamBuffer = NULL;
	nRealTime = 0;
	nActiveStreamers = 0;
	fFakeIndex = false;
	nFiles = 1;
	nCurrentFile = 0;
	pSegmentHint = NULL;
	cache=0;
	if (!g_disklockinited) {
		g_disklockinited=true;
		//pthread_mutex_init(&g_diskcs, NULL);
	}
	_construct(s);
}



AVIReadHandler::~AVIReadHandler() {
//	cout<<"~AVIReadHandler"<<endl;
	_destruct();
}






int AVIReadHandler::_construct(const char *pszFile)
{

	try {
		//AVIFileDesc *pDesc;

		// open file

//		hFile = CreateFile(pszFile, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN, NULL);
		hFile = open(pszFile, O_RDONLY);

		if (hFile<0)
		    throw FATAL("Couldn't open file");
		
		hFileUnbuffered = open(pszFile, O_RDONLY);
			
		i64FilePosition = 0;

		// recursively parse file
		cache=new InputStream(hFile, hFileUnbuffered);

		_parseFile(listStreams);
		
		AVIStreamNode *pasn;

//		printf("Initializing cache\n");
		for( unsigned int i=0;  i < listStreams.size(); i++ ) {
		    pasn = listStreams[i];
		
		    if(pasn->index.index2Ptr()==NULL)
		        pasn->index.makeIndex2();
		
		    cache->AddStream( i, pasn->index.index2Ptr(), 0, pasn->index.indexLen() );
		}
		
		cache->Async();

		
		// Create first link
		/*
		if (!(pDesc = new AVIFileDesc))
		    throw MEMORY_FATAL;

		pDesc->hFile			= hFile;
		pDesc->hFileUnbuffered	= hFileUnbuffered;
		pDesc->i64Size			= i64Size = _sizeFile();

	    listFiles.push_back( pDesc );
	    */
	}
	
	catch(FatalError&)
	{
	    _destruct();
	    throw;
	}
	return 0;
}







int AVIReadHandler::_parseFile( vector<AVIStreamNode*>& streamlist ) {
	uint32_t fccType;
	uint32_t dwLength;
	uint32_t dwLengthLeft;
	bool index_found = false;
	bool size_invalid = false;
	bool fAcceptIndexOnly = true;
	AVIStreamNode *pasn;

	__int64	i64ChunkMoviPos = 0;
	uint32_t dwChunkMoviLength = 0;

	if (!_readChunkHeader(fccType, dwLength))
		throw FATAL("Invalid AVI file: File is less than 8 bytes");

	if (fccType != FOURCC_RIFF)
		throw FATAL("Invalid AVI file: Not a RIFF file");

	if (dwLength < 4) {
		dwLength = 0xFFFFFFF0;
		size_invalid = true;
	}

	dwLengthLeft = dwLength-4;

	_readFile2(&fccType, 4);

	if (fccType != mmioFOURCC( 'A', 'V', 'I', ' ') )
		throw FATAL("Invalid AVI file: RIFF type is not 'AVI'");

	// begin parsing chunks

	while(dwLengthLeft >= 8 && _readChunkHeader(fccType, dwLength)) {

		char s[5];
		memcpy(s, &fccType, 4);
		s[4]=0;
//		printf("%08Lx %08Lx ", _posFile()+dwLengthLeft, _posFile());
//		printf("Chunk '%.4s', length %08lx\n", s, dwLength);

		if (!size_invalid) {
			dwLengthLeft -= 8;

			if (dwLength > dwLengthLeft)
			{
//			    printf("Invalid AVI file\n");
			    return -1;
			}    

			dwLengthLeft -= (dwLength + (dwLength&1));
		}
		int hdrSize;
		switch(fccType) {
		case ckidAVIMAINHDR:
			hdrSize=min(dwLength, sizeof(m_header));
			_readFile2(&m_header, hdrSize);
//			printf("Found main header, size %d\n", hdrSize);
			dwLength-=hdrSize;
			
			break;
		case FOURCC_LIST:
			_readFile2(&fccType, 4);

			if (dwLength<4 && (fccType != listtypeAVIMOVIE|| !size_invalid))
			    throw FATAL("Invalid AVI file: LIST chunk <4 bytes");

			dwLength -= 4;

//			_RPT1(0,"\tList type '%-4s'\n", &fccType);

			switch(fccType) {
			case listtypeAVIMOVIE:
//				printf("Found MOVI chunk. Position %Lx\n", _posFile());
				if (dwLength < 8 && size_invalid) {
					i64ChunkMoviPos = _posFile();
					dwChunkMoviLength = 0xFFFFFFF0;
					dwLength = 0;
				} else {
					i64ChunkMoviPos = _posFile();
					dwChunkMoviLength = dwLength;
				}

				if (fAcceptIndexOnly)
					goto terminate_scan;

				break;
			case listtypeAVIHEADER:
				dwLengthLeft += (dwLength + (dwLength&1)) + 4;
				dwLength = 0;	// silently enter the header block
				break;
			case listtypeSTREAMHEADER:
				if (!_parseStreamHeader(streamlist, dwLength))
					fAcceptIndexOnly = false;

				++streams;
				dwLength = 0;
				break;
			}

			break;

		case ckidAVINEWINDEX:	// idx1
			if (!hyperindexed) {
				index_found = _parseIndexBlock(streamlist, dwLength/16, i64ChunkMoviPos);
				dwLength &= 15;
			}
			break;

		case ckidAVIPADDING:	// JUNK
			break;

		case mmioFOURCC( 's', 'e', 'g', 'm' ):			// VirtualDub segment hint block
			delete pSegmentHint;
			if (!(pSegmentHint = new char[dwLength]))
				throw MEMORY_FATAL;

			_readFile2(pSegmentHint, dwLength);

			if (dwLength&1)
				_skipFile2(1);

			dwLength = 0;
			break;

		}

		if (dwLength) {
			if (!_skipFile2(dwLength + (dwLength&1)))
				break;
		}

		// If we're trying to read a broken file, quit as soon as we see the index
		// block.

		if (size_invalid && fccType == ckidAVINEWINDEX)
			break;
	}

	if (i64ChunkMoviPos == 0)
		throw FATAL("This AVI file doesn't have a movie data block (movi)!");

terminate_scan:
	if (!index_found && !hyperindexed) {
		printf("Reconstructing table\n");

		for( unsigned int i=0; i < streamlist.size(); i++ ) {
		    pasn = streamlist[i];
		    pasn->hdr.dwLength=0;
		}

//		ProgressDialog pd(NULL, "AVI Import Filter", "Reconstructing missing index block", (dwChunkMoviLength+1023)/1024, true);

//		pd.setValueFormat("%ldK of %ldK");

		// obtain length of file and limit scanning if so
		int progress=0;
		__int64 i64FileSize = 0x7FFFFFFFFFFFFFFFLL;

		dwLengthLeft = dwChunkMoviLength;

		__int64 pos=lseek64(hFile, 0, SEEK_CUR);
		i64FileSize=lseek64(hFile, 0, SEEK_END);
		lseek64(hFile, pos, SEEK_SET);
	
		fFakeIndex = true;

		_seekFile(i64ChunkMoviPos);

		while(dwLengthLeft >= 8 && _readChunkHeader(fccType, dwLength)) {
			int stream;
			__int64 cpos=_posFile();
			while(cpos/(i64FileSize/10)>progress)
			{
			    cerr<<".";
			    progress++;
			}    
			
//			_RPT2(0,"(stream header) Chunk '%-4s', length %08lx\n", &fccType, dwLength);

			dwLengthLeft -= 8+(dwLength + (dwLength&1));

			if (dwLength) {
				if (!_skipFile2(dwLength + (dwLength&1)))
					break;
			}

			if (_posFile() > i64FileSize)
				break;

			if (isxdigit(fccType&0xff) && isxdigit((fccType>>8)&0xff)) {
				stream = StreamFromFOURCC(fccType);

				if (stream >=0 && stream < streams) {

					if( stream < (int)streamlist.size() ) {
					    pasn = streamlist[ stream ];
						pasn->index.add(fccType, _posFile()-(dwLength + (dwLength&1))-8, dwLength, 1);
						pasn->bytes += dwLength;
						if(pasn->hdr.fccType==streamtypeAUDIO)
						{
						    if(pasn->hdr.dwSampleSize)
							pasn->hdr.dwLength+=dwLength/pasn->hdr.dwSampleSize;
						    else
							pasn->hdr.dwLength+=dwLength;							
						}
						else pasn->hdr.dwLength++;
					}
				}

			}

//			pd.advance((long)((_posFile() - i64ChunkMoviPos)/1024));
//			pd.check();
		}
		cerr<<endl;

	}

	// glue together indices

	for( unsigned int i=0; i < streamlist.size(); i++) {
    	pasn = streamlist[ i ];
		
		if (!pasn->index.makeIndex2())
			throw MEMORY_FATAL;

		pasn->frames = pasn->index.indexLen();

		// Clear sample size for video streams!

		if (pasn->hdr.fccType == streamtypeVIDEO)
			pasn->hdr.dwSampleSize=0;

		if (pasn->hdr.dwSampleSize)
			pasn->length = pasn->bytes / pasn->hdr.dwSampleSize;
		else
			pasn->length = pasn->frames;
	}

    return 0;
}




bool AVIReadHandler::_parseStreamHeader( vector<AVIStreamNode*>& streamlist, uint32_t dwLengthLeft )
{
	AVIStreamNode *pasn;
	uint32_t fccType;
	uint32_t dwLength;
	BITMAPINFOHEADER *binfo;
	
	pasn = new AVIStreamNode;
	assert( pasn != NULL );
	
	while( dwLengthLeft >= 8 && _readChunkHeader(fccType, dwLength) ) {

		dwLengthLeft -= 8;

		if (dwLength > dwLengthLeft)
			throw FATAL("Invalid AVI file: chunk size extends outside of parent");

		dwLengthLeft -= (dwLength + (dwLength&1));

		switch(fccType) {

		case ckidSTREAMHEADER:
			memset(&pasn->hdr, 0, sizeof pasn->hdr);

			if (dwLength < sizeof pasn->hdr) {
				_readFile2(&pasn->hdr, dwLength);
				if (dwLength & 1)
					_skipFile(1);
			} else {
				_readFile2(&pasn->hdr, sizeof pasn->hdr);
				_skipFile(dwLength+(dwLength&1) - sizeof pasn->hdr);
			}
			dwLength = 0;
    				
		    pasn->keyframe_only = false;

			break;

		case ckidSTREAMFORMAT:
			pasn->pFormat = new char[pasn->lFormatLen = dwLength];
			assert( pasn->pFormat != NULL );

			_readFile2(pasn->pFormat, dwLength);

			if (pasn->hdr.fccType == streamtypeVIDEO) {
			    binfo = (BITMAPINFOHEADER *)pasn->pFormat;
				switch( binfo->biCompression ) {
					case 0:
					case mmioFOURCC('R', 'A', 'W', ' '):
					case mmioFOURCC('D', 'I', 'B', ' '):
					case mmioFOURCC('d', 'm', 'b', '1'):
					case mmioFOURCC('m', 'j', 'p', 'g'):
					case mmioFOURCC('M', 'J', 'P', 'G'):
					case mmioFOURCC('Y', 'U', 'Y', '2'):
					case mmioFOURCC('U', 'Y', 'V', 'Y'):
					case mmioFOURCC('Y', 'V', 'Y', 'U'):
					case mmioFOURCC('Y', 'V', '1', '2'):
					case mmioFOURCC('I', '4', '2', '0'):
					case mmioFOURCC('Y', '4', '1', 'P'):
					case mmioFOURCC('c', 'y', 'u', 'v'):
					    pasn->keyframe_only = true;
				}
			}

			if (dwLength & 1)
				_skipFile(1);
			dwLength = 0;
			break;
			
		case mmioFOURCC('i', 'n', 'd', 'x'):			// OpenDML extended index
			_parseExtendedIndexBlock(streamlist, pasn, -1, dwLength);
			hyperindexed = true;
			break;

		case ckidAVIPADDING:	// JUNK
			break;
		}

		if (dwLength) {
			if (!_skipFile2(dwLength + (dwLength&1)))
				break;
		}
	}

	if (dwLengthLeft)
		_skipFile2(dwLengthLeft);
		
	
	streamlist.push_back( pasn );

    return hyperindexed;
}






bool AVIReadHandler::_parseIndexBlock( vector<AVIStreamNode*>& streamlist, int count, __int64 movi_offset )
{
	AVIINDEXENTRY avie[32];
	AVIStreamNode *pasn;
	bool absolute_addr = true;

	// Some AVI files have relative addresses in their AVI index chunks, and some
	// relative.  They're supposed to be relative to the 'movi' chunk; all versions
	// of VirtualDub using fast write routines prior to build 4936 generate absolute
	// addresses (oops). AVIFile and ActiveMovie are both ambivalent.  I guess we'd
	// better be as well.

	while(count > 0) {
		int tc = count;
		int i;

		if (tc>32) tc=32;
		count -= tc;

		if( tc*(int)sizeof(AVIINDEXENTRY) != _readFile(avie, tc*sizeof(AVIINDEXENTRY)) ) {
		
		    for( unsigned int i=0; i < streamlist.size(); i++ ) {
			    pasn = streamlist[ i ];
				pasn->index.clear();
				pasn->bytes = 0;
			}
			
			return false;
		}

		for(i=0; i<tc; i++) {
			int stream = StreamFromFOURCC(avie[i].ckid);

			if (absolute_addr && avie[i].dwChunkOffset<movi_offset)
				absolute_addr = false;

			if( (int)streamlist.size() > stream ) {
				pasn = streamlist[ stream ];
				if (absolute_addr)
					pasn->index.add(&avie[i]);
				else
					pasn->index.add(avie[i].ckid, (movi_offset-4) + (__int64)avie[i].dwChunkOffset, avie[i].dwChunkLength, !!(avie[i].dwFlags & AVIIF_KEYFRAME));

				pasn->bytes += avie[i].dwChunkLength;
			}
		}
	}

	return true;

}

int AVIReadHandler::_parseExtendedIndexBlock( vector<AVIStreamNode*>& streamlist, AVIStreamNode *pasn, __int64 fpos, uint32_t dwLength) {
	union {
		AVISUPERINDEX idxsuper;
		AVISTDINDEX idxstd;
	};

	union {
		struct	avisuperindexentry		superent[64];
		struct	avistdindexentry			stdent[128];
		struct	avifieldindexentry		fieldent[85];
	};

	int entries, tp;
	int i;
	__int64 i64FPSave = _posFile();

	if (fpos>=0)
		_seekFile(fpos);
	_readFile2((char *)&idxsuper + 8, sizeof(AVISUPERINDEX) - 8);

	switch(idxsuper.bIndexType) {
	case AVI_INDEX_OF_INDEXES:
		// sanity check

		if (idxsuper.wLongsPerEntry != 4)
		    return false;
//			throw MyError("Invalid superindex block in stream");

//		if (idxsuper.bIndexSubType != 0)
//			throw MyError("Field indexes not supported");

		entries = idxsuper.nEntriesInUse;

		while(entries > 0) {
			tp = sizeof superent / sizeof superent[0];
			if (tp>entries) tp=entries;

			_readFile2(superent, tp*sizeof superent[0]);

			for(i=0; i<tp; i++)
				_parseExtendedIndexBlock(streamlist, pasn, superent[i].qwOffset+8, superent[i].dwSize-8);

			entries -= tp;
		}

		break;

	case AVI_INDEX_OF_CHUNKS:

//		if (idxstd.bIndexSubType != 0)
//			throw MyError("Frame indexes not supported");

		entries = idxstd.nEntriesInUse;

		if (idxstd.bIndexSubType == AVI_INDEX_2FIELD) {
			if (idxstd.wLongsPerEntry != 3)
			    return false;
//				throw MyError("Invalid field stdindex block in video stream");

			while(entries > 0) {
				tp = sizeof fieldent / sizeof fieldent[0];
				if (tp>entries) tp=entries;

				_readFile2(fieldent, tp*sizeof fieldent[0]);

				for(i=0; i<tp; i++) {
					pasn->index.add(idxstd.dwChunkId, idxstd.qwBaseOffset+fieldent[i].dwOffset-8, fieldent[i].dwSize&0x7FFFFFFF, !(fieldent[i].dwSize&0x80000000));

					pasn->bytes += fieldent[i].dwSize & 0x7FFFFFFF;
				}

				entries -= tp;
			}
		} else {
			if (idxstd.wLongsPerEntry != 2)
			    return false;
//			  throw MyError("Invalid frame stdindex block in video stream");

			while(entries > 0) {
				tp = sizeof stdent / sizeof stdent[0];
				if (tp>entries) tp=entries;

				_readFile2(stdent, tp*sizeof stdent[0]);

				for(i=0; i<tp; i++) {
					pasn->index.add(idxstd.dwChunkId, idxstd.qwBaseOffset+stdent[i].dwOffset-8, stdent[i].dwSize&0x7FFFFFFF, !(stdent[i].dwSize&0x80000000));

					pasn->bytes += stdent[i].dwSize & 0x7FFFFFFF;
				}

				entries -= tp;
			}
		}

		break;

	default:
//		throw MyError("Unknown hyperindex type");
	    return false;
	}

	_seekFile(i64FPSave);
	return false;
}








void AVIReadHandler::_destruct()
{
	//AVIFileDesc *pDesc;
	
	while( listStreams.size() ) {
	    delete listStreams.back();
		listStreams.pop_back();
    }

	delete streamBuffer;

	if (hFile >=0)
		close(hFile);
	if( (hFileUnbuffered >=0) )
		close(hFileUnbuffered);
	
	if(cache)
	    delete cache;
	
	delete pSegmentHint;
}










///////////////////////////////////////////////////////////////////////////

void AVIReadHandler::Release() {
	if (!--ref_count)
		delete this;
}

void AVIReadHandler::AddRef() {
	++ref_count;
}

IvAVIReadStream *AVIReadHandler::GetStream(uint32_t fccType, long lParam) {
		 {
		AVIStreamNode *pasn = NULL;
		int streamno = 0;
		unsigned int i=0;

		for( i=0; i < listStreams.size(); i++ ) {
		    pasn = listStreams[ i ];
		
			if (pasn->hdr.fccType == fccType && !lParam--)
				break;

			++streamno;
		}

		if( i < listStreams.size() ) {
			return new AVIReadStream(this, pasn, streamno);
		}

		return NULL;
	}
}

void AVIReadHandler::EnableFastIO(bool f) {
	fDisableFastIO = !f;
}

bool AVIReadHandler::isOptimizedForRealtime() {
	return nRealTime!=0;
}



bool AVIReadHandler::isIndexFabricated() {
	return fFakeIndex;
}

bool AVIReadHandler::getSegmentHint(const char **ppszPath) {
	if (!pSegmentHint) {
		if (ppszPath)
			*ppszPath = NULL;

		return false;
	}

	if (ppszPath)
		*ppszPath = pSegmentHint+1;

	return !!pSegmentHint[0];
}







///////////////////////////////////////////////////////////////////////////

void AVIReadHandler::AdjustRealTime(bool fInc) {
	if (fInc)
		++nRealTime;
	else
		--nRealTime;
}

char *AVIReadHandler::_StreamRead(long& bytes) {
	if (nCurrentFile<0 || nCurrentFile != (int)(i64StreamPosition>>48))
		_SelectFile((int)(i64StreamPosition>>48));

	if (sbPosition >= sbSize) {
		if (nRealTime || ((i64StreamPosition+sbSize) & -STREAM_BLOCK_SIZE)+STREAM_SIZE > i64Size) {
			i64StreamPosition += sbSize;
			sbPosition = 0;
			_seekFile(i64StreamPosition & 0x0000FFFFFFFFFFFFLL);

			sbSize = _readFile(streamBuffer, STREAM_RT_SIZE);

			if (sbSize < 0) {
				sbSize = 0;
				return NULL;
//				 throw MyWin32Error("Failure streaming AVI file: %%s.",GetLastError());
			}
		} else {
			i64StreamPosition += sbSize;
			sbPosition = i64StreamPosition & (STREAM_BLOCK_SIZE-1);
			i64StreamPosition &= -STREAM_BLOCK_SIZE;
			_seekFileUnbuffered(i64StreamPosition & 0x0000FFFFFFFFFFFFLL);

			sbSize = _readFileUnbuffered(streamBuffer, STREAM_SIZE);

			if (sbSize < 0) {
				sbSize = 0;
				return NULL;
//				throw MyWin32Error("Failure streaming AVI file: %%s.",GetLastError());
			}
		}
	}

	if (sbPosition >= sbSize)
		return NULL;

	if (bytes > sbSize - sbPosition)
		bytes = sbSize - sbPosition;

	sbPosition += bytes;

	return streamBuffer + sbPosition - bytes;
}
__int64 AVIReadHandler::getStreamPtr() {
	return i64StreamPosition + sbPosition;
}



long AVIReadHandler::ReadData(int stream, void *buffer, __int64 position, long len)
{
	if (nCurrentFile<0 || nCurrentFile != (int)(position>>48))
		_SelectFile((int)(position>>48));

	if (!_seekFile2(position & 0x0000FFFFFFFFFFFFLL))
		return -1;
	return _readFile(buffer, len);
}






void AVIReadHandler::_SelectFile(int file)
{
/*
	AVIFileDesc *pDesc;

	nCurrentFile = file;

	if( file < (int)listFiles.size() ) {
	    pDesc = listFiles8 file ];
	    hFile			= pDesc->hFile;
	    hFileUnbuffered	= pDesc->hFileUnbuffered;
	    i64Size			= pDesc->i64Size;
	}
*/
}
