/********************************************************

	Classes for reading AVIs
	Copyright 2000 Eugene Kuznetsov (divx@euro.ru)

*********************************************************/
#include "AviRead.h"
#include <iostream>
using namespace std;    
#define __MODULE__ "AviRead"
#ifndef TIMING
#define Debug if(0)
#else
#define Debug
#endif




/********************************************************************
 
 AviReadStream functions that are common for video and audio streams

*********************************************************************/
AviReadStream::~AviReadStream()
{
    delete rem_buffer;
}


HRESULT AviReadStream::Eof() const
{
//    cerr<<m_sample<<" "<<GetPos()<<" "<<m_table_size<<" "<<m_header.dwLength<<" "<<m_pIStream->Length()<<endl;
    return ((int)GetPos() >= m_pIStream->Length());
}



HRESULT AviReadStream::Init(Unsigned id, IvAVIReadStream* stream)
{
    if(stream==0)
        return -1;
    m_pIStream=stream;
    m_id=id;
    
    m_pIStream->Header(&m_header);
    m_pIStream->FormatSize(0, &m_format_size);
    m_format=new char[m_format_size];
    m_pIStream->ReadFormat(0, m_format, &m_format_size);

    m_table_size=stream->Length();	
//    cout<<"Successfully initialized stream "<<id<<endl;
//    cout<<"Chunk table size "<<m_table_size<<", format size "<<m_format_size<<endl;
    return 0;   
}



AviReadStream::StreamType AviReadStream::GetType() const
{
    if(m_header.fccType==streamtypeVIDEO)
        return Video;
    else if(m_header.fccType==streamtypeAUDIO)
        return Audio;
    else
        return Other;
}


HRESULT AviReadStream::Seek(Unsigned pos)
{
//    cout<<"AviReadStream::Seek"<<endl;
    m_sample=pos;
    rem_size=0;
    if(pos>=GetEndPos())
        return -1;
    m_pIStream->ClearAVICache();
    return 0;
}



double AviReadStream::SeekToTime(double time)
{
//    cout<<"AviReadStreamA::SeekToTime"<<endl;
    if(time<0)
        return -1;
    if(Seek((unsigned int)(time*(double)m_header.dwRate/m_header.dwScale))!=0)
        return -1;
    return time;
}



long AviReadStream::GetNextKeyFrame( long frame ) const
{
    if(m_pIStream==0)return 0;
    return m_pIStream->NextKeyFrame((frame<0)?(int)m_sample:frame);
}


long AviReadStream::GetPrevKeyFrame( long frame ) const
{
    if(m_pIStream==0)return 0;
    if(frame==0)return 0;
    if(frame<0 && m_sample<2)return 0;
    return m_pIStream->PrevKeyFrame((frame<0)?int(m_sample-(Unsigned)2):frame);
}

/*
int AviReadStream::ToNextKeyFrame()
{
    if(m_pIStream==0)return -1;
    int newpos = m_pIStream->NextKeyFrame(m_sample);
    if(newpos==-1)
        return -1;
    m_sample=newpos;
    return 0;
}


int AviReadStream::ToPrevKeyFrame()
{
    if(m_pIStream==0)return -1;
    int newpos=m_pIStream->PrevKeyFrame(m_sample-(Unsigned)2);
    if(newpos==-1)
        return -1;
    m_sample=newpos;
    return 0;
}
*/

long AviReadStream::GetDataSize( void )
{
    long size = 0;
    m_pIStream->Read( m_sample, 1, 0, 0, &size, 0 );
    return size;
}


HRESULT AviReadStream::ReadDirect(char* buffer, Unsigned bufsize, Unsigned samples,
        Unsigned &samples_read, Unsigned &bytes_read)
{
    if(m_pIStream==0)
        return -1;

    HRESULT hr=m_pIStream->Read( m_sample, samples, buffer, bufsize, (long*)&bytes_read, (long*)&samples_read );
    m_sample += samples_read;
    return hr;
}



HRESULT AviReadStream::GetFrameFlags(int* flags) const
{
    if(flags==0)return -1;
    if(!m_pIStream)return 0;
#warning FIXME
    *flags=(m_pIStream->IsKeyFrame(m_sample)?AVIIF_KEYFRAME:0);
    return 0;
}


double AviReadStream::GetTime() const
{
    return GetPos()*(double)m_header.dwScale/m_header.dwRate;
}



Unsigned AviReadStream::GetPos() const
{
    return m_sample;
}



double AviReadStream::GetEndTime() const
{
    return m_header.dwLength*(double)m_header.dwScale/m_header.dwRate;
}



/******************************************************************

    These functions are meaningless for 'unknown type' stream
    
******************************************************************/    

HRESULT AviReadStream::GetVideoFormatInfo( BITMAPINFOHEADER* bi) const
{
    cerr<<"WARNING: GetVideoFormatInfo() for non-video stream"<<endl;
    return -1;
}


HRESULT AviReadStream::GetAudioFormatInfo( WAVEFORMATEX* bi, char** ext ) const
{
    cerr<<"WARNING: GetAudioFormatInfo() for non-audio stream"<<endl;
    return -1;
}  
    















/*************************************************************

        AviReadStreamV ( video AVI stream ) functions
	
*************************************************************/	

HRESULT AviReadStreamV::GetVideoFormatInfo( BITMAPINFOHEADER* bi ) const
{
    if(bi==0)
        return E_ERROR;

    // memcpy(bi, m_format, m_format_size);
    memcpy(bi, m_format, sizeof(BITMAPINFOHEADER) );
    
    return 0;
}    



double AviReadStreamV::SeekToTime(double time)
{
//    cout<<"AviReadStreamV::SeekToTime"<<endl;
    Unsigned pos=(Unsigned)(time*(double)m_header.dwRate/m_header.dwScale);
    if(pos>=m_table_size)
    {
        Seek(pos);
        return time;
    }
    pos=m_pIStream->NearestKeyFrame(pos);
    Seek(pos);
    double qq=pos*(double)m_header.dwScale/m_header.dwRate;	
    return qq;
}


Unsigned AviReadStreamV::SeekToKeyframe(Unsigned pos)
{
    if(pos>=m_table_size)
    return (Unsigned)0;
    pos=m_pIStream->NearestKeyFrame(pos);
    Seek(pos);
    return pos;
}













/*************************************************************

	AviReadStreamA ( audio AVI stream ) Functions 

**************************************************************/

const int wfmtx_size = sizeof( WAVEFORMATEX );

HRESULT AviReadStreamA::GetAudioFormatInfo( WAVEFORMATEX* bi, char** ext) const
{
    if(bi==0)
        return E_ERROR;
        
    memcpy(bi, m_format, wfmtx_size);
    
    if(ext==0)
        return 0;
        
    if(((LPWAVEFORMATEX)bi)->cbSize>0)
    {
        if(m_format_size<wfmtx_size)
        {
//	        cout<<"WARNING: GetAudioFormatInfo: bad audio structure"<<endl;
            *ext=0;
            return E_ERROR;
        }
        
        if(m_format_size<(wfmtx_size+((LPWAVEFORMATEX)bi)->cbSize))
        {
            ((LPWAVEFORMATEX)bi)->cbSize=wfmtx_size-m_format_size;
//	        cout<<"WARNING: GetAudioFormatInfo: bad audio structure"<<endl;
//	        *ext=0;
//	        return E_ERROR;
        }

        *ext = new char[((LPWAVEFORMATEX)bi)->cbSize+18];
        memcpy(*ext, m_format, ((LPWAVEFORMATEX)bi)->cbSize+18);
        return 0;
        
    } else {
        *ext=0;
    }
    
    return 0;
}





Unsigned AviReadStreamA::GetPos() const
{
//    Debug cout<<"Sample "<<m_sample<<", rem_size"<<rem_size<<", sample_size "<<m_header.dwSampleSize<<endl;
    if((m_header.dwSampleSize==0))
        return m_sample;
    else
        if( rem_size / m_header.dwSampleSize > (int)m_sample )
            return 0;
        else
            return m_sample - int( rem_size / m_header.dwSampleSize );
}










/************************************************************

		    AviReadFile functions
		    
************************************************************/		    

AviReadFile::AviReadFile(const char* name)
{
    try {
        m_streams.clear();
        m_pIFile=0;
        try {
            m_pIFile=CreateAVIReadHandler(name);
        }

        catch(FatalError& e) {
            e.Print();
        }

        catch(...) {
//          cout<<"Some strange exception"<<endl;
        }

        if(m_pIFile==0)
            throw FATAL("Can't open file");//never get here
//        cout<<"Successfully opened"<<endl;
//      if(m_pIFile==0)return 0;
        IvAVIReadStream* stream;
        int i=0;
        m_vcount=m_acount=0;
        while( (stream=m_pIFile->GetStream(mmioFOURCC('v', 'i', 'd', 's'), m_vcount++)) );
        m_vcount--;
        while( (stream=m_pIFile->GetStream(mmioFOURCC('a', 'u', 'd', 's'), m_acount++)) );
        m_acount--;
        m_streams.clear();
    //    m_streams=new AviReadStream[m_vcount+m_acount];
        for(i=0; i<m_vcount; i++) {
            AviReadStream* str=new AviReadStreamV;
            str->Init(i, m_pIFile->GetStream(mmioFOURCC('v', 'i', 'd', 's'), i));
            m_streams.push_back(str);
        }
        for(i=0; i<m_acount; i++) {
            AviReadStream* str=new AviReadStreamA;
            str->Init(m_vcount+i, m_pIFile->GetStream(mmioFOURCC('a', 'u', 'd', 's'), i));
            m_streams.push_back(str);
        }

        m_pIFile->Header(&m_header);	
    //	cout<<"Successfully opened "<<name<<". "<<m_vcount<<" video streams, "<<m_acount<<" audio streams"<<endl;
//            cout<<"Length "<<m_header.dwTotalFrames<<endl;
    }

    catch(...) {
        _destruct();
        throw;
    }
}


Unsigned AviReadFile::StreamCount()
{
    return m_vcount+m_acount;
}	
Unsigned AviReadFile::VideoStreamCount()
{
    return m_vcount;
}
Unsigned AviReadFile::AudioStreamCount()
{
    return m_acount;
}	


AviReadStream* AviReadFile::GetStream(Unsigned stream_id, AviStream::StreamType type)
{
    if(type==AviStream::Video)
        if(stream_id<VideoStreamCount())
            return m_streams[stream_id];
    if(type==AviStream::Audio)
        if(stream_id<AudioStreamCount())
            return m_streams[m_vcount+stream_id];
    return NULL;
}


HRESULT AviReadFile::GetFileHeader(MainAVIHeader* header)
{
    if(header)*header=m_header;
    return 0;
}
void AviReadFile::_destruct()
{
    if(m_streams.size())
    for( unsigned int i=0; i<m_streams.size(); i++)
        delete m_streams[i];
    m_streams.clear();
    delete m_pIFile;
    m_pIFile=0;
    return;
}    
AviReadFile::~AviReadFile()
{
    _destruct();
}
