/********************************************************

	Classes for reading AVIs
	Copyright 2000 Eugene Kuznetsov  (divx@euro.ru)

*********************************************************/
#include "config.h"

#include "avifmt.h"
#include "default.h"
#include <unistd.h>
#ifdef HAVE_MALLOC_H
#include <malloc.h>
#else
#include <stdlib.h>
#endif
#include <string.h>
#include <stdio.h>
#include <pthread.h>

#include "AVIReadHandler.h"
#include "avifile.h"
#include <vector>


using namespace std;


class AviReadStream : public AviStream
{
    friend class AviReadFile;
protected:
    IvAVIReadStream* m_pIStream;
    AVIIndexEntry2* m_pTable;
    AVIStreamHeader m_header;
    Unsigned m_id;
    char* m_format;
    long m_format_size;
    Unsigned m_lockcount;
    
    Unsigned m_position;
    Unsigned m_ch_off; //chunk offset for streams with fixed sample size
    
    Unsigned m_sample; //for streams with fixed sample size: total number of samples sent
    
    Unsigned m_table_size;
    offset_t start_offset;

    char* rem_buffer;
    int rem_size;    
    int rem_limit;
public:
//AviReadStream-specific
    AviReadStream(): m_pIStream(0), m_pTable(0), m_format_size(0), m_position(0),
                     m_ch_off(0), m_sample(0), rem_buffer(0), rem_size(0) {}
    virtual HRESULT Init(Unsigned id, IvAVIReadStream* stream);
//Generic
    virtual ~AviReadStream();
    virtual StreamType GetType() const;
    virtual const AVIStreamHeader& GetHeader() const {return m_header;}
    virtual HRESULT GetVideoFormatInfo( BITMAPINFOHEADER* bi ) const;
    virtual HRESULT GetAudioFormatInfo( WAVEFORMATEX* bi, char** ext ) const;
//Size of stream and one frame
    virtual Unsigned GetEndPos() const {return m_header.dwLength;}
    virtual double GetEndTime() const;
    virtual double GetFrameTime() const {return (double)m_header.dwScale / m_header.dwRate;}
    virtual long GetSampleSize() const {return m_header.dwSampleSize;}
//Positioning in stream
    virtual HRESULT Seek(Unsigned pos);
    virtual double SeekToTime(double time);
    virtual Unsigned SeekToKeyframe(Unsigned pos){Seek(pos);return pos;}
    virtual double GetTime() const;
    virtual Unsigned GetPos() const;
    virtual HRESULT GetFrameFlags(int* flags) const;

//Reading data
    virtual long GetDataSize( void );
    virtual HRESULT ReadDirect( char* buffer, Unsigned bufsize, Unsigned samples,
                        Unsigned& samples_read, Unsigned& bytes_read);

    virtual long GetNextKeyFrame( long frame=-1 ) const;
    virtual long GetPrevKeyFrame( long frame=-1 ) const;

    //virtual int ToNextKeyFrame();
    //virtual int ToPrevKeyFrame();

    virtual HRESULT Eof() const;
};




class AviReadStreamV : public AviReadStream
{
    int flip;
public:
    AviReadStreamV():AviReadStream(),flip(0){}
    ~AviReadStreamV(){}
    HRESULT GetVideoFormatInfo( BITMAPINFOHEADER* bi ) const;
    double SeekToTime(double time);
    Unsigned SeekToKeyframe(Unsigned pos);
};




class AviReadStreamA : public AviReadStream
{
public:
    AviReadStreamA():AviReadStream(){}
    ~AviReadStreamA(){}
    HRESULT GetAudioFormatInfo( WAVEFORMATEX *bi, char** ext ) const;
    Unsigned GetPos() const;

    HRESULT ReadFrames( void* buffer, Unsigned bufsize, Unsigned samples,
                        Unsigned& samples_read, Unsigned& bytes_read);
};









class AviReadFile
{
private:
    MainAVIHeader m_header;
    IvAVIReadHandler* m_pIFile;
//    AviReadStream* m_streams;
    vector<AviReadStream*> m_streams;
    int m_vcount;
    int m_acount;
    void _destruct();
public:
//    AviReadFile():m_pIFile(0), m_streams(0){}
    AviReadFile(const char* name);
    virtual ~AviReadFile();
//    HRESULT OpenFile(const char* name);
    virtual Unsigned StreamCount();
    virtual Unsigned VideoStreamCount();
    virtual Unsigned AudioStreamCount();
    virtual AviReadStream* GetStream(Unsigned stream_id, AviStream::StreamType type);
    virtual HRESULT GetFileHeader(MainAVIHeader* header);
    
//    HRESULT CloseFile();
};

