/********************************************************

	AviWriteStream/AviWriteFile class implementation
	Copyright 2000 Eugene Kuznetsov  (divx@euro.ru)

*********************************************************/

#include <stdio.h>
#include <fcntl.h>
#include <iostream>

#include "config.h"

#include "except.h"

#include "AviWriteFile.h"
#include "AviWriteStream.h"


#ifdef __FreeBSD__
#define lseek64 lseek
#define O_LARGEFILE 0
#endif

#define __MODULE__ "AviWriteFile"

using namespace std;
AviWriteFile::AviWriteFile(const char* name, int flags, int mask):m_fd(0)
{
    try
    {
        m_status=0;
        m_fd=new FileBuffer(name, O_WRONLY | O_CREAT | O_TRUNC | O_LARGEFILE, mask);
        const int junk_size=0x800;
        char* junk=new char[junk_size];
        memset(junk, 0, junk_size);
        m_fd->write(junk, junk_size);
        delete junk;
        m_status=1;

        memset(&m_header, 0, sizeof(m_header));
        m_header.dwFlags=flags;
    }
    catch(FatalError&)
    {
        delete m_fd;
        throw;
    }
}







AviWriteStream* AviWriteFile::AddVideoStream( const char *format, unsigned int format_size,
                                              double frame_rate, int flags )
{
    int ckid;
    ckid=MAKEAVICKID(cktypeDIBcompressed, m_streams.size());

    BITMAPINFOHEADER *binfo = (BITMAPINFOHEADER *)format;

    binfo->biSize = sizeof(BITMAPINFOHEADER);
    //binfo.biWidth = width;
    //binfo.biHeight = height;
    if( binfo->biPlanes == 0 ) binfo->biPlanes = 1;
    if( binfo->biBitCount == 0 )binfo->biBitCount = 24;
    //binfo.biCompression = mmioFOURCC( fourcc[0], fourcc[1], fourcc[2], fourcc[3] );
    if( binfo->biSizeImage == 0 )binfo->biSizeImage = binfo->biWidth * binfo->biHeight;
    //binfo.biXPelsPerMeter = 0;
    //binfo.biYPelsPerMeter = 0;
    //binfo.biClrUsed = 0;
    //binfo.biClrImportant = 0;
    
    AviWriteStream* result=new AviWriteStream( this, ckid, AviStream::Video,
                                (char *)binfo, format_size, frame_rate, flags );
    if(result==0)
        throw FATAL("Failed to create new video stream");

    m_streams.push_back(result);
    return result;
}






AviWriteStream* AviWriteFile::AddAudioStream( const char *format, unsigned int format_size, int flags )
{
    int ckid;
    ckid=MAKEAVICKID(cktypeWAVEbytes, m_streams.size());

    WAVEFORMATEX *wformat = (WAVEFORMATEX *)format ;
    //int sampsize = (bits + 7) / 8 * channels;

    //wformat.wFormatTag = format ;
    //wformat.nChannels = channels;
    //wformat.nSamplesPerSec = rate;
    //wformat.nAvgBytesPerSec = sampsize * rate;
    //wformat.nBlockAlign = 1;
    //wformat.wBitsPerSample = bits;
    //wformat.cbSize = 0;
    
    AviWriteStream* result=new AviWriteStream(this, ckid, AviStream::Audio,
                                (char *)wformat, format_size, 25, flags );

    if(result==0)
        throw FATAL("Failed to create new audio stream");

    m_streams.push_back(result);
    return result;
}



#define write_int(X) m_fd->write_int(X)
//    temp=(X);			
//    write(m_fd, &temp, 4);	
AviWriteFile::~AviWriteFile()
{
    if(m_status==0)
        return;
    try
    {
    
    offset_t videoendpos=m_fd->lseek(0, SEEK_CUR);
    if(videoendpos&1)
    {
        int junk=-1;
        m_fd->write( &junk, 1);
        videoendpos++;
    }
    WriteHeaders();

    write_int(ckidAVINEWINDEX);
    write_int(m_index.size()*sizeof(AVIINDEXENTRY));
    m_fd->write( &m_index[0], m_index.size()*sizeof(AVIINDEXENTRY));
    
    videoendpos=m_fd->lseek(0, SEEK_END);
    m_fd->lseek(4, SEEK_SET);
    write_int(videoendpos-8);
    _destruct();
    }
    catch(...)
    {
        _destruct();
        throw;
    }
}




void AviWriteFile::_destruct()
{
    if(m_fd)delete m_fd;
    m_fd=0;
    for( unsigned int i=0; i < m_streams.size(); i++)
        delete m_streams[i];
    m_streams.clear();
    m_index.clear();
    m_status=0;
    return;
}




void AviWriteFile::AddChunk( offset_t offset, Unsigned size, Unsigned id, Unsigned flags )
{
//    m_index=(AVIINDEXENTRY*)realloc(m_index, (m_indsize+1)*sizeof(AVIINDEXENTRY));
    AVIINDEXENTRY entry;
    entry.ckid=id;
    entry.dwFlags=flags;
    entry.dwChunkOffset=offset;
    entry.dwChunkLength=size;
    try
    {
        m_index.push_back(entry);
    }
    catch(...)
    {
        throw FATAL("Chunk table reallocation failure");
    }
    if(m_index.size()%1000==0)
        WriteHeaders();
    return;
}





long long AviWriteFile::FileSize()
{
    return m_fd->lseek(0, SEEK_CUR);
}





void AviWriteFile::WriteHeaders()
{
    unsigned int i;
//    cerr<<"Writing headers"<<endl;
    if(m_status==0)
        return;
    m_header.dwFlags |= AVIF_HASINDEX | AVIF_TRUSTCKTYPE;
    m_header.dwPaddingGranularity=0;
    m_header.dwTotalFrames=0;
    for(i=0; i<m_streams.size(); i++)
    {
        if(m_streams[i]->GetType()==AviWriteStream::Video)
        {
            m_header.dwTotalFrames=m_streams[i]->GetLength();
            m_header.dwMicroSecPerFrame=m_streams[i]->m_header.dwScale;
            m_header.dwWidth=m_streams[i]->m_header.rcFrame.right;
            m_header.dwHeight=m_streams[i]->m_header.rcFrame.bottom;
            break;
        }
    }
    if(m_header.dwTotalFrames==0)
        if(m_streams.size())
        {
            m_header.dwTotalFrames=m_streams[0]->GetLength();
            m_header.dwWidth=m_header.dwHeight=0;
        }
    m_header.dwStreams=m_streams.size();
    offset_t endpos=m_fd->lseek( 0, SEEK_END);
    m_fd->lseek( 0, SEEK_SET);
    write_int(mmioFOURCC('R', 'I', 'F', 'F'));
    write_int(endpos-8);
    write_int(formtypeAVI);// Here goes chunk with all headers
    write_int(mmioFOURCC('L', 'I', 'S', 'T'));
    int hdr_pos=0x10;
    write_int(0);//here header chunk size hdr_size will be
    int hdr_size=12+sizeof(MainAVIHeader);
    write_int(listtypeAVIHEADER);
    write_int(ckidAVIMAINHDR);
    write_int(sizeof(MainAVIHeader));
    m_fd->write( &m_header, sizeof(MainAVIHeader));
    for(unsigned int i=0; i<m_streams.size(); i++)
    {
        write_int(mmioFOURCC('L', 'I', 'S', 'T'));
        write_int(20+sizeof(AVIStreamHeader)+m_streams[i]->m_forsize+(m_streams[i]->m_forsize&1));
        write_int(listtypeSTREAMHEADER);
        write_int(ckidSTREAMHEADER);
        write_int(sizeof(AVIStreamHeader));
        m_fd->write( &m_streams[i]->m_header, sizeof(AVIStreamHeader));
        write_int(ckidSTREAMFORMAT);
        write_int(m_streams[i]->m_forsize);
        m_fd->write( m_streams[i]->m_format, m_streams[i]->m_forsize);
        if(m_streams[i]->m_forsize&1)m_fd->write( &m_fd, 1);
        hdr_size+=28+sizeof(AVIStreamHeader)+m_streams[i]->m_forsize+(m_streams[i]->m_forsize&1);
    }

    if(hdr_size>0x700)
        throw FATAL("Too large header. Aborting");

    int curpos=m_fd->lseek( 0, SEEK_CUR);
    write_int(ckidAVIPADDING);
    write_int(0x7F4-(curpos+8));

    m_fd->lseek( 0x7F4, SEEK_SET);
    write_int(mmioFOURCC('L', 'I', 'S', 'T'));
    write_int(endpos-0x7FC);
    write_int(listtypeAVIMOVIE);

    m_fd->lseek( hdr_pos, SEEK_SET);
    write_int(hdr_size);
    m_fd->lseek( 0, SEEK_END);
}
