/********************************************************

	AviWriteStream/AviWriteFile class implementation
	Copyright 2000 Eugene Kuznetsov  (divx@euro.ru)

*********************************************************/

#include "config.h"

#include "AviWriteStream.h"
#include "AviWriteFile.h"

#ifdef __FreeBSD__
#define lseek64 lseek
#define O_LARGEFILE 0
#endif

#define __MODULE__ "AviWriteStream"

using namespace std;

/****************************************************************************************/
/***** AviWriteStream *******************************************************************/
/****************************************************************************************/
AviWriteStream::AviWriteStream( AviWriteFile* file, int ckid,
                                enum AviStream::StreamType type,
                                const char* format, unsigned int format_size,
                                double frame_rate, int flags )
    :m_file(file), m_ckid(ckid)
{
    int sampsize = 0;

    if(format==0)
        throw FATAL("Bad format");
    if(type==Other)
        throw FATAL("Unsupported stream type");
    
    m_type=type;

    m_fd=file->m_fd;
    m_header.dwLength=0;
    
    memset(&m_header, 0, sizeof(m_header));
    m_header.fccType=((m_type==Video)?streamtypeVIDEO:streamtypeAUDIO);
    m_header.dwFlags=flags;
    if(m_type==Video) {
        //format_size = sizeof( BITMAPINFOHEADER );
        m_header.fccHandler = ((BITMAPINFOHEADER*)format)->biCompression;
        m_header.dwRate = 1000000;
        m_header.dwScale = (int32_t)( 1000000.0 / frame_rate );
        m_header.rcFrame.right  = ((BITMAPINFOHEADER*)format)->biWidth;
        m_header.rcFrame.bottom = ((BITMAPINFOHEADER*)format)->biHeight;
    } else {
        m_header.fccHandler = 0;
        m_header.dwRate  = 1000000;
        m_header.dwScale = (int32_t)( 1000000.0 / frame_rate );

        // das hier ist fr den Windoof-Mediaplayer
        // der braucht das zum positionieren - absolut schwachsinnig!!!
        // Der Wert ist nicht zuverlssig, wenn sample_rate/frame_rate nicht ganzzahlig!
        ((WAVEFORMATEX*)format)->nBlockAlign = (int16_t)( (double)((WAVEFORMATEX*)format)->nSamplesPerSec / frame_rate );
    }

    m_header.dwSampleSize = sampsize;
    m_header.dwQuality = 0;
    m_header.dwFlags = flags;

    m_format = new char[format_size];
    m_forsize = format_size;
    memcpy(m_format, format, format_size);

    return;
}








HRESULT AviWriteStream::AddChunk( const char* chunk, Unsigned size, Unsigned flags )
{
    if((chunk==0) && (size!=0))
    {
        cerr<<"Invalid argument to AviWriteStream::AddChunk()"<<endl;
        return -1;
    }
    offset_t offset=m_fd->lseek(0, SEEK_CUR);
    if(offset>0x7f000000)
        return -1;
    m_fd->write( &m_ckid, 4);
    m_fd->write( &size, 4);
    if(chunk)
    {
        m_fd->write( chunk, size);
        if(size&1)m_fd->write( chunk, 1);
    }
//    fsync(m_fd);	
    m_file->AddChunk(offset-0x7FC, size, m_ckid, flags);
    if( m_header.dwSampleSize==0 )
        m_header.dwLength++;
    else
        m_header.dwLength += size / m_header.dwSampleSize;
    return 0;
}






AviWriteStream::~AviWriteStream()
{
    if(m_format)
        delete m_format;
}






void AviWriteStream::SetFrameRate( double frame_rate )
{
    m_header.dwSampleSize = 0;
    m_header.dwRate  = 1000000;
    m_header.dwScale = (int32_t)( 1000000.0 / frame_rate );

    // das hier ist fr den Windoof-Mediaplayer
    // der braucht das zum positionieren - absolut schwachsinnig!!!
    ((WAVEFORMATEX*)m_format)->nBlockAlign = (int16_t)( (double)((WAVEFORMATEX*)m_format)->nSamplesPerSec / frame_rate );
}





void AviWriteStream::SetBytesPerSec( int byterate )
{
    if( m_type == Audio ) {
        ((WAVEFORMATEX*)m_format)->nAvgBytesPerSec = byterate;
    }
}




unsigned int AviWriteStream::GetFormatInfo( char *&format )
{
    format = m_format;
    return m_forsize;
}
