/********************************************************************\
*****   Methoden fr IO in Avi Dateien                   *****
\********************************************************************/

#ifndef _AVI_H_
#define _AVI_H_

using namespace std;

#include <unistd.h>
#include <vector>
using std::vector;

#include "VideoFile/VideoFile.h"
#include "plugins/RIFFio/RIFF_File.h"
#include "plugins/RIFFio/RIFF_Header.h"
#include "types/AVI_Header.h"





class AviVtr;
class AviAtr;
class AviTrack;


class AviIO:public VideoFile {
    friend class AviVtr;
    friend class AviAtr;
    friend class AviTrack;
public:
    AviIO( const String datei, IO_Mode mode=write );
    ~AviIO();

protected:
    /*** Videofunktionen ***/
    /**Video codec initialisieren
      */
    V_Track *openVideoTrack( double Rate,
                             int x,
                             int y,
                             const char *Codec = NULL );
    /*** Audiofunktionen ***/
    /**Audio codec initialisieren
      */
    A_Track *openAudioTrack( int Sample_rate,
                             int Channels,
                             const char *Codec = NULL );

    /**Video-Track ffnen
      */
    V_Track *openVideoTrack( int track );
    
    /**Audio-Track ffnen
      */
    A_Track *openAudioTrack( int track );
private:



    // Funktionen zum Lesen
    /**Testet Dateityp und geht zum Anfang des Avi-Headers
      */
    bool sync_avi_start( void );

    /**Avi Datei einlesen, Index einlesen
      */
    bool parse_avi_file( void );

    /**Header Abschnitt einlesen
      */
    bool parse_headers( uint32_t &length_left );

    /**Stream-Header einlesen
      */
    bool parse_stream_header( uint32_t &length_left );

    /**OpenDML-Header einlesen
      */
    bool parse_odml_header( uint32_t &length_left );

    /**Index einlesen
      */
    bool read_index( uint32_t &length );

    /**Index neu aufbauen
      */
    bool rebuild_index( void );





    
    // Funktionen zum Schreiben
    /**Avi-Header schreiben
      */
    bool write_avi_header( size_t &length );

    /**OpenDML-Avi-Header schreiben
      */
    bool write_avi_ext_header( size_t &length );
    
    /**Sreibzugriff auf den Movi-Chunk vorbereiten
      */
    bool write_to_movi_chunk( size_t bytes );

    /**Header schreiben
      */
    bool write_headers( void );


    /**Index schreiben
      */
    bool write_index( void );

    /**OpenDML Index schreiben
      */
    bool write_odml_index( void );


            

    /**Avi-Datei fr Eingabe/Ausgabe
      */
    RIFF_File *avi_datei;

    /**Avi-Header
      */
    AVI_Header avi_header;

    /**erweiterter Avi-Header fr Dateien mit mehr als einem RIFF-Chunk
      */
    AVI_ExtHeader avi_ext_header;

    /**Liste mit Tracks der Avi-Datei
      */
    vector<AviTrack*> track_list;

    /**Liste mit RIFF-Headern
      */
    vector<RIFF_Header*> riff_list;

    /**Liste mit movi-Headern
      */
    vector<RIFF_Header*> movi_list;

    /**Die Grenangabe im RIFF-Chunk und im movi-Chunk
      *sind falsch
      */
    bool size_invalid;
       
    /**Erweitertes OpenDML-Format verwenden
      */
    bool open_dml_format;

    /**Anzahl der geschriebenen Blcke
      */
    long chunks_written;
    
};

#endif
