/********************************************************************\
*****   Audio / Video - Track fr Avi Dateien                    *****
\********************************************************************/

#ifndef _AVI_TRACK_H_
#define _AVI_TRACK_H_

#include "Avi.h"
#include "Frame_Index.h"
#include "types/Stream_Header.h"
#include "types/Bitmap_Info.h"
#include "plugins/RIFFio/Wave_Format.h"
#include "ODML_Index.h"


class AviTrack {
    friend class AviIO;
    friend class AviAtr;
    friend class AviVtr;
public:

    AviTrack( AviIO *File, int track=0 );

    virtual ~AviTrack();

    FrameIndex *Index;


    /**Typ des Track setzen
      */
    void SetType( uint32_t type );

    /**Typ des Tracks
      */
    uint32_t Type( void );

    /**Framerate setzen
      */
    void SetFrameRate( double rate, int byte_rate=0, int frame_samples=0, int frame_bytes=0 );

    /**Framerate
      */
    double FrameRate( void );

    /**Framerate oder Samplerate aus Streamheader
      */
    double Rate( void );

    /**Lnge aus StreamHeader (sollte nicht verwendet werden)
      */
    uint32_t Length() { return stream_header->Length; }
    
    /**Lngenangabe im Streamheader aktualisieren
      */
    void UpdateLength( void );


    
    
    
    
    
    
    
    //
    // Videotrack - Funktionen
    //

        
    /**Codecname fr Videotracks setzen
      */
    void SetCodecName( const String &codec );

    /**Codecname fr Videotracks
      */
    const String CodecName( void );
        
    /**Bildgre setzen
      */
    void SetSize( int x, int y );

    /**Breite des Videobildes
      */
    int Width( void );

    /**Hhe des Videobildes
      */
    int Height( void );

    /**Anzahl der Bilder
      */
    long Frames();


    
    
    
    
    
    
    
    
    //
    // Funktionen fr Audiotracks
    //
    
    
    /**Formatcode fr Audiotracks
      */
    uint32_t FormatCode( void );
    
    /**Formatcode fr Audiotracks
      */
    void SetFormatCode( uint32_t code );

    /**Samplerate fr Audiotracks
      */
    uint32_t SampleRate( void );
     
    /**Samplerate fr Audiotracks
      */
    void SetSampleRate( uint32_t rate );

    /**Anzahl der Knle fr Audiotracks
      */
    void SetChannels( int n );

    /**Anzahl der Knle fr Audiotracks
      */
    int Channels( void );
    
    /**Anzahl der Bits pro Sample
      */
    void SetBitsPerSample( int n );

    /**Anzahl der Bits pro Sample
      */
    int BitsPerSample( void );

    /**Block Align - Blockgre in Bytes
      */
    int BlockAlign( void );

    /**Bytes pro Sekunde fr Audiotracks
      */
    uint32_t BytesPerSec( void );

    /**SampleSize
      */
    uint32_t SampleSize( void ) { return stream_header->SampleSize; }

    /**Anzahl der Samples
      */
    uint64_t Samples( void );


    
    
    
    
    
    

    //
    // lesen und schreiben
    //    


    /**Position im Stream fr den nchsen Lesezugriff festlegen
      */
    void SetNextRead( uint64_t block, off_t offset );

    /**Daten lesen
      */
    bool ReadData( uint8_t *data, size_t bytes );

    /**Daten schreiben
      */
    bool WriteData( uint8_t *data, size_t bytes, uint32_t flags = 0 );


    
    

protected:
    AviIO *file;
    RIFF_File *avi_datei;

private:
    /**Avi-Tracknummer
     */
    int track_num;

    /**Stream-Header
     */
    StreamHeader *stream_header;

    /**Bitmap-Info
     */
    BitmapInfo *bitmap_info;

    /**Wave-Format
     */
    WaveFormat *wave_format;

    /**OpenDMl-Index
     */
    ODML_Index *odml_index;

    /**nchster Chunk zum lesen
     */
    int64_t next_chunk;

    /**nchster Offset im Chunk
     */
    int64_t next_chunk_offset;

    
    /**Bytes fr Header-Informationen
     */
    size_t Size(void);
   

    /**Header-Informationen schreiben
     */
    virtual bool WriteHeaders( size_t &length );

    /**Header-Informationen lesen
     */
    virtual bool ReadHeaders( size_t &length );

    /**Alle noch nicht geschriebenen Indexeintrge in einen Indexchunk
     *ans Ende der Datei schreiben<br>
     *Rckgabewert: geschriebene Bytes
     */
    size_t WriteIndex( void );

    /**Track-ID
     */
    uint32_t TrackID( void );

    /**ID fr einen OpenDML Index-Chunk
     */
    uint32_t IndexID( void );
};

#endif
