/*****************************************************************************\
**  Author  : Christian Tusche                                               **
**  E-Mail  : ctusche@gmx.net                        **
**                                                                           **
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/


#include "Avi_Vtr.h"
#include "debug.h"
#include "VideoFile/codecplugins.h"
#include "VideoFile/translate.h"
#include "types/AVI_Codes.h"






/*****************************************************************************/
/***** Videotrack-Funktionen *************************************************/
/*****************************************************************************/

AviVtr::AviVtr( AviIO *File, double Rate, int x, int y, const char *Codec_name, int track )
:Track( File, track, Codec_name ), V_Track( File, Rate, x, y, Codec_name )
{
    file = File;

    buffer_size = 0;
    buffer_data = NULL;
                
    // zugehrigen Avi-Track anlegen
    avi_track = new AviTrack( file, file->track_list.size() );
    file->track_list.push_back( avi_track );

    avi_track->SetType( STREAM_CODE_VIDEO );
            
    codec = AV_Codecs->CreateVCodec( this, "Avi", CodecName() );

    if( codec == NULL ) {
        ERROR_TXT( _("Video codec \"%s\" not implemented!"), CodecName().chars() );
        codec = new V_Codec( this );
    }

    avi_track->SetCodecName( CodecID() );
    avi_track->SetFrameRate( Rate );
    avi_track->SetSize( x, y );

}







AviVtr::AviVtr( AviIO *File, int track )
: Track(File, track), V_Track( File, track )
{
    file = File;

    buffer_size = 0;
    buffer_data = NULL;
    
    // Avi-Track finden
    avi_track = NULL;
    int t = track;
    unsigned int i = 0;
    for( i = 0; i < file->track_list.size(); i++ ) {
        if( file->track_list[i]->Type() == STREAM_CODE_VIDEO && --t < 0 )
            break;
    }
    avi_track = file->track_list[i];

    
    SetCodecName( avi_track->CodecName() );

    SetFrames( avi_track->Frames() );
    breite = avi_track->Width();
    hoehe = avi_track->Height();
    SetRate( avi_track->FrameRate() );
    
    codec = AV_Codecs->CreateVCodec( this, "Avi", CodecName() );

    if( codec == NULL ) {
        ERROR_TXT( _("Video codec \"%s\" not implemented!"), CodecName().chars() );
        codec = new V_Codec( this );
    }

    // vom Codec gendertes Format bernehmen
    (V_Format)*this = (V_Format)*codec;
}






AviVtr::~AviVtr()
{
    delete codec;
    codec = NULL;
}




int AviVtr::Seek( long block )
{
    if( block < Frames() ) {
        SetPosition( block );
        return 1;
    }
    
    return 0;
}






/**liefert den letzten Keyframe zur angegebenen Position
  */
long AviVtr::GetKeyFrame( long bild )
{
    if( !avi_track->Index->KeyframesOk() )
        return bild;

    if( (unsigned long)bild >= avi_track->Index->size() ) {
        return bild;
    }

    while( bild > 0 && !avi_track->Index->at(bild)->IsKeyframe() ) {
        bild--;
    }

    return bild;
}






// Rohdaten lesen und schreiben
int AviVtr::readRaw( unsigned char *&data, long &bytes )
{
    if( Position() >= Frames() ) {
        return 0;
    }
    
    // Datengre den Blocks feststellen
    bytes = avi_track->Index->at(Position())->ChunkSize;

    // Speicher fr Daten erstellen
    if( buffer_size < bytes ) {
        delete [] buffer_data;
        buffer_data = new uint8_t[bytes];
        buffer_size = bytes;
    }
    data = buffer_data;

    // Daten lesen
    avi_track->SetNextRead( Position(), 0 );
    if( !avi_track->ReadData( data, bytes ) )
        return 0;
    
    return 1;
}






int AviVtr::writeRaw( unsigned char *data, long bytes, int keyframe )
{
    uint32_t flags = 0;
    if( keyframe == IS_KEYFRAME )
        flags = AVI_INDEX_KEYFRAME;
        
    // Daten schreiben
    if( !avi_track->WriteData( data, bytes, flags ) )
        return 0;

    return 1;
}
