/********************************************************************\
*****   Methoden fr IO in Avi Dateien                           *****
*****   Verwaltung eines Index der Frames                        *****
\********************************************************************/


#ifndef _FRAME_INDEX_H_
#define _FRAME_INDEX_H_


#include <inttypes.h>
#include <vector>
using std::vector;

#include "types/AVI_Index_Entry.h"
#include "plugins/RIFFio/RIFF_File.h"


class ODML_Index;
class ODML_StandardIndexEntry;


class FrameIndexEntry {
public:
    FrameIndexEntry();
    
    uint64_t AbsFileOffset;
    uint32_t ChunkSize;
    uint32_t Flags;

    /**Prft, ob dieser Eintrag ein Keyframe ist
      */
    bool IsKeyframe( void );
};






class FrameIndex: public vector<FrameIndexEntry*> {
public:
    FrameIndex();
    ~FrameIndex();

    /**Einen Indexeintrag anhand eines vorhandenen AviIndexEntry erstellen.
      */
    void AddEntry( AviIndexEntry &e, uint64_t offset );

    /**Einen Indexeintrag erstellen
      */
    void AddEntry( uint64_t offset, uint32_t length, uint32_t flags = 0 );

    /**Einen Indexeintrag anhand eines vorhandenen OpenDML-Index-Eintrags erstellen.
      */
    void AddEntry( ODML_StandardIndexEntry &e, uint64_t offset );
    
    /**Einen OpenDML-Index hinzufgen.
      */
    bool AddIndex( ODML_Index &index, RIFF_File *file );

    /**Einen OpenDML-Index erstellen
      */
    ODML_Index *ODML_GetIndex( uint32_t chunk_id );


    /**Bytes im Stream
      */
    uint64_t DataBytes() { return bytes; }

    /**Sind die Keyframe-Informationen vorhanden?
      */
    bool KeyframesOk( void );

private:
    /**Bytes im Stream
      */
    uint64_t bytes;

    /**Anzahl der Keyframes
      */
    uint32_t keyframes;

    /**Anzahl der bisher in die Datei geschriebenen Indexeintrge
      */
    uint32_t entries_written;
};


#endif
