/*****************************************************************************\
**  Author  : Christian Tusche                                               **
**  E-Mail  : ctusche@gmx.net                        **
**                                                                           **
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/


#include "ODML_Index.h"
#include <stdio.h>




ODML_Index::ODML_Index()
{
    standard_index_null = NULL;
    super_index_null = NULL;  
}





ODML_Index::~ODML_Index()
{
    // Listen lschen
    while( standard_index.size() > 0 ) {
        delete standard_index.back();
        standard_index.pop_back();
    }

    while( field_index.size() > 0 ) {
        delete field_index.back();
        field_index.pop_back();
    }

    while( super_index.size() > 0 ) {
        delete super_index.back();
        super_index.pop_back();
    }

    delete standard_index_null;
    delete super_index_null;
}



/**Einlesen der Datenstruktur aus einem Puffer
  */
bool ODML_Index::Read( uint8_t *buf, size_t length )
{
    if( length < ODML_IndexHeader::Size() )
        return false;

    ODML_IndexHeader::Read( buf, length );
    buf    += ODML_IndexHeader::Size();
    length -= ODML_IndexHeader::Size();

    if( length < Size() - ODML_IndexHeader::Size() )
        return true;

    // lese die Indexeintrge
    if( (IndexType & 0x0f) == AVI_INDEX_OF_CHUNKS && IndexSubType == 0 ) {
        read_standard_entries( buf, length );
    } else if( (IndexType & 0x0f) == AVI_INDEX_OF_CHUNKS && IndexSubType == AVI_INDEX_2FIELD ) {
        read_field_entries( buf, length );
    } else {
        read_super_entries( buf, length );
    }


    return true;
}





/**Schreiben der Datenstruktur in einen Puffer
  */
bool ODML_Index::Write( uint8_t *buf, size_t length )
{
    if( length < Size() )
        return false;

    ODML_IndexHeader::Write( buf, length );
    buf    += ODML_IndexHeader::Size();
    length -= ODML_IndexHeader::Size();

    // schreibe die Indexeintrge
    if( (IndexType & 0x0f) == AVI_INDEX_OF_CHUNKS && IndexSubType == 0 ) {
        write_standard_entries( buf, length );
    } else if( (IndexType & 0x0f ) == AVI_INDEX_OF_CHUNKS && IndexSubType == AVI_INDEX_2FIELD ) {
        write_field_entries( buf, length );
    } else {
        write_super_entries( buf, length );
    }
    
    return true;
    
}




/**Werte Ausgeben
  */
void ODML_Index::printInfo( void )
{
    ODML_IndexHeader::printInfo();
}





/**Typ des Index festlegen
  */
void ODML_Index::SetIndexType( uint8_t type, uint8_t subtype )
{
    IndexType = type;
    IndexSubType = subtype;

    if( (IndexType & 0x0f) == AVI_INDEX_OF_CHUNKS && IndexSubType == 0 ) {
        LongsPerEntry = 2;
    } else if( (IndexType & 0x0f) == AVI_INDEX_OF_CHUNKS && IndexSubType == AVI_INDEX_2FIELD ) {
        LongsPerEntry = 3;
    } else {
        LongsPerEntry = 4;
    }
}






/**Einen Indexeintrag aus der Liste lesen.
  */
ODML_StandardIndexEntry &ODML_Index::GetFrame( unsigned long n )
{
    if( (IndexType & 0x0f) == AVI_INDEX_OF_CHUNKS && IndexSubType == 0 ) {
        if( n < standard_index.size() )
            return *standard_index[n];
    } else if( (IndexType & 0x0f) == AVI_INDEX_OF_CHUNKS && IndexSubType == AVI_INDEX_2FIELD ) {
        if( n < field_index.size() )
            return *field_index[n];
    }
    
    delete standard_index_null;
    standard_index_null = new ODML_StandardIndexEntry;
    return *standard_index_null;
}





/**Einen Indexeintrag aus der Liste lesen.
  */
ODML_SuperIndexEntry &ODML_Index::GetIndex( unsigned long n )
{
    if( (IndexType & 0x0f)  == AVI_INDEX_OF_INDEXES ) {
        if( n < super_index.size() )
            return *super_index[n];
    }

    delete super_index_null;
    super_index_null = new ODML_SuperIndexEntry;
    return *super_index_null;
}



/**Einen Frame zum Index hinzufgen
  */
void ODML_Index::AddFrame( ODML_StandardIndexEntry *entry )
{
    if( (IndexType & 0x0f) == AVI_INDEX_OF_CHUNKS && IndexSubType == 0 ) {
        standard_index.push_back( entry );
    }

    EntriesInUse = standard_index.size();
}



/**Einen Frame zum Index hinzufgen
  */
void ODML_Index::AddFrame( ODML_FieldIndexEntry *entry )
{
    if( (IndexType & 0x0f) == AVI_INDEX_OF_CHUNKS && IndexSubType == AVI_INDEX_2FIELD ) {
        field_index.push_back( entry );
    }

    EntriesInUse = field_index.size();
}




/**Einen Index zum Index hinzufgen
  */
void ODML_Index::AddIndex( ODML_SuperIndexEntry *entry )
{
    if( (IndexType & 0x0f) == AVI_INDEX_OF_INDEXES ) {
        super_index.push_back( entry );
    }

    EntriesInUse = super_index.size();
}



/**Anzahl der Indexeintrge
  */
unsigned long ODML_Index::Entries( void )
{
    if( (IndexType & 0x0f) == AVI_INDEX_OF_CHUNKS && IndexSubType == 0 ) {
        return standard_index.size();
    } else if( (IndexType & 0xf) == AVI_INDEX_OF_CHUNKS && IndexSubType == AVI_INDEX_2FIELD ) {
        return field_index.size();
    }

    return super_index.size();
}








/**Eintrge einlesen
  */
void ODML_Index::read_standard_entries( uint8_t *buf, size_t length )
{
    ODML_StandardIndexEntry *entry = NULL;
    unsigned int i, n=EntriesInUse;

   
    for( i=0; i < n; i++ ) {
        if( length < 8 )
            return;
            
        entry = new ODML_StandardIndexEntry;
        
        // einlesen
        entry->Read( buf, length );
        buf    += entry->Size();
        length -= entry->Size();

        // Eintrag hinzufgen
        AddFrame( entry );
    }
}



void ODML_Index::read_field_entries( uint8_t *buf, size_t length )
{
    ODML_FieldIndexEntry *entry = NULL;
    unsigned int i, n=EntriesInUse;

    for( i=0; i < n; i++ ) {
        if( length < 12 )
            return;

        entry = new ODML_FieldIndexEntry;

        // einlesen
        entry->Read( buf, length );
        buf    += entry->Size();
        length -= entry->Size();

        // Eintrag hinzufgen
        AddFrame( entry );
    }
}



void ODML_Index::read_super_entries( uint8_t *buf, size_t length )
{
    ODML_SuperIndexEntry *entry = NULL;
    unsigned int i, n=EntriesInUse;

    for( i=0; i < n; i++ ) {
        if( length < 16 )
            return;
        entry = new ODML_SuperIndexEntry;

        // einlesen
        entry->Read( buf, length );
        buf    += entry->Size();
        length -= entry->Size();

        // Eintrag hinzufgen
        AddIndex( entry );
    }
}









/**Eintrge schreibe
  */
void ODML_Index::write_standard_entries( uint8_t *buf, size_t length )
{
    ODML_StandardIndexEntry *entry = NULL;

    for( unsigned int i=0; i<standard_index.size(); i++ ) {
        entry = standard_index[i];
        
        if( length < entry->Size() )
            break;

        // schreiben
        entry->Write( buf, length );
        buf    += entry->Size();
        length -= entry->Size();
    }
}




void ODML_Index::write_field_entries( uint8_t *buf, size_t length )
{
    ODML_FieldIndexEntry *entry = NULL;

    for( unsigned int i=0; i<field_index.size(); i++ ) {
        entry = field_index[i];

        if( length < entry->Size() )
            break;

        // schreiben
        entry->Write( buf, length );
        buf    += entry->Size();
        length -= entry->Size();
    }
}




void ODML_Index::write_super_entries( uint8_t *buf, size_t length )
{
    ODML_SuperIndexEntry *entry = NULL;

    for( unsigned int i=0; i<super_index.size(); i++ ) {
        entry = super_index[i];

        if( length < entry->Size() )
            break;

        // schreiben
        entry->Write( buf, length );
        buf    += entry->Size();
        length -= entry->Size();
    }
}
