/********************************************************************\
*****   Methoden fr IO in Avi Dateien                           *****
*****   Erweiterter OpenDML-Index                                *****
\********************************************************************/

#ifndef _ODML_INDEX_H_
#define _ODML_INDEX_H_

#include <vector>
using std::vector;

#include "types/ODML_Index_Header.h"
#include "types/ODML_Index_Entry.h"

#include "Frame_Index.h"





class ODML_Index: public ODML_IndexHeader {
public:
    ODML_Index();
    virtual ~ODML_Index();

    /**Einlesen der Datenstruktur aus einem Puffer
      */
    bool Read( uint8_t *buf, size_t length );

    /**Schreiben der Datenstruktur in einen Puffer
      */
    bool Write( uint8_t *buf, size_t length );

    /**liefert die Gre des Datenpuffers
      */
    size_t Size(void) { return ODML_IndexHeader::Size() + LongsPerEntry * 4 * EntriesInUse; }


    /**Informationen rausschreiben
      */
    void printInfo( void ) ;


    /**Typ des Index festlegen
      */
    void SetIndexType( uint8_t type, uint8_t subtype = 0 );
    
    /**Einen Indexeintrag aus der Liste lesen.
      */
    ODML_StandardIndexEntry &GetFrame( unsigned long n );

    /**Einen Indexeintrag aus der Liste lesen.
      */
    ODML_SuperIndexEntry &GetIndex( unsigned long n );

    /**Einen Frame zum Index hinzufgen
      */
    void AddFrame( ODML_StandardIndexEntry *entry );

    /**Einen Frame zum Index hinzufgen
      */
    void AddFrame( ODML_FieldIndexEntry *entry );

    /**Einen Index zum Index hinzufgen
      */
    void AddIndex( ODML_SuperIndexEntry *entry );


    /**Anzahl der Indexeintrge
      */
    unsigned long Entries( void );    

    
private:
    /**Eintrge einlesen
      */
    void read_standard_entries( uint8_t *buf, size_t length );
    void read_field_entries( uint8_t *buf, size_t length );
    void read_super_entries( uint8_t *buf, size_t length );
    
    /**Eintrge schreibe
      */
    void write_standard_entries( uint8_t *buf, size_t length );
    void write_field_entries( uint8_t *buf, size_t length );
    void write_super_entries( uint8_t *buf, size_t length );


    vector<ODML_StandardIndexEntry*>  standard_index;
    vector<ODML_FieldIndexEntry*>     field_index;
    vector<ODML_SuperIndexEntry*>     super_index;

    // leerer Eintrag
    ODML_StandardIndexEntry *standard_index_null;
    ODML_SuperIndexEntry    *super_index_null;    
};

#endif
