/********************************************************************\
*****   Methoden fr IO in Avi Dateien                           *****
*****   Konstanten                                               *****
\********************************************************************/

#ifndef _AVI_CODES_H_
#define _AVI_CODES_H_

#include <inttypes.h>

#define fcc_uint32(a,b,c,d)         ( (uint32_t)(d) << 24 | (uint32_t)(c) << 16 | (uint32_t)(b) << 8 | (uint32_t)(a) )
#define uint32_fcc(x,z)             {(uint8_t)z[3] = x >> 24 &0x0F; (uint8_t)z[2] = x >> 16 & 0x0F; (uint8_t)z[1] = x >> 8 & 0xF; (uint8_t)z[0] = x & 0x0F }
#define tcc_uint16(a,b)             ( (uint32_t)(b) << 8 | (uint32_t)(a) )


#define FORMAT_CODE_RIFF            fcc_uint32( 'R','I','F','F' )
#define FORMAT_CODE_AVI             fcc_uint32( 'A','V','I',' ' )
#define FORMAT_CODE_AVIX            fcc_uint32( 'A','V','I','X' )

#define LIST_TYPE_AVIHEADER         fcc_uint32( 'h','d','r','l' )
#define LIST_TYPE_STREAMHEADER      fcc_uint32( 's','t','r','l' )
#define LIST_TYPE_AVIMOVIE          fcc_uint32( 'm','o','v','i' )
#define LIST_TYPE_AVIRECORD         fcc_uint32( 'r','e','c',' ' )
#define LIST_TYPE_ODML              fcc_uint32( 'o','d','m','l' )

#define CHUNK_CODE_LIST                 fcc_uint32( 'L','I','S','T' )
#define CHUNK_CODE_AVIMAINHDR           fcc_uint32( 'a','v','i','h' )
#define CHUNK_CODE_EXT_AVIHEADER        fcc_uint32( 'd','m','l','h' )
#define CHUNK_CODE_STREAMHEADER         fcc_uint32( 's','t','r','h' )
#define CHUNK_CODE_STREAMFORMAT         fcc_uint32( 's','t','r','f' )
#define CHUNK_CODE_STREAMHANDLERDATA    fcc_uint32( 's','t','r','d' )
#define CHUNK_CODE_STREAMNAME           fcc_uint32( 's','t','r','n' )
#define CHUNK_CODE_AVINEWINDEX          fcc_uint32( 'i','d','x','1' )
#define CHUNK_CODE_PADDING_DATA         fcc_uint32( 'J','U','N','K' )
#define CHUNK_CODE_SEGMENT_HINT         fcc_uint32( 's','e','g','m' )
#define CHUNK_CODE_EXTENDED_INDEX       fcc_uint32( 'i','n','d','x' )
                                                                     

#define STREAM_CODE_VIDEO               fcc_uint32( 'v','i','d','s' )
#define STREAM_CODE_AUDIO               fcc_uint32( 'a','u','d','s' )
#define STREAM_CODE_MIDI                fcc_uint32( 'm','i','d','s' )
#define STREAM_CODE_TEXT                fcc_uint32( 't','x','t','s' )

#define CHUNK_TYPE_DIBBITS              tcc_uint16( 'd','b' )
#define CHUNK_TYPE_DIBCOMPRESSED        tcc_uint16( 'd','c' )
#define CHUNK_TYPE_PALCHANGE            tcc_uint16( 'p','c' )
#define CHUNK_TYPE_WAVEBYTES            tcc_uint16( 'w','b' )

#define CHUNK_NUMBER(x)                 (uint16_t)(x & 0xffff )


#define AVI_INDEX_LIST                  0x00000001L     // LIST-Chunk
#define AVI_INDEX_KEYFRAME              0x00000010L     // Keyframe
#define AVI_INDEX_NOTIME                0x00000100L     // Darstellung des Frames bentigt keine Zeit
#define AVI_INDEX_CODEC                 0x0FFF0000L     // Diese Bit sollen vom Codec verwendet werden



#endif




/* The following is a short description of the AVI file format.
 * Please see the accompanying documentation for a full explanation.
 *
 * An AVI file is the following RIFF form:
 *
 *      RIFF( length, 'AVI') {
 *          LIST(length, 'hdrl' ) {
 *              avih(<MainAVIHeader>)
 *              LIST( length, 'strl' ) {
 *                  strh(length, <Stream header>)
 *                  strf(length, <Stream format>)
 *              }
 *              LIST( length, 'strl' ) {
 *                  strh(length, <Stream header>)
 *                  strf(lrngth, <Stream format>)
 *              }
 *              ...
 *          }
 *
 *          JUNK( length ) {
 *              data
 *          }
 *
 *          LIST( length, 'movi' ) {
 *              { LIST('rec'
 *                SubChunk...
 *                 )
 *               | SubChunk }
 *               ....
 *          }
 *
 *          [ <AVIIndex> ]
 *      }
 *
 *	The main file header specifies how many streams are present.  For
 *	each one, there must be a stream header chunk and a stream format
 *	chunk, enlosed in a 'strl' LIST chunk.  The 'strf' chunk contains
 *	type-specific format information; for a video stream, this should
 *	be a BITMAPINFO structure, including palette.  For an audio stream,
 *	this should be a WAVEFORMAT (or PCMWAVEFORMAT) structure.
 *
 *	The actual data is contained in subchunks within the 'movi' LIST
 *	chunk.  The first two characters of each data chunk are the
 *	stream number with which that data is associated.
 *
 *	Some defined chunk types:
 *           Video Streams:
 *                  ##db:	RGB DIB bits
 *                  ##dc:	RLE8 compressed DIB bits
 *                  ##pc:	Palette Change
 *
 *           Audio Streams:
 *                  ##wb:	waveform audio bytes
 *
 * The grouping into LIST 'rec' chunks implies only that the contents of
 * the chunk should be read into memory at the same time.  This grouping
 * is used for files specifically intended to be played from CD-ROM.
 *
 * The index chunk at the end of the file should contain one entry for
 * each data chunk in the file.
 *
 * Limitations for the current software:
 *	Only one video stream and one audio stream are allowed.
 *	The streams must start at the beginning of the file.
 *
 *
 * To register codec types please obtain a copy of the Multimedia
 * Developer Registration Kit from:
 *
 *  Microsoft Corporation
 *  Multimedia Systems Group
 *  Product Marketing
 *  One Microsoft Way
 *  Redmond, WA 98052-6399
 *
 */
