/*****************************************************************************\
**  Author  : Christian Tusche                                               **
**  E-Mail  : ctusche@gmx.net                        **
**                                                                           **
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/

#include <string.h>
#include "AVI_Header.h"
#include "debug.h"




AVI_Header::AVI_Header()
{
    // Datenstruktur initialisieren
    MicroSecPerFrame = 40000;     // 25 Bilder / sec.
    MaxBytesPerSec = 1000000;
    PaddingGranularity = 0;
    Flags = AVI_TRUSTCKTYPE | AVI_HASINDEX;
    TotalFrames = 0;
    InitialFrames = 0;
    Streams = 0;
    SuggestedBufferSize = 100000;

    Width = 640;
    Height = 480;

    Reserved[0] = 0;
    Reserved[1] = 0;
    Reserved[2] = 0;
    Reserved[3] = 0;
}





AVI_Header::~AVI_Header()
{

}



/**Einlesen der Datenstruktur aus einem Puffer
  */
bool AVI_Header::Read( uint8_t *buf, size_t length )
{
    if( length < Size() )
        return false;
        
    read_int( MicroSecPerFrame, buf, length );
    read_int( MaxBytesPerSec, buf, length );
    read_int( PaddingGranularity, buf, length );
    read_int( Flags, buf, length );
    read_int( TotalFrames, buf, length );
    read_int( InitialFrames, buf, length );
    read_int( Streams, buf, length );
    read_int( SuggestedBufferSize, buf, length );

    read_int( Width, buf, length );
    read_int( Height, buf, length );

    read_int( Reserved[0], buf, length );
    read_int( Reserved[1], buf, length );
    read_int( Reserved[2], buf, length );
    read_int( Reserved[3], buf, length );

    return true;
}





/**Schreiben der Datenstruktur in einen Puffer
  */
bool AVI_Header::Write( uint8_t *buf, size_t length )
{
    if( length < Size() )
        return false;

    write_int( MicroSecPerFrame, buf, length );
    write_int( MaxBytesPerSec, buf, length );
    write_int( PaddingGranularity, buf, length );
    write_int( Flags, buf, length );
    write_int( TotalFrames, buf, length );
    write_int( InitialFrames, buf, length );
    write_int( Streams, buf, length );
    write_int( SuggestedBufferSize, buf, length );

    write_int( Width, buf, length );
    write_int( Height, buf, length );

    write_int( Reserved[0], buf, length );
    write_int( Reserved[1], buf, length );
    write_int( Reserved[2], buf, length );
    write_int( Reserved[3], buf, length );
    
    return true;
    
}




/**Werte Ausgeben
  */
void AVI_Header::printInfo( void )
{
    printf( "    MicroSecPerFrame = %i\n", MicroSecPerFrame );
    printf( "    MaxBytesPerSec = %i\n", MaxBytesPerSec );
    printf( "    PaddingGranularity = %i\n", PaddingGranularity );
    printf( "    Flags = 0x%08x\n", Flags );
    printf( "    TotalFrames = %i\n", TotalFrames );
    printf( "    InitialFrames = %i\n", InitialFrames );
    printf( "    Streams = %i\n", Streams );
    printf( "    SuggestedBufferSize = %i\n", SuggestedBufferSize );
    
    printf( "    Width = %i\n", Width );
    printf( "    Height = %i\n", Height );

    printf( "    Reserved = 0x%08x, 0x%08x, 0x%08x, 0x%08x\n",
            Reserved[0], Reserved[1],Reserved[2],Reserved[3] );
}



















AVI_ExtHeader::AVI_ExtHeader()
{
    // Datenstruktur initialisieren
    TotalFrames = 0;
}





AVI_ExtHeader::~AVI_ExtHeader()
{

}



/**Einlesen der Datenstruktur aus einem Puffer
  */
bool AVI_ExtHeader::Read( uint8_t *buf, size_t length )
{
    if( length < Size() )
        return false;

    read_int( TotalFrames, buf, length );

    return true;
}





/**Schreiben der Datenstruktur in einen Puffer
  */
bool AVI_ExtHeader::Write( uint8_t *buf, size_t length )
{
    if( length < Size() )
        return false;

    write_int( TotalFrames, buf, length );

    // rest mit 0 auffllen
    memset( buf, 0, Size() - 4 );

    return true;

}




/**Werte Ausgeben
  */
void AVI_ExtHeader::printInfo( void )
{
    printf( "    TotalFrames = %i\n", TotalFrames );
}
