/********************************************************************\
*****   Methoden fr IO in Avi Dateien                           *****
*****   Datentyp AVI-Header                                      *****
\********************************************************************/

#ifndef _AVI_HEADER_H_
#define _AVI_HEADER_H_

#include "plugins/RIFFio/RIFF_Chunk.h"
#include "plugins/RIFFio/RIFF_ITypes.h"



/* The AVI File Header LIST chunk should be padded to this size */
#define AVI_HEADERSIZE  2048                    // size of AVI header list


/* flags for use in <Flags> in AVIFileHdr */
#define AVI_HASINDEX            0x00000010      // Index at end of file?
#define AVI_MUSTUSEINDEX        0x00000020
#define AVI_ISINTERLEAVED       0x00000100
#define AVI_TRUSTCKTYPE         0x00000800      // Use CKType to find key frames?
#define AVI_WASCAPTUREFILE      0x00010000
#define AVI_COPYRIGHTED         0x00020000



class AVI_Header: public RIFF_Chunk, public RIFF_ITypes {
public:
    AVI_Header();
    ~AVI_Header();

    // Datenstruktur
    uint32_t MicroSecPerFrame;     // Zeit pro Frame (oder 0L) (das macht hier berhaupt
                                   // keinen Sinn!) sollte der Rate aus StremHeader fr
                                   // irgendeinen? Videotrack entsprechen
    uint32_t MaxBytesPerSec;       // Dateigre/Lnge???
    uint32_t PaddingGranularity;   // pad to multiples of this
                                   // size; normally 2K.
    uint32_t Flags;                // Flags (s.o.)
    uint32_t TotalFrames;          // Anzahl der Frames in der Datei (das gleiche wie in StremHeader)
    uint32_t InitialFrames;        // ?
    uint32_t Streams;              // Anzahl der Streams in dieser Datei
    uint32_t SuggestedBufferSize;  // ?

    uint32_t Width;                // Das gleiche wie in BitmapInfo (fr irgendeinen
    uint32_t Height;               // Videotrack). WAS SOLL DAS HIER?!

    uint32_t Reserved[4];

    
    // Methoden
    /**Einlesen der Datenstruktur aus einem Puffer
      */
    bool Read( uint8_t *buf, size_t length );

    /**Schreiben der Datenstruktur in einen Puffer
      */
    bool Write( uint8_t *buf, size_t length );

    /**liefert die Gre des Datenpuffers
      */
    size_t Size(void) { return 56; }

    void printInfo( void ) ;
};







class AVI_ExtHeader: public RIFF_Chunk, public RIFF_ITypes {
public:
    AVI_ExtHeader();
    ~AVI_ExtHeader();

    uint32_t TotalFrames;          // Anzahl der Frames in der Datei (das gleiche wie in StremHeader)

    // Methoden
    /**Einlesen der Datenstruktur aus einem Puffer
      */
    bool Read( uint8_t *buf, size_t length );

    /**Schreiben der Datenstruktur in einen Puffer
      */
    bool Write( uint8_t *buf, size_t length );

    /**liefert die Gre des Datenpuffers<br>
      *es sind hier nur die ersten 4 Bytes verwendet
      */
    size_t Size(void) { return 248; }

    void printInfo( void ) ;
};

#endif
