/*****************************************************************************\
**  Author  : Christian Tusche                                               **
**  E-Mail  : ctusche@gmx.net                        **
**                                                                           **
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/


#include "ODML_Index_Entry.h"
#include "debug.h"




ODML_StandardIndexEntry::ODML_StandardIndexEntry()
{
    // Datenstruktur initialisieren
    DataOffset = 0;
    DataSize = 0;
}





ODML_StandardIndexEntry::~ODML_StandardIndexEntry()
{

}



/**Einlesen der Datenstruktur aus einem Puffer
  */
bool ODML_StandardIndexEntry::Read( uint8_t *buf, size_t length )
{
    if( length < Size() )
        return false;
        
    read_int( DataOffset, buf, length );
    read_int( DataSize, buf, length );

    return true;
}





/**Schreiben der Datenstruktur in einen Puffer
  */
bool ODML_StandardIndexEntry::Write( uint8_t *buf, size_t length )
{
    if( length < Size() )
        return false;

    write_int( DataOffset, buf, length );
    write_int( DataSize, buf, length );
    
    return true;
    
}












ODML_FieldIndexEntry::ODML_FieldIndexEntry()
{
    // Datenstruktur initialisieren
    DataOffsetField2 = 0;
}





ODML_FieldIndexEntry::~ODML_FieldIndexEntry()
{

}



/**Einlesen der Datenstruktur aus einem Puffer
  */
bool ODML_FieldIndexEntry::Read( uint8_t *buf, size_t length )
{
    if( length < Size() )
        return false;

    ODML_StandardIndexEntry::Read( buf, length );
    buf += ODML_StandardIndexEntry::Size();
    read_int( DataOffsetField2, buf, length );

    return true;
}





/**Schreiben der Datenstruktur in einen Puffer
  */
bool ODML_FieldIndexEntry::Write( uint8_t *buf, size_t length )
{
    if( length < Size() )
        return false;

    ODML_StandardIndexEntry::Write( buf, length );
    buf += ODML_StandardIndexEntry::Size();
    write_int( DataOffsetField2, buf, length );


    return true;

}










ODML_SuperIndexEntry::ODML_SuperIndexEntry()
{
    // Datenstruktur initialisieren
    ChunkOffset = 0;
    DataSize = 0;
    Duration = 0;
}





ODML_SuperIndexEntry::~ODML_SuperIndexEntry()
{

}



/**Einlesen der Datenstruktur aus einem Puffer
  */
bool ODML_SuperIndexEntry::Read( uint8_t *buf, size_t length )
{
    if( length < Size() )
        return false;

    read_int( ChunkOffset, buf, length );
    read_int( DataSize, buf, length );
    read_int( Duration, buf, length );

    return true;
}





/**Schreiben der Datenstruktur in einen Puffer
  */
bool ODML_SuperIndexEntry::Write( uint8_t *buf, size_t length )
{
    if( length < Size() )
        return false;

    write_int( ChunkOffset, buf, length );
    write_int( DataSize, buf, length );
    write_int( Duration, buf, length );

    return true;

}
