/********************************************************************\
*****   Methoden fr IO in Avi Dateien                           *****
*****   Datentyp Indexeintrag fr erweiterten OpenDML-Index      *****
\********************************************************************/

#ifndef _ODML_INDEX_ENTRY_H_
#define _ODML_INDEX_ENTRY_H_

#include "plugins/RIFFio/RIFF_Chunk.h"
#include "plugins/RIFFio/RIFF_ITypes.h"





class ODML_StandardIndexEntry: public RIFF_Chunk, public RIFF_ITypes {
public:
    ODML_StandardIndexEntry();
    virtual ~ODML_StandardIndexEntry();

    // Datenstruktur
    uint32_t DataOffset;       // Beginn der Daten, der Chunk fngt
                               // bei <BaseOffset + DataOffset - 8> Bytes an
                               // DataOffset ist relativ zu BaseOffset im Index-Header
    uint32_t DataSize;
    
    // Methoden
    /**Einlesen der Datenstruktur aus einem Puffer
      */
    virtual bool Read( uint8_t *buf, size_t length );

    /**Schreiben der Datenstruktur in einen Puffer
      */
    virtual bool Write( uint8_t *buf, size_t length );

    /**liefert die Gre des Datenpuffers
      */
    virtual size_t Size(void) { return 8; }

};






class ODML_FieldIndexEntry: public ODML_StandardIndexEntry {
public:
    ODML_FieldIndexEntry();
    ~ODML_FieldIndexEntry();

    // Datenstruktur
    uint32_t DataOffsetField2;

    // Methoden
    /**Einlesen der Datenstruktur aus einem Puffer
      */
    bool Read( uint8_t *buf, size_t length );

    /**Schreiben der Datenstruktur in einen Puffer
      */
    bool Write( uint8_t *buf, size_t length );

    /**liefert die Gre des Datenpuffers
      */
    size_t Size(void) { return ODML_StandardIndexEntry::Size() + 4; }
};






class ODML_SuperIndexEntry: public RIFF_Chunk, public RIFF_ITypes {
public:
    ODML_SuperIndexEntry();
    virtual ~ODML_SuperIndexEntry();

    // Datenstruktur
    uint64_t ChunkOffset;       // Begin des Chunks, die Daten
                                // fangen bei ChunkOffset + 8 an
    uint32_t DataSize;          // Gre der Daten
    uint32_t Duration;

    // Methoden
    /**Einlesen der Datenstruktur aus einem Puffer
      */
    bool Read( uint8_t *buf, size_t length );

    /**Schreiben der Datenstruktur in einen Puffer
      */
    bool Write( uint8_t *buf, size_t length );

    /**liefert die Gre des Datenpuffers
      */
    size_t Size(void) { return 16; }

};

#endif
