/*****************************************************************************\
**  Author  : Christian Tusche                                               **
**  E-Mail  : ctusche@gmx.net                        **
**                                                                           **
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/


#include "ODML_Index_Header.h"
#include <stdio.h>




ODML_IndexHeader::ODML_IndexHeader()
{
    // Datenstruktur initialisieren
    LongsPerEntry = 2;
    IndexSubType = 0;
    IndexType = AVI_INDEX_OF_CHUNKS; 
    EntriesInUse = 0;
    ChunkId = 0;
    BaseOffset=0;
    Reserved=0;
}





ODML_IndexHeader::~ODML_IndexHeader()
{

}



/**Einlesen der Datenstruktur aus einem Puffer
  */
bool ODML_IndexHeader::Read( uint8_t *buf, size_t length )
{
    if( length < Size() )
        return false;
        
    read_int( LongsPerEntry, buf, length );
    read_int( IndexSubType, buf, length );
    read_int( IndexType, buf, length );
    read_int( EntriesInUse, buf, length );
    read_int( ChunkId, buf, length );
    read_int( BaseOffset, buf, length );
    read_int( Reserved, buf, length );

    return true;
}





/**Schreiben der Datenstruktur in einen Puffer
  */
bool ODML_IndexHeader::Write( uint8_t *buf, size_t length )
{
    if( length < Size() )
        return false;

    write_int( LongsPerEntry, buf, length );
    write_int( IndexSubType, buf, length );
    write_int( IndexType, buf, length );
    write_int( EntriesInUse, buf, length );
    write_int( ChunkId, buf, length );
    write_int( BaseOffset, buf, length );
    write_int( Reserved, buf, length );
    
    return true;
    
}




/**Werte Ausgeben
  */
void ODML_IndexHeader::printInfo( void )
{
    uint8_t Z[4];
    
    printf( "    LongsPerEntry = %i\n", LongsPerEntry );
    printf( "    IndexSubType = %i\n", IndexSubType );
    printf( "    IndexType = %i\n", IndexType );
    printf( "    EntriesInUse = %i\n", EntriesInUse );
    write_int( ChunkId, Z );
    printf( "    ChunkId = %c%c%c%c\n", Z[0], Z[1], Z[2], Z[3] );
    printf( "    BaseOffset = %lli\n", BaseOffset );
    printf( "    Reserved = %i\n", Reserved );
}
