/********************************************************************\
*****   Methoden fr IO in Avi Dateien                           *****
*****   Datentyp Header fr erweiterten OpenDML-Index            *****
\********************************************************************/

#ifndef _ODML_INDEX_HEADER_H_
#define _ODML_INDEX_HEADER_H_

#include "plugins/RIFFio/RIFF_Chunk.h"
#include "plugins/RIFFio/RIFF_ITypes.h"

// Werte fr IndexType
#define AVI_INDEX_OF_INDEXES            0x00
#define AVI_INDEX_OF_CHUNKS             0x01
#define AVI_INDEX_IS_DATA               0x80

// Werte fr IndexSubType
#define AVI_INDEX_2FIELD                0x01





class ODML_IndexHeader: public RIFF_Chunk, public RIFF_ITypes {
public:
    ODML_IndexHeader();
    virtual ~ODML_IndexHeader();

    // Datenstruktur
    uint16_t LongsPerEntry;        // Gre eines Indexeintrages als Anzahl der 32 Bit Wrter
                                   // LongsPerEntry * 4 ist die Anzahl der Bytes
    uint8_t  IndexSubType;         // =1 : Field-Index
    uint8_t  IndexType;            // Typ des Index-Headers und der Index-Eintrge
    uint32_t EntriesInUse;         // Anzahl der Indexeintrge
    uint32_t ChunkId;              // Id des Tracks zudem dieser Index gehrt

    uint64_t BaseOffset;           // Die Offsets im Indexeintrag sind relativ dazu
                                   // <Offset in der Datei> = <BaseOffset> + <Offset>
                                   // verwendet fr InfexType = AVI_INDEX_OF_CHUNKS
                                   // sonst =0
    uint32_t Reserved;             // Platzhalter = 0

    // Die Indexeintrge folgen nach dieser Datenstruktur
    
    // Methoden
    /**Einlesen der Datenstruktur aus einem Puffer
      */
    virtual bool Read( uint8_t *buf, size_t length );

    /**Schreiben der Datenstruktur in einen Puffer
      */
    virtual bool Write( uint8_t *buf, size_t length );

    /**liefert die Gre des Datenpuffers
      */
    virtual size_t Size(void) { return 24; }

    virtual void printInfo( void ) ;

};

#endif
