/*****************************************************************************\
**  Author  : Christian Tusche                                               **
**  E-Mail  : ctusche@gmx.net                        **
**                                                                           **
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/

/********************************************************************\
*****   Methoden fr IO in Avi Dateien                           *****
*****   Datentyp AVI-Header                                      *****
\********************************************************************/


#include "Stream_Header.h"
#include "debug.h"
#include "AVI_Codes.h"


StreamHeader::StreamHeader()
{
    // Datenstruktur
    StreamType = 0;
    StreamHandler = 0;
    Flags = 0;
    Priority = 0;
    Language = 0;
    InitialFrames = 0;
    Scale = 1000;
    Rate = 25000;
    Start = 0;
    Length = 0;
    SuggestedBufferSize = 0;
    Quality = 0;
    SampleSize = 0;
    left = 0;
    top = 0;
    right = 0;
    bottom = 0;
}










/**Einlesen der Datenstruktur aus einem Puffer
  */
bool StreamHeader::Read( uint8_t *buf, size_t length )
{
    if( length < Size() )
        return false;

    read_int( StreamType, buf, length );
    read_int( StreamHandler, buf, length );
    read_int( Flags, buf, length );
    read_int( Priority, buf, length );
    read_int( Language, buf, length );
    read_int( InitialFrames, buf, length );
    read_int( Scale, buf, length );
    read_int( Rate, buf, length );
    read_int( Start, buf, length );
    read_int( Length, buf, length );
    read_int( SuggestedBufferSize, buf, length );
    read_int( Quality, buf, length );
    read_int( SampleSize, buf, length );
    read_int( left, buf, length );
    read_int( top, buf, length );
    read_int( right, buf, length );
    read_int( bottom, buf, length );

    return true;
}





/**Schreiben der Datenstruktur in einen Puffer
  */
bool StreamHeader::Write( uint8_t *buf, size_t length )
{
    if( length < Size() )
        return false;

    write_int( StreamType, buf, length );
    write_int( StreamHandler, buf, length );
    write_int( Flags, buf, length );
    write_int( Priority, buf, length );
    write_int( Language, buf, length ); 
    write_int( InitialFrames, buf, length );
    write_int( Scale, buf, length );
    write_int( Rate, buf, length );
    write_int( Start, buf, length );
    write_int( Length, buf, length );
    write_int( SuggestedBufferSize, buf, length );
    write_int( Quality, buf, length );
    write_int( SampleSize, buf, length );
    write_int( left, buf, length );
    write_int( top, buf, length );
    write_int( right, buf, length );
    write_int( bottom, buf, length );

    return true;
}







/**Werte Ausgeben
  */
void StreamHeader::printInfo( void )
{
    uint8_t Z[4];

    write_int( StreamType, Z );
    printf( "    StreamType = %c%c%c%c\n", Z[0], Z[1], Z[2], Z[3] );
    write_int( StreamHandler, Z );
    printf( "    StreamHandler = %c%c%c%c\n", Z[0], Z[1], Z[2], Z[3] );
    printf( "    Flags = 0x%08x\n", Flags );
    printf( "    Priority = %i\n", Priority );
    printf( "    Language = %i\n", Language );
    printf( "    InitialFrames = %i\n", InitialFrames );
    printf( "    Scale = %i\n", Scale );
    printf( "    Rate = %i\n", Rate );
    printf( "    Start = %i\n", Start );
    printf( "    Length = %i\n", Length );
    printf( "    SuggestedBufferSize = %i\n", SuggestedBufferSize );
    printf( "    Quality = %i\n", Quality );
    printf( "    SampleSize = %i\n", SampleSize );
    printf( "    left = %i\n", left );
    printf( "    top = %i\n", top );
    printf( "    right = %i\n", right );
    printf( "    bottom = %i\n", bottom );
}
