/********************************************************************\
*****   Methoden fr IO in MPPEG-Video Dateien                   *****
\********************************************************************/

#ifndef _FILE_PUFFER_H_
#define _FILE_PUFFER_H_

#include <inttypes.h>
#include <unistd.h>
#include "puffer.h"

class FilePuffer : public FifoPuffer {
public:
    FilePuffer( int file, size_t gr );
    
    ~FilePuffer();

    /**Daten in Puffer laden<br>
      *Rckgabe:  1  Daten gelesen<br>
      *           0  keine Daten gelesen / eof<br>
      *          -1  Fehler
      */
    int NeedData( size_t n );
    
    /**Puffer um n Bytes weiterschieben
      */
    void ClearData( size_t n );

    /**Puffer leeren
      */
    void Reset( void );

    /**Finde "muster" im Puffer<br>
      *gesucht wird ab der Position "offset"<br>
      *Rckgabe: Anfangsposition<br>
      *          oder -1, wenn nicht gefunden
      */
    long Finden( size_t offset, const unsigned char *muster, size_t n );
    
    /**Finde "muster" im Puffer<br>
      *Verglichen werden nur die Bits aus "maske", die 1 sind<br>
      *gesucht wird ab der Position "offset"<br>
      *das gefundene Muster wird in "&gefunden" gespeichert
      *Rckgabe: Anfangsposition<br>
      *          oder -1, wenn nicht gefunden
      */
    long Finden( size_t offset, uint32_t muster, uint32_t maske, uint32_t &gefunden );

    /**Zeiger auf Datenfeld an Stelle "offset"
      */
    uint8_t Data( size_t offset );

    /**Daten aus Puffer kopieren
      */
    int GetData( size_t offset, uint8_t *data, size_t n );

    /**End of File
      */
    bool Eof( void ) { return eof && bytes == 0; }

    /**Offset am Start des Puffers
      */
    off_t Offset( void ) { return file_offset - bytes; }

    /**Eingabedatei positionieren<br>
      *Rckgabe: 1   Erfolg<br>
      *          0   eof<br>
      *          -1  Fehler
      */
    int Seek( off_t offset );

    /**testet, ob die Datei positioniert werden kann
      */
    bool CanSeek( void ) { return stream != STDIN_FILENO; }

    /**Gre der Datei<br>
      *Rckgabe: >=0   Bytes<br>
      *           -1   Fehler / keine Datei
      */
    off_t FileSize( void );

private:
    /**Puffer aus Eingabedatei fllen<br>
      *Rckgabe:  1  Daten gelesen<br>
      *           0  keine Daten gelesen / eof<br>
      *          -1  Fehler
      */
    int Fuellen( void );

    
    unsigned char *puffer;      // Puffer fr gelesene Daten
    
    int stream;                 // Datei aus der gelesen wird
    
    size_t groesse;             // Gre des Puffers
    size_t position;            // aktuelle Startposition der Daten
    bool eof;
    off64_t file_offset;        // Offset am Start des Puffers
    off64_t file_length;        // Dateilnge

    /**Offset einstellen
      */
    void set_offset( size_t bytes_read );
};

#endif
