/********************************************************************\
*****   Methoden fr IO in MPEG-Video Dateien                   *****
\********************************************************************/

#ifndef _MPEG_H_
#define _MPEG_H_

using namespace std;

#include <fstream>
#include <unistd.h>
#include "VideoFile/VideoFile.h"
#include "keytable.h"
#include "FilePuffer.h"
#include "System.h"
#include "System_Write.h"
#include "StreamPuffer.h"
#include "MpegTrack.h"

// Puffer festlegen - maximale Gre: 2 umkompremierte Bilder mit 800*600 Punkten RGB24
#define FILE_PUFFER_GR 2*800*600*3
#define PACK_PUFFER 1000


class MpegTrack;
class MpegVtr;
class MpegAtr;
class Chunk;


class MpegIO:public VideoFile {
    friend class MpegTrack;
    friend class MpegVtr;
    friend class MpegAtr;
    friend class Chunk;
public:
    MpegIO( const String datei, IO_Mode mode=write, int mpegtype=1 );
    ~MpegIO();

protected:
    /*** Videofunktionen ***/
    /**Video codec initialisieren
      */
    V_Track *openVideoTrack( double Rate,
                             int x,
                             int y,
                             const char *Codec = NULL );
    /*** Audiofunktionen ***/
    /**Audio codec initialisieren
      */
    A_Track *openAudioTrack( int Sample_rate,
                             int Channels,
                             const char *Codec = NULL );

    /**Video-Track ffnen
      */
    V_Track *openVideoTrack( int track );
    
    /**Audio-Track ffnen
      */
    A_Track *openAudioTrack( int track );
private:
    /**Dateityp prfen und auf Start der Daten synchronisieren
      */
    bool sync_block_start( void );

    /**Stream-Puffer aus Eingabedatei fllen<br>
      *Rckgabe:  1  Daten gelesen<br>
      *           0  keine Daten gelesen / eof<br>
      *          -1  Fehler
      */
    int FillStreamPuffer( MpegTrack *tr = NULL );

    /**kleinste Anzahl der Packs aus allen Puffern
      */
    //int stream_puffer_packs( void );

    /**Stream Puffer zu Track finden / anlegen
      */
    StreamPuffer *finde_stream_puffer( int type, int num=-1 );

    /**Stream-Puffer zu einem Stream multiplexen
      */
    int multiplex_puffer( MpegTrack *tr = NULL );
    


    int mpeg_datei;                  // Datei - lesen/schreiben

    FilePuffer *puffer;
    System *system;                  // Lesen
    System_Write *wsystem;           // Schreiben

    StreamPuffer *vstream[16];       // Video Streams
    StreamPuffer *astream[16];       // Audio Streams

    int mpeg_system_type;
    off_t mpeg_start_offset;         // Start der Mpeg-Daten im Stream
};

#endif
