/********************************************************************\
*****   Methoden fr IO in MPEG-Video Dateien                   *****
\********************************************************************/

#ifndef _MPEG_VTR_H_
#define _MPEG_VTR_H_

using namespace std;
#include <fstream>

#include "Mpeg.h"
#include "MpegTrack.h"

#include "VideoFile/VideoFile.h"
#include "chunk.h"



class MpegVtr: public MpegTrack, public V_Track{
public:

    MpegVtr( MpegIO *File, int track=0 );
    MpegVtr( MpegIO *File,
            double Rate,
            int x,
            int y,
            const char *Codec_name,
            int track = 0 );


    ~MpegVtr();


    /**Audio-Position setzen
      */
    int Seek( long block );

    /**liefert den letzten Keyframe zur angegebenen Position
      */
    long GetKeyFrame( long bild );

private:
    Chunk *chunk;

    // Informationen fr Videotrack
    int aspect_ratio_code;
    int bitrate;
    
    // Ein Bild berspringen
    int skip_frame( void );

    unsigned char *sequence_header;     // Zwischenspeicher fr Sequence Header
    size_t sequence_header_gr;
    bool sequence_header_gelesen;       // Der Header wurde schon einmal vom Codec gelesen

    /**decodiere Sequence-Header
      */
    int decode_header( unsigned char *head, int bytes );

    /**finde einen Sequence-Header und kopiere ihn
      */
    int sync_header( void );

    /**Mpeg-Frameratecode in Bilder/Sec. umwandeln
      */
    double decode_framerate( int code );
 
    unsigned char *data_puffer;         // Puffer fr gelesene Daten
    long data_puffer_gr;

    /**Prfen, ob "data" einen I-Frame enthlt
      */
    bool test_keyframe( void );

    /**Timestam berechnen
      */
    void calc_timestamp( uint8_t *buf, size_t bytes, double &pts, double &dts );
    uint32_t group_start_pic, group_start_field, fields_presented, decoding_order;


    static const double mpeg_framerates[];
protected:
    int readRaw( unsigned char *&data, long &bytes );
    int writeRaw( unsigned char *data, long bytes, int keyframe );

};

#endif
