/*****************************************************************************\
**  Author  : Christian Tusche                                               **
**  E-Mail  : ctusche@gmx.net                        **
**                                                                           **
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/




#include <string.h>
#include <assert.h>

#include "StreamPuffer.h"



StreamPuffer::StreamPuffer( int n_packs )
{
    puffer_gesamt = n_packs;
    puffer = new PACK_DATA[puffer_gesamt];

    stream_type = -1;
    stream_num = -1;

    //Puffer initialisieren
    for( int i=0; i<puffer_gesamt; i++ ) {
        puffer[i].data = NULL;
        puffer[i].data_size = 0;
    }

    // Pufferverwaltung initialisieren
    Reset();
}





StreamPuffer::~StreamPuffer()
{
    for( int i=0; i<puffer_gesamt; i++ ) {
        delete [] puffer[i].data;
    }

    delete [] puffer;
}




/**Paket in Puffer einreihen<br>
  *Rckgabe:  1  Paket eingereiht<br>
  *           0  Puffer voll / eof<br>
  *          -1  Fehler
  */
int StreamPuffer::AddPack( uint8_t *&data, size_t n, double timestamp, off_t file_offset )
{

    // prfen, ob noch Platz im Puffer ist
    if( FreePacks() == 0 )
        return 0;
    else if( FreePacks() < 0 )
        return -1;
        
    // freien Puffer finden
    int i;
    for( i=0; i<puffer_gesamt; i++ ) {
        if( puffer[i].bytes == 0 )
            break;
    }
    assert( i < puffer_gesamt );

    // Daten in Puffer kopieren
    if( puffer[i].data_size < n ) {
        delete [] puffer[i].data;
        puffer[i].data_size = n;
        puffer[i].data = new uint8_t[n];
    }
    data = puffer[i].data;
    
    puffer[i].bytes = n;
    // memcpy( puffer[i].data, data, n );
    puffer[i].timestamp = timestamp;
    puffer[i].file_offset = file_offset;
    puffer[i].nummer = ++pakete_gelesen;
    puffer[i].position = 0;
    
    if( timestamp > 0.0 )
        have_timestamps = true;

    // Puffer in Liste einreihen
    if( letztes_paket >= 0 ) {
        puffer[letztes_paket].naechstes_paket = i;
        letztes_paket = i;
    } else {
        erstes_paket = i;
        letztes_paket = i;
    }
    puffer[i].naechstes_paket = -1;            

    ++puffer_belegt;
    bytes += n;

    return 1;
}









/**Puffer weiterschieben
  */
void StreamPuffer::ClearData( size_t n )
{
    size_t pos = 0;

    if( n > bytes )
        n = bytes;

    // finde Paket, in dem offset liegt
    while( puffer_belegt > 0 ) {
        pos += puffer[erstes_paket].bytes - puffer[erstes_paket].position;
        if( pos > n )
            break;

        // Paket lschen
        ClearPack();
    }

    // Rest vom ersten Paket lschen
    if( erstes_paket >= 0 ) {
        bytes -= n + puffer[erstes_paket].bytes - puffer[erstes_paket].position - pos;
        puffer[erstes_paket].position += n + puffer[erstes_paket].bytes - puffer[erstes_paket].position - pos;
    }

    return;
}




void StreamPuffer::ClearPack( void )
{
    int pack = erstes_paket;

    if( pack < 0 )
        return;
        
    erstes_paket = puffer[pack].naechstes_paket;
    bytes -= puffer[pack].bytes - puffer[pack].position;
    
    puffer[pack].bytes=0;
    puffer[pack].position = 0;
    puffer[pack].naechstes_paket = -1;
    puffer_belegt--;
    if( pack == letztes_paket ) {
        letztes_paket = -1;
        bytes = 0;
    }
}





/**Puffer leeren
  */
void StreamPuffer::Reset( void )
{
    //Puffer lschen
    for( int i=0; i<puffer_gesamt; i++ ) {
        puffer[i].timestamp = 0.0;
        puffer[i].file_offset = 0;
        puffer[i].nummer = 0;
        puffer[i].bytes = 0;
        puffer[i].position = 0;
        puffer[i].naechstes_paket = -1;
    }
    
    bytes = 0;
    puffer_belegt = 0;
    pakete_gelesen = 0;
    erstes_paket = -1;
    letztes_paket = -1;
    have_timestamps = false;
}














/**Zeiger auf Datenfeld an Stelle "offset"
  */
uint8_t StreamPuffer::Data( size_t offset )
{
    size_t pos = 0;
    int pack;
    
    if( offset >= bytes )
        return 0;

    // finde Paket, in dem offset liegt
    pack = erstes_paket;
    for( int i=0; i< puffer_gesamt; ++i ) {
        pos += puffer[pack].bytes - puffer[pack].position;
        if( pos > offset )
            break;
        pack = puffer[pack].naechstes_paket;
    }

    return puffer[pack].data[offset + puffer[pack].bytes - pos];
}




/**Zeiger auf Paket n im Puffer
  */
int StreamPuffer::Pack( int n )
{
    int buf = -1;
    
    if( erstes_paket >= 0 && Packs() > n ) {
        buf = erstes_paket;

        for( int i = 0; i<n; i++ ) {
            buf = puffer[buf].naechstes_paket;
        }
    }

    return buf;
}

