/********************************************************************\
*****   Methoden fr IO in MPEG-Video Dateien                    *****
\********************************************************************/

#ifndef _STREAM_PUFFER_H_
#define _STREAM_PUFFER_H_

#include <unistd.h>
#include <inttypes.h>
#include "puffer.h"

class StreamPuffer : public FifoPuffer{
public:
    StreamPuffer( int n_packs );
    ~StreamPuffer();

    /**Anzahl der freien Pakete
      */
    int FreePacks( void ) { return puffer_gesamt - puffer_belegt; }

    /**Anzahl der belegten Pakete
      */
    int Packs( void ) { return puffer_belegt; }

    /**Paket in Puffer einreihen<br>
      **data wird auf das Datenfeld des Puffers gesetzt, und mu anschlieend<br>
      *beschrieben werden. *data hat mindestens die Gre n.
      *Rckgabe:  1  Paket eingereiht<br>
      *           0  Puffer voll / eof<br>
      *          -1  Fehler
      */
    int AddPack( uint8_t *&data, size_t n, double timestamp, off_t file_offset = -1 );
    
    /**n Bytes aus dem Puffer entfernen
      */
    void ClearData( size_t n );

    /**erstes Paket aus dem Puffer entfernen
      */
    void ClearPack( void );

    /**Puffer leeren
      */
    void Reset( void );

    /**Zeiger auf Datenfeld an Stelle "offset"
      */
    unsigned char Data( size_t offset );

    /**Streamtype und Nummer setzen
      */
    void SetTrackID( int type, int num ) { stream_type = type; stream_num = num; }

    /**Streamtype<br>
      *Typ des Streams: 0xC0 - mp123, 0xE0 - Video, 0x80 - AC3, ...
      */
    int Tracktype( void ) { return stream_type; }

    /**Streamnummer<br>
      *Tracknummer 0..15
      */
    int Tracknumber( void ) { return stream_num; }

    /**gibt es Timestamps im Stream?
      */
    bool HaveTimestamps() { return have_timestamps; }

    /**Timestamp des n-ten Pakets
      */
    double Timestamp( int n=0 ) { if( Pack(n) >= 0 ) return puffer[Pack(n)].timestamp; return 0; }

    /**Offset des n-ten Puffers
      */
    off_t FileOffset( int n=0 ) { if( Pack(n) >= 0 ) return puffer[Pack(n)].file_offset; return 0; }
    
    /**Bytes im n-ten Paket im Puffer
      */
    size_t PackBytes( int n=0 ) { if( Pack(n) >= 0 ) return puffer[Pack(n)].bytes; return 0; }
    

private:
    
    typedef struct {
        uint8_t *data;
        size_t data_size;
        double timestamp;
        off_t file_offset;
        uint32_t nummer;
        size_t bytes;
        size_t position;
        int naechstes_paket;
    } PACK_DATA;

    /**Index von Paket n im Puffer
      */
    int Pack( int n );
    
    PACK_DATA *puffer;          // Puffer fr gelesene MP3-Daten
    int puffer_gesamt;          // Anzahl der Pakete
    int puffer_belegt;          // Anzahl der Pakete im Puffer
    uint32_t pakete_gelesen;    // laufende Nummer des Paketes
    int erstes_paket;           // erstes Paket in der Liste
    int letztes_paket;          // letztes Paket in der Liste
    int stream_type;            // Typ des Streams: 0xC0 - mp123, 0xE0 - Video, 0x80 - AC3, ...
    int stream_num;             // Nummer des Streams: 0..15
    bool have_timestamps;       // gibt es Timestamps im Stream
};

#endif
