/********************************************************************\
*****   Methoden fr IO in MPEG-Video Dateien                    *****
*****   System Stream Demultiplexen                              *****
\********************************************************************/

#ifndef _SYSTEM_H_
#define _SYSTEM_H_

#include <unistd.h>
#include "FilePuffer.h"


class System {
public:
    /**bernimmt alle informationen ber den Mpeg-Stream aus *sys
      */
    System( System *sys, FilePuffer *Puffer, off_t offset = 0 );
    
    System( FilePuffer *Puffer, off_t offset = 0 );
    ~System();

    /**Nchsten Pack-Header lesen und parsen<br>
      *Rckgabe: >0 Erfolg<br>
      *           0 keine weiteren Daten<br>
      *          -1 Fehler
      */
    int NextPack( void );
     
    /**Das nchste Paket aus dem Puffer in das Datenfeld kopieren<br>
      *Rckgabe: >0  erfolg<br>
      *           0  keine weiteren Daten<br>
      *          -1  Fehler
      */
    int GetPackData( uint8_t *data, size_t &bytes );

    /**Bytes im Paket
      */
    long DataBytes( void );
    
    /**Liefert den Tracktyp des nchsten Blocks<br>
      *Rckgabe: >=0  Tracktyp<br>
      *               0xC0 - mp2 Audio<br>
      *               0xE0 - mpeg Video<br>
      *               0x20 - Subtitel<br>
      *               0x80 - AC3 Audio<br>
      *               0xA0 - PCM Audio<br>
      *           -1  Fehler
      */
    int Tracktype( void ) { return (track_code & 0xE0); }

    /**Tracknummer des nchsten Pakets<br>
      * Rckgabe: 0..31 Track Nummer<br>
      *           -1    Fehler
      */
    int Tracknumber( void ) { return (track_code & 0x1F); }

    /**Timestamp des nchsten Pakets
      */
    double Timestamp( void ) { return timestamp; }

    /**Offset des aktuellen Paketes in der Datei
      */
    off_t FileOffset( void ) { return file_offset; }

    /**Bytes im nchsten Paket
      */
    long Bytes( void );

    /**Mpeg 1 oder 2<br>
      * Rckgabe: 1/2   Mpeg1/2<br>
      *           0     Fehler
      */
    int MpegType( void ) { return mpeg_type; }
   

    /**Zeichen an Offset im Chunk
      */
    uint8_t Data( size_t offset );

    /**System Stream positionieren<br>
      *Rckgabe: >0  erfolg<br>
      *           0  eof<br>
      *          -1  Fehler
      */
    int SeekTime( double sec );
    
    /**System Stream positionieren<br>
      *Rckgabe: >0  erfolg<br>
      *           0  eof<br>
      *          -1  Fehler
      */
    int SeekOffset( off_t offset );
    
    /**gibt es Timestamps im Stream?
      */
    bool HaveTimestamps() { return have_timestamps; }

    /**EndZeit des Streams
      */
    double TotalTime() { return end_time; }

    
private:
    /**Auf den Start eines Pakets synchronisieren<br>
      *Rckgabe: >0 Erfolg<br>
      *           0 keine weiteren Daten<br>
      *          -1 Fehler
      */
    int Sync( void );

    /**Pack Header lesen und parsen<br>
      *Rckgabe: >0 Erfolg<br>
      *          -1 Fehler
      */
    int ParsePackHeader( void );

    /**Sonstige Header lesen und parsen<br>
      *Rckgabe: >0 Erfolg<br>
      *          -1 Fehler
      */
    int ParseExtHeader( void );
    
    /**PES Header lesen und parsen<br>
      *Rckgabe: >0 Erfolg<br>
      *          -1 Fehler
      */
    int ParsePesHeader( void );
    
    /**Timestamp lesen<br>
      *Rckgabe: >0 Timestamp in Sec.<br>
      *          <0 Fehler
      */
    double DecodeTimestamp( size_t offset );

    /**Endzeit und Dateigre einlesen
     */
    void get_end_time( void );

    /**Datei initialisieren<br>
      *such nach Timestamps und ermittle Dateigre und End-Zeit
      */
    int init_system( void );


          
    FilePuffer *puffer;
    double timestamp, last_timestamp, start_timestamp, end_time;

    // 0xC0..0xCF  Mp123 Audio
    // 0xE0..0xEF  Mpeg 1/2 Video
    // 0x20..0x3F  subtitle
    // 0x80..0x9F  AC3 Audio
    // 0xA0..0xBF  PCM Audio
    int track_code;
    long data_size;
    bool css;
    int mpeg_type;
    off_t start_offset, file_offset, file_size;
    bool have_timestamps, init;
};

#endif
