/********************************************************************\
*****   Methoden fr IO in MPEG-Video Dateien                    *****
*****   System Stream schreiben                                  *****
\********************************************************************/

#ifndef _SYSTEM_WRITE_H_
#define _SYSTEM_WRITE_H_


#include <unistd.h>
#include "StreamPuffer.h"


class System_Write {
public:
    System_Write( int fileno, int MpegType=1 );
    ~System_Write();

    /**Nchstes Packet schreiben<br>
      *Rckgabe: >0 Erfolg<br>
      *          -1 Fehler
      */
    int WritePack( StreamPuffer *Puffer );

    /**setze Video-Tracks
      */
    void SetVideoTracks( int n ) { v_tracks = n; }

    /**setze Audio-Tracks
      */
    void SetAudioTracks( int n ) { a_tracks = n; }
    
private:
    /**Pack Header schreiben<br>
      *Rckgabe: geschriebene Bytes
      */
    int WritePackHeader( uint8_t *buf, double timestamp );

    /**Mpeg1-PES Header schreiben<br>
      *Rckgabe: geschriebene Bytes
      */
    int WriteMpeg1PesHeader( uint8_t *buf, int track_code , int data_size, double pts, double dts );

    /**Mpeg2-PES Header schreiben<br>
      *Rckgabe: geschriebene Bytes
      */
    int WriteMpeg2PesHeader( uint8_t *buf, int track_code , int data_size, double pts, double dts );

    
    /**Sys-Headerschreiben
      *Rckgabe: geschriebene Bytes
      */
    int WriteSysHeader( uint8_t *buf, uint32_t mux_rate, bool vbr );
    
    /**Timestamp schreiben<br>
      *Rckgabe: >0 Timestamp in Sec.<br>
      *          <0 Fehler
      */
    void WriteTimestamp( double Seconds, uint8_t *data );

    /**Mpeg2 SCR-Timestamp schreiben<br>
      *Rckgabe: >0 Timestamp in Sec.<br>
      *          <0 Fehler
      */
    void WriteTimestampMpeg2( double Seconds, uint8_t *data );
          
    int stream;

    // 0xC0..0xCF  Mp123 Audio
    // 0xE0..0xEF  Mpeg 1/2 Video
    // 0x20..0x3F  subtitle
    // 0x80..0x9F  AC3 Audio
    // 0xA0..0xBF  PCM Audio
    double last_timestamp[256];

    int mpeg_type;

    uint8_t *pack_data, *header_data;
    uint64_t bytes_written;

    // Video- und Audio-Tracks
    int v_tracks, a_tracks;

    //System Header schon geschrieben
    bool sys_header;

    //variable Bitrate
    bool VBR;
};

#endif
