/********************************************************************\
*****   Methoden fr IO in MPEG-Video Dateien                    *****
\********************************************************************/

#ifndef _CHUNK_H_
#define _CHUNK_H_

#include "StreamPuffer.h"
#include "Mpeg.h"
#include "MpegTrack.h"



class Chunk {
public:
    Chunk( MpegIO *file, MpegTrack *tr );
    ~Chunk();

    /**Den nchsten Block aus dem Puffer in das Datenfeld kopieren<br>
      *Rckgabe: >0  erfolg<br>
      *           0  keine weiteren Daten<br>
      *          -1  Fehler
      */
    int Lesen( uint8_t *data, size_t &bytes );

    /**Liefert den Startcode des nchsten Blocks<br>
      *Rckgabe: >=0  Startcode<br>
      *           -1  Fehler
      */
    int Startcode( void );

    /**berspringt den nchsten Block
      */
    int Skip( void );

    /**Bytes im nchsten Block
      */
    long Bytes( void );

    /**Zeichen an Offset im Chunk
      */
    uint8_t Data( size_t offset );

    /**Puffer aus Eingabedatei fllen<br>
      *Rckgabe:  1  Daten gelesen<br>
      *           0  keine Daten gelesen / eof<br>
      *          -1  Fehler
      */
    int PufferFuellen( void ) { return mpeg_file->FillStreamPuffer( track ); }
    
private:
    /**Auf den Start eines Blockes synchronisieren<br>
      *Rckgabe: >0 Erfolg<br>
      *           0 keine weiteren Daten<br>
      *          -1 Fehler
      */
    int Sync( void );
    
    StreamPuffer *puffer;
    MpegTrack *track;
    MpegIO *mpeg_file;
};

#endif
