/*****************************************************************************\
**  Author  : Christian Tusche                                               **
**  E-Mail  : ctusche@gmx.net                        **
**                                                                           **
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/


#include "keytable.h"
#include "debug.h"




KeyTable::KeyTable()
{

}



KeyTable::~KeyTable()
{
    DEBUG_TXT( "lsche Keytable mit %i Eintrgen", table.size() );
    while( table.size() > 0 ) {
        delete table.back();
        table.pop_back();
    }
    DEBUG_TXT( "Keytable gelscht" );
}




void KeyTable::AddKeyFrame( long frame, long long offset )
{
    // suche nach Key-Frame in Tabelle
    unsigned int i, size;
    list<key_entry*>::iterator element;

    size = table.size();
    element = table.begin();

    // Suche Bild in Liste
    for( i=0; i < size; i++ ) {
        if( (*element)->frame >= frame )
            break;
        element++;
    }

    // Bild schon in Liste
    if( i < size && (*element)->frame == frame )
        return;

    // Bild in Liste einreihen    
    key_entry *entry = new key_entry;

    entry->frame = frame;
    entry->offset = offset;

    // Bild vor gefundenem element einreihen
    if( i < table.size() ) {
        table.insert( element, entry );
    } else {
        table.push_back( entry );
    }

    //DEBUG_TXT( "insert Keyframe %i" , frame );
}






long KeyTable::GetKeyFrame( long frame, long long &offset )
{
    long key = -1;
    unsigned int i, size;
    list<key_entry*>::iterator element;
     
    size = table.size();
    element = table.begin();

    if( size == 0 )
        return -1;
        
    // Suche Bild in Liste
    for( i=0; i < size; i++ ) {
        if( (*element)->frame >= frame )
            break;
        element++;
    }

    if( i < size && (*element)->frame == frame ) {
        key = (*element)->frame;
        offset = (*element)->offset;
    } else if( i < size && i > 0 && (*element)->frame > frame ) {
        element--;
        key = (*element)->frame;
        offset = (*element)->offset;
    } else if( i==0 && size > 0 ) {
        key = 0;
    }

    //DEBUG_TXT( "get Keyframe %i" , key );
    
    return key;
}





long KeyTable::GetLastKeyFrame( long frame, long long &offset )
{
    long key = -1;
    unsigned int i, size;
    list<key_entry*>::iterator element;
    
    size = table.size();
    element = table.begin();

    if( size == 0 )
        return -1;

    // Suche Bild in Liste
    for( i=0; i < size; i++ ) {
        if( (*element)->frame >= frame )
            break;
        element++;
    }

    if( i < size && (*element)->frame == frame ) {
        key = (*element)->frame;
        offset = (*element)->offset;
    } else if( i < size && i > 0 && (*element)->frame > frame ) {
        element--;
        key = (*element)->frame;
        offset = (*element)->offset;
    } else if( i==0 && size > 0 ) {
        key = 0;
    } else if( i==size && size > 0 ) {
        element = table.end();
        element--;
        key = (*element)->frame;
        offset = (*element)->offset;
    }

    //DEBUG_TXT( "get Keyframe %i" , key );

    return key; 
}
