/*****************************************************************************\
**  Author  : Christian Tusche                                               **
**  E-Mail  : ctusche@gmx.net                        **
**                                                                           **
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/




#include <string.h>
#include <assert.h>

#include "puffer.h"



FifoPuffer::FifoPuffer()
{
    bytes = 0;
}





FifoPuffer::~FifoPuffer()
{
}








/**Puffer weiterschieben
  */
void FifoPuffer::ClearData( size_t n )
{
    if( n >= bytes ) {
        bytes = 0;
    }

    bytes -= n;
}







/**Puffer leeren
  */
void FifoPuffer::Reset( void )
{
    bytes = 0;
}






/**Finde "muster" im Puffer<br>
  *Rckgabe: Anfangsposition<br>
  *          oder -1, wenn nicht gefunden
  */
long FifoPuffer::Finden( size_t offset, const unsigned char *muster, size_t n )
{
    //unsigned char *gefunden;

    if( offset + n > bytes )
        return -1;
        
    //gefunden = (unsigned char *)memmem( puffer + offset, bytes - offset,
    //                                    muster, n);

    unsigned int i = 0, j = 0;
    for( i=0; i < bytes - n && j < n; i++ ) {
        for( j=0; j < n; j++ ) {
            if( Data(offset + i + j) != muster[j] )
                break;
        }
    }
    
    if( i >= bytes - n )
        return -1;
                                                
    return offset + i - 1;                                        
}







/**Finde "muster" im Puffer<br>
  *Verglichen werden nur die Bits aus "maske", die 1 sind<br>
  *gesucht wird ab der Position "offset"<br>
  *das gefundene Muster wird in "&gefunden" gespeichert
  *Rckgabe: Anfangsposition<br>
  *          oder -1, wenn nicht gefunden
  */
long FifoPuffer::Finden( size_t offset, uint32_t muster, uint32_t maske, uint32_t &gefunden )
{
    long begin;
    uint32_t wort = 0;

    if( Bytes() < 4 )
        return -1;

    wort = Data(offset)<<16 | Data(offset+1)<<8 | Data(offset+2);

    for( begin = offset; begin < (long)bytes - 3; begin++ ) {
        wort = (wort<<8) | Data(begin + 3);

        if( (wort & maske) == muster ) {
            gefunden = wort;
            return begin;
        }
    }

    if( begin >= (long)bytes - 3 ) {
        begin = -1;
    }

    return begin;
}








/**Zeiger auf Datenfeld an Stelle "offset"
  */
unsigned char FifoPuffer::Data( size_t offset )
{
    return 0;
}




/**Daten aus Puffer kopieren<br>
  *kopiert n Bytes aus dem Puffer in Position offset<br>
  *in das Datenfeld *data
  */
int FifoPuffer::GetData( size_t offset, uint8_t *data, size_t n )
{
    // prfe, ob Daten vorhanden sind
    if( bytes < offset + n )
        return -1;

    // kopiere Daten
    for( unsigned int i=0; i < n; ++i ) {
        data[i] = Data(offset+i);
    }

    return 1;
}
