/********************************************************************\
*****   Methoden fr IO in Quicktime Dateien                     *****
\********************************************************************/

#ifndef _O_QUICKTIME_H_
#define _O_QUICKTIME_H_

// die config.h von openquicktime darf nicht eingelesen werden!!!
#include <openquicktime/openquicktime.h>
#include <openquicktime/colormodels.h>

#undef PACKAGE
#undef PACKAGE_BUGREPORT
#undef PACKAGE_NAME
#undef PACKAGE_STRING
#undef PACKAGE_TARNAME
#undef PACKAGE_VERSION
#undef VERSION

#include "VideoFile/VideoFile.h"

//#define USE_FLOAT 1

class OQuicktimeA;
class OQuicktimeV;
class OQuicktimeVtr;







/****************************************************************************************/
/***** Quicktime - Klasse ***************************************************************/
/****************************************************************************************/

class OQuicktimeIO:public VideoFile {
    friend class OQuicktimeV;
    friend class OQuicktimeVtr;
    friend class OQuicktimeA;
    friend class OQuicktimeAtr;
public:
    OQuicktimeIO( const String Dateiname, IO_Mode mode=write );
    ~OQuicktimeIO();

    // Quicktime Datei-Descriptor
    oqt_t *QuicktimeDatei( void );


protected:
    /*** Videofunktionen ***/
    /**Video codec initialisieren
      */
    V_Track *openVideoTrack( double Rate,
                             int x,
                             int y,
                             const char *Codec = NULL );
    /*** Audiofunktionen ***/
    /**Audio codec initialisieren
      */
    A_Track *openAudioTrack( int Sample_rate,
                             int Channels,
                             const char *Codec = NULL );

    // Video-Track ffnen
    V_Track *openVideoTrack( int track );
    // Audio-Track ffnen
    A_Track *openAudioTrack( int track );

    oqt_t *Datei;         // Quicktime Datei
};









/****************************************************************************************/
/***** Quicktime - Videotrack ***********************************************************/
/****************************************************************************************/
class OQuicktimeVtr:public V_Track {
    friend class OQuicktimeV;
public:
    OQuicktimeVtr( OQuicktimeIO *File, int track );

    OQuicktimeVtr( OQuicktimeIO *File,
                   int track,
                   double rate,
                   int x, int y,
                   const char *Codec = NULL );

    ~OQuicktimeVtr();


    // Bild-Position setzen
    int Seek( long bild );

    // letzter Keyframe vor bild
    long GetKeyFrame( long bild );

    //int readData( unsigned char *&data, long &bytes );
    //int writeData( unsigned char *data, long bytes );


private:
    OQuicktimeIO *file;          // bergeordnete Klasse
    oqt_t *Datei;         // Quicktime Datei
    unsigned char *FrameBuf;    // Datenpuffer fr gelesene Bilddaten
    long BufGr;                 // Puffergre

protected:
    int readRaw( unsigned char *&data, long &bytes );
    int writeRaw( unsigned char *data, long bytes, int keyframe=0 );
};










/****************************************************************************************/
/***** Quicktime - Videocodec ***********************************************************/
/****************************************************************************************/
class OQuicktimeV:public V_Codec {
public:
    OQuicktimeV( OQuicktimeVtr *Track );

    ~OQuicktimeV();

    // Farbraum festlegen
    bool ColorModel( int cmodel );

    // JPEG-Qualitt
    bool jpegQuality( int q );

private:
    oqt_t *Datei;         // Quicktime Datei
    unsigned char **row_pointers;
    OQuicktimeVtr *track;

protected:
    // Video-Bild schreiben
    int encode( VideoFrame *bild );

    // Video-Bild lesen
    int decode( VideoFrame *bild );

    int qt_cmodel( int cmodel );     // Quicktime-C-Model -> Colormodel 
    int cmodel_qt( int cmodel );     // Colormodel -> Quicktime-C-Model
    void erz_row_pointers( unsigned char **row_pointers, unsigned char *buf );  // erzeuge row_pointers
                                                                              // abhngig von Farbraum
};









/****************************************************************************************/
/***** Qicktime - Audiotrack ************************************************************/
/****************************************************************************************/
class OQuicktimeAtr:public A_Track {
    friend class OQuicktimeA;
public:
    OQuicktimeAtr( OQuicktimeIO *File, int track );
    OQuicktimeAtr( OQuicktimeIO *File,
                   int track,
                   int Sample_rate,
                   int Channels,
                   const char *Codec_name = NULL );

    ~OQuicktimeAtr();
    
    // Position setzen
    int Seek( long block );
    
    /** Aktuelle Position in Sekunden
      */
    double CurrentTime( void );

private:
    OQuicktimeIO *file;             // bergeordnete Klasse
    oqt_t *Datei;                   // Quicktime Datei
    unsigned char *FrameBuf;        // Datenpuffer fr gelesene Bilddaten
    long BufGr;                     // Puffergre
    long BufBytes;                  // belegter Platz
    long BufSamples;                // Samples im Puffer
    bool internal_codec;            // interner Audio-Codec aus OpenQuicktime
    
protected:
    int readRaw( unsigned char *&data, long &bytes );
    int writeRaw( unsigned char *data, long bytes, int samp );
};







/****************************************************************************************/
/***** Qicktime - Audiocodec ************************************************************/
/****************************************************************************************/
class OQuicktimeA:public A_Codec {
public:
    OQuicktimeA( OQuicktimeAtr *Track );
    ~OQuicktimeA();

protected:
    // Audiodaten schreiben
    int encode( float **daten, int samples );

    // Audiodaten lesen
    int decode( float **daten, int samples );

    // positionieren
    int SeekSample( long long sample );

private:
    oqt_t *Datei;         // Quicktime Datei
    long long sample_pos;       // aktuelle leseposition
    uint8_t *sample_puffer;
    long sample_puffer_gr;
};



#endif
