/********************************************************************\
*****   Methoden fr IO in Quicktime Dateien                     *****
\********************************************************************/

#ifndef _QUICKTIME_H_
#define _QUICKTIME_H_

#include <quicktime.h>
#include <colormodels.h>

#include "VideoFile/VideoFile.h"

#define USE_FLOAT 1

class QuicktimeA;
class QuicktimeV;
class QuicktimeVtr;







/****************************************************************************************/
/***** Quicktime - Klasse ***************************************************************/
/****************************************************************************************/

class QuicktimeIO:public VideoFile {
    friend class QuicktimeV;
    friend class QuicktimeVtr;
    friend class QuicktimeA;
    friend class QuicktimeAtr;
public:
    QuicktimeIO( const String Dateiname, IO_Mode mode=write );
    ~QuicktimeIO();

    // Quicktime Datei-Descriptor
    quicktime_t *QuicktimeDatei( void );


protected:
    /*** Videofunktionen ***/
    /**Video codec initialisieren
      */
    V_Track *openVideoTrack( double Rate,
                             int x,
                             int y,
                             const char *Codec = NULL );
    /*** Audiofunktionen ***/
    /**Audio codec initialisieren
      */
    A_Track *openAudioTrack( int Sample_rate,
                             int Channels,
                             const char *Codec = NULL );

    // Video-Track ffnen
    V_Track *openVideoTrack( int track );
    // Audio-Track ffnen
    A_Track *openAudioTrack( int track );

    quicktime_t *Datei;         // Quicktime Datei
};









/****************************************************************************************/
/***** Quicktime - Videotrack ***********************************************************/
/****************************************************************************************/
class QuicktimeVtr:public V_Track {
    friend class QuicktimeV;
public:
    QuicktimeVtr( QuicktimeIO *File, int track );

    QuicktimeVtr( QuicktimeIO *File,
                  double rate,
                  int x, int y,
                  const char *Codec = NULL );

    ~QuicktimeVtr();


    // Bild-Position setzen
    int Seek( long bild );

    // letzter Keyframe vor bild
    long GetKeyFrame( long bild );

    //int readData( unsigned char *&data, long &bytes );
    //int writeData( unsigned char *data, long bytes );


private:
    QuicktimeIO *file;          // bergeordnete Klasse
    quicktime_t *Datei;         // Quicktime Datei
    unsigned char *FrameBuf;    // Datenpuffer fr gelesene Bilddaten
    long BufGr;                 // Puffergre

protected:
    int readRaw( unsigned char *&data, long &bytes );
    int writeRaw( unsigned char *data, long bytes, int keyframe=0 );
};










/****************************************************************************************/
/***** Quicktime - Videocodec ***********************************************************/
/****************************************************************************************/
class QuicktimeV:public V_Codec {
public:
    QuicktimeV( QuicktimeVtr *Track );

    ~QuicktimeV();

    // Farbraum festlegen
    bool ColorModel( int cmodel );

    // JPEG-Qualitt
    bool jpegQuality( int q );

private:
    quicktime_t *Datei;         // Quicktime Datei
    unsigned char **row_pointers;
    QuicktimeVtr *track;

protected:
    // Video-Bild schreiben
    int encode( VideoFrame *bild );

    // Video-Bild lesen
    int decode( VideoFrame *bild );

    int qt_c_model( int cmodel );     // Colormodel gem Quicktime
    void erz_row_pointers( unsigned char **row_pointers, unsigned char *buf );  // erzeuge row_pointers
                                                                              // abhngig von Farbraum
};









/****************************************************************************************/
/***** Qicktime - Audiotrack ************************************************************/
/****************************************************************************************/
class QuicktimeAtr:public A_Track {
    friend class QuicktimeA;
public:
    QuicktimeAtr( QuicktimeIO *File, int track );
    QuicktimeAtr( QuicktimeIO *File,
                  int Sample_rate,
                  int Channels,
                  const char *Codec_name = NULL );

    ~QuicktimeAtr();

private:
    QuicktimeIO *file;          // bergeordnete Klasse
    quicktime_t *Datei;         // Quicktime Datei
};







/****************************************************************************************/
/***** Qicktime - Audiocodec ************************************************************/
/****************************************************************************************/
class QuicktimeA:public A_Codec {
public:
    QuicktimeA( QuicktimeAtr *Track );
    ~QuicktimeA();

protected:
    // Audiodaten schreiben
    int encode( float **daten, int sampels );

    // Audiodaten lesen
    int decode( float **daten, int sampels );

    // positionieren
    int SeekSample( long long sample );

private:
    quicktime_t *Datei;         // Quicktime Datei
    long long sample_pos;       // aktuelle leseposition
    int16_t *decode_puffer;
    long decode_puffer_gr;
};



#endif
