/********************************************************************\
*****   Methoden fr IO in Quicktime Dateien                     *****
\********************************************************************/

#ifndef _WAV_H_
#define _WAV_H_

#include <fstream>
using namespace std;

#include "VideoFile/VideoFile.h"


class WavAtr;


class WavIO:public VideoFile {
    friend class WavAtr;
public:
    WavIO( const String datei, IO_Mode mode=write );
    ~WavIO();

protected:
    /*** Audiofunktionen ***/
    // Audio codec initialisieren
    A_Track *openAudioTrack( int Sample_rate,
                             int Channels,
                             const char *Codec = NULL );
    // Audio-Track ffnen
    A_Track *openAudioTrack( int track );

private:
    struct riff_header {
        unsigned char id[4];   /* RIFF */
        unsigned long len;
        unsigned char wave_id[4]; /* WAVE */
    };

    int write_riff_header( void );
    int read_riff_header( void );

    fstream *wav_datei;         // Datei - lesen/schreiben
    istream *wav_istream;       // Stream - lesen (stdin)
    ostream *wav_ostream;       // Stream - schreiben (stout)
};






class WavAtr:public A_Track {
public:

    WavAtr( WavIO *File );
    WavAtr( WavIO *File,
            int Sample_rate,
            int Channels,
            const char *Codec );


    ~WavAtr();


    // Audio-Position setzen
    int Seek( long block );

private:
    WavIO *file;

    // Datenfelder im Wav-Header
    struct chunk_struct {
        unsigned char id[4];
        unsigned long len;
    };

    struct common_struct {
        unsigned short wFormatTag;
        unsigned short wChannels;
        unsigned long dwSamplesPerSec;
        unsigned long dwAvgBytesPerSec;
        unsigned short wBlockAlign;
        unsigned short wBitsPerSample;
    };

    struct wav_header {
        struct chunk_struct  format;
        struct common_struct common;
        struct chunk_struct  data;
    };

    int write_wav_header( void );
    int read_wav_header( void );

    //Info aus dem Wave-Header;
    struct wav_header *wav_info;
    short format;

    unsigned char *puffer;
    long puffer_gr;

protected:
    int readRaw( unsigned char *&data, long &bytes );
    int writeRaw( unsigned char *data, long bytes, int samp );
};

#endif
