/********************************************************************\
*****   Methoden fr IO in MPPEG-Video Dateien                   *****
\********************************************************************/

#ifndef _MPEGVIDEO_H_
#define _MPEGVIDEO_H_

using namespace std;

#include <iostream>
#include <fstream>
#include "VideoFile/VideoFile.h"
#include "keytable.h"
#include "puffer.h"
#include "chunk.h"


class MpegVideoVtr;


class MpegVideoIO:public VideoFile {
    friend class MpegVideoVtr;
public:
    MpegVideoIO( const String datei, IO_Mode mode=write );
    ~MpegVideoIO();


protected:
    // Video-Track ffnen
    V_Track *openVideoTrack( double Rate,
                             int x,
                             int y,
                             const char *Codec = NULL );
                             
    V_Track *openVideoTrack( int track );

private:
    fstream *mpeg_datei;         // Datei - lesen/schreiben
    istream *mpeg_istream;       // Stream - lesen (stdin)
    ostream *mpeg_ostream;       // Stream - schreiben (stout)

    FifoPuffer *puffer;
    Chunk *chunk;

    // Informationen fr Videotrack
    int width, height;
    int aspect_ratio_code;
    int bitrate;
    double framerate;

    bool sync_block_start( void );
    int decode_header( unsigned char *head, int bytes );
    
    static const double mpeg_framerates[];
    double decode_framerate( int code );

    int version;

    unsigned char *sequence_header;     // Zwischenspeicher fr Sequence Header
    size_t sequence_header_gr;
};






class MpegVideoVtr:public V_Track {
public:

    MpegVideoVtr( MpegVideoIO *File );
    MpegVideoVtr( MpegVideoIO *File,
            double Rate,
            int x,
            int y,
            const char *Codec_name );


    ~MpegVideoVtr();


    /**Audio-Position setzen
      */
    int Seek( long block );

    /**liefert den letzten Keyframe zur angegebenen Position
      */
    long GetKeyFrame( long bild );

private:
    MpegVideoIO *file;

    // Ein Bild berspringen
    int skip_frame( void );
    
    bool sequence_header_gelesen;       // Der Header wurde schon einmal vom Codec gelesen
    unsigned char *data_puffer;         // Puffer fr gelesene Daten
    long data_puffer_gr;

    KeyTable key_table;         // Liste mit I-Frames
    
    /**Prfen, ob "data" einen I-Frame enthlt
      */
    bool test_keyframe( void );

    long max_pos;

protected:
    int readRaw( unsigned char *&data, long &bytes );
    int writeRaw( unsigned char *data, long bytes, int keyframe );
};

#endif
