/*****************************************************************************\
**  Author  : Christian Tusche                                               **
**  E-Mail  : ctusche@gmx.net                        **
**                                                                           **
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/




#include <string.h>
#include <assert.h>

#include "puffer.h"



FifoPuffer::FifoPuffer( istream *file, size_t gr )
{
    groesse = gr;
    puffer = new unsigned char[groesse];
    bytes = 0;
    position = 0;
    eof = false;
    datei = file;
}





FifoPuffer::~FifoPuffer()
{
    delete [] puffer;
}





/**Puffer aus Eingabedatei fllen<br>
  *Rckgabe:  1  Daten gelesen<br>
  *           0  keine Daten gelesen / eof<br>
  *          -1  Fehler
  */
int FifoPuffer::Fuellen( void )
{
    long read_bytes=0, read_bytes2=0;

    // eof zurcksetzern
    eof = false;
    
    // Endposition der Daten
    size_t end_pos = position + bytes;
    if( end_pos >= groesse )
        end_pos -= groesse;

    // Restlnge bis zum Ende
    size_t rest = groesse - bytes;
    if( end_pos + rest > groesse )
        rest = groesse - end_pos;

    // Restlnge vom Anfang
    size_t anfang = groesse - bytes - rest;
    
    // einen Block am Ende enlesen
    if( rest > 0 ) {
        datei->read( (char*)&(puffer[end_pos]), rest );
        read_bytes = datei->gcount();
        bytes += read_bytes;
        if( !datei->good() ) {
            if( datei->eof() ) {
                eof = true;
                if( read_bytes > 0 )
                    return 1;
                return 0;
            } else {
                return -1;
            }
        }
    }

    if( anfang > 0 ) {
        datei->read( (char*)puffer, anfang );
        read_bytes2 = datei->gcount();
        bytes += read_bytes2;
        if( !datei->good() ) {
            if( datei->eof() ) {
                eof = true;
                if( read_bytes > 0 || read_bytes2 > 0 )
                    return 1;
                return 0;
            } else {
                return -1;
            }
        }
    }
    
    return 1;

}





/**Puffer weiterschieben
  */
void FifoPuffer::Schieben( size_t n )
{
    if( n >= bytes ) {
        bytes = 0;
        position = 0;
        return;
    }

    position += n;
    if( position >= groesse )
        position -= groesse;

    bytes -= n;
}







/**Puffer leeren
  */
void FifoPuffer::Reset( void )
{
    bytes = 0;
    position = 0;
}






/**Finde "muster" im Puffer<br>
  *Rckgabe: Anfangsposition<br>
  *          oder -1, wenn nicht gefunden
  */
long FifoPuffer::Finden( size_t offset, const unsigned char *muster, size_t n )
{
    //unsigned char *gefunden;

    if( offset + n > bytes )
        return -1;
        
    //gefunden = (unsigned char *)memmem( puffer + offset, bytes - offset,
    //                                    muster, n);

    unsigned int i = 0, j = 0;
    for( i=0; i < bytes - n && j < n; i++ ) {
        for( j=0; j < n; j++ ) {
            if( *Data(offset + i + j) != muster[j] )
                break;
        }
    }
    
    if( i >= bytes - n )
        return -1;
                                                
    return offset + i - 1;                                        
}







/**Zeiger auf Datenfeld an Stelle "offset"
  */
unsigned char *FifoPuffer::Data( size_t offset )
{
    //assert( offset < groesse );

    size_t pos = position + offset;
    if( pos >= groesse )
        pos -= groesse;
        
    return &(puffer[pos]);
}






/**Daten aus Puffer kopieren
  */
void FifoPuffer::Kopieren( size_t offset, uint8_t *data, size_t n )
{
    assert( n + offset <= bytes );

    size_t pos = 0;

    long bytes = n;
    long anfang = position + offset;
    
    if( anfang >= (long)groesse )
        anfang -= groesse;

    if( anfang + bytes > (long)groesse )
        bytes = groesse - anfang;

    if( bytes > 0 ) {
        memcpy( data, puffer + anfang, bytes );
        pos += bytes;
    }

    bytes = n - bytes;
    
    if( bytes > 0 ) {
        memcpy( data + pos, puffer, bytes );
    }
    
}
