/*****************************************************************************\
**  Author  : Christian Tusche                                               **
**  E-Mail  : ctusche@gmx.net                        **
**                                                                           **
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/

#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <sys/types.h>
#include <assert.h>

#include "DVCodec.h"

#define DV_VERSION_NEW 1


/*****************************************************************************/
/***** Videofunktionen *******************************************************/
/*****************************************************************************/

DVCodec::DVCodec( V_Track *Track )
:V_Codec( Track )
{
    // initialisiere DV-Lib
#ifdef DV_VERSION_NEW
    // libdv - Version > 0.9
    dv_decoder = dv_decoder_new(0,0,0);
#else
    // libdv - Version = 0.9
    dv_init();
    dv_decoder = dv_decoder_new();
#endif
    
    // Qualitt fr Dekodierung setzen (wichtig!!!)
    dv_decoder->quality = DV_QUALITY_BEST;

    // Color Model einstellen - Standard RGB24
    ColorModel( RGB24 );
}



DVCodec::~DVCodec()
{
#ifdef DV_VERSION_NEW
    // libdv - Version > 0.9
    dv_decoder_free( dv_decoder );
#endif
}




// Farbraum festlegen
bool DVCodec::ColorModel( int cmodel )
{
    // verwende intern Standardfarbraum und benutze Konvertierung
    // z.Z nur RGB24 mgich
    if( cmodel != RGB24 ) {
        V_Codec::ColorModel( RGB24 );
        return false;
    }

    return V_Codec::ColorModel( cmodel );
}






// Video-Bild lesen
int DVCodec::decode( VideoFrame *bild )
{
    uint8_t *data;
    uint8_t *yuv[3];
    long bytes;
    int pitches[3];

    track->readData( data, bytes );

    // Zeiger auf YUV-Planes / RGB-Data
    bild->FramePointer( yuv[0] );

    // YUV-planar
    //bild->FramePointer( yuv[0], yuv[1], yuv[2] );
    
    pitches[0] = breite * 3;
    pitches[1] = 0;
    pitches[2] = 0;

    /*
    pitches[0] = breite;
    pitches[1] = breite / 2;
    pitches[2] = breite / 2;
    */

    // Header dekodieren
    if( dv_parse_header(dv_decoder, data) < 0 )
        return 0;

    // Bild decodieren
    dv_decode_full_frame( dv_decoder, data, e_dv_color_rgb, yuv, pitches );
    //dv_decode_full_frame( dv_decoder, data, e_dv_color_yuv, yuv, pitches );

    return 1;
}
