/*****************************************************************************\
                          ffmpegCodec.C  -  description
                             -------------------
    begin                : Sun Nov 25 2001
    copyright            : (C) 2001 by Christian Tusche
    email                : ctusche@gmx.net
\*****************************************************************************/

/*****************************************************************************\
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/

#include <stdio.h>
#include <assert.h>
#include "debug.h"
#include "ffmpegCodec.h"
#include "VideoFile/translate.h"

ffmpegCodec::ffmpegCodec( V_Track *Track )
:V_Codec( Track )
{
    enum CodecID cid = CODEC_ID_NONE;
    
    frame_bytes = 0;
    frame_data = NULL;
    init = false;

    // Optionen
    bitrate = 0;
    min_quality = 0;
    max_quality = 0;
    gop_size = 0;
    high_quality = 0;
    motion_estimation = 0;

    // Codec-Context initialisieren
    memset( &context, 0, sizeof(AVCodecContext) );


        
    // Compressor / Dekompressor initialisieren
    if( CodecName() == "Mpeg-1" ) {
        cid = CODEC_ID_MPEG1VIDEO;
    } else if( CodecName() == "Mpeg-2" ) {
        cid = CODEC_ID_MPEG2VIDEO;
    } else if ( CodecName() == "DivX-v3" || CodecName() == "MS-MPEG4-v3" ) {
        cid = CODEC_ID_MSMPEG4V3;
    } else if ( CodecName() == "MS-MPEG4-v2" ) {
        cid = CODEC_ID_MSMPEG4V2;
    } else if ( CodecName() == "MS-MPEG4-v1" ) {
        cid = CODEC_ID_MSMPEG4V1;
    } else if ( CodecName() == "OpenDivX" ) {
        cid = CODEC_ID_MPEG4;
    } else if ( CodecName() == "H263" || CodecName() == "U263" ) {
        cid = CODEC_ID_H263P;
    } else if ( CodecName() == "I263" ) {
        cid = CODEC_ID_H263I;
    } else if ( CodecName() == "M-JPEG" ) {
        cid = CODEC_ID_MJPEG;
    } else if ( CodecName() == "L-JPEG" ) {
        cid = CODEC_ID_LJPEG;
    } else if ( CodecName() == "Indeo3" ) {
        cid = CODEC_ID_INDEO3;
    } else if ( CodecName() == "WMV-2" ) {
        cid = CODEC_ID_WMV2;
    } else if ( CodecName() == "VCR1" ) {
        cid = CODEC_ID_VCR1;
    } else if ( CodecName() == "On2-VP3" ) {
        cid = CODEC_ID_VP3;
    } else if ( CodecName() == "ASUS-V1" ) {
        cid = CODEC_ID_ASV1;
    } else if ( CodecName() == "ASUS-V2" ) {
        cid = CODEC_ID_ASV2;
    } else {
        ERROR_TXT( _("Codec %s not implemented!"), CodecName().chars() );
        cid = CODEC_ID_NONE;
    }
    
    
    if( iomode == VideoFile::read )
        avcodec = avcodec_find_decoder( cid );
    else
        avcodec = avcodec_find_encoder( cid );
    
    
    
    // Datenpuffer fr kompremierte Daten
    // nehme maximale gre fr unkomopremierte RGB32 Daten an
    // das sollte auf jeden Fall reichen
    data_buf_bytes = breite * hoehe * 4;
    data_buf = new unsigned char[ data_buf_bytes ];
    data_bytes = 0;

    AddOption( "Bitrate", _("average bitrate in kbps") ) = 1200;
    AddOption( "KeyInterval", _("distance between I-frames") ) = 16;
    AddOption( "MotionEstimation", _("method used for motion estimation (0:zero 1:full 2:log 3:phods)")  ) = 1;
    AddOption( "MinQuant", _("lowest quantisation (1: high quality - 32: low quality)") ) = 2;
    AddOption( "MaxQuant", _("highest quantisation (1: high quality - 32: low quality)") ) = 8;
        
    // Optionen fr Mpeg4
    if( cid == CODEC_ID_MPEG4 ) {
    
        AddOption( "HighQuality", _("quality of the encoder (0: low / 1: high)") ) = 1;
    }

    ColorModel( YUV420P );
}






ffmpegCodec::~ffmpegCodec()
{
    if( init )
        avcodec_close( &context );
    
    delete [] data_buf;
}






// Codec ffen und Optionen setzen
void ffmpegCodec::init_codec( void )
{
    avcodec_get_context_defaults( &context );
    
    context.me_method = motion_estimation;
    
    // Ratecontrol
    context.bit_rate = bitrate * 1000;
    context.bit_rate_tolerance = bitrate;
    context.rc_min_rate = bitrate * 999;
    context.rc_max_rate = bitrate * 1001;
    context.max_qdiff = 1;
    context.qcompress = 0.5;
    context.qblur = 0.5;
    context.qmin = min_quality;
    context.qmax = max_quality;
    context.b_quant_factor = 1.25;
    context.b_quant_offset = 1.25;
    context.i_quant_factor = -0.8;
    context.i_quant_offset = 0.0;
    context.rc_initial_cplx = 1.0;
    
    // Nur fr Mpeg-4
    if( high_quality )
        context.flags |= CODEC_FLAG_4MV | CODEC_FLAG_QPEL;


    // context.mpeg_quant;

    

    /* audio only */
    //int sample_rate; ///< samples per sec
    //int channels;
    //int sample_fmt;  ///< sample format, currenly unused
 
    
    /* video only */
    context.frame_rate = (int)(track->Rate() * 10000); /* frames per sec multiplied by FRAME_RATE_BASE */
    context.frame_rate_base = 10000;
    context.width = breite;
    context.height = hoehe;

    context.gop_size = gop_size; /* 0 = intra only */
    //context.max_b_frames = 0;

    
    context.pix_fmt = PIX_FMT_YUV420P;  /* pixel format, see PIX_FMT_xxx */

    if( avcodec_open( &context, avcodec) < 0 ) {
        ERROR_TXT( _("Error opening the video-codec!") );
        return;
    }

    init = true;
}





// Farbraum festlegen
bool ffmpegCodec::ColorModel( int cmodel )
{
    //DEBUG_TXT( "ffmpegCodec::ColorModel: setze Colormodel %i", cmodel );

    // verwende intern Standardfarbraum und benutze Konvertierung
    if( cmodel != YUV420P ) {
        V_Codec::ColorModel( YUV420P );
        return false;
    }

    return V_Codec::ColorModel( cmodel );
}







int ffmpegCodec::SetOption( OptionEntry *opt )
{

    if( opt->Name() == "Bitrate" ) {
        if( (int)*opt < 10 )
            *opt = 10;
        bitrate = (int)*opt;

    } else if( opt->Name() == "KeyInterval" ) {
        if( (int)*opt < 0 )
            *opt = 0;
        gop_size = (int)*opt;

    } else if( opt->Name() == "MinQuant" ) {
        if( (int)*opt < 1 )
            *opt = 1;
        else if( (int)*opt > 32 )
            *opt = 32;
        min_quality = (int)*opt;

    } else if( opt->Name() == "MaxQuant" ) {
        if( (int)*opt < 1 )
            *opt = 1;
        else if( (int)*opt > 32 )
            *opt = 32;
        max_quality = (int)*opt;

    } else if( opt->Name() == "HighQuality" ) {
        if( (int)*opt != 0 && (int)*opt != 1 )
            *opt = 1;
        high_quality = (int)*opt;

    } else if( opt->Name() == "MotionEstimation" ) {
        if( (int)*opt < 0 )
            *opt = 0;
        else if( (int)*opt > 3 )
            *opt = 3;
        motion_estimation = (int)*opt;

    } else {
        return 0;
    }

    return 1;
}







// Bild schreiben
int ffmpegCodec::encode( VideoFrame *bild )
{
    unsigned char *yuv[3];
    AVFrame pic;

    // Codec-Context initialisieren
    memset( &pic, 0, sizeof(AVFrame) );


    if( !init )
        init_codec();

    if( !init )
        return 0;
        
    // Zeiger auf Planes erstellen
    if( bild->GetFrame( yuv[0], yuv[1], yuv[2] ) <= 0 )
        return 0;
        
    pic.data[0] =yuv[0];
    pic.data[1] =yuv[1];
    pic.data[2] =yuv[2];

    pic.linesize[0] = breite;
    pic.linesize[1] = pic.linesize[2] = breite / 2;

    data_bytes = avcodec_encode_video( &context, data_buf, data_buf_bytes, &pic);

    // Daten schreiben
    return track->writeData( data_buf, data_bytes, pic.key_frame ? V_Track::IS_KEYFRAME : 0 );
}







// Video-Bild lesen
int ffmpegCodec::decode( VideoFrame *bild )
{
    int i=0, pic_ptr=0, n=0;
    AVFrame pic;
    uint8_t *bild_buf = NULL;

    if( !init )
        init_codec();

    if( !init )
        return 0;
        
    while( pic_ptr == 0 ) {
        if( frame_bytes <= 0 ) {
            // Daten einlesen
            int j=0;
            if( (j = track->readData( frame_data, frame_bytes )) < 1 ) {
                return j;
            }
        }

        i = avcodec_decode_video( &context, &pic, &pic_ptr, frame_data, frame_bytes );
        frame_data += i;
        frame_bytes -= i;
    
        if( i < 0 )
            return 0;

        if( n++ > 2 )
            return 0;
    }


    
    // Bild kopieren
    if( pic_ptr == 0 )
        return 0;

    if( bild == NULL )
        return 1;

    if( bild->FramePointer( bild_buf ) <= 0 )
        return 0;
        
    long offset = 0;
    for( i=0; i<hoehe; i++ ) {
        memcpy( bild_buf + offset,
                pic.data[0] + i*pic.linesize[0],
                breite );
        offset += breite;   
    }

    for( i=0; i<(hoehe+1)/2; i++ ) {
        memcpy( bild_buf + offset,
               pic.data[1] + i*pic.linesize[1],
               breite/2 );
        offset += breite/2;
    }

    for( i=0; i<(hoehe+1)/2; i++ ) {
        memcpy( bild_buf + offset,
                pic.data[2] + i*pic.linesize[2],
                breite/2 );
        offset += breite/2;
    }
     
    return 1;
}







int ffmpegCodec::SeekFrame( long bild )
{
    long keyframe = track->GetKeyFrame( bild );

    // auf jeden Fall erstes Bild decodieren, um Puffer zu fllen
    if( track->Position() == 0 ) {
        decode( NULL );
    }

    // wenn keine Keyframeinformation vorhanden, einfach 32 Bilder zurck gehen
    // (maximale GOP-Gre?)
    if( bild > 0 && keyframe == bild && track->GetKeyFrame( bild - 1 ) == bild - 1 ) {
        keyframe = bild - 32;
        if( keyframe < 0 )
            keyframe = 0;
    }

    DEBUG_TXT( _("Seek to keyframe %li for frame %li"), keyframe, bild );

    // auf Keyframe positionieren
    if( !track->Seek( keyframe ) ) {
        return 0;
    }
    frame_bytes = 0;

    // Puffer mit erstem Bild fllen
    //track->readData( data, bytes );
    //mpeg2_decode_data( &mpeg2dec, data, data + bytes );

    // decodiere Frames bis zur aktuellen Position
    while( track->Position() < bild ) {
        decode( NULL );
    }

    return 1;
}








int ffmpegCodec::CopyFrame( void )
{
    if( iomode != VideoFile::write )
        return 0;
        
    return track->writeData( data_buf, data_bytes );
}
