#include <unistd.h>

#include "VideoFile/PluginList.h"
#include "VideoFile/Codec_Info.h"
#include "VideoFile/translate.h"

#include "ffmpegCodec.h"
#include "ffmpegAudio.h"


static PluginList<Codec_Info> *Info;


V_Codec *create_codec( V_Track *track )
{
    return new ffmpegCodec( track );
}


A_Codec *create_audio( A_Track *track )
{
    return new ffmpegAudio( track );
}



extern "C" {

const char *plugin_init( void )
{
    // libavcodec initialisieren
    avcodec_init();
    avcodec_register_all();


    // Plugininformation erstellen
    Info = new PluginList<Codec_Info>;

    //
    // Video-Codecs
    //  
    String Text = _("ffmpeg: read and write mpeg video");

    //                        DateiTyp    Codecalias  Konstruktor    Codecname  Beschreibung lesen  schreiben
    *Info += new Codec_Info( "MpegVideo", "Mpeg-1",   &create_codec, "Mpeg-1",   Text,        true,  true );
    *Info += new Codec_Info( "MpegVideo", "Mpeg-2",   &create_codec, "Mpeg-2",   Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "Mpeg-1",   &create_codec, "mpg1",     Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "Mpeg-1",   &create_codec, "PIM1",     Text,        true,  false );
    *Info += new Codec_Info( "Avi",       "Mpeg-1",   &create_codec, "vcr2",     Text,        true,  false );
    *Info += new Codec_Info( "Avi",       "Mpeg-2",   &create_codec, "mpg2",     Text,        true,  false );


    Text = _("ffmpeg: read and write DivX3 video");

    //                        DateiTyp    Codecalias  Konstruktor    Codecname  Beschreibung lesen  schreiben
    *Info += new Codec_Info( "Avi",       "DivX-v3",  &create_codec, "DIV3",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "DivX-v3",  &create_codec, "div3",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "DivX-v3",  &create_codec, "DIV5",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "DivX-v3",  &create_codec, "div5",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "DivX-v3",  &create_codec, "DIV6",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "DivX-v3",  &create_codec, "div6",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "DivX-v3",  &create_codec, "MP41",    Text,        true,  true );    

    Text = _("ffmpeg: read and write OpenDivX video");

    //                        DateiTyp    Codecalias  Konstruktor    Codecname  Beschreibung lesen  schreiben
    *Info += new Codec_Info( "Avi",       "OpenDivX", &create_codec, "DIVX",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "OpenDivX", &create_codec, "divx",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "OpenDivX", &create_codec, "DX50",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "OpenDivX", &create_codec, "MP4S",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "OpenDivX", &create_codec, "M4S2",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "OpenDivX", &create_codec, "DIV1",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "OpenDivX", &create_codec, "BLZ0",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "OpenDivX", &create_codec, "mp4v",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "OpenDivX", &create_codec, "UMP4",    Text,        true,  true );


    Text = _("ffmpeg: read and write ms-mpeg4 video");

    //                        DateiTyp    Codecalias     Konstruktor    Codecname  Beschreibung lesen  schreiben
    *Info += new Codec_Info( "Avi",       "MS-MPEG4-v3", &create_codec, "MP43",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "MS-MPEG4-v3", &create_codec, "mp43",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "MS-MPEG4-v2", &create_codec, "MP42",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "MS-MPEG4-v2", &create_codec, "mp42",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "MS-MPEG4-v2", &create_codec, "DIV2",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "MS-MPEG4-v2", &create_codec, "div2",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "MS-MPEG4-v1", &create_codec, "MPG4",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "MS-MPEG4-v1", &create_codec, "mpg4",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "MS-MPEG4-v1", &create_codec, "DIV4",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "MS-MPEG4-v1", &create_codec, "div4",    Text,        true,  true );
                
    Text = _("ffmpeg: read and write H263 video");

    //                        DateiTyp    Codecalias  Konstruktor    Codecname  Beschreibung lesen  schreiben
    *Info += new Codec_Info( "Avi",       "H263",     &create_codec, "H263",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "H263",     &create_codec, "h263",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "H263",     &create_codec, "viv1",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "U263",     &create_codec, "U263",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "U263",     &create_codec, "u263",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "I263",     &create_codec, "I263",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "I263",     &create_codec, "i263",    Text,        true,  true );

    Text = _("ffmpeg: read and write MJPEG video");

    //                        DateiTyp    Codecalias  Konstruktor    Codecname  Beschreibung lesen  schreiben
    *Info += new Codec_Info( "Avi",       "M-JPEG",   &create_codec, "MJPG",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "M-JPEG",   &create_codec, "mjpg",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "L-JPEG",   &create_codec, "LJPG",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "L-JPEG",   &create_codec, "JPGL",    Text,        true,  true );


    Text = _("ffmpeg: read and write Windows-Media video");

    //                        DateiTyp    Codecalias  Konstruktor    Codecname  Beschreibung lesen  schreiben
    *Info += new Codec_Info( "Avi",       "WMV-2",    &create_codec, "WMV2",    Text,        true,  true );

    
    Text = _("ffmpeg: read and write Intel-Indeo video");

    //                        DateiTyp    Codecalias  Konstruktor    Codecname  Beschreibung lesen  schreiben
    *Info += new Codec_Info( "Avi",       "Indeo3",   &create_codec, "IV31",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "Indeo3",   &create_codec, "IV32",    Text,        true,  true );
    

    Text = _("ffmpeg: read and write ATI VCR1 video");

    //                        DateiTyp    Codecalias  Konstruktor    Codecname  Beschreibung lesen  schreiben
    *Info += new Codec_Info( "Avi",       "VCR1",     &create_codec, "VCR1",    Text,        true,  true );
  

    Text = _("ffmpeg: read and write On2 VP3 video");

    //                        DateiTyp    Codecalias  Konstruktor    Codecname  Beschreibung lesen  schreiben
    *Info += new Codec_Info( "Avi",       "On2-VP3",  &create_codec, "VP31",    Text,        true,  false );


    Text = _("ffmpeg: read and write ASUS V1/V2 video");

    //                        DateiTyp    Codecalias  Konstruktor    Codecname  Beschreibung lesen  schreiben
    *Info += new Codec_Info( "Avi",       "ASUS-V1",  &create_codec, "ASV1",    Text,        true,  true );
    *Info += new Codec_Info( "Avi",       "ASUS-V2",  &create_codec, "ASV2",    Text,        true,  true );

    




    
    //
    // Audio-Codecs
    //
    Text = _("ffmpeg: read and write AC3 audio");

    //                        DateiTyp    Codecalias  Kontruktor     Codecname  Beschreibung lesen  schreiben
    *Info += new Codec_Info( "Avi",       "ac3",      &create_audio, "2000",    Text,        true,  true );
    *Info += new Codec_Info( "Mpeg",      "ac3",      &create_audio, "ac3",     Text,        true,  true );


    Text = _("ffmpeg: write mp2 audio");
    
    *Info += new Codec_Info( "Avi",       "mp2",      &create_audio, "0050",    Text,        false,  true );
    *Info += new Codec_Info( "mp3",       "mp2",      &create_audio, "mp2",     Text,        false,  true );
    *Info += new Codec_Info( "Mpeg",      "mp2",      &create_audio, "mp2",     Text,        false,  true );

    Text = _("ffmpeg: read ADPCM audio");

    *Info += new Codec_Info( "Avi",       "ADPCM-MS",  &create_audio, "0002",    Text,        true,  false );
    *Info += new Codec_Info( "Wav",       "ADPCM-MS",  &create_audio, "0002",    Text,        true,  false );
    
    Text = _("ffmpeg: read and write ADPCM audio");

    *Info += new Codec_Info( "Avi",       "ADPCM-IMA", &create_audio, "0011",    Text,        true,  true );
    *Info += new Codec_Info( "Wav",       "ADPCM-IMA", &create_audio, "0011",    Text,        true,  true );


    Text = _("ffmpeg: write Windows-Media audio");

    *Info += new Codec_Info( "Avi",       "WMA1",      &create_audio, "0160",    Text,        true,  false );
    *Info += new Codec_Info( "Avi",       "WMA2",      &create_audio, "0161",    Text,        true,  false );


        
    return "V_Codec";
}




void *plugin_info( void )
{
    return Info;
}




void plugin_delete( void )
{
    delete Info;
}


}



