/*****************************************************************************\
**  Author  : Christian Tusche                                               **
**  E-Mail  : ctusche@gmx.net                        **
**                                                                           **
** This program is free software; you can redistribute it and/or modify      **
** it under the terms of the GNU General Public License as published by      **
** the Free Software Foundation; either version 2 of the License, or         **
** (at your option) any later version.                                       **
**                                                                           **
** This program is distributed in the hope that it will be useful,           **
** but WITHOUT ANY WARRANTY; without even the implied warranty of            **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             **
** GNU General Public License for more details.                              **
**                                                                           **
** You should have received a copy of the GNU General Public License         **
** along with this program; if not, write to the Free Software               **
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                 **
\*****************************************************************************/

#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <sys/types.h>
#include <assert.h>

#include "MpegCodec.h"
#include "debug.h"
#include "VideoFile/translate.h"




/*****************************************************************************/
/***** Videofunktionen *******************************************************/
/*****************************************************************************/

MpegCodec::MpegCodec( V_Track *Track )
:V_Codec( Track )
{
    uint32_t accel=0;
    vo_driver_t * drivers = vo_drivers();
    
    // initialisiere Mpeg2dec-Lib
    accel = mm_accel () | MM_ACCEL_MLIB;
    vo_accel (accel);

    vo_output = vo_open( drivers[0].open );

    assert( vo_output != NULL );

    mpeg2_init( &mpeg2dec, accel, vo_output );

    // Color Model einstellen - Standard YUV420P
    ColorModel( YUV420P );
}



MpegCodec::~MpegCodec()
{
    mpeg2_close( &mpeg2dec );
    vo_close( vo_output );
}




// Farbraum festlegen
bool MpegCodec::ColorModel( int cmodel )
{
    // verwende intern Standardfarbraum und benutze Konvertierung
    if( cmodel != YUV420P ) {
        V_Codec::ColorModel( YUV420P );
        return false;
    }

    return V_Codec::ColorModel( cmodel );
}





int MpegCodec::SeekFrame( long bild )
{
    long keyframe = track->GetKeyFrame( bild );
    unsigned char *data;
    long bytes=0;

    // wenn keine Keyframeinformation vorhanden, einfach 32 Bilder zurck gehen
    // (maximale GOP-Gre?)
    if( bild > 0 && keyframe == bild && track->GetKeyFrame( bild - 1 ) == bild - 1 ) {
        keyframe = bild - 32;
        if( keyframe < 0 )
            keyframe = 0;
    }

    // auf jeden Fall erstes Bild decodieren, um Puffer zu fllen
    if( track->Position() == 0 && keyframe > 0 ) {
        if( !track->readData( data, bytes ) )
            return 0;
        mpeg2_decode_data( &mpeg2dec, data, data + bytes );
    }

    
    DEBUG_TXT( _("seek to keyframe %li for frame %li"), keyframe, bild );
                
    // auf Keyframe positionieren
    if( !track->Seek( keyframe ) ) {
        return 0;
    }

    // Puffer mit erstem Bild fllen
    //track->readData( data, bytes );
    //mpeg2_decode_data( &mpeg2dec, data, data + bytes );
    
    // decodiere Frames bis zur aktuellen Position
    while( track->Position() < bild ) {
        if( !track->readData( data, bytes ) )
            return 0;
        mpeg2_decode_data( &mpeg2dec, data, data + bytes );
    }

    return 1;
}





// Video-Bild lesen
int MpegCodec::decode( VideoFrame *bild )
{
    unsigned char *data;
    long bytes;
    null_instance_t *vo_inst = (null_instance_t *) vo_output;
    int frames=0;

    // Frame lesen und decodieren
    int i = 0;

    if( track->Position() == 0 ) {
        if( !track->readData( data, bytes ) )
            return 0;

        frames = mpeg2_decode_data( &mpeg2dec, data, data + bytes );
    }

    if( !track->readData( data, bytes ) )
        return 0;
        
    frames = mpeg2_decode_data( &mpeg2dec, data, data + bytes );

    if( frames == 0 && i > 0) {
        DEBUG_TXT( _("No frame encoded!") );
        return 1;
    }
    
    // Frame in Puffer kopieren
    bild->SetFrame( vo_inst->yuv_frame[0], vo_inst->yuv_frame[1], vo_inst->yuv_frame[2] );
    
    return 1;
}
