/* 
 * video_out_null.c
 * Copyright (C) 1999-2001 Aaron Holtzman <aholtzma@ess.engr.uvic.ca>
 *
 * This file is part of mpeg2dec, a free MPEG-2 video stream decoder.
 *
 * mpeg2dec is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * mpeg2dec is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "config.h"

#include <stdlib.h>
#include <inttypes.h>

#include "video_out_null.h"




static void null_draw_frame (vo_frame_t * frame)
{
    null_instance_t *instance = (null_instance_t *) frame->instance;
     
    instance->yuv_frame[0] = frame->base[0];
    instance->yuv_frame[1] = frame->base[1];
    instance->yuv_frame [2] = frame->base[2];
}



static int null_setup (vo_instance_t * instance, int width, int height, int frame_rate_code)
{
    return libvo_common_alloc_frames (instance, width, height,
				      sizeof (null_frame_t),
				      NULL, NULL, null_draw_frame);
}



vo_instance_t * vo_null_open (void)
{
    null_instance_t * instance;
    int i = 0;

    instance = malloc (sizeof (null_instance_t));
    if (instance == NULL)
        return NULL;

    for( i=0; i<3; i++ ) {
        instance->vo.frame_ptr[i] = NULL;
    }
    instance->vo.setup = null_setup;
    instance->vo.close = libvo_common_free_frames;
    instance->vo.get_frame = libvo_common_get_frame;

    return (vo_instance_t *) instance;
}
