#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/shm.h>
#include <sys/ipc.h>
#include "audio.h"
#include "../XJ.h"
#include "../RTjpeg.h"
#include "NS.h"

RTjpeg_t *rtj;

#define ABUF 262144
#define AOFS 25
unsigned char abuf[ABUF];

int shmid=0;
pid_t aud_pid=0;
pid_t vid_pid=0;

uint8_t Q=0;
uint16_t width=0;
uint16_t height=0;
uint32_t vsize=0;

#define VBUFS 15
int8_t *vbufD[VBUFS+1];
uint8_t vbufQ[VBUFS];
uint32_t vbufF[VBUFS];
int vbuf_head=0;
int vbuf_tail=0;
int vbuf_queue=0;

unsigned char *vid_buf=NULL;
char *vid_sbuf;
uint8_t *rows[576];
uint8_t *planes[3];

struct sharedataS
{
 volatile unsigned int cur_field;
} *sharedata;

volatile int playing=0;

void (*vidconv)(RTjpeg_t *rtj, uint8_t **planes, uint8_t **rows);

void vidplay(void)
{
  RTjpeg_decompress(rtj, vbufD[vbuf_tail], planes);
  vidconv(rtj, planes, rows);
  XJ_show();
  if(++vbuf_tail==VBUFS)vbuf_tail=0;
  vbuf_queue--;
  fprintf(stderr, "P ");
}

void viddrop(void)
{
 if(++vbuf_tail==VBUFS)vbuf_tail=0;
 vbuf_queue--;
 fprintf(stderr, "D! ");
}

int next_tail(void)
{
 int i=vbuf_tail;
 
 if(++i==VBUFS)i=0;
 return i;
}

void falarm(int i)
{
 if(sharedata->cur_field!=0)
  sharedata->cur_field++;
 fprintf(stderr, "%d ", vbuf_queue);

 if(vbuf_queue>0)
 {
  if(!(playing++))
  {
   if(sharedata->cur_field>vbufF[vbuf_tail]) /* OK the current tail is ready to play */
   {
    /* if there are more queued test if we are behind */
    while((sharedata->cur_field>vbufF[next_tail()])&&(vbuf_queue>1)) viddrop();
    vidplay();
   }
  }
  playing--;
 }
}

void quit(int i)
{
 signal(SIGALRM, SIG_IGN);
 if(shmid!=0)
 {
  shmctl(shmid, IPC_RMID, NULL);
  shmdt((char *)(sharedata));
 }
 if (aud_pid!=0) kill(aud_pid, SIGINT);
 if (vid_pid!=0) kill(vid_pid, SIGINT);
 XJ_exit();
 fprintf(stderr, "\nExiting on signal %d\n", i);
                   
 exit(i);
}

void vidQ(struct B_head_vid vhead)
{
 int i;
 
 if(!vid_buf) /* initialise */
 {
  width=vhead.width;
  height=vhead.height;
  vsize=width*height+((width*height)>>1);
  Q=vhead.Q;
 
  rtj = RTjpeg_init();
  i = RTJ_YUV420;
  RTjpeg_set_format(rtj, &i);
  for(i=0; i<=VBUFS; i++)
  {
   vbufD[i]=malloc(vsize);
   if(!vbufD[i])
   {
    fprintf(stderr, "Could not allocate memory for video buffers\n");
    quit(-1);
   }
  }
  vid_sbuf=XJ_init(width, height, "NetStream", "NetStream");
  if(!vid_sbuf)
  {
   fprintf(stderr, "Could not initialise X\n");
   quit(-1);
  }
  vid_buf=malloc(vsize);
  if(!vid_buf)
  {
   fprintf(stderr, "Could not allocate memory for decompress buffer\n");
   quit(-1);
  }
  planes[0] = vid_buf;
  planes[1] = planes[0]+width*height;
  planes[2] = planes[1]+(width*height)/4;
  for(i=0; i<height; i++)
   rows[i] = vid_sbuf + width*(XJ_depth/8)*i;
  switch (XJ_depth)
  {
   case 8: vidconv=RTjpeg_yuv420rgb8;
   	   break;
   case 16: vidconv=RTjpeg_yuv420rgb16;
   	    break;
   case 24: vidconv=RTjpeg_yuv420bgr24;
   	    // break;
   case 32: vidconv=RTjpeg_yuv420bgr32; break;
   	    break;
   default: fprintf(stderr, "Unsupported display depth\n");
   	    quit(-1);
   	    break;
  }
  // temp
//  if(sharedata->cur_field==0)sharedata->cur_field=vhead.field-AOFS; /* half sec */
 }
 if((vhead.width!=width)||(vhead.height!=height)||(vhead.size>vsize))
 {
  fprintf(stderr, "B ");
  return;
 }
 if(vbuf_queue==VBUFS)
 {
  fprintf(stderr, "D ");
  si_read(vbufD[VBUFS], vhead.size);
//  sharedata->cur_field=vhead.field-AOFS;
  return;
 }
 vbufQ[vbuf_head]=vhead.Q;
 vbufF[vbuf_head]=vhead.field;
 si_read(vbufD[vbuf_head], vhead.size);
 if(++vbuf_head==VBUFS)vbuf_head=0;
 vbuf_queue++;
}


void getsync(struct B_head_vid *vhead)
{
 unsigned int t=0;
 unsigned char *cbuf=(unsigned char *)vhead;
 
 while(t!=SYNCW)
 {
  si_read(cbuf, 1);
  t=(t>>8)&0x00ffffffU;
  t|=(cbuf[0]<<24)&0xff000000U;
 }
 si_read(&(cbuf[4]), HSIZE-4);
 fprintf(stderr, "S ");
}

int main(int argc, char **argv)
{
 struct B_head_vid *vhead=(struct B_head_vid *)abuf;
 struct B_head_aud *ahead=(struct B_head_aud *)abuf;
 struct itimerval ttime;
 struct sigaction asig={falarm, 0, SA_RESTART|SA_NOMASK, NULL};
 
 signal(SIGINT, quit);

 if((shmid=shmget(0, sizeof(struct sharedataS), IPC_CREAT|IPC_EXCL|S_IRWXU))<0)
 {
  perror("shmget");
  quit(-1);
 }
 if((sharedata=(struct sharedataS *)shmat(shmid, NULL, 0))<0)
 {
  perror("shmat");
  quit(-1);
 }
 sharedata->cur_field=0;
 vbuf_head=0;
 vbuf_tail=0;
 vbufF[0]=0;
 
 ttime.it_interval.tv_sec=0;
 ttime.it_interval.tv_usec=20000; /* PAL 50fps */
 ttime.it_value.tv_sec=0;
 ttime.it_value.tv_usec=20000; /* PAL 50fps */
 if(setitimer(ITIMER_REAL, &ttime, NULL)!=0)
 {
  perror("setitimer");
  quit(-1);
 }
 
 if(sigaction(SIGALRM, &asig, NULL)!=0)
  {
   fprintf(stderr, "Could not capture alarm signal\n");
   quit(-1);
  }
                  
 audio_init(22050, 8, 0, O_WRONLY, 50);                  
                         
 while(1)
 {
  si_read(vhead, HSIZE);
  if(vhead->sync!=SYNCW)getsync(vhead);
  if(vhead->type==TYPE_VIDEO)
  {
   vidQ(*vhead);
  } else
  {
   if(ahead->size>ABUF)ahead->size=ABUF;
   si_read(&(ahead->data), ahead->size);
   a_write(&(ahead->data), ahead->size);
   if(abs(sharedata->cur_field-ahead->field-AOFS)>12)
    sharedata->cur_field=ahead->field-AOFS;
  }
 }
 return 0;
}
