#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <errno.h>
#include <unistd.h>
#include <linux/videodev.h>
#include "audio.h"
#include "../RTjpeg.h"
#include "NS.h"

RTjpeg_t *rtj;

#define ABUF 262144

unsigned char abuffer[ABUF];

int fd;
int8_t *vdata;
struct B_head_vid *vbuf;
struct B_head_aud *abuf=(struct B_head_aud *)abuffer;
unsigned long fsize;
unsigned long ftot=0;
unsigned long stot=0;
unsigned long dropped=0;
unsigned int lf=0, laf=0, lvf=0, lrf=0;
int M,Q;
int w=384, h=288;

int Vsync(int f)
{
 int e;

 do
 {
  e=ioctl(fd, VIDIOCSYNC, &f);
  if(e>=0) return e;
 } while(errno==-EINTR);
 perror("VIDIOCSYNC");
 return -1;
}

void writeit(unsigned char *buf, uint32_t field)
{
 int tmp;
 static uint8_t *planes[3];

 if(lrf==0)
 {
  lrf=field;
  lvf=field;
  laf=field;
 }

 if(field>=(lrf+4))
 {
  planes[0] = buf;
  planes[1] = planes[0]+w*h;
  planes[2] = planes[1]+(w*h)/4;
  tmp=RTjpeg_compress(rtj, &(vbuf->data), planes);
  vbuf->size=tmp;
  vbuf->field=field;
  so_write(vbuf, tmp+HSIZE);
  if(ftot>0x0fffffff)
  {
   ftot=0;
   stot=0;
  }
 
  ftot+=fsize;
  stot+=tmp;
  
  dropped+=(field-(lrf+4));
  lrf=field;
 }
 
 if((field-laf)>15)
 {
  fprintf(stderr, "aud\n\n");
  tmp=a_read(&(abuf->data));
  fprintf(stderr, "aud %d\n\n", tmp);
  abuf->field=field;
  abuf->size=tmp;
  so_write(abuf, tmp+HSIZE); 
  laf=field;
 }
 
 fprintf(stderr, "dropped %ld %u ratio %f      \r", dropped, field,  (float)ftot/(float)stot);
}

int main(int argc, char** argv)
{
  uint32_t field = 0;

  struct video_mmap mm;
  struct video_mbuf vm;
  unsigned char *buf;
  int frame;
  int i, j;

  if(argc!=3)
  {
   fprintf(stderr, "Usage: test3 <Q factor> <motion threshold>\n\n");
   exit(-1);
  }

  Q=atoi(argv[1]);
  M=atoi(argv[2]);
  fsize=(w*h)+((w*h)>>1);
  
  fd = open("/dev/video0", O_RDWR|O_CREAT);
  if(fd<=0){
    perror("open");
    exit(-1);
  }
  if(ioctl(fd, VIDIOCGMBUF, &vm)<0)
  {
  	perror("VIDIOCMCAPTUREi0");
  	exit(-1);
  }
  if(vm.frames<2)
  {
  	fprintf(stderr, "stoopid prog want min 2 cap buffs!\n");
  	exit(-1);
  }
  buf = (unsigned char*)mmap(0, vm.size, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0);
  if (buf<=0)
  {
   perror("mmap");
   exit(-1);
  }	
  
  vbuf=malloc(HSIZE+fsize);
  vbuf->sync=SYNCW;
  vbuf->type=TYPE_VIDEO;
  vbuf->width=w;
  vbuf->height=h;
  vbuf->Q=Q;
  
  rtj = RTjpeg_init();
  RTjpeg_set_size(rtj, &w, &h);
  RTjpeg_set_quality(rtj, &Q);
  i = RTJ_YUV420;
  RTjpeg_set_format(rtj, &i);
  i = 25;
  j = M/2;
  RTjpeg_set_intra(rtj, &i, &M, &j);

  audio_init(22050, 8, 0, O_RDONLY, 50);
  abuf->sync=SYNCW;
  abuf->type=TYPE_AUDIO;
  abuf->speed=16000;
  abuf->channels=0;
  abuf->bits=16;

  mm.frame  = 0;
  mm.height = h;
  mm.width  = w;
  mm.format = VIDEO_PALETTE_YUV420P    ; /* YCrCb422 */
  if(ioctl(fd, VIDIOCMCAPTURE, &mm)<0) perror("VIDIOCMCAPTUREi0");
  mm.frame  = 1;
  if(ioctl(fd, VIDIOCMCAPTURE, &mm)<0) perror("VIDIOCMCAPTUREi1");
  while(1)
  {
   frame=0;
   mm.frame  = 0;
   if(Vsync(frame)>=0)
    {
     //ioctl(fd, BTTV_FIELDNR, &field);
     field+=2;
     writeit(buf+vm.offsets[0], field);
    }
   if(ioctl(fd, VIDIOCMCAPTURE, &mm)<0) perror("VIDIOCMCAPTURE0");
   frame=1;
   mm.frame  = 1;
   if(Vsync(frame)>=0)
    {
     //ioctl(fd, BTTV_FIELDNR, &field);
     field+=2;
     writeit(buf+vm.offsets[1], field);
    }
   if(ioctl(fd, VIDIOCMCAPTURE, &mm)<0) perror("VIDIOCMCAPTURE1");
  }
 return 0;
}
