#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/soundcard.h>
#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <stdlib.h>

int _dsp=-1;
int _bsize;
int _bpf;
int _aud_init=0;

void audio_init(int speed, int bits, int channels, int mode, int fps)
{
 int f;

 if(_dsp!=-1)
 {
  close(_dsp);
 }
 _dsp=open("/dev/dsp", mode);
 if(_dsp<=0)
 {
  perror("dsp open");
  exit(-1);
 }

 f=0x00800008U;
 if (ioctl(_dsp, SNDCTL_DSP_SETFRAGMENT, &f)<0)
 {
  perror("setfrag");
  exit(-1);
 }


 f=bits;
 if (ioctl(_dsp, SNDCTL_DSP_SAMPLESIZE, &f)<0)
 {
  perror("samplesize");
  exit(-1);
 }

 f=channels;
 if (ioctl(_dsp, SNDCTL_DSP_STEREO, &f)<0)
 {
  perror("stereo");
  exit(-1);
 }

 f=speed;
 ioctl(_dsp, SNDCTL_DSP_SPEED, &f);

 if (ioctl(_dsp, SNDCTL_DSP_GETBLKSIZE, &_bsize)<0)
 {
  perror("getblk");
  exit(-1);
 }

// fprintf(stderr, "BLK: %d\n", _bsize);

 _bpf=(speed*(channels+1)*((bits==16)?2:1))/fps;
}

void a_write(unsigned char * buf, int count)
{
 struct audio_buf_info info;
 int t;
 
 if(_aud_init==0)
 {
  info.bytes=_bsize;
  _aud_init=1;
 } else
 {
  if (ioctl(_dsp, SNDCTL_DSP_GETOSPACE, &info)<0)
  {
   perror("getospace");
   exit(-1);
  }
 }
// fprintf(stderr, "ISP: %d   CNT: %d ", info.bytes, count);
 
 if(count>=_bsize)
  while(info.bytes>=_bsize)
  {
   t=write(_dsp, buf, _bsize);
   if(t==-1)
   {
    if(errno!=EINTR)perror("audio write");
    t=0;
//    exit(-1);
   }
   info.bytes-=t;
   count-=t;
   if(count>=_bsize)buf+=t;
    else break;
//   fprintf(stderr, ".");
  }
// fprintf(stderr, "\n");
}

int a_read(unsigned char * buf)
{
 struct audio_buf_info info;
 int count=0, t;
 

 if(_aud_init==0)
 {
  info.bytes=_bsize;
  _aud_init=1;
 } else
 {
  if (ioctl(_dsp, SNDCTL_DSP_GETISPACE, &info)<0)
  {
   perror("getispace");
   exit(-1);
  }
 }
// fprintf(stderr, "ISP: %d ", info.bytes);

 if(info.bytes==0) info.bytes = _bsize;

 while(info.bytes>=_bsize)
 {
  t=read(_dsp, buf, _bsize);
  if(t==0)
  {
   fprintf(stderr, "\nPossible DMA overrun, restarting audio.\n");
   ioctl(_dsp, SNDCTL_DSP_RESET);
  }
  if(t==-1)
  {
   perror("audio read");
   t=0;
  }
  buf+=t;
  count+=t;
  info.bytes-=t;
//  fprintf(stderr, ".");
 }
// fprintf(stderr, "\n");

 return count;
}
