#include <stdio.h>
#include <string.h>
#include "funcprotos.h"
#include "quicktime.h"
#include "colormodels.h"
#include "../RTjpeg.h"


static inline void out_colormodels( quicktime_t *file, int track, int in_cmodel, unsigned char **yuv_pointers, unsigned char **row_pointers );
static inline void in_colormodels( quicktime_t *file, int track, int out_cmodel, unsigned char **yuv_pointers, unsigned char **row_pointers );
static inline int make_row_pointers( unsigned char **row_pointers, unsigned char *buf, int cmodel, int w, int h );


typedef struct
{
    int quality;                    /* 0 - 100 quality */
    char *output_buffer;            /* Buffer for RTJPEG data */
    char *input_buffer;             /* Buffer for RTJPEG data */
    unsigned char *cplanes[3];      /* Buffer for YUV output */
    unsigned char *dplanes[3];      /* Buffer for YUV output */
    unsigned char **row_pointers;
    RTjpeg_t *compress_struct;
    RTjpeg_t *decompress_struct;
    int (*quicktime_encode_video_external)(quicktime_t *, unsigned char **, int );
    int (*quicktime_decode_video_external)(quicktime_t *, unsigned char **, int );
    long last_position;             // last frame decoded
    int recursive;                  // codec was called recursive -> skip color transform
} quicktime_rtjpeg_codec_t;







static int quicktime_writes_colormodel_rtjpeg(quicktime_t *file,
		int colormodel,
		int track)
{
    fprintf(stderr, "WRITE_CMODEL, %d\n", colormodel);
    return( colormodel == BC_YUV420P || colormodel == BC_RGB888 );
}









static int quicktime_reads_colormodel_rtjpeg(quicktime_t *file,
		int colormodel,
		int track)
{
    fprintf(stderr, "READ_CMODEL, %d\n", colormodel);
    return( (colormodel == BC_YUV420P) || (colormodel == BC_RGB888) );
}




/****************************************************************************************/
/***** Encoder **************************************************************************/
/****************************************************************************************/

int qt_extern_encode(quicktime_t *file, int track, unsigned char *input, unsigned char *output)
{
    long bytes = 0;
    quicktime_video_map_t *vtrack = &(file->vtracks[track]);
    quicktime_trak_t *trak = vtrack->track;
    quicktime_rtjpeg_codec_t *codec;
    int w = trak->tkhd.track_width;
    int h = trak->tkhd.track_height;

    // fprintf( stderr, "encode_extern %d\n", file->color_model );

    codec = ((quicktime_codec_t*)file->vtracks[track].codec)->priv;

    if( !make_row_pointers( codec->row_pointers, input, file->color_model, w, h ) ) {
        fprintf(stderr, "Invalid compress color format!");
        return -1;
    }

    switch(file->color_model) {
    case BC_YUV420P:
        bytes = RTjpeg_compress(codec->compress_struct, output, codec->row_pointers);
        break;
    default:
        in_colormodels( file, track, BC_YUV420P, codec->dplanes, codec->row_pointers );
        bytes = RTjpeg_compress(codec->compress_struct, output, codec->dplanes);
    }
    return bytes;
}







static int quicktime_encode_rtjpeg(quicktime_t *file, unsigned char **row_pointers, int track )
{
    quicktime_rtjpeg_codec_t *codec;
    int result;

    codec = ((quicktime_codec_t*)file->vtracks[track].codec)->priv;

    // save row_pointers in privat codec data
    codec->row_pointers = row_pointers;

    // call original quicktime plugin encode function
    result = codec->quicktime_encode_video_external( file, row_pointers, track );

    return result;
}








/****************************************************************************************/
/***** Decoder **************************************************************************/
/****************************************************************************************/
int qt_extern_decode(quicktime_t *file, int track, unsigned long inputsize, unsigned char *input, unsigned char *output)
{
    int result = 0;
    //int i;
    quicktime_video_map_t *vtrack = &(file->vtracks[track]);
    quicktime_rtjpeg_codec_t *codec;
    int position = vtrack->current_position;
    int w, h, key;

    // fprintf(stderr, "decode_extern %d\n", file->color_model);
    w = vtrack->track->tkhd.track_width;
    h = vtrack->track->tkhd.track_height;

    codec = ((quicktime_codec_t*)file->vtracks[track].codec)->priv;

    /*
    if( !make_row_pointers( codec->row_pointers, output, file->color_model, w, h ) ) {
        fprintf(stderr, "Invalid compress color format!");
        return -1;
    }
    */

    // is this a key frame?
    key = RTjpeg_frame_key( input );
    if( key != 0 && position != codec->last_position + 1 && !codec->recursive ) {
        // go back to last key frame
        vtrack->current_position = position - key;

        // call codec recursive until current frame reached
        codec->recursive = 1;
        while( vtrack->current_position < position ) {
            codec->quicktime_decode_video_external( file, codec->row_pointers, track );
            vtrack->current_position++;
        }
	codec->quicktime_decode_video_external( file, codec->row_pointers, track );
        codec->recursive = 0;
    } else {
        // decode frame
        RTjpeg_decompress( codec->decompress_struct, input, codec->dplanes);
    }

    codec->last_position = position;

    if( codec->recursive ) {
        return result;
    }

    switch (file->color_model) {
        case BC_YUV420P:
            memcpy( codec->row_pointers[0], codec->dplanes[0], w * h );
            memcpy( codec->row_pointers[1], codec->dplanes[1], w/2 * ((h+1)/2) );
            memcpy( codec->row_pointers[2], codec->dplanes[2], w/2 * ((h+1)/2) );
            break;
/*
        case BC_RGB8:
            RTjpeg_yuv420rgb8( codec->decompress_struct, codec->dplanes, codec->row_pointers);
            break;
        case BC_RGB565:
			RTjpeg_yuv420rgb16( codec->decompress_struct, codec->dplanes, codec->row_pointers);
			break;
		case BC_RGB888:
			RTjpeg_yuv420rgb24( codec->decompress_struct, codec->dplanes, codec->row_pointers);
			break;
		case BC_RGBA8888:
			RTjpeg_yuv420rgb32( codec->decompress_struct, codec->dplanes, codec->row_pointers);
			break;
		case BC_BGR888:
			RTjpeg_yuv420bgr24( codec->decompress_struct, codec->dplanes, codec->row_pointers);
			break;
		case BC_BGR8888:
			RTjpeg_yuv420bgr32( codec->decompress_struct, codec->dplanes, codec->row_pointers);
			break;
*/
        default:
            // color transfer
            out_colormodels( file, track, BC_YUV420P, codec->dplanes, codec->row_pointers );
    }
	return result;
}









static int quicktime_decode_rtjpeg( quicktime_t *file, unsigned char **row_pointers, int track )
{
    quicktime_rtjpeg_codec_t *codec;
    int result;

    codec = ((quicktime_codec_t*)file->vtracks[track].codec)->priv;

    // save row_pointers in privat codec data
    codec->row_pointers = row_pointers;

    // call original quicktime plugin decode function
    result = codec->quicktime_decode_video_external( file, row_pointers, track );

    return result;
}











int quicktime_delete_codec_rtjpeg(quicktime_video_map_t *vtrack)
{
	quicktime_rtjpeg_codec_t *codec = ((quicktime_codec_t*)vtrack->codec)->priv;

    //fprintf(stderr, "\nDELETE\n");

	if(codec->output_buffer)
		free(codec->output_buffer);
	codec->output_buffer = 0;
	if(codec->input_buffer)
		free(codec->input_buffer);
	codec->input_buffer = 0;
	if(codec->cplanes[0])
		free(codec->cplanes[0]);
	codec->cplanes[0] = 0;
	if(codec->dplanes[0])
		free(codec->dplanes[0]);
	codec->dplanes[0] = 0;
	RTjpeg_close( codec->compress_struct );
	RTjpeg_close( codec->decompress_struct );
	free(codec);
	return 0;
}








/****************************************************************************************/
/***** init codec ***********************************************************************/
/****************************************************************************************/
void quicktime_init_codec_rtjpeg(quicktime_video_map_t *vtrack)
{
	quicktime_rtjpeg_codec_t *codec;
	int i, w, h;

	w = vtrack->track->tkhd.track_width;
	h = vtrack->track->tkhd.track_height;

    codec = malloc(sizeof(quicktime_rtjpeg_codec_t));
	((quicktime_codec_t*)vtrack->codec)->priv = (void *)codec;
	((quicktime_codec_t*)vtrack->codec)->reads_colormodel = quicktime_reads_colormodel_rtjpeg;
	((quicktime_codec_t*)vtrack->codec)->writes_colormodel = quicktime_writes_colormodel_rtjpeg;

    //overload decode / encode functions of quicktime plugin-module
    codec->quicktime_encode_video_external = ((quicktime_codec_t*)vtrack->codec)->encode_video;
    codec->quicktime_decode_video_external = ((quicktime_codec_t*)vtrack->codec)->decode_video;
    ((quicktime_codec_t*)vtrack->codec)->decode_video = quicktime_decode_rtjpeg;
    ((quicktime_codec_t*)vtrack->codec)->encode_video = quicktime_encode_rtjpeg;

	codec->quality = 100;
	codec->compress_struct = RTjpeg_init();
	i = RTJ_YUV420;
	RTjpeg_set_format( codec->compress_struct, &i);
	RTjpeg_set_size( codec->compress_struct, &w, &h);
	i = (int)((float)codec->quality / 100.0 * 160.0) + 32;
	RTjpeg_set_quality( codec->compress_struct, &i);

	codec->decompress_struct = RTjpeg_init();
	i = RTJ_YUV420;
	RTjpeg_set_format( codec->decompress_struct, &i);

	codec->output_buffer = malloc(w*h*2);
	codec->input_buffer = malloc(w*h*2);
	codec->cplanes[0] = malloc(w*h+(w*h)/2);
	codec->cplanes[1] = codec->cplanes[0]+w*h;
	codec->cplanes[2] = codec->cplanes[1]+(w*h)/4;
	codec->dplanes[0] = malloc(w*h+(w*h)/2);
	codec->dplanes[1] = codec->dplanes[0]+w*h;
	codec->dplanes[2] = codec->dplanes[1]+(w*h)/4;
	//codec->row_pointers = malloc(sizeof(unsigned char *) * h);
    codec->last_position = 0;
    codec->recursive = 0;
	//fprintf(stderr, "\nINIT %ld\n", (unsigned long int)codec);
}




void quicktime_codec_register(quicktime_extern_video_t * vcodec)
{
    //fprintf(stderr, "Register\n" );
	strcpy(vcodec->fourcc, "RTJ0");
	vcodec->init = quicktime_init_codec_rtjpeg;
	vcodec->encode = qt_extern_encode;
	vcodec->decode = qt_extern_decode;
	vcodec->delete_codec = quicktime_delete_codec_rtjpeg;
}
















/***** transfer colormodel after decoding ***********************************************/
static inline void out_colormodels( quicktime_t *file, int track, int in_cmodel, unsigned char **yuv_pointers, unsigned char **row_pointers )
{

    quicktime_video_map_t *vtrack = &(file->vtracks[track]);
    int track_width = vtrack->track->tkhd.track_width;
    int track_height = vtrack->track->tkhd.track_height;
    //printf( "colormodel = %i\n", file->color_model );
    cmodel_transfer(row_pointers,
		            yuv_pointers,
		            row_pointers[0],
		            row_pointers[1],
		            row_pointers[2],
		            yuv_pointers[0],
		            yuv_pointers[1],
		            yuv_pointers[2],
		            file->in_x,
		            file->in_y,
		            file->in_w,
		            file->in_h,
		            0,
		            0,
		            file->out_w,
		            file->out_h,
		            in_cmodel,
		            file->color_model,
		            0,
		            track_width,
		            track_height);
}






/***** transfer colormodel before encoding **********************************************/
static inline void in_colormodels( quicktime_t *file, int track, int out_cmodel, unsigned char **yuv_pointers, unsigned char **row_pointers )
{
    quicktime_video_map_t *vtrack = &(file->vtracks[track]);
    int track_width = vtrack->track->tkhd.track_width;
    int track_height = vtrack->track->tkhd.track_height;
    //printf( "colormodel = %i\n", file->color_model );
    cmodel_transfer(yuv_pointers,
		            row_pointers,
		            yuv_pointers[0],
		            yuv_pointers[1],
		            yuv_pointers[2],
		            row_pointers[0],
		            row_pointers[1],
		            row_pointers[2],
    	            0,
		            0,
		            track_width,
		            track_height,
		            0,
		            0,
		            track_width,
		            track_height,
		            file->color_model,
		            out_cmodel,
		            0,
		            track_width,
		            track_height);
}






static inline int make_row_pointers( unsigned char **row_pointers, unsigned char *buf, int cmodel, int w, int h )
{
    int i;

    switch( cmodel ) {
    case BC_YUV420P:
        row_pointers[0] = buf;                                          // y - Plane
        row_pointers[1] = row_pointers[0] + w * h;                      // u - Plane
        row_pointers[2] = row_pointers[1] + w/2 * ((h + 1)/2) ;         // v - Plane
        break;
    case BC_YUV422P:
    case BC_YUV422:
        row_pointers[0] = buf;                                          // y - Plane
        row_pointers[1] = row_pointers[0] + w * h;                      // u - Plane
        row_pointers[2] = row_pointers[1] + w/2 * h;                    // v - Plane
        break;
    case BC_RGB888:
    case BC_BGR888:
    case BC_YUV888:
        for( i = 0; i<h; i++ )
            row_pointers[i] = buf + i * 3 * w;
        break;
    case BC_RGB8:
        for(i = 0; i < h; i++)
            row_pointers[i] = buf + i * w;
        break;
    case BC_RGB565:
    case BC_BGR565:
        for(i = 0; i < h; i++)
            row_pointers[i] = buf + i * w * 2;
        break;
    case BC_RGBA8888:
    case BC_BGR8888:
        for(i = 0; i < h; i++)
            row_pointers[i] = buf + i * w * 4;
        break;
    default:
        fprintf( stderr, "Colormodel %i", cmodel );
        return -1;
    }

    return 1;
}
