/* PGM reader c/o "Rene K. Mueller" <kiwi@the-labs.com> */

#include <stdlib.h>
#include <stdio.h>
#include "RTjpeg.h"

void rgb_yuv(int w, int h, unsigned char *rgb, unsigned char *buf) {
   register int i,j; char *off = buf+w*h, *off2 = buf+w*h+w*h/4;
   for(j=0; j<h; j++) {
      for(i=0; i<w; i++) {
         int b = *rgb++;
         int g = *rgb++;
         int r = *rgb++;
         int y,cb,cr;
         y  =  (0.257 * r) + (0.504 * g) + (0.098 * b) + 16;
         cr =  (0.439 * r) - (0.368 * g) - (0.071 * b) + 128;
         cb = -(0.148 * r) - (0.291 * g) + (0.439 * b) + 128;
         if(y<0) y = 0; if(y>255) y = 255;
         if(cb<0) cb = 0; if(cb>255) cb = 255;
         if(cr<0) cr = 0; if(cr>255) cr = 255;
         *buf++ = y;
         if(j&1&&i&1)
            *off++ = cr;
         if(j&1&&i&1)
            *off2++ = cb;
      }
   }
}

int main(int argc, char **argv)
{
 int w, h, tmp;
 FILE *f;
 unsigned char *yuv, *rgb;
 char t[256];
 int8_t *strm;
 int Q;
 int i;
 RTjpeg_t *rtj;
 uint8_t **rows;
 uint8_t *planes[3];
 
 if(argc!=3)
 {
  fprintf(stderr, "usage: test21 <filename> <Q factor (32..255)>\n\n");
  exit(-1);
 }
 
 f=fopen(argv[1], "r");
 Q=atoi(argv[2]);
 fgets(t, 256, f);
 do
 {
  fgets(t, 256, f);
 } while(t[0]=='#');
 sscanf(t, "%d %d", &w, &h);
 fgets(t, 256, f);

 rtj = RTjpeg_init();
 i = RTJ_YUV420;
 RTjpeg_set_format(rtj, &i);
 RTjpeg_set_size(rtj, &w, &h);
 RTjpeg_set_quality(rtj, &Q);
 
 yuv=(unsigned char *)malloc(w*h+(w*h)/2);
 strm=malloc(w*h+(w*h)/2+10);
 rgb=(unsigned char *)malloc(w*h*3);
 planes[0] = yuv;
 planes[1] = planes[0]+w*h;
 planes[2] = planes[1]+(w*h)/4;
 rows = malloc(sizeof(uint8_t *)*h);
 for(i=0; i< h; i++)
  rows[i] = rgb+w*3*i;

 tmp=fread(rgb, 1, w*h*3, f);
 fclose(f);
 rgb_yuv(w,h,rgb,yuv);
  
 fprintf(stderr, "Read %d bytes (%dx%d)\n", tmp, w, h);

 for(i=0; i<100; i++)
 {
   tmp=RTjpeg_compress(rtj, strm, planes);
 }
 
 RTjpeg_close(rtj);
 
 fprintf(stderr, "strmlen: %d (%f)\n", tmp, (w*h*3)/(float)tmp);
 
 rtj = RTjpeg_init();
 i = RTJ_YUV420;
 RTjpeg_set_format(rtj, &i);

for(i=0; i<100; i++)
{
   RTjpeg_decompress(rtj, strm, planes);
}
for(i=0; i<100; i++)
{
   RTjpeg_yuv420rgb24(rtj, planes, rows);
 }
 fprintf(stdout, "P6\n");
 fprintf(stdout, "%d %d\n", w, h);
 fprintf(stdout, "255\n");
 fwrite(rgb, 1, w*h*3, stdout);

 
 return 0;
}
